/*
 * WinUsbIF.cpp: WinUSB Interface
 * Copyright: Keil - An ARM Company
 * Version: 1.0.1
 */

#include "stdafx.h"
#include "windows.h"

#include <stdio.h>

#undef _MP  // Required for WinDDK USB headers

extern "C"
{
#include "setupapi.h"
#include "winusb.h"
}

#include "WinUsbIF.h"

// Configuration defines
DEFINE_GUID( DEFAULT_WINUSB_DEVICE_INTERFACE_GUID, 0x7D9ADCFCL, 0xE570, 0x4B38, 0xBF, 0x4E, 0x8F, 0x81, 0xF6, 0x89, 0x64, 0xE0 );

// Local variables
static GUID           WinUsbDevIfGUID = DEFAULT_WINUSB_DEVICE_INTERFACE_GUID;   // Device Interface GUID
static int            WinUsbDevCount;                                           // Number of Devices
static WinUsbDevice_t WinUsbDevice [MAX_DEV_NUM];                               // Device runtime information

// Local functions
static BOOL WinUsbIF_LoadDeviceInfo (UCHAR dev);
static void StrDesc2Str             (PUSB_STRING_DESCRIPTOR strDscr, TCHAR *str);


// ---- User API functions -------------------------------------

/*
 *  WinUSB Interface: Initialization
 *    Parameters:      None
 *    Return Value:    None
 */
void WinUsbIF_Initialize (void) {

  WinUsbDevCount = 0;
  memset(WinUsbDevice, 0, sizeof(WinUsbDevice));
}


/*
 *  WinUSB Interface: Uninitialization
 *    Parameters:      None
 *    Return Value:    None
 */
void WinUsbIF_Uninitialize (void) {
  UCHAR dev;

  for (dev = 0; dev < MAX_DEV_NUM; dev++) {
    if (WinUsbDevice[dev].Open) {
      WinUsbIF_CloseDevice(dev);
    }
  }
}


/*
 *  WinUSB Interface: Set Device Interface GUID
 *    Parameters:      guid:  Pointer to GUID
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_SetGUID (GUID *guid) {

  memcpy(&WinUsbDevIfGUID, guid, sizeof(GUID));

  return TRUE;
}


/*
 *  WinUSB Interface: Get Device Interface GUID
 *    Parameters:      guid:  Pointer to GUID
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_GetGUID (GUID *guid) {

  memcpy(guid, &WinUsbDevIfGUID, sizeof(GUID));

  return TRUE;
}


/*
 *  WinUSB Interface: Find Devices
 *    Parameters:      None
 *    Return Value:    Number of Devices found
 */
int WinUsbIF_FindDevices (void) {
  HDEVINFO                          deviceInfo;
  SP_DEVICE_INTERFACE_DATA          interfaceData;
  PSP_DEVICE_INTERFACE_DETAIL_DATA  detailData = NULL;
  TCHAR                            *ptr_mi, *ptr_str;
  TCHAR                             path[MAX_PATH];
  ULONG                             length;
  UCHAR                             dev, idx, mi, i, j;
  BOOL                              result;

  // If there is an open Device close it first
  for (dev = 0; dev < MAX_DEV_NUM; dev++) {
    if (WinUsbDevice[dev].Open) { 
      WinUsbIF_CloseDevice(dev);
    }
  }

  // Initialize local variable and structure keeping Device information
  WinUsbDevCount = 0;
  memset(WinUsbDevice, 0, sizeof(WinUsbDevice));

  // Get Device Information for all present devices
  deviceInfo = SetupDiGetClassDevs(&WinUsbDevIfGUID,
                                    NULL,
                                    NULL,
                                    DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
  if (deviceInfo == INVALID_HANDLE_VALUE) { return 0; }

  interfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);

  dev =  0;
  idx = -1;

  // Scan all Interfaces
  do {
    idx++;

    // Device Interface Element of a Device Information set
    result = SetupDiEnumDeviceInterfaces( deviceInfo,
                                          NULL,
                                         &WinUsbDevIfGUID,
                                          idx,
                                         &interfaceData);
    if (!result) { break; }

    // Get Device Interface Details - Get Length
    SetupDiGetDeviceInterfaceDetail( deviceInfo,
                                    &interfaceData,
                                     NULL,
                                     0,
                                    &length,
                                     NULL);

    // Free/Allocate memory for Device Detailed Data
    if (detailData != NULL) { free(detailData); }
    detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(length);
    if (detailData == NULL) { return 0; }

    // Set cbSize in the detailData structure
    detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

    // Get Device Interface Details
    result = SetupDiGetDeviceInterfaceDetail( deviceInfo,
                                             &interfaceData,
                                              detailData,
                                              length,
                                              NULL,
                                              NULL);
    if (!result) { continue; }

	  if (detailData->DevicePath == NULL) { continue; }

  	ptr_mi = _tcsstr(detailData->DevicePath, TEXT("mi_"));

    if (ptr_mi != NULL) {               // Any other Interface of Device with Multiple Interfaces
      mi = _ttoi(ptr_mi + 3);           // Current mi from string to number
      if (mi != 0) {
        // Create path of Device (First Interface) this Interface belongs to
        _tcscpy_s(path, MAX_PATH, detailData->DevicePath);
        ptr_str = _tcsstr(path, TEXT("mi_"));
        *(ptr_str + 3) = TCHAR('0');
        *(ptr_str + 4) = TCHAR('0');
        ptr_str = _tcsstr(ptr_str, TEXT("&"));
        ptr_str++;
        ptr_str = _tcsstr(ptr_str, TEXT("&"));
        ptr_str++;
        ptr_str = _tcsstr(ptr_str, TEXT("&"));
        ptr_str++;
        *ptr_str = TCHAR(0);

        for (i = 0; i < dev; i++) {
          if (memcmp(WinUsbDevice[i].Interface[0].DeviceInterfacePath, path, _tcslen(path) * sizeof(TCHAR)) == 0) {
            // Store Path to Device Interface mi
            _tcscpy_s(WinUsbDevice[i].Interface[mi].DeviceInterfacePath, MAX_PATH, detailData->DevicePath);

            // Increment Number of Interfaces that Device contains
            WinUsbDevice[i].NumInterfaces++;
            break;
          }
        }
      }
    }
    if ((ptr_mi == NULL) || (mi == 0)) { // First Interface of Device with Multiple Interfaces or Device with Single Interface
      // Store Path to Device Interface 0
      _tcscpy_s(WinUsbDevice[dev].Interface[0].DeviceInterfacePath, MAX_PATH, detailData->DevicePath);

      // Number of Interfaces that Device contains is 1 at the moment
      WinUsbDevice[dev].NumInterfaces = 1;

      dev++;
    }
  } while (dev < MAX_DEV_NUM);

  // Load info of all found devices, if loading of info fails clear device as if it was not found
  for (i = 0; i < dev; i++) {
    result = WinUsbIF_LoadDeviceInfo(i);
    if (!result) {
      dev--;
      for (j = i; j < dev; j++) {
        memcpy(&WinUsbDevice[j], &WinUsbDevice[j+1], sizeof(WinUsbDevice_t));
      }
      memset(&WinUsbDevice[dev], 0, sizeof(WinUsbDevice_t));
      i--;
    }
  }

  WinUsbDevCount = dev;

  if (detailData != NULL) { free(detailData); }

  SetupDiDestroyDeviceInfoList(deviceInfo);

  return WinUsbDevCount;
}


/*
 *  WinUSB Interface: Get Device's number of Interfaces
 *    Parameters:      dev:    Device index
 *    Return Value:    Number of interfaces
 */
int  WinUsbIF_GetNumOfInterfaces (UCHAR dev) {

  if (dev >= MAX_DEV_NUM) { return 0; }

  return WinUsbDevice[dev].NumInterfaces;
}


/*
 *  WinUSB Interface: Get Device's Interface available Endpoints Mask
 *    Parameters:      dev:    Device index
 *                     intf:   Interface index
 *    Return Value:    32-bit mask representing available endpoints
 *                     (16-bits IN endpoints | 16-bits OUT endpoints)
 */
ULONG WinUsbIF_GetEndpointsMask (UCHAR dev, UCHAR intf) {

  if (dev  >= MAX_DEV_NUM)                     { return 0; }
  if (intf >= WinUsbDevice[dev].NumInterfaces) { return 0; }

  return WinUsbDevice[dev].Interface[intf].EpMask;
}


/*
 *  WinUSB Interface: Get Device's Name string (VID + PID + Serial Number or Unique ID)
 *    Parameters:      dev:     Device index
 *                     buf:     Pointer to buffer that receives string (depends on selected project character set)
 *                     max_len: Maximum length in number of characters that string can contain
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_GetName(UCHAR dev, TCHAR *buf, UCHAR max_len) {

  if (dev >= MAX_DEV_NUM)                                                                           { return FALSE; }
  if ((_tcslen(WinUsbDevice[dev].Product) + _tcslen(WinUsbDevice[dev].SerialNumber) + 7) > max_len) { return FALSE; }

  _stprintf_s(buf, max_len, TEXT("%s (SN: %s)"), WinUsbDevice[dev].Product, WinUsbDevice[dev].SerialNumber);

  return TRUE;
}


/*
 *  WinUSB Interface: Get Device's Manufacturer String
 *    Parameters:      dev:     Device index
 *                     buf:     Pointer to buffer that receives string (depends on selected project character set)
 *                     max_len: Maximum length in number of characters that string can contain
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_GetManufacturerString(UCHAR dev, TCHAR *buf, UCHAR max_len) {

  if (dev >= MAX_DEV_NUM)                                { return FALSE; }
  if (_tcslen(WinUsbDevice[dev].Manufacturer) > max_len) { return FALSE; }

  _tcscpy_s(buf, max_len, WinUsbDevice[dev].Manufacturer);

  return TRUE;
}


/*
 *  WinUSB Interface: Get Device's Product String
 *    Parameters:      dev:     Device index
 *                     buf:     Pointer to buffer that receives string (depends on selected project character set)
 *                     max_len: Maximum length in number of characters that string can contain
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_GetProductString(UCHAR dev, TCHAR *buf, UCHAR max_len) {

  if (dev >= MAX_DEV_NUM)                           { return FALSE; }
  if (_tcslen(WinUsbDevice[dev].Product) > max_len) { return FALSE; }

  _tcscpy_s(buf, max_len, WinUsbDevice[dev].Product);

  return TRUE;
}


/*
*  WinUSB Interface: Get Device's Serial Number String
*    Parameters:      dev:     Device index
*                     buf:     Pointer to character array receiving Serial Number string (depends on selected project character set)
*                     max_len: Maximum length in number of characters that string can contain
*    Return Value:    TRUE - Success, FALSE - Error
*/
BOOL WinUsbIF_GetSerialNumberString(UCHAR dev, TCHAR *buf, UCHAR max_len) {

    if (dev >= MAX_DEV_NUM)                                { return FALSE; }
    if (_tcslen(WinUsbDevice[dev].SerialNumber) > max_len) { return FALSE; }

    _tcscpy_s(buf, max_len, WinUsbDevice[dev].SerialNumber);

    return TRUE;
}


/*
*  WinUSB Interface: Get Device's Interface String
*    Parameters:      dev:     Device index
*                     intf:    Interface index
*                     buf:     Pointer to character array receiving Interface string (depends on selected project character set)
*                     max_len: Maximum length in number of characters that string can contain
*    Return Value:    TRUE - Success, FALSE - Error
*/
BOOL WinUsbIF_GetInterfaceString(UCHAR dev, UCHAR intf, TCHAR *buf, UCHAR max_len) {

    if (dev >= MAX_DEV_NUM)                                          { return FALSE; }
    if (intf >= WinUsbDevice[dev].NumInterfaces)                     { return FALSE; }
    if (_tcslen(WinUsbDevice[dev].Interface[intf].String) > max_len) { return FALSE; }

    _tcscpy_s(buf, max_len, WinUsbDevice[dev].Interface[intf].String);

    return TRUE;
}


/*
*  WinUSB Interface: Get Device's Vendor ID (VID)
*    Parameters:      dev:     Device index
*                     vid:     Pointer to variable receiving Vendor ID
*    Return Value:    TRUE - Success, FALSE - Error
*/
BOOL WinUsbIF_GetVID(UCHAR dev, WORD *vid) {

  if (dev >= MAX_DEV_NUM) { return FALSE; }

  *vid = WinUsbDevice[dev].VID;

  return TRUE;
}


/*
*  WinUSB Interface: Get Device's Product ID (PID)
*    Parameters:      dev:     Device index
*                     pid:     Pointer to variable receiving Product ID
*    Return Value:    TRUE - Success, FALSE - Error
*/
BOOL WinUsbIF_GetPID(UCHAR dev, WORD *pid) {

  if (dev >= MAX_DEV_NUM) { return FALSE; }

  *pid = WinUsbDevice[dev].PID;

  return TRUE;
}


/*
 *  WinUSB Interface: Open Device (and all containing Interfaces)
 *    Parameters:      dev:   Device index
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_OpenDevice (UCHAR dev) {
  HANDLE                   devIntfHandle;
  WINUSB_INTERFACE_HANDLE  winusbIntfHandle;
  USB_INTERFACE_DESCRIPTOR intfDscr;
  WINUSB_PIPE_INFORMATION  pipeInfo;
  UCHAR                    intf_idx, ep_idx, ep_num, pipe_idx;
  BOOL                     result;

  if (dev >= MAX_DEV_NUM)                                         { return FALSE; }
  if (WinUsbDevice[dev].Interface[0].DeviceInterfacePath == NULL) { return FALSE; }

  // Go through all Interfaces
  for (intf_idx = 0; intf_idx < WinUsbDevice[dev].NumInterfaces; intf_idx++) {

    // Create File for Device Interface Read/Write
    devIntfHandle = CreateFile(WinUsbDevice[dev].Interface[intf_idx].DeviceInterfacePath,
                               GENERIC_READ | GENERIC_WRITE,
                               FILE_SHARE_READ | FILE_SHARE_WRITE,
                               (LPSECURITY_ATTRIBUTES)NULL,
                               OPEN_EXISTING,
                               FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
                               NULL);

    if (devIntfHandle == INVALID_HANDLE_VALUE) {
      return FALSE;
    }

    // Store Device Interface handle
    WinUsbDevice[dev].Interface[intf_idx].DeviceInterfaceHandle = devIntfHandle;

    // Initialize WinUSB for Device Interface and get WinUSB Device Interface handle
    result = WinUsb_Initialize(devIntfHandle, &winusbIntfHandle);

    if (result) {
      // Store WinUSB Device Interface handle
      WinUsbDevice[dev].Interface[intf_idx].WinUsbInterfaceHandle = winusbIntfHandle;

      // Get information about Interface
      result = WinUsb_QueryInterfaceSettings( winusbIntfHandle,
                                              0,
                                             &intfDscr);
      if (result) {
        ep_num = intfDscr.bNumEndpoints;                // Get number of endpoints this Interface contains
        for (ep_idx = 0; ep_idx < ep_num; ep_idx++) {   // Go through all available endpoints of Interface
          // Get information about Pipe used for communication with endpoint
          result = WinUsb_QueryPipe( winusbIntfHandle,
                                     0,
                                     (UCHAR)ep_idx,
                                    &pipeInfo);
          if (result) {
            if (USB_ENDPOINT_DIRECTION_OUT(pipeInfo.PipeId)) {
              // Pipe for communicating with OUT endpoint
              // Increment number of OUT pipes, store PipeID and update mask of used endpoints
              pipe_idx = WinUsbDevice[dev].Interface[intf_idx].NumOutPipes++;
              WinUsbDevice[dev].Interface[intf_idx].OutPipeID[pipe_idx] = pipeInfo.PipeId;
              WinUsbDevice[dev].Interface[intf_idx].EpMask |= 1UL << pipeInfo.PipeId;
            } else {
              // Pipe for communicating with IN endpoint
              // Increment number of IN pipes, store PipeID and update mask of used endpoints
              pipe_idx = WinUsbDevice[dev].Interface[intf_idx].NumInPipes++;
              WinUsbDevice[dev].Interface[intf_idx].InPipeID [pipe_idx] = pipeInfo.PipeId;
              WinUsbDevice[dev].Interface[intf_idx].EpMask |= 1UL << 16 << (pipeInfo.PipeId & 0x0F);
            }
          }
        }
      }
    }
  }

  // Store opening status
  WinUsbDevice[dev].Open = result;

  return result;
}


/*
 *  WinUSB Interface: Close Device
 *    Parameters:      dev:   Device index
 *    Return Value:    None
 */
void WinUsbIF_CloseDevice (UCHAR dev) {
  UCHAR  intf_idx;

  if (dev >= MAX_DEV_NUM) { return; }

  if (WinUsbDevice[dev].Open) {
    for (intf_idx = 0; intf_idx < WinUsbDevice[dev].NumInterfaces; intf_idx++) {
      if (WinUsbDevice[dev].Interface[intf_idx].WinUsbInterfaceHandle != NULL) {
        // Close WinUSB Device Interface
        WinUsb_Free(WinUsbDevice[dev].Interface[intf_idx].WinUsbInterfaceHandle);
        WinUsbDevice[dev].Interface[intf_idx].WinUsbInterfaceHandle = NULL;
      }
      if (WinUsbDevice[dev].Interface[intf_idx].DeviceInterfaceHandle != NULL) {
        // Close Device Interface
        CloseHandle(WinUsbDevice[dev].Interface[intf_idx].DeviceInterfaceHandle);
        WinUsbDevice[dev].Interface[intf_idx].DeviceInterfaceHandle = NULL;
      }
    }

    WinUsbDevice[dev].Open = FALSE;
  }
}


/*
 *  WinUSB Interface: Control Transfer
 *    Parameters:      dev:   Device index
 *                     sp:    Pointer to Setup Packet
 *                     buf:   Pointer to buffer containing data for stage data
 *                     sz:    Number of bytes to transfer in data stage
 *                     cnt:   Pointer to number of bytes transferred in data stage
 *                     ov:    Pointer to OVERLAPPED structure
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_ControlTransfer (UCHAR dev, WINUSB_SETUP_PACKET *sp, UCHAR *buf, ULONG sz, ULONG *cnt, OVERLAPPED *ov) {

  // WinUSB Control Transfer supports up to 4KB data stage transfer

  if (dev >= MAX_DEV_NUM)                                           { return FALSE; }
  if (sz  >  4096)                                                  { return FALSE; }
  if (sp  == NULL)                                                  { return FALSE; }
  if ((buf == NULL) && (sz != 0))                                   { return FALSE; }
  if (WinUsbDevice[dev].Interface[0].WinUsbInterfaceHandle == NULL) { return FALSE; }

  return (WinUsb_ControlTransfer( WinUsbDevice[dev].Interface[0].WinUsbInterfaceHandle,
                                 *sp,
                                  buf,
                                  sz,
                                  cnt,
                                  ov));
}


/*
 *  WinUSB Interface: Write Pipe (Bulk/Interrupt Endpoint)
 *    Parameters:      dev:   Device index
 *                     intf:  Interface index
 *                     pipe:  Pipe index
 *                     buf:   Pointer to buffer with data to write
 *                     sz:    Number of bytes to write
 *                     cnt:   Pointer to number of bytes written
 *                     ov:    Pointer to OVERLAPPED structure
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_WritePipe (UCHAR dev, UCHAR intf, UCHAR pipe, UCHAR *buf, ULONG sz, ULONG *cnt, OVERLAPPED *ov) {

  if (dev  >= MAX_DEV_NUM)                                             { return FALSE; }
  if (intf >= WinUsbDevice[dev].NumInterfaces)                         { return FALSE; }
  if ((buf == NULL) && (sz != 0))                                      { return FALSE; }
  if (WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle == NULL) { return FALSE; }
  if (WinUsbDevice[dev].Interface[intf].OutPipeID[pipe]       == NULL) { return FALSE; }

  return (WinUsb_WritePipe(WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle,
                           WinUsbDevice[dev].Interface[intf].OutPipeID[pipe],
                           buf,
                           sz,
                           cnt,
                           ov));
}


/*
 *  WinUSB Interface: Read Pipe (Bulk/Interrupt Endpoint)
 *    Parameters:      dev:   Device index
 *                     intf:  Interface index
 *                     pipe:  Pipe index
 *                     buf:   Pointer to buffer that receives data
 *                     sz:    Number of bytes to read
 *                     cnt:   Pointer to number of bytes read
 *                     ov:    Pointer to OVERLAPPED structure
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_ReadPipe (UCHAR dev, UCHAR intf, UCHAR pipe, UCHAR *buf, ULONG sz, ULONG *cnt, OVERLAPPED *ov) {

  if (dev  >= MAX_DEV_NUM)                                             { return FALSE; }
  if (intf >= WinUsbDevice[dev].NumInterfaces)                         { return FALSE; }
  if ((buf == NULL) && (sz != 0))                                      { return FALSE; }
  if (WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle == NULL) { return FALSE; }
  if (WinUsbDevice[dev].Interface[intf].InPipeID[pipe]        == NULL) { return FALSE; }

  // Execute WinUSB call
  return (WinUsb_ReadPipe(WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle,
                          WinUsbDevice[dev].Interface[intf].InPipeID[pipe],
                          buf,
                          sz,
                          cnt,
                          ov));
}


/*
 *  WinUSB Interface: Abort Pipe pending transfers (Bulk/Interrupt Endpoint)
 *    Parameters:      dev:      Device index
 *                     intf:     Interface index
 *                     pipe:     Pipe index
 *                     pipe_dir: Pipe direction (0 = OUT, 1 = IN)
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_AbortPipe (UCHAR dev, UCHAR intf, UCHAR pipe, UCHAR pipe_dir) {
  UCHAR pipe_id;

  if (dev  >= MAX_DEV_NUM)                                             { return FALSE; }
  if (intf >= WinUsbDevice[dev].NumInterfaces)                         { return FALSE; }
  if (WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle == NULL) { return FALSE; }

  if (pipe_dir == 1) {
    pipe_id = WinUsbDevice[dev].Interface[intf].InPipeID[pipe];
  } else {
    pipe_id = WinUsbDevice[dev].Interface[intf].OutPipeID[pipe];
  }
  if (pipe_id == 0) { return FALSE; }

  return (WinUsb_AbortPipe(WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle,
                           pipe_id));
}


/*
 *  WinUSB Interface: Flush Pipe cached data (Bulk/Interrupt Endpoint)
 *    Parameters:      dev:      Device index
 *                     intf:     Interface index
 *                     pipe:     Pipe index
 *                     pipe_dir: Pipe direction (0 = OUT, 1 = IN)
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_FlushPipe (UCHAR dev, UCHAR intf, UCHAR pipe, UCHAR pipe_dir) {
  UCHAR pipe_id;

  if (dev  >= MAX_DEV_NUM)                                             { return FALSE; }
  if (intf >= WinUsbDevice[dev].NumInterfaces)                         { return FALSE; }
  if (WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle == NULL) { return FALSE; }

  if (pipe_dir == 1) {
    pipe_id = WinUsbDevice[dev].Interface[intf].InPipeID[pipe];
  } else {
    pipe_id = WinUsbDevice[dev].Interface[intf].OutPipeID[pipe];
  }
  if (pipe_id == 0) { return FALSE; }

  return (WinUsb_FlushPipe(WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle,
                           pipe_id));
}


/*
 *  WinUSB Interface: Reset Pipe (Bulk/Interrupt Endpoint)
 *    Parameters:      dev:      Device index
 *                     intf:     Interface index
 *                     pipe:     Pipe index
 *                     pipe_dir: Pipe direction (0 = OUT, 1 = IN)
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_ResetPipe (UCHAR dev, UCHAR intf, UCHAR pipe, UCHAR pipe_dir) {
  UCHAR pipe_id;

  if (dev  >= MAX_DEV_NUM)                                             { return FALSE; }
  if (intf >= WinUsbDevice[dev].NumInterfaces)                         { return FALSE; }
  if (WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle == NULL) { return FALSE; }

  if (pipe_dir == 1) {
    pipe_id = WinUsbDevice[dev].Interface[intf].InPipeID[pipe];
  } else {
    pipe_id = WinUsbDevice[dev].Interface[intf].OutPipeID[pipe];
  }
  if (pipe_id == 0) { return FALSE; }

  // Execute WinUSB call
  return (WinUsb_ResetPipe(WinUsbDevice[dev].Interface[intf].WinUsbInterfaceHandle,
                           pipe_id));
}


/*
 *  WinUSB Interface: Get Overlapped Result
 *    Parameters:      dev:   Device index
 *                     ov:    Pointer to OVERLAPPED structure
 *                     cnt:   Pointer to number of bytes transfered
 *                     wait:  Wait until operation completed
 *    Return Value:    TRUE - Success, FALSE - Error
 */
BOOL WinUsbIF_GetOverlappedResult (UCHAR dev, OVERLAPPED *ov, DWORD *cnt, BOOL wait) {

  if (dev >= MAX_DEV_NUM)                                           { return FALSE; }
  if (WinUsbDevice[dev].Interface[0].WinUsbInterfaceHandle == NULL) { return FALSE; }

  return (WinUsb_GetOverlappedResult(WinUsbDevice[dev].Interface[0].WinUsbInterfaceHandle, ov, cnt, wait));
}


// ---- Auxiliary Functions (local) ----------------------------

/*
 *  WinUSB Interface: Load Device Info (load device parameters and strings (open device read info and close it))
 *    Parameters:      dev:   Device index
 *    Return Value:    TRUE - Success, FALSE - Error
 */
static BOOL WinUsbIF_LoadDeviceInfo (UCHAR dev) {
  HANDLE                   devIntfHandle;
  WINUSB_INTERFACE_HANDLE  winusbIntfHandle;
  USB_DEVICE_DESCRIPTOR    devDscr;
  USB_INTERFACE_DESCRIPTOR intfDscr;
  UCHAR                    strDscr[MAXIMUM_USB_STRING_LENGTH+2];
  ULONG                    length;
  UCHAR                    intf_idx;
  BOOL                     result;

  if (dev >= MAX_DEV_NUM)                                         { return FALSE; }
  if (WinUsbDevice[dev].Interface[0].DeviceInterfacePath[0] == 0) { return FALSE; }

  // Go through all Interfaces
  for (intf_idx = 0; intf_idx < WinUsbDevice[dev].NumInterfaces; intf_idx++) {

    // Create File for Device Interface Read/Write
    devIntfHandle = CreateFile(WinUsbDevice[dev].Interface[intf_idx].DeviceInterfacePath,
                               GENERIC_READ | GENERIC_WRITE,
                               FILE_SHARE_READ | FILE_SHARE_WRITE,
                               (LPSECURITY_ATTRIBUTES)NULL,
                               OPEN_EXISTING,
                               FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
                               NULL);

    if (devIntfHandle == INVALID_HANDLE_VALUE) {
      return FALSE;
    }

    // Initialize WinUSB for Device Interface and get WinUSB Device Interface handle
    result = WinUsb_Initialize(devIntfHandle, &winusbIntfHandle);

    if (intf_idx == 0) {
      // Get additional information from device (for Interface 0)

      if (result) {
        length = sizeof(UCHAR);

        // Get USB Bus speed and store it at the same time
        result = WinUsb_QueryDeviceInformation( winusbIntfHandle,
                                                DEVICE_SPEED,
                                               &length,
                                               &WinUsbDevice[dev].Speed);
      }

      if (result) {
        // Get Device Descriptor
        result = WinUsb_GetDescriptor( winusbIntfHandle,
                                       USB_DEVICE_DESCRIPTOR_TYPE,
                                       0, 0,
                                       (PUCHAR)&devDscr,
                                       sizeof(USB_DEVICE_DESCRIPTOR),
                                      &length);
      }

      if (result) {
        // Store VID and PID
        WinUsbDevice[dev].VID = devDscr.idVendor;
        WinUsbDevice[dev].PID = devDscr.idProduct;

        // Get Manufacturer string
        result = WinUsb_GetDescriptor( winusbIntfHandle,
                                       USB_STRING_DESCRIPTOR_TYPE,
                                       devDscr.iManufacturer, 0x0409,
                                       (PUCHAR)strDscr,
                                       MAXIMUM_USB_STRING_LENGTH+2,
                                      &length);
      }

      if (result) {
        // Store Manufacturer string
        StrDesc2Str((PUSB_STRING_DESCRIPTOR)strDscr, WinUsbDevice[dev].Manufacturer);

        // Get Product string
        // devDscr.iProduct for multiple interface contains number of interfaces string
        // so we use hardcoded index 2 which is almost always product string index
        result = WinUsb_GetDescriptor( winusbIntfHandle,
                                       USB_STRING_DESCRIPTOR_TYPE,
                                       2, 0x0409,
                                       (PUCHAR)strDscr,
                                       MAXIMUM_USB_STRING_LENGTH+2,
                                      &length);
      }

      if (result) {
        // Store Product string
        StrDesc2Str((PUSB_STRING_DESCRIPTOR)strDscr, WinUsbDevice[dev].Product);

        // Clear stored Serial Number string
        memset(WinUsbDevice[dev].SerialNumber, 0, sizeof(WinUsbDevice[dev].SerialNumber));

        if (devDscr.iSerialNumber != 0) {
          // Get Serial Number string if it exists
          result = WinUsb_GetDescriptor( winusbIntfHandle,
                                         USB_STRING_DESCRIPTOR_TYPE,
                                         devDscr.iSerialNumber, 0x0409,
                                         (PUCHAR)strDscr,
                                         MAXIMUM_USB_STRING_LENGTH+2,
                                        &length);
          if (result) {
            // Store Serial Number string
            StrDesc2Str((PUSB_STRING_DESCRIPTOR)strDscr, WinUsbDevice[dev].SerialNumber);
          }
        }
      }
    }

    if (result) {
      // Store WinUSB Device Interface handle
      WinUsbDevice[dev].Interface[intf_idx].WinUsbInterfaceHandle = winusbIntfHandle;

      // Get information about Interface
      result = WinUsb_QueryInterfaceSettings( winusbIntfHandle,
                                              0,
                                             &intfDscr);
      if (result) {
        // Clear stored Interface string
        memset(WinUsbDevice[dev].Interface[intf_idx].String, 0, sizeof(WinUsbDevice[dev].Interface[intf_idx].String));

        if (intfDscr.iInterface != 0) {
          // Get Interface string if it exists
          result = WinUsb_GetDescriptor( winusbIntfHandle,
                                         USB_STRING_DESCRIPTOR_TYPE,
                                         intfDscr.iInterface, 0x0409,
                                         (PUCHAR)strDscr,
                                         MAXIMUM_USB_STRING_LENGTH+2,
                                        &length);
          if (result) {
            // Store Interface string
            StrDesc2Str((PUSB_STRING_DESCRIPTOR)strDscr, WinUsbDevice[dev].Interface[intf_idx].String);
          }
        }
      }
    }

    // Close WinUSB Device Interface
    WinUsb_Free(winusbIntfHandle);

    // Close Device Interface
    CloseHandle(devIntfHandle);
  }

  return result;
}

/*
 *  Convert String Descriptor to String
 *    Parameters:      str_desc: Pointer to String Descriptor
 *    Parameters:      str:      Pointer to converted String
 *    Return Value:    None
 */
static void StrDesc2Str (PUSB_STRING_DESCRIPTOR str_desc, TCHAR *str) {
  int n;

  n = 0;
  if ((str_desc->bDescriptorType == USB_STRING_DESCRIPTOR_TYPE) &&
      (str_desc->bLength > 2)                                   &&
      ((str_desc->bLength % 2) == 0)                            &&
      (str_desc->bString[0] != 0x0409)) {
    n = (str_desc->bLength - 2) / 2;
#if (TCHAR == WCHAR)
    wmemcpy_s(str, n, (TCHAR *)str_desc->bString, n);
#else
    int i;
    for (i = 0; i < n; i++) {
      str[i] = (UCHAR)str_desc->bString[i];
    }
#endif
  }
  str[n] = 0;                           // Terminate String
}
