/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2018 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    net_evr.h
 * Purpose: Network definitions for Event Recorder
 * Rev.:    V7.9.0
 *----------------------------------------------------------------------------*/

#ifdef DEBUG_EVR
#include <string.h>
#include "EventRecorder.h"

#ifndef __STATIC_INLINE
  #define __STATIC_INLINE   static __inline __attribute__((always_inline))
#endif

extern uint32_t net_dbg_buf[6];         ///< Buffer for complex debug structures 

/// Union for accessing the debug parameters. 
union evr_access {
  uint32_t u32[6];                      ///< 4-byte access
  uint16_t u16[12];                     ///< 2-byte access
  uint8_t  u8 [24];                     ///< 1-byte access
};
#define evr_buf             (*(union evr_access *)&net_dbg_buf)

// Network components ----------------------------------------------------------
#define EvtNetSYS           0xC0        ///< System Core
#define EvtNetMEM           0xC1        ///< Memory Management
#define EvtNetETH           0xC2        ///< ETH Interface
#define EvtNetPPP           0xC3        ///< PPP Interface
#define EvtNetSLIP          0xC4        ///< SLIP Interface
#define EvtNetLOOP          0xC5        ///< Loopback Interface

#define EvtNetIP4           0xC6        ///< IPv4 Core
#define EvtNetICMP          0xC7        ///< ICMP Control
#define EvtNetIGMP          0xC8        ///< IGMP Group Management
#define EvtNetNBNS          0xC9        ///< NetBIOS Name Service Client
#define EvtNetDHCP          0xCA        ///< Dynamic Host Configuration Client
#define EvtNetARP           0xCB        ///< Ethernet Address Resolution

#define EvtNetIP6           0xCC        ///< IPv6 Core
#define EvtNetICMP6         0xCD        ///< ICMPv6 Control
#define EvtNetDHCP6         0xCE        ///< Dynamic Host Configuration Client for IPv6
#define EvtNetNDP           0xCF        ///< Neighbor Discovery for IPv6

#define EvtNetUDP           0xD0        ///< Socket UDP
#define EvtNetTCP           0xD1        ///< Socket TCP
#define EvtNetBSD           0xD2        ///< Socket BSD

#define EvtNetHTTPs         0xD3        ///< Web Server
#define EvtNetFTPs          0xD4        ///< File Transfer Server
#define EvtNetFTPc          0xD5        ///< File Transfer Client
#define EvtNetTeln          0xD6        ///< Telnet Server
#define EvtNetTFTPs         0xD7        ///< Trivial File Transfer Server
#define EvtNetTFTPc         0xD8        ///< Trivial File Transfer Client
#define EvtNetSMTP          0xD9        ///< Simple Mail Transfer Client
#define EvtNetDNS           0xDA        ///< Domain Name Service Client
#define EvtNetSNMP          0xDB        ///< Simple Network Management Agent
#define EvtNetSNTP          0xDC        ///< Simple Network Time Client
#endif


// NetSYS event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetSYS_InitSystem            EventID (EventLevelOp,    EvtNetSYS, 0)
#define EvtNetSYS_ThreadCreateFailed    EventID (EventLevelError, EvtNetSYS, 1)
#define EvtNetSYS_TimerCreateFailed     EventID (EventLevelError, EvtNetSYS, 2)
#define EvtNetSYS_InitComplete          EventID (EventLevelOp,    EvtNetSYS, 3)
#define EvtNetSYS_GetOption             EventID (EventLevelAPI,   EvtNetSYS, 4)
#define EvtNetSYS_SetOption             EventID (EventLevelAPI,   EvtNetSYS, 5)
#define EvtNetSYS_SetDefault            EventID (EventLevelAPI,   EvtNetSYS, 6)
#define EvtNetSYS_SetHostName           EventID (EventLevelAPI,   EvtNetSYS, 7)
#define EvtNetSYS_UninitSystem          EventID (EventLevelOp,    EvtNetSYS, 8)
#define EvtNetSYS_UninitComplete        EventID (EventLevelOp,    EvtNetSYS, 9)
#endif

/**
  \brief  Event on \ref netInitialize start (Op)
  \param  lib_version   encoded library version BCD (MMmmbbbb)
                         - MM:   Major
                         - mm:   minor
                         - bbbb: build
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_InitSystem(uint32_t lib_version) {
    uint16_t version = (uint16_t)(lib_version >> 16);
    uint16_t build   = (uint16_t)(lib_version);
    EventRecord2 (EvtNetSYS_InitSystem, version, build);
  }
#else
  #define EvrNetSYS_InitSystem(lib_version)
#endif

/**
  \brief  Event on \ref netInitialize thread create failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_ThreadCreateFailed(void) {
    EventRecord2 (EvtNetSYS_ThreadCreateFailed, 0, 0);
  }
#else
  #define EvrNetSYS_ThreadCreateFailed()
#endif

/**
  \brief  Event on \ref netInitialize timer create failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_TimerCreateFailed(void) {
    EventRecord2 (EvtNetSYS_TimerCreateFailed, 0, 0);
  }
#else
  #define EvrNetSYS_TimerCreateFailed()
#endif

/**
  \brief  Event on \ref netInitialize complete (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_InitComplete(void) {
    EventRecord2 (EvtNetSYS_InitComplete, 0, 0);
  }
#else
  #define EvrNetSYS_InitComplete()
#endif

/**
  \brief  Event on \ref netIF_SetOption (API)
  \param  if_id         interface identification (class and number)
  \param  option        interface option as specified by \ref netIF_Option
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_SetOption(uint32_t if_id, int32_t option) {
    EventRecord2 (EvtNetSYS_SetOption, if_id, (uint32_t)option);
  }
#else
  #define EvrNetSYS_SetOption(if_id, option)
#endif

/**
  \brief  Event on \ref netIF_GetOption (API)
  \param  if_id         interface identification (class and number)
  \param  option        interface option as specified by \ref netIF_Option
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_GetOption(uint32_t if_id, int32_t option) {
    EventRecord2 (EvtNetSYS_GetOption, if_id, (uint32_t)option);
  }
#else
  #define EvrNetSYS_GetOption(if_id, option)
#endif

/**
  \brief  Event on \ref netIF_SetDefault (API)
  \param  if_id         interface identification (class and number)
  \param  ip_version    internet protocol version as specified by \ref netIF_Version
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_SetDefault(uint32_t if_id, int32_t ip_version) {
    EventRecord2 (EvtNetSYS_SetDefault, if_id, (uint32_t)ip_version);
  }
#else
  #define EvrNetSYS_SetDefault(if_id, ip_version)
#endif

/**
  \brief  Event on \ref netSYS_SetHostName entry (API)
  \param  name          pointer to host name string
  \param  length        length of host name string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_SetHostName(const char *name, uint32_t length) {
    EventRecordData (EvtNetSYS_SetHostName, name, length);
  }
#else
  #define EvrNetSYS_SetHostName(name, length)
#endif

/**
  \brief  Event on \ref netUninitialize start (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_UninitSystem(void) {
    EventRecord2 (EvtNetSYS_UninitSystem, 0, 0);
  }
#else
  #define EvrNetSYS_UninitSystem()
#endif

/**
  \brief  Event on \ref netUninitialize complete (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSYS_UninitComplete(void) {
    EventRecord2 (EvtNetSYS_UninitComplete, 0, 0);
  }
#else
  #define EvrNetSYS_UninitComplete()
#endif


// NetMEM event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetMEM_InitMemory                EventID (EventLevelOp,    EvtNetMEM,  0)
#define EvtNetMEM_AllocMemory               EventID (EventLevelOp,    EvtNetMEM,  1)
#define EvtNetMEM_AllocLimitExceeded        EventID (EventLevelOp,    EvtNetMEM,  2)
#define EvtNetMEM_AllocOutOfMemory          EventID (EventLevelError, EvtNetMEM,  3)
#define EvtNetMEM_ShrinkMemory              EventID (EventLevelOp,    EvtNetMEM,  4)
#define EvtNetMEM_FreeMemory                EventID (EventLevelOp,    EvtNetMEM,  5)
#define EvtNetMEM_FreeInvalidBlock          EventID (EventLevelError, EvtNetMEM,  6)
#define EvtNetMEM_FreeLinkCorrupted         EventID (EventLevelError, EvtNetMEM,  7)
#define EvtNetMEM_UninitMemory              EventID (EventLevelOp,    EvtNetMEM,  8)
#endif

/**
  \brief  Event on memory management initialize (Op)
  \param  pool_size     size of the memory pool
  \param  limit0        usage limit 0
                        (limit used in ethernet and bsd receive buffering)
  \param  limit1        usage limit 1
                        (limit used in TCP send buffering)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_InitMemory(uint32_t pool_size, uint32_t limit0, uint32_t limit1) {
    EventRecord4 (EvtNetMEM_InitMemory, pool_size, limit0, limit1, 0);
  }
#else
  #define EvrNetMEM_InitMemory(pool_size, limit0, limit1)
#endif

/**
  \brief  Event on memory allocate request (Op)
  \param  size          requested memory size in bytes
  \param  used          used memory status in bytes
  \param  blocks        number of used blocks status
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_AllocMemory(uint32_t size, uint32_t used, uint32_t blocks) {
    EventRecord4 (EvtNetMEM_AllocMemory, size, used, blocks, 0);
  }
#else
  #define EvrNetMEM_AllocMemory(size, used, blocks)
#endif

/**
  \brief  Event on memory allocate limit exceeded (Op)
  \param  size          requested memory size in bytes
  \param  used          used memory status in bytes
  \param  blocks        number of used blocks status
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_AllocLimitExceeded(uint32_t size, uint32_t used, uint32_t blocks) {
    EventRecord4 (EvtNetMEM_AllocLimitExceeded, size, used, blocks, 0);
  }
#else
  #define EvrNetMEM_AllocLimitExceeded(size, used, blocks)
#endif

/**
  \brief  Event on memory allocate out of memory error (Error)
  \param  size          requested memory size in bytes
  \param  used          used memory status in bytes
  \param  blocks        number of used blocks status
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_AllocOutOfMemory(uint32_t size, uint32_t used, uint32_t blocks) {
    EventRecord4 (EvtNetMEM_AllocOutOfMemory, size, used, blocks, 0);
  }
#else
  #define EvrNetMEM_AllocOutOfMemory(size, used, blocks)
#endif

/**
  \brief  Event on memory shrink block request (Op)
  \param  new_size      new block size
  \param  old_size      current block size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_ShrinkMemory(uint32_t new_size, uint32_t old_size) {
    EventRecord2 (EvtNetMEM_ShrinkMemory, new_size, old_size);
  }
#else
  #define EvrNetMEM_ShrinkMemory(new_size, old_size)
#endif

/**
  \brief  Event on memory free request (Op)
  \param  size          size of the memory block to free
  \param  used          used memory status in bytes
  \param  blocks        number of used blocks status
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_FreeMemory(uint32_t size, uint32_t used, uint32_t blocks) {
    EventRecord4 (EvtNetMEM_FreeMemory, size, used, blocks, 0);
  }
#else
  #define EvrNetMEM_FreeMemory(size, used, blocks)
#endif

/**
  \brief  Event on memory free an invalid block (Error)
  \param  size          size of the memory block to free
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_FreeInvalidBlock(uint32_t size) {
    EventRecord2 (EvtNetMEM_FreeInvalidBlock, size, 0);
  }
#else
  #define EvrNetMEM_FreeInvalidBlock(size)
#endif

/**
  \brief  Event on memory free link corrupted (Error)
  \param  size          size of the memory block to free
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_FreeLinkCorrupted(uint32_t size) {
    EventRecord2 (EvtNetMEM_FreeLinkCorrupted, size, 0);
  }
#else
  #define EvrNetMEM_FreeLinkCorrupted(size)
#endif

/**
  \brief  Event on memory management de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetMEM_UninitMemory(void) {
    EventRecord2 (EvtNetMEM_UninitMemory, 0, 0);
  }
#else
  #define EvrNetMEM_UninitMemory()
#endif


// NetETH event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetETH_InitInterface             EventID (EventLevelOp,    EvtNetETH,  0)
#define EvtNetETH_MacAddressConfigError     EventID (EventLevelError, EvtNetETH,  1)
#define EvtNetETH_PhyDriverConfigError      EventID (EventLevelError, EvtNetETH,  2)
#define EvtNetETH_PhyDriverInitError        EventID (EventLevelError, EvtNetETH,  3)
#define EvtNetETH_Ip4ConfigError            EventID (EventLevelError, EvtNetETH,  4)
#define EvtNetETH_Ip6ConfigError            EventID (EventLevelError, EvtNetETH,  5)
#define EvtNetETH_ThreadCreateFailed        EventID (EventLevelError, EvtNetETH,  6)
#define EvtNetETH_GetOptionInvalidParameter EventID (EventLevelError, EvtNetETH,  7)
#define EvtNetETH_SetOptionInvalidParameter EventID (EventLevelError, EvtNetETH,  8)
#define EvtNetETH_SetMacAddress             EventID (EventLevelOp,    EvtNetETH,  9)
#define EvtNetETH_SetIp4Address             EventID (EventLevelOp,    EvtNetETH, 10)
#define EvtNetETH_SetIp4SubnetMask          EventID (EventLevelOp,    EvtNetETH, 11)
#define EvtNetETH_SetIp4DefaultGateway      EventID (EventLevelOp,    EvtNetETH, 12)
#define EvtNetETH_SetIp4PrimaryDNS          EventID (EventLevelOp,    EvtNetETH, 13)
#define EvtNetETH_SetIp4SecondaryDNS        EventID (EventLevelOp,    EvtNetETH, 14)
#define EvtNetETH_SetIp4Mtu                 EventID (EventLevelOp,    EvtNetETH, 15)
#define EvtNetETH_SetIp6Address             EventID (EventLevelOp,    EvtNetETH, 16)
#define EvtNetETH_SetIp6DefaultGateway      EventID (EventLevelOp,    EvtNetETH, 17)
#define EvtNetETH_SetIp6PrimaryDNS          EventID (EventLevelOp,    EvtNetETH, 18)
#define EvtNetETH_SetIp6SecondaryDNS        EventID (EventLevelOp,    EvtNetETH, 19)
#define EvtNetETH_SetIp6PrefixLength        EventID (EventLevelOp,    EvtNetETH, 20)
#define EvtNetETH_SendFrame                 EventID (EventLevelOp,    EvtNetETH, 21)
#define EvtNetETH_LinkDownError             EventID (EventLevelError, EvtNetETH, 22)
#define EvtNetETH_SendDataTooLarge          EventID (EventLevelError, EvtNetETH, 23)
#define EvtNetETH_Ip4LocalAddressUndefined  EventID (EventLevelError, EvtNetETH, 24)
#define EvtNetETH_Ip4MacAddressUnresolved   EventID (EventLevelError, EvtNetETH, 25)
#define EvtNetETH_Ip6LocalAddressUndefined  EventID (EventLevelError, EvtNetETH, 26)
#define EvtNetETH_Ip6MacAddressUnresolved   EventID (EventLevelError, EvtNetETH, 27)
#define EvtNetETH_InvalidIpVersion          EventID (EventLevelError, EvtNetETH, 28)
#define EvtNetETH_ShowFrameHeader           EventID (EventLevelDetail,EvtNetETH, 29)
#define EvtNetETH_LinkDownStatus            EventID (EventLevelOp,    EvtNetETH, 30)
#define EvtNetETH_LinkUpStatus              EventID (EventLevelOp,    EvtNetETH, 31)
#define EvtNetETH_ReceiveFrame              EventID (EventLevelOp,    EvtNetETH, 32)
#define EvtNetETH_Ip4Disabled               EventID (EventLevelOp,    EvtNetETH, 33)
#define EvtNetETH_Ip6Disabled               EventID (EventLevelOp,    EvtNetETH, 34)
#define EvtNetETH_ProtocolUnknown           EventID (EventLevelOp,    EvtNetETH, 35)
#define EvtNetETH_SendRawFrame              EventID (EventLevelAPI,   EvtNetETH, 36)
#define EvtNetETH_SendRawInvalidParameter   EventID (EventLevelError, EvtNetETH, 37)
#define EvtNetETH_UninitInterface           EventID (EventLevelOp,    EvtNetETH, 38)
#endif

/**
  \brief  Event on Ethernet interface initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_InitInterface(void) {
    EventRecord2 (EvtNetETH_InitInterface, 0, 0);
  }
#else
  #define EvrNetETH_InitInterface()
#endif

/**
  \brief  Event on Ethernet MAC address configuration error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_MacAddressConfigError(void) {
    EventRecord2 (EvtNetETH_MacAddressConfigError, 0, 0);
  }
#else
  #define EvrNetETH_MacAddressConfigError()
#endif

/**
  \brief  Event on Ethernet PHY configuration error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_PhyDriverConfigError(void) {
    EventRecord2 (EvtNetETH_PhyDriverConfigError, 0, 0);
  }
#else
  #define EvrNetETH_PhyDriverConfigError()
#endif

/**
  \brief  Event on Ethernet PHY initialization error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_PhyDriverInitError(void) {
    EventRecord2 (EvtNetETH_PhyDriverInitError, 0, 0);
  }
#else
  #define EvrNetETH_PhyDriverInitError()
#endif

/**
  \brief  Event on Ethernet IPv4 configuration error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_Ip4ConfigError(void) {
    EventRecord2 (EvtNetETH_Ip4ConfigError, 0, 0);
  }
#else
  #define EvrNetETH_Ip4ConfigError()
#endif

/**
  \brief  Event on Ethernet IPv6 configuration error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_Ip6ConfigError(void) {
    EventRecord2 (EvtNetETH_Ip6ConfigError, 0, 0);
  }
#else
  #define EvrNetETH_Ip6ConfigError()
#endif

/**
  \brief  Event on Ethernet thread create failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_ThreadCreateFailed(void) {
    EventRecord2 (EvtNetETH_ThreadCreateFailed, 0, 0);
  }
#else
  #define EvrNetETH_ThreadCreateFailed()
#endif

/**
  \brief  Event on Ethernet get-option invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_GetOptionInvalidParameter(void) {
    EventRecord2 (EvtNetETH_GetOptionInvalidParameter, 0, 0);
  }
#else
  #define EvrNetETH_GetOptionInvalidParameter()
#endif

/**
  \brief  Event on Ethernet set-option invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetOptionInvalidParameter(void) {
    EventRecord2 (EvtNetETH_SetOptionInvalidParameter, 0, 0);
  }
#else
  #define EvrNetETH_SetOptionInvalidParameter()
#endif

/**
  \brief  Event on Ethernet set interface MAC address (Op)
  \param  mac_addr      pointer to MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetMacAddress(const uint8_t *mac_addr) {
    EventRecordData (EvtNetETH_SetMacAddress, mac_addr, 6);
  }
#else
  #define EvrNetETH_SetMacAddress(mac_addr)
#endif

/**
  \brief  Event on Ethernet set interface IPv4 address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp4Address(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetETH_SetIp4Address, ip4_addr, 4);
  }
#else
  #define EvrNetETH_SetIp4Address(ip4_addr)
#endif

/**
  \brief  Event on Ethernet set interface IPv4 subnet mask (Op)
  \param  net_mask      pointer to IPv4 subnet mask
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp4SubnetMask(const uint8_t *net_mask) {
    EventRecordData (EvtNetETH_SetIp4SubnetMask, net_mask, 4);
  }
#else
  #define EvrNetETH_SetIp4SubnetMask(net_mask)
#endif

/**
  \brief  Event on Ethernet set interface IPv4 default gateway (Op)
  \param  ip4_addr      pointer to IPv4 address of a gateway
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp4DefaultGateway(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetETH_SetIp4DefaultGateway, ip4_addr, 4);
  }
#else
  #define EvrNetETH_SetIp4DefaultGateway(ip4_addr)
#endif

/**
  \brief  Event on Ethernet set interface primary DNS server (Op)
  \param  ip4_addr      pointer to IPv4 address of a DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp4PrimaryDNS(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetETH_SetIp4PrimaryDNS, ip4_addr, 4);
  }
#else
  #define EvrNetETH_SetIp4PrimaryDNS(ip4_addr)
#endif

/**
  \brief  Event on Ethernet set interface secondary DNS server (Op)
  \param  ip4_addr      pointer to IPv4 address of a DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp4SecondaryDNS(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetETH_SetIp4SecondaryDNS, ip4_addr, 4);
  }
#else
  #define EvrNetETH_SetIp4SecondaryDNS(ip4_addr)
#endif

/**
  \brief  Event on Ethernet set interface MTU (Op)
  \param  mtu           maximum transmission unit
                         - 0: fragmentation disabled
  \note   MTU is limited in the range from 576 to 1500 bytes.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp4Mtu(uint16_t mtu) {
    EventRecord2 (EvtNetETH_SetIp4Mtu, mtu, 0);
  }
#else
  #define EvrNetETH_SetIp4Mtu(mtu)
#endif

/**
  \brief  Event on Ethernet set interface IPv6 static address (Op)
  \param  ip6_addr      pointer to IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp6Address(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetETH_SetIp6Address, ip6_addr, 16);
  }
#else
  #define EvrNetETH_SetIp6Address(ip6_addr)
#endif

/**
  \brief  Event on Ethernet set interface IPv6 default gateway (Op)
  \param  ip6_addr      pointer to IPv6 address of a gateway
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp6DefaultGateway(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetETH_SetIp6DefaultGateway, ip6_addr, 16);
  }
#else
  #define EvrNetETH_SetIp6DefaultGateway(ip6_addr)
#endif

/**
  \brief  Event on Ethernet set interface primary DNS server (Op)
  \param  ip6_addr      pointer to IPv6 address of a DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp6PrimaryDNS(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetETH_SetIp6PrimaryDNS, ip6_addr, 16);
  }
#else
  #define EvrNetETH_SetIp6PrimaryDNS(ip6_addr)
#endif

/**
  \brief  Event on Ethernet set interface secondary DNS server (Op)
  \param  ip6_addr      pointer to IPv6 address of a DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp6SecondaryDNS(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetETH_SetIp6SecondaryDNS, ip6_addr, 16);
  }
#else
  #define EvrNetETH_SetIp6SecondaryDNS(ip6_addr)
#endif

/**
  \brief  Event on Ethernet set interface IPv6 subnet prefix length (Op)
  \param  pref_len      subnet prefix length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SetIp6PrefixLength(uint8_t pref_len) {
    EventRecord2 (EvtNetETH_SetIp6PrefixLength, pref_len, 0);
  }
#else
  #define EvrNetETH_SetIp6PrefixLength(pref_len)
#endif

/**
  \brief  Event on Ethernet send frame (Op)
  \param  length        frame length in bytes
  \param  ip_version    internet protocol version
                         - 0: IPv4
                         - 1: IPv6
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SendFrame(uint32_t length, uint8_t ip_version) {
    EventRecord2 (EvtNetETH_SendFrame, length, ip_version);
  }
#else
  #define EvrNetETH_SendFrame(length, ip_version)
#endif

/**
  \brief  Event on Ethernet link down error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_LinkDownError(void) {
    EventRecord2 (EvtNetETH_LinkDownError, 0, 0);
  }
#else
  #define EvrNetETH_LinkDownError()
#endif

/**
  \brief  Event on Ethernet send payload too large error (Error)
  \param  length        data length in bytes
  \param  max_length    maximum length of data in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SendDataTooLarge(uint32_t length, uint32_t max_length) {
    EventRecord2 (EvtNetETH_SendDataTooLarge, length, max_length);
  }
#else
  #define EvrNetETH_SendDataTooLarge(length, max_length)
#endif

/**
  \brief  Event on Ethernet local IPv4 address undefined (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_Ip4LocalAddressUndefined(void) {
    EventRecord2 (EvtNetETH_Ip4LocalAddressUndefined, 0, 0);
  }
#else
  #define EvrNetETH_Ip4LocalAddressUndefined()
#endif

/**
  \brief  Event on Ethernet MAC address unresolved (Error)
  \param  ip4_addr      pointer to IPv4 destination address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_Ip4MacAddressUnresolved(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetETH_Ip4MacAddressUnresolved, ip4_addr, 4);
  }
#else
  #define EvrNetETH_Ip4MacAddressUnresolved(ip4_addr)
#endif

/**
  \brief  Event on Ethernet local IPv6 address undefined (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_Ip6LocalAddressUndefined(void) {
    EventRecord2 (EvtNetETH_Ip6LocalAddressUndefined, 0, 0);
  }
#else
  #define EvrNetETH_Ip6LocalAddressUndefined()
#endif

/**
  \brief  Event on Ethernet MAC address unresolved (Error)
  \param  ip6_addr      pointer to IPv6 destination address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_Ip6MacAddressUnresolved(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetETH_Ip6MacAddressUnresolved, ip6_addr, 16);
  }
#else
  #define EvrNetETH_Ip6MacAddressUnresolved(ip6_addr)
#endif

/**
  \brief  Event on Ethernet invalid IP version (Error)
  \param  ip_version    invalid IP protocol version value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_InvalidIpVersion(uint8_t ip_version) {
    EventRecord2 (EvtNetETH_InvalidIpVersion, ip_version, 0);
  }
#else
  #define EvrNetETH_InvalidIpVersion(ip_version)
#endif

/**
  \brief  Event on Ethernet display send/receive frame header (Detail)
  \param  eth_header    pointer to ethernet frame header of 14 bytes
                         - DstAddr  (6 bytes)
                         - SrcAddr  (6 bytes)
                         - Protocol (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_ShowFrameHeader(const void *eth_header) {
    EventRecordData (EvtNetETH_ShowFrameHeader, eth_header, 14);
  }
#else
  #define EvrNetETH_ShowFrameHeader(eth_header)
#endif

/**
  \brief  Event on Ethernet link down status (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_LinkDownStatus(void) {
    EventRecord2 (EvtNetETH_LinkDownStatus, 0, 0);
  }
#else
  #define EvrNetETH_LinkDownStatus()
#endif

/**
  \brief  Event on Ethernet link up status (Op)
  \param  speed         link speed
                         - 0: 10 Mbit
                         - 1: 100 Mbit
                         - 2: 1 Gbit
  \param  duplex        link duplex
                         - 0: half duplex
                         - 1: full duplex
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_LinkUpStatus(int32_t speed, int32_t duplex) {
    EventRecord2 (EvtNetETH_LinkUpStatus, (uint32_t)speed, (uint32_t)duplex);
  }
#else
  #define EvrNetETH_LinkUpStatus(speed, duplex)
#endif

/**
  \brief  Event on Ethernet receive frame (Op)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_ReceiveFrame(uint32_t length) {
    EventRecord2 (EvtNetETH_ReceiveFrame, length, 0);
  }
#else
  #define EvrNetETH_ReceiveFrame(length)
#endif

/**
  \brief  Event on Ethernet receive IPv4 disabled (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_Ip4Disabled(void) {
    EventRecord2 (EvtNetETH_Ip4Disabled, 0, 0);
  }
#else
  #define EvrNetETH_Ip4Disabled()
#endif

/**
  \brief  Event on Ethernet receive IPv6 disabled (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_Ip6Disabled(void) {
    EventRecord2 (EvtNetETH_Ip6Disabled, 0, 0);
  }
#else
  #define EvrNetETH_Ip6Disabled()
#endif

/**
  \brief  Event on Ethernet receive protocol unknown (Op)
  \param  protocol      unknown ethernet protocol type
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_ProtocolUnknown(uint16_t protocol) {
    EventRecord2 (EvtNetETH_ProtocolUnknown, protocol, 0);
  }
#else
  #define EvrNetETH_ProtocolUnknown(protocol)
#endif

/**
  \brief  Event on Ethernet send raw frame (API)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SendRawFrame(uint32_t length) {
    EventRecord2 (EvtNetETH_SendRawFrame, length, 0);
  }
#else
  #define EvrNetETH_SendRawFrame(length)
#endif

/**
  \brief  Event on Ethernet send raw frame invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_SendRawInvalidParameter(void) {
    EventRecord2 (EvtNetETH_SendRawInvalidParameter, 0, 0);
  }
#else
  #define EvrNetETH_SendRawInvalidParameter()
#endif

/**
  \brief  Event on Ethernet interface de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetETH_UninitInterface(void) {
    EventRecord2 (EvtNetETH_UninitInterface, 0, 0);
  }
#else
  #define EvrNetETH_UninitInterface()
#endif


// NetPPP event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
/* PPP-Core events */
#define EvtNetPPP_InitInterface             EventID (EventLevelOp,    EvtNetPPP,  0)
#define EvtNetPPP_ThreadCreateFailed        EventID (EventLevelError, EvtNetPPP,  1)
#define EvtNetPPP_GetOptionInvalidParameter EventID (EventLevelError, EvtNetPPP,  2)
#define EvtNetPPP_SetOptionInvalidParameter EventID (EventLevelError, EvtNetPPP,  3)
#define EvtNetPPP_SetIp4Address             EventID (EventLevelOp,    EvtNetPPP,  4)
#define EvtNetPPP_SetIp4PrimaryDNS          EventID (EventLevelOp,    EvtNetPPP,  5)
#define EvtNetPPP_SetIp4SecondaryDNS        EventID (EventLevelOp,    EvtNetPPP,  6)
#define EvtNetPPP_SetIp4Mtu                 EventID (EventLevelOp,    EvtNetPPP,  7)
#define EvtNetPPP_ReceiveFrame              EventID (EventLevelOp,    EvtNetPPP,  8)
#define EvtNetPPP_FrameTooShort             EventID (EventLevelOp,    EvtNetPPP,  9)
#define EvtNetPPP_ChecksumFailed            EventID (EventLevelError, EvtNetPPP, 10)
#define EvtNetPPP_CtrlByteInvalid           EventID (EventLevelError, EvtNetPPP, 11)
#define EvtNetPPP_SendFrame                 EventID (EventLevelOp,    EvtNetPPP, 12)
#define EvtNetPPP_NetworkLayerDown          EventID (EventLevelError, EvtNetPPP, 13)
#define EvtNetPPP_SendDataTooLarge          EventID (EventLevelError, EvtNetPPP, 14)
#define EvtNetPPP_NoMemoryError             EventID (EventLevelError, EvtNetPPP, 15)
#define EvtNetPPP_QueueAddTransmit          EventID (EventLevelOp,    EvtNetPPP, 16)
#define EvtNetPPP_RejectProtocol            EventID (EventLevelOp,    EvtNetPPP, 17)
#define EvtNetPPP_RejectCode                EventID (EventLevelOp,    EvtNetPPP, 18)
#define EvtNetPPP_Connect                   EventID (EventLevelAPI,   EvtNetPPP, 19)
#define EvtNetPPP_Listen                    EventID (EventLevelAPI,   EvtNetPPP, 20)
#define EvtNetPPP_Close                     EventID (EventLevelAPI,   EvtNetPPP, 21)
#define EvtNetPPP_ShowUsername              EventID (EventLevelOp,    EvtNetPPP, 22)
#define EvtNetPPP_ShowPassword              EventID (EventLevelOp,    EvtNetPPP, 23)
#define EvtNetPPP_ModemOffline              EventID (EventLevelOp,    EvtNetPPP, 24)
#define EvtNetPPP_DataLinkDown              EventID (EventLevelOp,    EvtNetPPP, 25)
#define EvtNetPPP_UninitInterface           EventID (EventLevelOp,    EvtNetPPP, 26)
/* PPP-LCP events */
#define EvtNetPPP_LcpInit                   EventID (EventLevelOp,    EvtNetPPP, 27)
#define EvtNetPPP_LcpSendConfigRequest      EventID (EventLevelOp,    EvtNetPPP, 28)
#define EvtNetPPP_LcpNoRetriesLeft          EventID (EventLevelError, EvtNetPPP, 29)
#define EvtNetPPP_LcpOptionCharMap          EventID (EventLevelOp,    EvtNetPPP, 30)
#define EvtNetPPP_LcpOptionPfc              EventID (EventLevelOp,    EvtNetPPP, 31)
#define EvtNetPPP_LcpOptionAcfc             EventID (EventLevelOp,    EvtNetPPP, 32)
#define EvtNetPPP_LcpOptionMagicNumber      EventID (EventLevelOp,    EvtNetPPP, 33)
#define EvtNetPPP_LcpOptionAuthPap          EventID (EventLevelOp,    EvtNetPPP, 34)
#define EvtNetPPP_LcpOptionAuthChapMd5      EventID (EventLevelOp,    EvtNetPPP, 35)
#define EvtNetPPP_LcpSendEchoRequest        EventID (EventLevelOp,    EvtNetPPP, 36)
#define EvtNetPPP_LcpSendTerminateRequest   EventID (EventLevelOp,    EvtNetPPP, 37)
#define EvtNetPPP_LcpSendFrame              EventID (EventLevelOp,    EvtNetPPP, 38)
#define EvtNetPPP_LcpReceiveFrame           EventID (EventLevelOp,    EvtNetPPP, 39)
#define EvtNetPPP_LcpDataLinkUp             EventID (EventLevelOp,    EvtNetPPP, 40)
#define EvtNetPPP_LcpOptionMru              EventID (EventLevelOp,    EvtNetPPP, 41)
#define EvtNetPPP_LcpOptionAuth             EventID (EventLevelOp,    EvtNetPPP, 42)
#define EvtNetPPP_LcpOptionAuthChapNotMd5   EventID (EventLevelOp,    EvtNetPPP, 43)
#define EvtNetPPP_LcpOptionUnknown          EventID (EventLevelOp,    EvtNetPPP, 44)
#define EvtNetPPP_LcpSendReject             EventID (EventLevelOp,    EvtNetPPP, 45)
#define EvtNetPPP_LcpSendNak                EventID (EventLevelOp,    EvtNetPPP, 46)
#define EvtNetPPP_LcpSendAck                EventID (EventLevelOp,    EvtNetPPP, 47)
#define EvtNetPPP_LcpWrongAckReceived       EventID (EventLevelOp,    EvtNetPPP, 48)
#define EvtNetPPP_LcpWrongNakReceived       EventID (EventLevelOp,    EvtNetPPP, 49)
#define EvtNetPPP_LcpConfigAuthFailed       EventID (EventLevelError, EvtNetPPP, 50)
#define EvtNetPPP_LcpWrongRejectReceived    EventID (EventLevelOp,    EvtNetPPP, 51)
#define EvtNetPPP_LcpEchoMagicNumber        EventID (EventLevelOp,    EvtNetPPP, 52)
#define EvtNetPPP_LcpWrongPeerMagicNumber   EventID (EventLevelOp,    EvtNetPPP, 53)
#define EvtNetPPP_LcpSendEchoReply          EventID (EventLevelOp,    EvtNetPPP, 54)
#define EvtNetPPP_LcpWrongEchoReplyReceived EventID (EventLevelOp,    EvtNetPPP, 55)
#define EvtNetPPP_LcpSendTerminateAck       EventID (EventLevelOp,    EvtNetPPP, 56)
#define EvtNetPPP_LcpUninit                 EventID (EventLevelOp,    EvtNetPPP, 57)
/* PPP-PAP events */
#define EvtNetPPP_PapInit                   EventID (EventLevelOp,    EvtNetPPP, 58)
#define EvtNetPPP_PapSendAuthRequest        EventID (EventLevelOp,    EvtNetPPP, 59)
#define EvtNetPPP_PapShowPassword           EventID (EventLevelOp,    EvtNetPPP, 60)
#define EvtNetPPP_PapSendFrame              EventID (EventLevelOp,    EvtNetPPP, 61)
#define EvtNetPPP_PapReceiveFrame           EventID (EventLevelOp,    EvtNetPPP, 62)
#define EvtNetPPP_PapLoginSuccess           EventID (EventLevelOp,    EvtNetPPP, 63)
#define EvtNetPPP_PapLoginFailed            EventID (EventLevelError, EvtNetPPP, 64)
#define EvtNetPPP_PapWrongAckReceived       EventID (EventLevelOp,    EvtNetPPP, 65)
#define EvtNetPPP_PapWrongNakReceived       EventID (EventLevelOp,    EvtNetPPP, 66)
#define EvtNetPPP_PapUninit                 EventID (EventLevelOp,    EvtNetPPP, 67)
/* PPP-CHAP events */
#define EvtNetPPP_ChapInit                  EventID (EventLevelOp,    EvtNetPPP, 68)
#define EvtNetPPP_ChapSendChallenge         EventID (EventLevelOp,    EvtNetPPP, 69)
#define EvtNetPPP_ChapSendFrame             EventID (EventLevelOp,    EvtNetPPP, 70)
#define EvtNetPPP_ChapReceiveFrame          EventID (EventLevelOp,    EvtNetPPP, 71)
#define EvtNetPPP_ChapWrongResponseReceived EventID (EventLevelOp,    EvtNetPPP, 72)
#define EvtNetPPP_ChapWrongSuccessReceived  EventID (EventLevelOp,    EvtNetPPP, 73)
#define EvtNetPPP_ChapWrongFailureReceived  EventID (EventLevelOp,    EvtNetPPP, 74)
#define EvtNetPPP_ChapLoginSuccess          EventID (EventLevelOp,    EvtNetPPP, 75)
#define EvtNetPPP_ChapLoginFailed           EventID (EventLevelError, EvtNetPPP, 76)
#define EvtNetPPP_ChapUninit                EventID (EventLevelOp,    EvtNetPPP, 77)
/* PPP-IPCP events */
#define EvtNetPPP_IpcpInit                  EventID (EventLevelOp,    EvtNetPPP, 78)
#define EvtNetPPP_IpcpIp4ConfigError        EventID (EventLevelError, EvtNetPPP, 79)
#define EvtNetPPP_IpcpSendConfigRequest     EventID (EventLevelOp,    EvtNetPPP, 80)
#define EvtNetPPP_IpcpOptionIpAddress       EventID (EventLevelOp,    EvtNetPPP, 81)
#define EvtNetPPP_IpcpOptionPrimaryDns      EventID (EventLevelOp,    EvtNetPPP, 82)
#define EvtNetPPP_IpcpOptionSecondaryDns    EventID (EventLevelOp,    EvtNetPPP, 83)
#define EvtNetPPP_IpcpSendFrame             EventID (EventLevelOp,    EvtNetPPP, 84)
#define EvtNetPPP_IpcpReceiveFrame          EventID (EventLevelOp,    EvtNetPPP, 85)
#define EvtNetPPP_IpcpOptionUnknown         EventID (EventLevelOp,    EvtNetPPP, 86)
#define EvtNetPPP_IpcpSendReject            EventID (EventLevelOp,    EvtNetPPP, 87)
#define EvtNetPPP_IpcpSendNak               EventID (EventLevelOp,    EvtNetPPP, 88)
#define EvtNetPPP_IpcpSendAck               EventID (EventLevelOp,    EvtNetPPP, 89)
#define EvtNetPPP_IpcpWrongAckReceived      EventID (EventLevelOp,    EvtNetPPP, 90)
#define EvtNetPPP_IpcpNetworkLayerUp        EventID (EventLevelOp,    EvtNetPPP, 91)
#define EvtNetPPP_IpcpWrongNakReceived      EventID (EventLevelOp,    EvtNetPPP, 92)
#define EvtNetPPP_IpcpWrongRejectReceived   EventID (EventLevelOp,    EvtNetPPP, 93)
#define EvtNetPPP_IpcpWrongSubnet           EventID (EventLevelOp,    EvtNetPPP, 94)
#define EvtNetPPP_IpcpPrimaryDnsRejected    EventID (EventLevelOp,    EvtNetPPP, 95)
#define EvtNetPPP_IpcpSecondaryDnsRejected  EventID (EventLevelOp,    EvtNetPPP, 96)
#define EvtNetPPP_IpcpIpAddressRejected     EventID (EventLevelError, EvtNetPPP, 97)
#define EvtNetPPP_IpcpNotRequestedOption    EventID (EventLevelError, EvtNetPPP, 98)
#define EvtNetPPP_IpcpUninit                EventID (EventLevelOp,    EvtNetPPP, 99)
#endif

// PPP core event recorder functions -------------------------------------------

/**
  \brief  Event on PPP interface initialize (Op)
  \note   Point-to-Point Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_InitInterface(void) {
    EventRecord2 (EvtNetPPP_InitInterface, 0, 0);
  }
#else
  #define EvrNetPPP_InitInterface()
#endif

/**
  \brief  Event on PPP interface thread create failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ThreadCreateFailed(void) {
    EventRecord2 (EvtNetPPP_ThreadCreateFailed, 0, 0);
  }
#else
  #define EvrNetPPP_ThreadCreateFailed()
#endif

/**
  \brief  Event on PPP get-option invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_GetOptionInvalidParameter(void) {
    EventRecord2 (EvtNetPPP_GetOptionInvalidParameter, 0, 0);
  }
#else
  #define EvrNetPPP_GetOptionInvalidParameter()
#endif

/**
  \brief  Event on PPP set-option invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_SetOptionInvalidParameter(void) {
    EventRecord2 (EvtNetPPP_SetOptionInvalidParameter, 0, 0);
  }
#else
  #define EvrNetPPP_SetOptionInvalidParameter()
#endif

/**
  \brief  Event on PPP set interface IPv4 address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_SetIp4Address(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetPPP_SetIp4Address, ip4_addr, 4);
  }
#else
  #define EvrNetPPP_SetIp4Address(ip4_addr)
#endif

/**
  \brief  Event on PPP set interface primary DNS server (Op)
  \param  ip4_addr      pointer to IPv4 address of a DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_SetIp4PrimaryDNS(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetPPP_SetIp4PrimaryDNS, ip4_addr, 4);
  }
#else
  #define EvrNetPPP_SetIp4PrimaryDNS(ip4_addr)
#endif

/**
  \brief  Event on PPP set interface secondary DNS server (Op)
  \param  ip4_addr      pointer to IPv4 address of a DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_SetIp4SecondaryDNS(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetPPP_SetIp4SecondaryDNS, ip4_addr, 4);
  }
#else
  #define EvrNetPPP_SetIp4SecondaryDNS(ip4_addr)
#endif

/**
  \brief  Event on PPP set interface MTU (Op)
  \param  mtu           maximum transmission unit
                         - 0: fragmentation disabled
  \note   MTU is limited in the range from 576 to 1500 bytes.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_SetIp4Mtu(uint16_t mtu) {
    EventRecord2 (EvtNetPPP_SetIp4Mtu, mtu, 0);
  }
#else
  #define EvrNetPPP_SetIp4Mtu(mtu)
#endif

/**
  \brief  Event on PPP interface receive frame (Op)
  \param  protocol      PPP protocol type
                         - 0xC021: LCP
                         - 0xC023: PAP
                         - 0xC223: CHAP
                         - 0x8021: IPCP
                         - 0x0021: IP
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ReceiveFrame(uint16_t protocol, uint32_t length) {
    EventRecord2 (EvtNetPPP_ReceiveFrame, protocol, length);
  }
#else
  #define EvrNetPPP_ReceiveFrame(protocol, length)
#endif

/**
  \brief  Event on PPP interface received frame too short (Op)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_FrameTooShort(uint32_t length, uint32_t min_length) {
    if (length > 1536) length = 0;
    EventRecord2 (EvtNetPPP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetPPP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on PPP interface frame checksum check failed (Error)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChecksumFailed(uint32_t length) {
    EventRecord2 (EvtNetPPP_ChecksumFailed, length, 0);
  }
#else
  #define EvrNetPPP_ChecksumFailed(length)
#endif

/**
  \brief  Event on PPP interface frame control byte invalid (Error)
  \param  ctrl_byte     invalid control byte value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_CtrlByteInvalid(uint8_t ctrl_byte) {
    EventRecord2 (EvtNetPPP_CtrlByteInvalid, ctrl_byte, 0);
  }
#else
  #define EvrNetPPP_CtrlByteInvalid(ctrl_byte)
#endif

/**
  \brief  Event on PPP interface send frame (Op)
  \param  protocol      PPP protocol type
                         - 0xC021: LCP
                         - 0xC023: PAP
                         - 0xC223: CHAP
                         - 0x8021: IPCP
                         - 0x0021: IP
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_SendFrame(uint16_t protocol, uint32_t length) {
    EventRecord2 (EvtNetPPP_SendFrame, protocol, length);
  }
#else
  #define EvrNetPPP_SendFrame(protocol, length)
#endif

/**
  \brief  Event on PPP interface network layer down (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_NetworkLayerDown(void) {
    EventRecord2 (EvtNetPPP_NetworkLayerDown, 0, 0);
  }
#else
  #define EvrNetPPP_NetworkLayerDown()
#endif

/**
  \brief  Event on PPP interface send payload too large error (Error)
  \param  length        data length in bytes
  \param  max_length    maximum length of data in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_SendDataTooLarge(uint32_t length, uint32_t max_length) {
    EventRecord2 (EvtNetPPP_SendDataTooLarge, length, max_length);
  }
#else
  #define EvrNetPPP_SendDataTooLarge(length, max_length)
#endif

/**
  \brief  Event on PPP interface out of memory error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_NoMemoryError(void) {
    EventRecord2 (EvtNetPPP_NoMemoryError, 0, 0);
  }
#else
  #define EvrNetPPP_NoMemoryError()
#endif

/**
  \brief  Event on PPP interface add frame to transmit queue (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_QueueAddTransmit(void) {
    EventRecord2 (EvtNetPPP_QueueAddTransmit, 0, 0);
  }
#else
  #define EvrNetPPP_QueueAddTransmit()
#endif

/**
  \brief  Event on PPP interface reject unsupported protocol type (Op)
  \param  protocol      unsupported protocol type value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_RejectProtocol(uint16_t protocol) {
    EventRecord2 (EvtNetPPP_RejectProtocol, protocol, 0);
  }
#else
  #define EvrNetPPP_RejectProtocol(protocol)
#endif

/**
  \brief  Event on PPP interface reject unsupported cp code (Op)
  \param  protocol      PPP protocol type
                         - 0xC021: LCP
                         - 0x8021: IPCP
  \param  code          unsupported control code value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_RejectCode(uint16_t protocol, uint8_t code) {
    EventRecord2 (EvtNetPPP_RejectCode, protocol, code);
  }
#else
  #define EvrNetPPP_RejectCode(protocol, code)
#endif

/**
  \brief  Event on PPP interface \ref netPPP_Connect start (API)
  \param  dial_number   pointer to a dial number string
  \param  length        length of the dial number string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_Connect(const char *dial_number, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData (EvtNetPPP_Connect, dial_number, length);
  }
#else
  #define EvrNetPPP_Connect(dial_number, length)
#endif

/**
  \brief  Event on PPP interface \ref netPPP_Listen start (API)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_Listen(void) {
    EventRecord2 (EvtNetPPP_Listen, 0, 0);
  }
#else
  #define EvrNetPPP_Listen()
#endif

/**
  \brief  Event on PPP interface \ref netPPP_Close start (API)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_Close(void) {
    EventRecord2 (EvtNetPPP_Close, 0, 0);
  }
#else
  #define EvrNetPPP_Close()
#endif

/**
  \brief  Event on PPP interface display username (Op)
  \param  username      pointer to a username string
  \param  length        length of the username string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ShowUsername(const char *username, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData (EvtNetPPP_ShowUsername, username, length);
  }
#else
  #define EvrNetPPP_ShowUsername(username, length)
#endif

/**
  \brief  Event on PPP interface display password (Op)
  \param  password      pointer to a password string
  \param  length        length of the password string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ShowPassword(const char *password, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData (EvtNetPPP_ShowPassword, password, length);
  }
#else
  #define EvrNetPPP_ShowPassword(password, length)
#endif

/**
  \brief  Event on PPP interface modem offline (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ModemOffline(void) {
    EventRecord2 (EvtNetPPP_ModemOffline, 0, 0);
  }
#else
  #define EvrNetPPP_ModemOffline()
#endif

/**
  \brief  Event on PPP interface data-link layer down (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_DataLinkDown(void) {
    EventRecord2 (EvtNetPPP_DataLinkDown, 0, 0);
  }
#else
  #define EvrNetPPP_DataLinkDown()
#endif

/**
  \brief  Event on PPP interface de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_UninitInterface(void) {
    EventRecord2 (EvtNetPPP_UninitInterface, 0, 0);
  }
#else
  #define EvrNetPPP_UninitInterface()
#endif

// PPP-LCP event recorder functions --------------------------------------------

/**
  \brief  Event on PPP-LCP initialize (Op)
  \note   Point-to-Point Link Control Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpInit(void) {
    EventRecord2 (EvtNetPPP_LcpInit, 0, 0);
  }
#else
  #define EvrNetPPP_LcpInit()
#endif

/**
  \brief  Event on PPP-LCP send configuration request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendConfigRequest(void) {
    EventRecord2 (EvtNetPPP_LcpSendConfigRequest, 0, 0);
  }
#else
  #define EvrNetPPP_LcpSendConfigRequest()
#endif

/**
  \brief  Event on PPP-LCP no retries left for LCP negotiation (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpNoRetriesLeft(void) {
    EventRecord2 (EvtNetPPP_LcpNoRetriesLeft, 0, 0);
  }
#else
  #define EvrNetPPP_LcpNoRetriesLeft()
#endif

/**
  \brief  Event on PPP-LCP option character map (Op)
  \param  char_map      character bit-map for non printable characters
                         - bit 0: ascii character 0
                         - bit 1: ascii character 1
                         - bit 2: ascii character 2
                         -  ...
                         - bit 31: ascii character 31
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionCharMap(uint32_t char_map) {
    EventRecord2 (EvtNetPPP_LcpOptionCharMap, char_map, 0);
  }
#else
  #define EvrNetPPP_LcpOptionCharMap(char_map)
#endif

/**
  \brief  Event on PPP-LCP option PFC (Protocol Field Compression) (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionPfc(void) {
    EventRecord2 (EvtNetPPP_LcpOptionPfc, 0, 0);
  }
#else
  #define EvrNetPPP_LcpOptionPfc()
#endif

/**
  \brief  Event on PPP-LCP option ACFC (Address and Control Field Compression) (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionAcfc(void) {
    EventRecord2 (EvtNetPPP_LcpOptionAcfc, 0, 0);
  }
#else
  #define EvrNetPPP_LcpOptionAcfc()
#endif

/**
  \brief  Event on PPP-LCP option magic number (Op)
  \param  magic_number  magic random number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionMagicNumber(uint32_t magic_number) {
    EventRecord2 (EvtNetPPP_LcpOptionMagicNumber, magic_number, 0);
  }
#else
  #define EvrNetPPP_LcpOptionMagicNumber(magic_number)
#endif

/**
  \brief  Event on PPP-LCP option Authentication PAP (Op)
  \note   Password Authentication Protocol - plain text
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionAuthPap(void) {
    EventRecord2 (EvtNetPPP_LcpOptionAuthPap, 0, 0);
  }
#else
  #define EvrNetPPP_LcpOptionAuthPap()
#endif

/**
  \brief  Event on PPP-LCP option Authentication CHAP (Op)
  \note   Challenge Handshake Authentication Protocol with MD5
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionAuthChapMd5(void) {
    EventRecord2 (EvtNetPPP_LcpOptionAuthChapMd5, 0, 0);
  }
#else
  #define EvrNetPPP_LcpOptionAuthChapMd5()
#endif

/**
  \brief  Event on PPP-LCP send echo request (Op)
  \param  magic_number  magic random number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendEchoRequest(uint32_t magic_number) {
    EventRecord2 (EvtNetPPP_LcpSendEchoRequest, magic_number, 0);
  }
#else
  #define EvrNetPPP_LcpSendEchoRequest(magic_number)
#endif

/**
  \brief  Event on PPP-LCP send LCP terminate request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendTerminateRequest(void) {
    EventRecord2 (EvtNetPPP_LcpSendTerminateRequest, 0, 0);
  }
#else
  #define EvrNetPPP_LcpSendTerminateRequest()
#endif

/**
  \brief  Event on PPP-LCP send frame (Op)
  \param  lcp_header    pointer to LCP frame header of 4 bytes
                         - Code (1 byte)
                         - Id   (1 byte)
                         - Len  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendFrame(const void *lcp_header) {
    EventRecordData (EvtNetPPP_LcpSendFrame, lcp_header, 4);
  }
#else
  #define EvrNetPPP_LcpSendFrame(lcp_header)
#endif

/**
  \brief  Event on PPP-LCP receive frame (Op)
  \param  lcp_header    pointer to LCP frame header of 4 bytes
                         - Code (1 byte)
                         - Id   (1 byte)
                         - Len  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpReceiveFrame(const void *lcp_header) {
    EventRecordData (EvtNetPPP_LcpReceiveFrame, lcp_header, 4);
  }
#else
  #define EvrNetPPP_LcpReceiveFrame(lcp_header)
#endif

/**
  \brief  Event on PPP-LCP data-link layer established (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpDataLinkUp(void) {
    EventRecord2 (EvtNetPPP_LcpDataLinkUp, 0, 0);
  }
#else
  #define EvrNetPPP_LcpDataLinkUp()
#endif

/**
  \brief  Event on PPP-LCP option MRU (Op)
  \param  mru_size      maximum receive unit size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionMru(uint16_t mru_size) {
    EventRecord2 (EvtNetPPP_LcpOptionMru, mru_size, 0);
  }
#else
  #define EvrNetPPP_LcpOptionMru(mru_size)
#endif

/**
  \brief  Event on PPP-LCP option authentication type (Op)
  \param  auth_type     authentication type code
                         - 0xC023: PAP
                         - 0xC223: CHAP
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionAuth(uint16_t auth_type) {
    EventRecord2 (EvtNetPPP_LcpOptionAuth, auth_type, 0);
  }
#else
  #define EvrNetPPP_LcpOptionAuth(auth_type)
#endif

/**
  \brief  Event on PPP-LCP option CHAP authentication not MD5 type (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionAuthChapNotMd5(void) {
    EventRecord2 (EvtNetPPP_LcpOptionAuthChapNotMd5, 0, 0);
  }
#else
  #define EvrNetPPP_LcpOptionAuthChapNotMd5()
#endif

/**
  \brief  Event on PPP-LCP unknown option received (Op)
  \param  lcp_option    unknown LCP option value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpOptionUnknown(uint8_t lcp_option) {
    EventRecord2 (EvtNetPPP_LcpOptionUnknown, lcp_option, 0);
  }
#else
  #define EvrNetPPP_LcpOptionUnknown(lcp_option)
#endif

/**
  \brief  Event on PPP-LCP send reject (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendReject(void) {
    EventRecord2 (EvtNetPPP_LcpSendReject, 0, 0);
  }
#else
  #define EvrNetPPP_LcpSendReject()
#endif

/**
  \brief  Event on PPP-LCP send not acknowledge (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendNak(void) {
    EventRecord2 (EvtNetPPP_LcpSendNak, 0, 0);
  }
#else
  #define EvrNetPPP_LcpSendNak()
#endif

/**
  \brief  Event on PPP-LCP send acknowledge (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendAck(void) {
    EventRecord2 (EvtNetPPP_LcpSendAck, 0, 0);
  }
#else
  #define EvrNetPPP_LcpSendAck()
#endif

/**
  \brief  Event on PPP-LCP wrong acknowledge received (Op)
  \param  lcp_id        wrong LCP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpWrongAckReceived(uint8_t lcp_id) {
    EventRecord2 (EvtNetPPP_LcpWrongAckReceived, lcp_id, 0);
  }
#else
  #define EvrNetPPP_LcpWrongAckReceived(lcp_id)
#endif

/**
  \brief  Event on PPP-LCP wrong not-acknowledge received (Op)
  \param  lcp_id        wrong LCP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpWrongNakReceived(uint8_t lcp_id) {
    EventRecord2 (EvtNetPPP_LcpWrongNakReceived, lcp_id, 0);
  }
#else
  #define EvrNetPPP_LcpWrongNakReceived(lcp_id)
#endif

/**
  \brief  Event on PPP-LCP authentication type negotiation failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpConfigAuthFailed(void) {
    EventRecord2 (EvtNetPPP_LcpConfigAuthFailed, 0, 0);
  }
#else
  #define EvrNetPPP_LcpConfigAuthFailed()
#endif

/**
  \brief  Event on PPP-LCP wrong reject received (Op)
  \param  lcp_id        wrong LCP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpWrongRejectReceived(uint8_t lcp_id) {
    EventRecord2 (EvtNetPPP_LcpWrongRejectReceived, lcp_id, 0);
  }
#else
  #define EvrNetPPP_LcpWrongRejectReceived(lcp_id)
#endif

/**
  \brief  Event on PPP-LCP echo magic number (Op)
  \param  magic_number  echo magic number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpEchoMagicNumber(uint32_t magic_number) {
    EventRecord2 (EvtNetPPP_LcpEchoMagicNumber, magic_number, 0);
  }
#else
  #define EvrNetPPP_LcpEchoMagicNumber(magic_number)
#endif

/**
  \brief  Event on PPP-LCP wrong peer magic number (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpWrongPeerMagicNumber(void) {
    EventRecord2 (EvtNetPPP_LcpWrongPeerMagicNumber, 0, 0);
  }
#else
  #define EvrNetPPP_LcpWrongPeerMagicNumber()
#endif

/**
  \brief  Event on PPP-LCP send echo reply (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendEchoReply(void) {
    EventRecord2 (EvtNetPPP_LcpSendEchoReply, 0, 0);
  }
#else
  #define EvrNetPPP_LcpSendEchoReply()
#endif

/**
  \brief  Event on PPP-LCP wrong echo reply received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpWrongEchoReplyReceived(void) {
    EventRecord2 (EvtNetPPP_LcpWrongEchoReplyReceived, 0, 0);
  }
#else
  #define EvrNetPPP_LcpWrongEchoReplyReceived()
#endif

/**
  \brief  Event on PPP-LCP send terminate acknowledge (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpSendTerminateAck(void) {
    EventRecord2 (EvtNetPPP_LcpSendTerminateAck, 0, 0);
  }
#else
  #define EvrNetPPP_LcpSendTerminateAck()
#endif

/**
  \brief  Event on PPP-LCP de-initialize (Op)
  \note   Point-to-Point Link Control Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_LcpUninit(void) {
    EventRecord2 (EvtNetPPP_LcpUninit, 0, 0);
  }
#else
  #define EvrNetPPP_LcpUninit()
#endif

// PPP-PAP event recorder functions --------------------------------------------

/**
  \brief  Event on PPP-PAP initialize (Op)
  \note   Password Authentication Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapInit(void) {
    EventRecord2 (EvtNetPPP_PapInit, 0, 0);
  }
#else
  #define EvrNetPPP_PapInit()
#endif

/**
  \brief  Event on PPP-PAP send authentication request (Op)
  \param  peer_id       peer identifier string (username)
  \param  length        length of the peer_id string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapSendAuthRequest(const char *peer_id, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData (EvtNetPPP_PapSendAuthRequest, peer_id, length);
  }
#else
  #define EvrNetPPP_PapSendAuthRequest(peer_id, length)
#endif

/**
  \brief  Event on PPP-PAP display password (Op)
  \param  password      authentication password
  \param  length        length of authentication password
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapShowPassword(const char *password, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData (EvtNetPPP_PapShowPassword, password, length);
  }
#else
  #define EvrNetPPP_PapShowPassword(password, length)
#endif

/**
  \brief  Event on PPP-PAP send frame (Op)
  \param  pap_header    pointer to PAP frame header of 4 bytes
                         - Code (1 byte)
                         - Id   (1 byte)
                         - Len  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapSendFrame(const void *pap_header) {
    EventRecordData (EvtNetPPP_PapSendFrame, pap_header, 4);
  }
#else
  #define EvrNetPPP_PapSendFrame(pap_header)
#endif

/**
  \brief  Event on PPP-PAP receive frame (Op)
  \param  pap_header    pointer to PAP frame header of 4 bytes
                         - Code (1 byte)
                         - Id   (1 byte)
                         - Len  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapReceiveFrame(const void *pap_header) {
    EventRecordData (EvtNetPPP_PapReceiveFrame, pap_header, 4);
  }
#else
  #define EvrNetPPP_PapReceiveFrame(pap_header)
#endif

/**
  \brief  Event on PPP-PAP login success (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapLoginSuccess(void) {
    EventRecord2 (EvtNetPPP_PapLoginSuccess, 0, 0);
  }
#else
  #define EvrNetPPP_PapLoginSuccess()
#endif

/**
  \brief  Event on PPP-PAP login failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapLoginFailed(void) {
    EventRecord2 (EvtNetPPP_PapLoginFailed, 0, 0);
  }
#else
  #define EvrNetPPP_PapLoginFailed()
#endif

/**
  \brief  Event on PPP-PAP wrong acknowledge received (Op)
  \param  pap_id        wrong PAP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapWrongAckReceived(uint8_t pap_id) {
    EventRecord2 (EvtNetPPP_PapWrongAckReceived, pap_id, 0);
  }
#else
  #define EvrNetPPP_PapWrongAckReceived(pap_id)
#endif

/**
  \brief  Event on PPP-PAP wrong not-acknowledge received (Op)
  \param  pap_id        wrong PAP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapWrongNakReceived(uint8_t pap_id) {
    EventRecord2 (EvtNetPPP_PapWrongNakReceived, pap_id, 0);
  }
#else
  #define EvrNetPPP_PapWrongNakReceived(pap_id)
#endif

/**
  \brief  Event on PPP-PAP de-initialize (Op)
  \note   Password Authentication Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_PapUninit(void) {
    EventRecord2 (EvtNetPPP_PapUninit, 0, 0);
  }
#else
  #define EvrNetPPP_PapUninit()
#endif

// PPP-CHAP event recorder functions -------------------------------------------

/**
  \brief  Event on PPP-CHAP initialize (Op)
  \note   Challenge-Handshake Authentication Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapInit(void) {
    EventRecord2 (EvtNetPPP_ChapInit, 0, 0);
  }
#else
  #define EvrNetPPP_ChapInit()
#endif

/**
  \brief  Event on PPP-CHAP send challenge (Op)
  \param  name          system identification string
  \param  length        length of the system identification string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapSendChallenge(const char *name, uint32_t length) {
    EventRecordData (EvtNetPPP_ChapSendChallenge, name, length);
  }
#else
  #define EvrNetPPP_ChapSendChallenge(name, length)
#endif

/**
  \brief  Event on PPP-CHAP send frame (Op)
  \param  chap_header   pointer to CHAP frame header of 4 bytes
                         - Code (1 byte)
                         - Id   (1 byte)
                         - Len  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapSendFrame(const void *chap_header) {
    EventRecordData (EvtNetPPP_ChapSendFrame, chap_header, 4);
  }
#else
  #define EvrNetPPP_ChapSendFrame(chap_header)
#endif

/**
  \brief  Event on PPP-CHAP receive frame (Op)
  \param  chap_header   pointer to CHAP frame header of 4 bytes
                         - Code (1 byte)
                         - Id   (1 byte)
                         - Len  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapReceiveFrame(const void *chap_header) {
    EventRecordData (EvtNetPPP_ChapReceiveFrame, chap_header, 4);
  }
#else
  #define EvrNetPPP_ChapReceiveFrame(chap_header)
#endif

/**
  \brief  Event on PPP-CHAP wrong response received (Op)
  \param  chap_id       wrong CHAP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapWrongResponseReceived(uint8_t chap_id) {
    EventRecord2 (EvtNetPPP_ChapWrongResponseReceived, chap_id, 0);
  }
#else
  #define EvrNetPPP_ChapWrongResponseReceived(chap_id)
#endif

/**
  \brief  Event on PPP-CHAP wrong success received (Op)
  \param  chap_id       wrong CHAP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapWrongSuccessReceived(uint8_t chap_id) {
    EventRecord2 (EvtNetPPP_ChapWrongSuccessReceived, chap_id, 0);
  }
#else
  #define EvrNetPPP_ChapWrongSuccessReceived(chap_id)
#endif

/**
  \brief  Event on PPP-CHAP wrong failure received (Op)
  \param  chap_id       wrong CHAP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapWrongFailureReceived(uint8_t chap_id) {
    EventRecord2 (EvtNetPPP_ChapWrongFailureReceived, chap_id, 0);
  }
#else
  #define EvrNetPPP_ChapWrongFailureReceived(chap_id)
#endif

/**
  \brief  Event on PPP-CHAP login success (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapLoginSuccess(void) {
    EventRecord2 (EvtNetPPP_ChapLoginSuccess, 0, 0);
  }
#else
  #define EvrNetPPP_ChapLoginSuccess()
#endif

/**
  \brief  Event on PPP-CHAP login failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapLoginFailed(void) {
    EventRecord2 (EvtNetPPP_ChapLoginFailed, 0, 0);
  }
#else
  #define EvrNetPPP_ChapLoginFailed()
#endif

/**
  \brief  Event on PPP-CHAP de-initialize (Op)
  \note   Challenge-Handshake Authentication Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_ChapUninit(void) {
    EventRecord2 (EvtNetPPP_ChapUninit, 0, 0);
  }
#else
  #define EvrNetPPP_ChapUninit()
#endif

// PPP-IPCP event recorder functions -------------------------------------------

/**
  \brief  Event on PPP-IPCP initialize (Op)
  \note   PPP Internet Protocol Control Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpInit(void) {
    EventRecord2 (EvtNetPPP_IpcpInit, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpInit()
#endif

/**
  \brief  Event on PPP-IPCP interface IPv4 configuration error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpIp4ConfigError(void) {
    EventRecord2 (EvtNetPPP_IpcpIp4ConfigError, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpIp4ConfigError()
#endif

/**
  \brief  Event on PPP-IPCP send configuration request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpSendConfigRequest(void) {
    EventRecord2 (EvtNetPPP_IpcpSendConfigRequest, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpSendConfigRequest()
#endif

/**
  \brief  Event on PPP-IPCP option IP-address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpOptionIpAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetPPP_IpcpOptionIpAddress, ip4_addr, 4);
  }
#else
  #define EvrNetPPP_IpcpOptionIpAddress(ip4_addr)
#endif

/**
  \brief  Event on PPP-IPCP option primary DNS server address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpOptionPrimaryDns(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetPPP_IpcpOptionPrimaryDns, ip4_addr, 4);
  }
#else
  #define EvrNetPPP_IpcpOptionPrimaryDns(ip4_addr)
#endif

/**
  \brief  Event on PPP-IPCP option secondary DNS server address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpOptionSecondaryDns(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetPPP_IpcpOptionSecondaryDns, ip4_addr, 4);
  }
#else
  #define EvrNetPPP_IpcpOptionSecondaryDns(ip4_addr)
#endif

/**
  \brief  Event on PPP-IPCP send frame (Op)
  \param  ipcp_header   pointer to IPCP frame header of 4 bytes
                         - Code (1 byte)
                         - Id   (1 byte)
                         - Len  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpSendFrame(const void *ipcp_header) {
    EventRecordData (EvtNetPPP_IpcpSendFrame, ipcp_header, 4);
  }
#else
  #define EvrNetPPP_IpcpSendFrame(ipcp_header)
#endif

/**
  \brief  Event on PPP-IPCP receive frame (Op)
  \param  ipcp_header   pointer to IPCP frame header of 4 bytes
                         - Code (1 byte)
                         - Id   (1 byte)
                         - Len  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpReceiveFrame(const void *ipcp_header) {
    EventRecordData (EvtNetPPP_IpcpReceiveFrame, ipcp_header, 4);
  }
#else
  #define EvrNetPPP_IpcpReceiveFrame(ipcp_header)
#endif

/**
  \brief  Event on PPP-IPCP unknown option received (Op)
  \param  ipcp_option   unknown IPCP option value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpOptionUnknown(uint8_t ipcp_option) {
    EventRecord2 (EvtNetPPP_IpcpOptionUnknown, ipcp_option, 0);
  }
#else
  #define EvrNetPPP_IpcpOptionUnknown(ipcp_option)
#endif

/**
  \brief  Event on PPP-IPCP send reject (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpSendReject(void) {
    EventRecord2 (EvtNetPPP_IpcpSendReject, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpSendReject()
#endif

/**
  \brief  Event on PPP-IPCP send not acknowledge (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpSendNak(void) {
    EventRecord2 (EvtNetPPP_IpcpSendNak, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpSendNak()
#endif

/**
  \brief  Event on PPP-IPCP send acknowledge (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpSendAck(void) {
    EventRecord2 (EvtNetPPP_IpcpSendAck, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpSendAck()
#endif

/**
  \brief  Event on PPP-IPCP wrong acknowledge received (Op)
  \param  ipcp_id       wrong IPCP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpWrongAckReceived(uint8_t ipcp_id) {
    EventRecord2 (EvtNetPPP_IpcpWrongAckReceived, ipcp_id, 0);
  }
#else
  #define EvrNetPPP_IpcpWrongAckReceived(ipcp_id)
#endif

/**
  \brief  Event on PPP-IPCP network-layer established (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpNetworkLayerUp(void) {
    EventRecord2 (EvtNetPPP_IpcpNetworkLayerUp, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpNetworkLayerUp()
#endif

/**
  \brief  Event on PPP-IPCP wrong not-acknowledge received (Op)
  \param  ipcp_id       wrong IPCP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpWrongNakReceived(uint8_t ipcp_id) {
    EventRecord2 (EvtNetPPP_IpcpWrongNakReceived, ipcp_id, 0);
  }
#else
  #define EvrNetPPP_IpcpWrongNakReceived(ipcp_id)
#endif

/**
  \brief  Event on PPP-IPCP wrong reject received (Op)
  \param  ipcp_id       wrong IPCP identification value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpWrongRejectReceived(uint8_t ipcp_id) {
    EventRecord2 (EvtNetPPP_IpcpWrongRejectReceived, ipcp_id, 0);
  }
#else
  #define EvrNetPPP_IpcpWrongRejectReceived(ipcp_id)
#endif

/**
  \brief  Event on PPP-IPCP advertised IP-address not in subnet (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpWrongSubnet(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetPPP_IpcpWrongSubnet, ip4_addr, 4);
  }
#else
  #define EvrNetPPP_IpcpWrongSubnet(ip4_addr)
#endif

/**
  \brief  Event on PPP-IPCP primary DNS server option rejected (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpPrimaryDnsRejected(void) {
    EventRecord2 (EvtNetPPP_IpcpPrimaryDnsRejected, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpPrimaryDnsRejected()
#endif

/**
  \brief  Event on PPP-IPCP secondary DNS server option rejected (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpSecondaryDnsRejected(void) {
    EventRecord2 (EvtNetPPP_IpcpSecondaryDnsRejected, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpSecondaryDnsRejected()
#endif

/**
  \brief  Event on PPP-IPCP IP-address option rejected (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpIpAddressRejected(void) {
    EventRecord2 (EvtNetPPP_IpcpIpAddressRejected, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpIpAddressRejected()
#endif

/**
  \brief  Event on PPP-IPCP not requested option rejected (Error)
  \param  ipcp_option   rejected option value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpNotRequestedOption(uint8_t ipcp_option) {
    EventRecord2 (EvtNetPPP_IpcpNotRequestedOption, ipcp_option, 0);
  }
#else
  #define EvrNetPPP_IpcpNotRequestedOption(ipcp_option)
#endif

/**
  \brief  Event on PPP-IPCP de-initialize (Op)
  \note   PPP Internet Protocol Control Protocol
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetPPP_IpcpUninit(void) {
    EventRecord2 (EvtNetPPP_IpcpUninit, 0, 0);
  }
#else
  #define EvrNetPPP_IpcpUninit()
#endif


// NetSLIP event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetSLIP_InitInterface            EventID (EventLevelOp,    EvtNetSLIP,  0)
#define EvtNetSLIP_Ip4ConfigError           EventID (EventLevelError, EvtNetSLIP,  1)
#define EvtNetSLIP_ThreadCreateFailed       EventID (EventLevelError, EvtNetSLIP,  2)
#define EvtNetSLIP_GetOptionInvalidParameter EventID (EventLevelError,EvtNetSLIP,  3)
#define EvtNetSLIP_SetOptionInvalidParameter EventID (EventLevelError,EvtNetSLIP,  4)
#define EvtNetSLIP_SetIp4Address            EventID (EventLevelOp,    EvtNetSLIP,  5)
#define EvtNetSLIP_SetIp4PrimaryDNS         EventID (EventLevelOp,    EvtNetSLIP,  6)
#define EvtNetSLIP_SetIp4SecondaryDNS       EventID (EventLevelOp,    EvtNetSLIP,  7)
#define EvtNetSLIP_SetIp4Mtu                EventID (EventLevelOp,    EvtNetSLIP,  8)
#define EvtNetSLIP_Connect                  EventID (EventLevelAPI,   EvtNetSLIP,  9)
#define EvtNetSLIP_Listen                   EventID (EventLevelAPI,   EvtNetSLIP, 10)
#define EvtNetSLIP_Close                    EventID (EventLevelAPI,   EvtNetSLIP, 11)
#define EvtNetSLIP_SendFrame                EventID (EventLevelOp,    EvtNetSLIP, 12)
#define EvtNetSLIP_InvalidIpVersion         EventID (EventLevelError, EvtNetSLIP, 13)
#define EvtNetSLIP_SendDataTooLarge         EventID (EventLevelError, EvtNetSLIP, 14)
#define EvtNetSLIP_NoMemoryError            EventID (EventLevelError, EvtNetSLIP, 15)
#define EvtNetSLIP_QueueAddTransmit         EventID (EventLevelOp,    EvtNetSLIP, 16)
#define EvtNetSLIP_ModemOffline             EventID (EventLevelOp,    EvtNetSLIP, 17)
#define EvtNetSLIP_ReceiveFrame             EventID (EventLevelOp,    EvtNetSLIP, 18)
#define EvtNetSLIP_UninitInterface          EventID (EventLevelOp,    EvtNetSLIP, 19)
#endif

/**
  \brief  Event on SLIP interface initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_InitInterface(void) {
    EventRecord2 (EvtNetSLIP_InitInterface, 0, 0);
  }
#else
  #define EvrNetSLIP_InitInterface()
#endif

/**
  \brief  Event on SLIP interface IPv4 configuration error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_Ip4ConfigError(void) {
    EventRecord2 (EvtNetSLIP_Ip4ConfigError, 0, 0);
  }
#else
  #define EvrNetSLIP_Ip4ConfigError()
#endif

/**
  \brief  Event on SLIP interface thread create failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_ThreadCreateFailed(void) {
    EventRecord2 (EvtNetSLIP_ThreadCreateFailed, 0, 0);
  }
#else
  #define EvrNetSLIP_ThreadCreateFailed()
#endif

/**
  \brief  Event on SLIP get-option invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_GetOptionInvalidParameter(void) {
    EventRecord2 (EvtNetSLIP_GetOptionInvalidParameter, 0, 0);
  }
#else
  #define EvrNetSLIP_GetOptionInvalidParameter()
#endif

/**
  \brief  Event on SLIP set-option invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_SetOptionInvalidParameter(void) {
    EventRecord2 (EvtNetSLIP_SetOptionInvalidParameter, 0, 0);
  }
#else
  #define EvrNetSLIP_SetOptionInvalidParameter()
#endif

/**
  \brief  Event on SLIP set interface IPv4 address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_SetIp4Address(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetSLIP_SetIp4Address, ip4_addr, 4);
  }
#else
  #define EvrNetSLIP_SetIp4Address(ip4_addr)
#endif

/**
  \brief  Event on SLIP set interface primary DNS server (Op)
  \param  ip4_addr      pointer to IPv4 address of a DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_SetIp4PrimaryDNS(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetSLIP_SetIp4PrimaryDNS, ip4_addr, 4);
  }
#else
  #define EvrNetSLIP_SetIp4PrimaryDNS(ip4_addr)
#endif

/**
  \brief  Event on SLIP set interface secondary DNS server (Op)
  \param  ip4_addr      pointer to IPv4 address of a DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_SetIp4SecondaryDNS(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetSLIP_SetIp4SecondaryDNS, ip4_addr, 4);
  }
#else
  #define EvrNetSLIP_SetIp4SecondaryDNS(ip4_addr)
#endif

/**
  \brief  Event on SLIP set interface MTU (Op)
  \param  mtu           maximum transmission unit
                         - 0: fragmentation disabled
  \note   MTU is limited in the range from 296 to 1500 bytes.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_SetIp4Mtu(uint16_t mtu) {
    EventRecord2 (EvtNetSLIP_SetIp4Mtu, mtu, 0);
  }
#else
  #define EvrNetSLIP_SetIp4Mtu(mtu)
#endif

/**
  \brief  Event on SLIP interface \ref netSLIP_Connect start (API)
  \param  dial_number   pointer to a number to dial string
  \param  length        length of the dial_number string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_Connect(const char *dial_number, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData (EvtNetSLIP_Connect, dial_number, length);
  }
#else
  #define EvrNetSLIP_Connect(dial_number, length)
#endif

/**
  \brief  Event on SLIP interface \ref netSLIP_Listen start (API)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_Listen(void) {
    EventRecord2 (EvtNetSLIP_Listen, 0, 0);
  }
#else
  #define EvrNetSLIP_Listen()
#endif

/**
  \brief  Event on SLIP interface \ref netSLIP_Close start (API)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_Close(void) {
    EventRecord2 (EvtNetSLIP_Close, 0, 0);
  }
#else
  #define EvrNetSLIP_Close()
#endif

/**
  \brief  Event on SLIP interface send frame (Op)
  \param  length        frame length in bytes
  \param  ip_version    internet protocol version
                         - 0: IPv4
                         - 1: IPv6
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_SendFrame(uint32_t length, uint8_t ip_version) {
    EventRecord2 (EvtNetSLIP_SendFrame, length, ip_version);
  }
#else
  #define EvrNetSLIP_SendFrame(length, ip_version)
#endif

/**
  \brief  Event on SLIP interface invalid IP version (Error)
  \param  ip_version    invalid IP protocol version value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_InvalidIpVersion(uint8_t ip_version) {
    EventRecord2 (EvtNetSLIP_InvalidIpVersion, ip_version, 0);
  }
#else
  #define EvrNetSLIP_InvalidIpVersion(ip_version)
#endif

/**
  \brief  Event on SLIP interface send payload too large error (Error)
  \param  length        data length in bytes
  \param  max_length    maximum length of data in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_SendDataTooLarge(uint32_t length, uint32_t max_length) {
    EventRecord2 (EvtNetSLIP_SendDataTooLarge, length, max_length);
  }
#else
  #define EvrNetSLIP_SendDataTooLarge(length, max_length)
#endif

/**
  \brief  Event on SLIP interface out of memory error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_NoMemoryError(void) {
    EventRecord2 (EvtNetSLIP_NoMemoryError, 0, 0);
  }
#else
  #define EvrNetSLIP_NoMemoryError()
#endif

/**
  \brief  Event on SLIP interface add frame to transmit queue (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_QueueAddTransmit(void) {
    EventRecord2 (EvtNetSLIP_QueueAddTransmit, 0, 0);
  }
#else
  #define EvrNetSLIP_QueueAddTransmit()
#endif

/**
  \brief  Event on SLIP interface modem offline (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_ModemOffline(void) {
    EventRecord2 (EvtNetSLIP_ModemOffline, 0, 0);
  }
#else
  #define EvrNetSLIP_ModemOffline()
#endif

/**
  \brief  Event on SLIP interface receive frame (Op)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_ReceiveFrame(uint32_t length) {
    EventRecord2 (EvtNetSLIP_ReceiveFrame, length, 0);
  }
#else
  #define EvrNetSLIP_ReceiveFrame(length)
#endif

/**
  \brief  Event on SLIP interface de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSLIP_UninitInterface(void) {
    EventRecord2 (EvtNetSLIP_UninitInterface, 0, 0);
  }
#else
  #define EvrNetSLIP_UninitInterface()
#endif


// NetLOOP event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetLOOP_InitInterface            EventID (EventLevelOp,    EvtNetLOOP, 0)
#define EvtNetLOOP_SendFrame                EventID (EventLevelOp,    EvtNetLOOP, 1)
#define EvtNetLOOP_NoMemoryError            EventID (EventLevelError, EvtNetLOOP, 2)
#define EvtNetLOOP_ReceiveFrame             EventID (EventLevelOp,    EvtNetLOOP, 3)
#define EvtNetLOOP_UninitInterface          EventID (EventLevelOp,    EvtNetLOOP, 4)
#endif

/**
  \brief  Event on Loopback interface initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetLOOP_InitInterface(void) {
    EventRecord2 (EvtNetLOOP_InitInterface, 0, 0);
  }
#else
  #define EvrNetLOOP_InitInterface()
#endif

/**
  \brief  Event on Loopback interface send frame (Op)
  \param  length        frame length in bytes
  \param  ip_version    internet protocol version
                         - 0: IPv4
                         - 1: IPv6
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetLOOP_SendFrame(uint32_t length, uint8_t ip_version) {
    EventRecord2 (EvtNetLOOP_SendFrame, length, ip_version);
  }
#else
  #define EvrNetLOOP_SendFrame(length, ip_version)
#endif

/**
  \brief  Event on Loopback interface out of memory error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetLOOP_NoMemoryError(void) {
    EventRecord2 (EvtNetLOOP_NoMemoryError, 0, 0);
  }
#else
  #define EvrNetLOOP_NoMemoryError()
#endif

/**
  \brief  Event on Loopback interface receive frame (Op)
  \param  length        frame length in bytes
  \param  ip_version    internet protocol version
                         - 0: IPv4
                         - 1: IPv6
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetLOOP_ReceiveFrame(uint32_t length, uint8_t ip_version) {
    EventRecord2 (EvtNetLOOP_ReceiveFrame, length, ip_version);
  }
#else
  #define EvrNetLOOP_ReceiveFrame(length, ip_version)
#endif

/**
  \brief  Event on Loopback interface de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetLOOP_UninitInterface(void) {
    EventRecord2 (EvtNetLOOP_UninitInterface, 0, 0);
  }
#else
  #define EvrNetLOOP_UninitInterface()
#endif


// NetIP4 event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetIP4_InitCore                  EventID (EventLevelOp,    EvtNetIP4,  0)
#define EvtNetIP4_ReceiveFrame              EventID (EventLevelOp,    EvtNetIP4,  1)
#define EvtNetIP4_FrameTooShort             EventID (EventLevelError, EvtNetIP4,  2)
#define EvtNetIP4_InvalidIpVersion          EventID (EventLevelError, EvtNetIP4,  3)
#define EvtNetIP4_ShowFrameHeader           EventID (EventLevelDetail,EvtNetIP4,  4)
#define EvtNetIP4_SourceIpAddressInvalid    EventID (EventLevelError, EvtNetIP4,  5)
#define EvtNetIP4_WrongMulticastProtocol    EventID (EventLevelError, EvtNetIP4,  6)
#define EvtNetIP4_WrongDestinationAddress   EventID (EventLevelOp,    EvtNetIP4,  7)
#define EvtNetIP4_ChecksumFailed            EventID (EventLevelError, EvtNetIP4,  8)
#define EvtNetIP4_FragmentDfFlagSet         EventID (EventLevelError, EvtNetIP4,  9)
#define EvtNetIP4_FragmentationNotEnabled   EventID (EventLevelError, EvtNetIP4, 10)
#define EvtNetIP4_SendFrame                 EventID (EventLevelOp,    EvtNetIP4, 11)
#define EvtNetIP4_ProtocolUnknown           EventID (EventLevelError, EvtNetIP4, 12)
#define EvtNetIP4_DestinationAddressNull    EventID (EventLevelError, EvtNetIP4, 13)
#define EvtNetIP4_UninitCore                EventID (EventLevelOp,    EvtNetIP4, 14)
#endif

/**
  \brief  Event on IPv4 core initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_InitCore(void) {
    EventRecord2 (EvtNetIP4_InitCore, 0, 0);
  }
#else
  #define EvrNetIP4_InitCore()
#endif

/**
  \brief  Event on IPv4 receive frame (Op)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_ReceiveFrame(uint32_t length) {
    EventRecord2 (EvtNetIP4_ReceiveFrame, length, 0);
  }
#else
  #define EvrNetIP4_ReceiveFrame(length)
#endif

/**
  \brief  Event on IPv4 received frame too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetIP4_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetIP4_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on IPv4 invalid protocol version (Error)
  \param  ip_version    not IPv4 protocol version value
                         - 0..3
                         - 5..15
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_InvalidIpVersion(uint8_t ip_version) {
    EventRecord2 (EvtNetIP4_InvalidIpVersion, ip_version, 0);
  }
#else
  #define EvrNetIP4_InvalidIpVersion(ip_version)
#endif

/**
  \brief  Event on IPv4 display frame header (Detail)
  \param  ip4_header    pointer to IPv4 frame header of 20 bytes
                         - VerHlen  (1 byte)
                         - Tos      (1 byte)
                         - Len      (2 bytes)
                         - Id       (2 bytes)
                         - FragOffs (2 bytes)
                         - Ttl      (1 byte)
                         - Proto    (1 byte)
                         - Chksum   (2 bytes)
                         - SrcAddr  (4 bytes)
                         - DstAddr  (4 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_ShowFrameHeader(const void *ip4_header) {
    EventRecordData (EvtNetIP4_ShowFrameHeader, ip4_header, 20);
  }
#else
  #define EvrNetIP4_ShowFrameHeader(ip4_header)
#endif

/**
  \brief  Event on IPv4 source IP address invalid (Error)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_SourceIpAddressInvalid(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIP4_SourceIpAddressInvalid, ip4_addr, 4);
  }
#else
  #define EvrNetIP4_SourceIpAddressInvalid(ip4_addr)
#endif

/**
  \brief  Event on IPv4 wrong multicast protocol type (Error)
  \param  ip4_protocol  wrong protocol type value
  \note   Only UDP and IGMP multicast frames are allowed!
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_WrongMulticastProtocol(uint8_t ip4_protocol) {
    EventRecord2 (EvtNetIP4_WrongMulticastProtocol, ip4_protocol, 0);
  }
#else
  #define EvrNetIP4_WrongMulticastProtocol(ip4_protocol)
#endif

/**
  \brief  Event on IPv4 wrong destination address (Op)
  \param  ip4_addr      pointer to IPv4 destination address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_WrongDestinationAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIP4_WrongDestinationAddress, ip4_addr, 4);
  }
#else
  #define EvrNetIP4_WrongDestinationAddress(ip4_addr)
#endif

/**
  \brief  Event on IP4 frame checksum check failed (Error)
  \param  length        header length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_ChecksumFailed(uint32_t length) {
    EventRecord2 (EvtNetIP4_ChecksumFailed, length, 0);
  }
#else
  #define EvrNetIP4_ChecksumFailed(length)
#endif

/**
  \brief  Event on IP4 fragmented frame received wit DF-flag set (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_FragmentDfFlagSet(void) {
    EventRecord2 (EvtNetIP4_FragmentDfFlagSet, 0, 0);
  }
#else
  #define EvrNetIP4_FragmentDfFlagSet()
#endif

/**
  \brief  Event on IP4 fragmented frame received (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_FragmentationNotEnabled(void) {
    EventRecord2 (EvtNetIP4_FragmentationNotEnabled, 0, 0);
  }
#else
  #define EvrNetIP4_FragmentationNotEnabled()
#endif

/**
  \brief  Event on IP4 send frame (Op)
  \param  ip4_protocol  IPv4 protocol type
                         - 1:  ICMP
                         - 2:  IGMP
                         - 6:  TCP
                         - 17: UDP
  \param  length        frame length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_SendFrame(uint8_t ip4_protocol, uint32_t length) {
    EventRecord2 (EvtNetIP4_SendFrame, ip4_protocol, length);
  }
#else
  #define EvrNetIP4_SendFrame(ip4_protocol, length)
#endif

/**
  \brief  Event on IP4 unknown protocol type in sending frame (Error)
  \param  ip4_protocol  unknown IPv4 protocol type value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_ProtocolUnknown(uint8_t ip4_protocol) {
    EventRecord2 (EvtNetIP4_ProtocolUnknown, ip4_protocol, 0);
  }
#else
  #define EvrNetIP4_ProtocolUnknown(ip4_protocol)
#endif

/**
  \brief  Event on IP4 destination address not provided (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_DestinationAddressNull(void) {
    EventRecord2 (EvtNetIP4_DestinationAddressNull, 0, 0);
  }
#else
  #define EvrNetIP4_DestinationAddressNull()
#endif

/**
  \brief  Event on IPv4 core de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP4_UninitCore(void) {
    EventRecord2 (EvtNetIP4_UninitCore, 0, 0);
  }
#else
  #define EvrNetIP4_UninitCore()
#endif


// NetICMP event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
/* ICMP-Core events */
#define EvtNetICMP_ReceiveFrame             EventID (EventLevelOp,    EvtNetICMP,  0)
#define EvtNetICMP_FrameTooShort            EventID (EventLevelError, EvtNetICMP,  1)
#define EvtNetICMP_ChecksumFailed           EventID (EventLevelError, EvtNetICMP,  2)
#define EvtNetICMP_EchoRequestWrongCode     EventID (EventLevelError, EvtNetICMP,  3)
#define EvtNetICMP_EchoRequestReceived      EventID (EventLevelOp,    EvtNetICMP,  4)
#define EvtNetICMP_SendEchoReply            EventID (EventLevelOp,    EvtNetICMP,  5)
#define EvtNetICMP_EchoReplyReceived        EventID (EventLevelOp,    EvtNetICMP,  6)
#define EvtNetICMP_EchoReplyWrongState      EventID (EventLevelError, EvtNetICMP,  7)
#define EvtNetICMP_EchoReplyWrongCode       EventID (EventLevelError, EvtNetICMP,  8)
#define EvtNetICMP_EchoReplyWrongIpAddress  EventID (EventLevelError, EvtNetICMP,  9)
#define EvtNetICMP_EchoReplyWrongId         EventID (EventLevelError, EvtNetICMP, 10)
#define EvtNetICMP_EchoReplyWrongPayload    EventID (EventLevelError, EvtNetICMP, 11)
#define EvtNetICMP_MessageTypeUnknown       EventID (EventLevelOp,    EvtNetICMP, 12)
#define EvtNetICMP_SendEchoRequest          EventID (EventLevelOp,    EvtNetICMP, 13)
/* ICMP-Ping client events */
#define EvtNetICMP_PingInit                 EventID (EventLevelOp,    EvtNetICMP, 14)
#define EvtNetICMP_PingEcho                 EventID (EventLevelAPI,   EvtNetICMP, 15)
#define EvtNetICMP_PingTargetNotValid       EventID (EventLevelError, EvtNetICMP, 16)
#define EvtNetICMP_PingDnsError             EventID (EventLevelError, EvtNetICMP, 17)
#define EvtNetICMP_PingInvalidParameter     EventID (EventLevelError, EvtNetICMP, 18)
#define EvtNetICMP_PingClientBusy           EventID (EventLevelError, EvtNetICMP, 19)
#define EvtNetICMP_PingSendRequest          EventID (EventLevelOp,    EvtNetICMP, 20)
#define EvtNetICMP_PingRetransmitRequest    EventID (EventLevelOp,    EvtNetICMP, 21)
#define EvtNetICMP_PingTimeout              EventID (EventLevelOp,    EvtNetICMP, 22)
#define EvtNetICMP_PingUninit               EventID (EventLevelOp,    EvtNetICMP, 23)
#endif

/**
  \brief  Event on ICMP receive frame (Op)
  \param  icmp_header   pointer to ICMP frame header of 8 bytes
                         - Type    (1 byte)
                         - Code    (1 byte)
                         - Chksum  (2 bytes)
                         - EchoId  (2 bytes)
                         - EchoSeq (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_ReceiveFrame(const void *icmp_header) {
    EventRecordData (EvtNetICMP_ReceiveFrame, icmp_header, 8);
  }
#else
  #define EvrNetICMP_ReceiveFrame(icmp_header)
#endif

/**
  \brief  Event on ICMP received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetICMP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetICMP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on ICMP frame checksum check failed (Error)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_ChecksumFailed(uint32_t length) {
    EventRecord2 (EvtNetICMP_ChecksumFailed, length, 0);
  }
#else
  #define EvrNetICMP_ChecksumFailed(length)
#endif

/**
  \brief  Event on ICMP wrong code in echo request received (Error)
  \param  icmp_code     wrong ICMP code value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_EchoRequestWrongCode(uint8_t icmp_code) {
    EventRecord2 (EvtNetICMP_EchoRequestWrongCode, icmp_code, 0);
  }
#else
  #define EvrNetICMP_EchoRequestWrongCode(icmp_code)
#endif

/**
  \brief  Event on ICMP echo request received (Op)
  \param  data_length   length of the payload data
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_EchoRequestReceived(uint32_t data_length) {
    EventRecord2 (EvtNetICMP_EchoRequestReceived, data_length, 0);
  }
#else
  #define EvrNetICMP_EchoRequestReceived(data_length)
#endif

/**
  \brief  Event on ICMP send echo reply (Op)
  \param  icmp_header   pointer to ICMP frame header of 8 bytes
                         - Type    (1 byte)
                         - Code    (1 byte)
                         - Chksum  (2 bytes)
                         - EchoId  (2 bytes)
                         - EchoSeq (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_SendEchoReply(const void *icmp_header) {
    EventRecordData (EvtNetICMP_SendEchoReply, icmp_header, 8);
  }
#else
  #define EvrNetICMP_SendEchoReply(icmp_header)
#endif

/**
  \brief  Event on ICMP echo reply received (Op)
  \param  data_length   length of the payload data
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_EchoReplyReceived(uint32_t data_length) {
    EventRecord2 (EvtNetICMP_EchoReplyReceived, data_length, 0);
  }
#else
  #define EvrNetICMP_EchoReplyReceived(data_length)
#endif

/**
  \brief  Event on ICMP echo reply received in wrong ping process state (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_EchoReplyWrongState(void) {
    EventRecord2 (EvtNetICMP_EchoReplyWrongState, 0, 0);
  }
#else
  #define EvrNetICMP_EchoReplyWrongState()
#endif

/**
  \brief  Event on ICMP wrong code in echo reply received (Error)
  \param  icmp_code     wrong ICMP code value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_EchoReplyWrongCode(uint8_t icmp_code) {
    EventRecord2 (EvtNetICMP_EchoReplyWrongCode, icmp_code, 0);
  }
#else
  #define EvrNetICMP_EchoReplyWrongCode(icmp_code)
#endif

/**
  \brief  Event on ICMP wrong source IPv4 address of echo reply (Error)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_EchoReplyWrongIpAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetICMP_EchoReplyWrongIpAddress, ip4_addr, 4);
  }
#else
  #define EvrNetICMP_EchoReplyWrongIpAddress(ip4_addr)
#endif

/**
  \brief  Event on ICMP wrong echo identifier value in echo reply (Error)
  \param  echo_id       wrong echo identifier value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_EchoReplyWrongId(uint16_t echo_id) {
    EventRecord2 (EvtNetICMP_EchoReplyWrongId, echo_id, 0);
  }
#else
  #define EvrNetICMP_EchoReplyWrongId(echo_id)
#endif

/**
  \brief  Event on ICMP echo reply payload data invalid (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_EchoReplyWrongPayload(void) {
    EventRecord2 (EvtNetICMP_EchoReplyWrongPayload, 0, 0);
  }
#else
  #define EvrNetICMP_EchoReplyWrongPayload()
#endif

/**
  \brief  Event on ICMP message type unknown (Op)
  \param  icmp_type     wrong ICMP message type value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_MessageTypeUnknown(uint8_t icmp_type) {
    EventRecord2 (EvtNetICMP_MessageTypeUnknown, icmp_type, 0);
  }
#else
  #define EvrNetICMP_MessageTypeUnknown(icmp_type)
#endif

/**
  \brief  Event on ICMP send echo request (Op)
  \param  icmp_header   pointer to ICMP frame header of 8 bytes
                         - Type    (1 byte)
                         - Code    (1 byte)
                         - Chksum  (2 bytes)
                         - EchoId  (2 bytes)
                         - EchoSeq (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_SendEchoRequest(const void *icmp_header) {
    EventRecordData (EvtNetICMP_SendEchoRequest, icmp_header, 8);
  }
#else
  #define EvrNetICMP_SendEchoRequest(icmp_header)
#endif

// ICMP ping client event recorder functions -----------------------------------

/**
  \brief  Event on ICMP ping client initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingInit(void) {
    EventRecord2 (EvtNetICMP_PingInit, 0, 0);
  }
#else
  #define EvrNetICMP_PingInit()
#endif

/**
  \brief  Event on ICMP \ref netPing_Echo request (API)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingEcho(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetICMP_PingEcho, ip4_addr, 4);
  }
#else
  #define EvrNetICMP_PingEcho(ip4_addr)
#endif

/**
  \brief  Event on \ref netPing_EchoX target name not valid (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingTargetNotValid(void) {
    EventRecord2 (EvtNetICMP_PingTargetNotValid, 0, 0);
  }
#else
  #define EvrNetICMP_PingTargetNotValid()
#endif

/**
  \brief  Event on \ref netPing_EchoX host name resolver error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingDnsError(void) {
    EventRecord2 (EvtNetICMP_PingDnsError, 0, 0);
  }
#else
  #define EvrNetICMP_PingDnsError()
#endif

/**
  \brief  Event on ICMP \ref netPing_Echo invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingInvalidParameter(void) {
    EventRecord2 (EvtNetICMP_PingInvalidParameter, 0, 0);
  }
#else
  #define EvrNetICMP_PingInvalidParameter()
#endif

/**
  \brief  Event on ICMP \ref netPing_Echo client busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingClientBusy(void) {
    EventRecord2 (EvtNetICMP_PingClientBusy, 0, 0);
  }
#else
  #define EvrNetICMP_PingClientBusy()
#endif

/**
  \brief  Event on ICMP \ref netPing_Echo send echo request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingSendRequest(void) {
    EventRecord2 (EvtNetICMP_PingSendRequest, 0, 0);
  }
#else
  #define EvrNetICMP_PingSendRequest()
#endif

/**
  \brief  Event on ICMP retransmit echo request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingRetransmitRequest(void) {
    EventRecord2 (EvtNetICMP_PingRetransmitRequest, 0, 0);
  }
#else
  #define EvrNetICMP_PingRetransmitRequest()
#endif

/**
  \brief  Event on ICMP timeout on ping send echo request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingTimeout(void) {
    EventRecord2 (EvtNetICMP_PingTimeout, 0, 0);
  }
#else
  #define EvrNetICMP_PingTimeout()
#endif

/**
  \brief  Event on ICMP ping client de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP_PingUninit(void) {
    EventRecord2 (EvtNetICMP_PingUninit, 0, 0);
  }
#else
  #define EvrNetICMP_PingUninit()
#endif


// NetIGMP event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetIGMP_InitManager              EventID (EventLevelOp,    EvtNetIGMP,  0)
#define EvtNetIGMP_Join                     EventID (EventLevelAPI,   EvtNetIGMP,  1)
#define EvtNetIGMP_AlreadyInGroup           EventID (EventLevelOp,    EvtNetIGMP,  2)
#define EvtNetIGMP_NoFreeEntries            EventID (EventLevelError, EvtNetIGMP,  3)
#define EvtNetIGMP_SendReport               EventID (EventLevelOp,    EvtNetIGMP,  4)
#define EvtNetIGMP_Leave                    EventID (EventLevelAPI,   EvtNetIGMP,  5)
#define EvtNetIGMP_NotInGroup               EventID (EventLevelError, EvtNetIGMP,  6)
#define EvtNetIGMP_SendLeave                EventID (EventLevelOp,    EvtNetIGMP,  7)
#define EvtNetIGMP_ReceiveFrame             EventID (EventLevelOp,    EvtNetIGMP,  8)
#define EvtNetIGMP_FrameTooShort            EventID (EventLevelError, EvtNetIGMP,  9)
#define EvtNetIGMP_ChecksumFailed           EventID (EventLevelError, EvtNetIGMP, 10)
#define EvtNetIGMP_GroupSpecificQuery       EventID (EventLevelOp,    EvtNetIGMP, 11)
#define EvtNetIGMP_DestAddressWrong         EventID (EventLevelError, EvtNetIGMP, 12)
#define EvtNetIGMP_DelayedReportScheduled   EventID (EventLevelOp,    EvtNetIGMP, 13)
#define EvtNetIGMP_GeneralQuery             EventID (EventLevelOp,    EvtNetIGMP, 14)
#define EvtNetIGMP_StartModeIGMPv1          EventID (EventLevelOp,    EvtNetIGMP, 15)
#define EvtNetIGMP_MaxTimeForReport         EventID (EventLevelOp,    EvtNetIGMP, 16)
#define EvtNetIGMP_GroupReportsScheduled    EventID (EventLevelOp,    EvtNetIGMP, 17)
#define EvtNetIGMP_ReportReceived           EventID (EventLevelOp,    EvtNetIGMP, 18)
#define EvtNetIGMP_OwnReportCanceled        EventID (EventLevelOp,    EvtNetIGMP, 19)
#define EvtNetIGMP_StartModeIGMPv2          EventID (EventLevelOp,    EvtNetIGMP, 20)
#define EvtNetIGMP_SendDelayedReport        EventID (EventLevelOp,    EvtNetIGMP, 21)
#define EvtNetIGMP_UninitManager            EventID (EventLevelOp,    EvtNetIGMP, 22)
#endif

/**
  \brief  Event on IGMP manager initialize (Op)
  \param  num_entries   number of entries available in membership table
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_InitManager(uint32_t num_entries) {
    EventRecord2 (EvtNetIGMP_InitManager, num_entries, 0);
  }
#else
  #define EvrNetIGMP_InitManager(num_entries)
#endif

/**
  \brief  Event on IGMP \ref netIGMP_Join start (API)
  \param  ip4_addr      pointer to IPv4 address of a group
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_Join(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_Join, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_Join(ip4_addr)
#endif

/**
  \brief  Event on IGMP \ref netIGMP_Join already a member (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_AlreadyInGroup(void) {
    EventRecord2 (EvtNetIGMP_AlreadyInGroup, 0, 0);
  }
#else
  #define EvrNetIGMP_AlreadyInGroup()
#endif

/**
  \brief  Event on IGMP \ref netIGMP_Join no free entries available (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_NoFreeEntries(void) {
    EventRecord2 (EvtNetIGMP_NoFreeEntries, 0, 0);
  }
#else
  #define EvrNetIGMP_NoFreeEntries()
#endif

/**
  \brief  Event on IGMP send report message (Op)
  \param  ip4_addr      pointer to IPv4 address of a group
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_SendReport(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_SendReport, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_SendReport(ip4_addr)
#endif

/**
  \brief  Event on IGMP \ref netIGMP_Leave start (API)
  \param  ip4_addr      pointer to IPv4 address of a group
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_Leave(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_Leave, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_Leave(ip4_addr)
#endif

/**
  \brief  Event on IGMP \ref netIGMP_Leave not a member (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_NotInGroup(void) {
    EventRecord2 (EvtNetIGMP_NotInGroup, 0, 0);
  }
#else
  #define EvrNetIGMP_NotInGroup()
#endif

/**
  \brief  Event on IGMP send leave message (Op)
  \param  ip4_addr      pointer to IPv4 address of a group
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_SendLeave(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_SendLeave, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_SendLeave(ip4_addr)
#endif

/**
  \brief  Event on IGMP receive frame (Op)
  \param  message_type  type of received message
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_ReceiveFrame(uint8_t message_type) {
    EventRecord2 (EvtNetIGMP_ReceiveFrame, message_type, 0);
  }
#else
  #define EvrNetIGMP_ReceiveFrame(message_type)
#endif

/**
  \brief  Event on IGMP received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetIGMP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetIGMP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on IGMP frame checksum check failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_ChecksumFailed(void) {
    EventRecord2 (EvtNetIGMP_ChecksumFailed, 0, 0);
  }
#else
  #define EvrNetIGMP_ChecksumFailed()
#endif

/**
  \brief  Event on IGMPv2 group specific query (Op)
  \param  ip4_addr      pointer to IPv4 address of a group
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_GroupSpecificQuery(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_GroupSpecificQuery, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_GroupSpecificQuery(ip4_addr)
#endif

/**
  \brief  Event on IGMPv2 destination address wrong (Error)
  \param  ip4_addr      pointer to IPv4 address of a group
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_DestAddressWrong(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_DestAddressWrong, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_DestAddressWrong(ip4_addr)
#endif

/**
  \brief  Event on IGMPv2 report message delayed (Op)
  \param  max_time      maximum delay time in 100 ms ticks
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_DelayedReportScheduled(uint8_t max_time) {
    EventRecord2 (EvtNetIGMP_DelayedReportScheduled, max_time, 0);
  }
#else
  #define EvrNetIGMP_DelayedReportScheduled(max_time)
#endif

/**
  \brief  Event on IGMPv1 general query (Op)
  \param  ip4_addr      pointer to IPv4 address of a group
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_GeneralQuery(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_GeneralQuery, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_GeneralQuery(ip4_addr)
#endif

/**
  \brief  Event on start IGMPv1 mode (Op)
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_StartModeIGMPv1(void) {
    EventRecord2 (EvtNetIGMP_StartModeIGMPv1, 0, 0);
  }
#else
  #define EvrNetIGMP_StartModeIGMPv1()
#endif

/**
  \brief  Event on IGMPv2 maximum time for report messages (Op)
  \param  max_time      maximum delay time in 100 ms ticks
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_MaxTimeForReport(uint8_t max_time) {
    EventRecord2 (EvtNetIGMP_MaxTimeForReport, max_time, 0);
  }
#else
  #define EvrNetIGMP_MaxTimeForReport(max_time)
#endif

/**
  \brief  Event on IGMP scheduled reports for all active groups (Op)
  \param  n_reports     number of scheduled reports
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_GroupReportsScheduled(uint32_t n_reports) {
    EventRecord2 (EvtNetIGMP_GroupReportsScheduled, n_reports, 0);
  }
#else
  #define EvrNetIGMP_GroupReportsScheduled(n_reports)
#endif

/**
  \brief  Event on IGMP received report message (Op)
  \param  ip4_addr      pointer to IPv4 address of a group
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_ReportReceived(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_ReportReceived, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_ReportReceived(ip4_addr)
#endif

/**
  \brief  Event on own report canceled (Op)
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_OwnReportCanceled(void) {
    EventRecord2 (EvtNetIGMP_OwnReportCanceled, 0, 0);
  }
#else
  #define EvrNetIGMP_OwnReportCanceled()
#endif

/**
  \brief  Event on start IGMPv2 mode (Op)
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_StartModeIGMPv2(void) {
    EventRecord2 (EvtNetIGMP_StartModeIGMPv2, 0, 0);
  }
#else
  #define EvrNetIGMP_StartModeIGMPv2()
#endif

/**
  \brief  Event on IGMP send delayed report message (Op)
  \param  ip4_addr      pointer to IPv4 address of a group
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_SendDelayedReport(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetIGMP_SendDelayedReport, ip4_addr, 4);
  }
#else
  #define EvrNetIGMP_SendDelayedReport(ip4_addr)
#endif

/**
  \brief  Event on IGMP manager de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIGMP_UninitManager(void) {
    EventRecord2 (EvtNetIGMP_UninitManager, 0, 0);
  }
#else
  #define EvrNetIGMP_UninitManager()
#endif


// NetNBNS event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetNBNS_InitService              EventID (EventLevelOp,    EvtNetNBNS, 0)
#define EvtNetNBNS_GetSocketFailed          EventID (EventLevelError, EvtNetNBNS, 1)
#define EvtNetNBNS_ReceiveFrame             EventID (EventLevelOp,    EvtNetNBNS, 2)
#define EvtNetNBNS_WrongPort                EventID (EventLevelError, EvtNetNBNS, 3)
#define EvtNetNBNS_FrameTooShort            EventID (EventLevelError, EvtNetNBNS, 4)
#define EvtNetNBNS_NameQueryRequest         EventID (EventLevelOp,    EvtNetNBNS, 5)
#define EvtNetNBNS_QueryFromAddress         EventID (EventLevelOp,    EvtNetNBNS, 6)
#define EvtNetNBNS_NameQueryResponse        EventID (EventLevelOp,    EvtNetNBNS, 7)
#define EvtNetNBNS_UninitService            EventID (EventLevelOp,    EvtNetNBNS, 8)
#endif

/**
  \brief  Event on NBNS name service initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_InitService(void) {
    EventRecord2 (EvtNetNBNS_InitService, 0, 0);
  }
#else
  #define EvrNetNBNS_InitService()
#endif

/**
  \brief  Event on NBNS failed to allocate UDP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_GetSocketFailed(void) {
    EventRecord2 (EvtNetNBNS_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetNBNS_GetSocketFailed()
#endif

/**
  \brief  Event on NBNS receive frame (Op)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_ReceiveFrame(uint32_t length) {
    EventRecord2 (EvtNetNBNS_ReceiveFrame, length, 0);
  }
#else
  #define EvrNetNBNS_ReceiveFrame(length)
#endif

/**
  \brief  Event on NBNS wrong UDP source port (Error)
  \param  udp_port      wrong port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_WrongPort(uint16_t udp_port) {
    EventRecord2 (EvtNetNBNS_WrongPort, udp_port, 0);
  }
#else
  #define EvrNetNBNS_WrongPort(udp_port)
#endif

/**
  \brief  Event on NBNS received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetNBNS_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetNBNS_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on NBNS name query request received (Op)
  \param  q_name        pointer to a question name string
  \param  length        length of the question name string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_NameQueryRequest(const char *q_name, uint32_t length) {
    if (length > 16) length = 16;
    EventRecordData (EvtNetNBNS_NameQueryRequest, q_name, length);
  }
#else
  #define EvrNetNBNS_NameQueryRequest(q_name, length)
#endif

/**
  \brief  Event on NBNS name query from IP-address (Op)
  \param  ip4_addr      pointer to IPv4 address of a host
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_QueryFromAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetNBNS_QueryFromAddress, ip4_addr, 4);
  }
#else
  #define EvrNetNBNS_QueryFromAddress(ip4_addr)
#endif

/**
  \brief  Event on NBNS response to a name query (Op)
  \param  ip4_addr      pointer to local IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_NameQueryResponse(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetNBNS_NameQueryResponse, ip4_addr, 4);
  }
#else
  #define EvrNetNBNS_NameQueryResponse(ip4_addr)
#endif

/**
  \brief  Event on NBNS name service de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNBNS_UninitService(void) {
    EventRecord2 (EvtNetNBNS_UninitService, 0, 0);
  }
#else
  #define EvrNetNBNS_UninitService()
#endif


// NetDHCP event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetDHCP_InitClient               EventID (EventLevelOp,    EvtNetDHCP,  0)
#define EvtNetDHCP_GetSocketFailed          EventID (EventLevelError, EvtNetDHCP,  1)
#define EvtNetDHCP_StartClient              EventID (EventLevelOp,    EvtNetDHCP,  2)
#define EvtNetDHCP_StopClient               EventID (EventLevelOp,    EvtNetDHCP,  3)
#define EvtNetDHCP_ClientState              EventID (EventLevelOp,    EvtNetDHCP,  4)
#define EvtNetDHCP_NextState                EventID (EventLevelOp,    EvtNetDHCP,  5)
#define EvtNetDHCP_StateRetransmit          EventID (EventLevelOp,    EvtNetDHCP,  6)
#define EvtNetDHCP_ChangeStateOnTimeout     EventID (EventLevelOp,    EvtNetDHCP,  7)
#define EvtNetDHCP_AutoIpAddressProbe       EventID (EventLevelOp,    EvtNetDHCP,  8)
#define EvtNetDHCP_AutoIpSuccess            EventID (EventLevelOp,    EvtNetDHCP,  9)
#define EvtNetDHCP_T2Expired                EventID (EventLevelOp,    EvtNetDHCP, 10)
#define EvtNetDHCP_LeaseExpired             EventID (EventLevelError, EvtNetDHCP, 11)
#define EvtNetDHCP_SendDhcpMessage          EventID (EventLevelOp,    EvtNetDHCP, 12)
#define EvtNetDHCP_ReceiveFrame             EventID (EventLevelOp,    EvtNetDHCP, 13)
#define EvtNetDHCP_WrongServerPort          EventID (EventLevelError, EvtNetDHCP, 14)
#define EvtNetDHCP_FrameTooShort            EventID (EventLevelError, EvtNetDHCP, 15)
#define EvtNetDHCP_MisformedReply           EventID (EventLevelError, EvtNetDHCP, 16)
#define EvtNetDHCP_WrongTransactionId       EventID (EventLevelError, EvtNetDHCP, 17)
#define EvtNetDHCP_WrongClientHwAddress     EventID (EventLevelError, EvtNetDHCP, 18)
#define EvtNetDHCP_WrongMagicCookie         EventID (EventLevelError, EvtNetDHCP, 19)
#define EvtNetDHCP_NotDhcpOfferMessage      EventID (EventLevelOp,    EvtNetDHCP, 20)
#define EvtNetDHCP_DhcpOfferReceived        EventID (EventLevelOp,    EvtNetDHCP, 21)
#define EvtNetDHCP_FileOverloadedOptions    EventID (EventLevelOp,    EvtNetDHCP, 22)
#define EvtNetDHCP_SnameOverloadedOptions   EventID (EventLevelOp,    EvtNetDHCP, 23)
#define EvtNetDHCP_DhcpOfferServerIdMissing EventID (EventLevelOp,    EvtNetDHCP, 24)
#define EvtNetDHCP_ForwardedMessage         EventID (EventLevelOp,    EvtNetDHCP, 25)
#define EvtNetDHCP_RelayAgentAddressInvalid EventID (EventLevelError, EvtNetDHCP, 26)
#define EvtNetDHCP_ShowRelayAgentAddress    EventID (EventLevelOp,    EvtNetDHCP, 27)
#define EvtNetDHCP_ShowOfferedIpAddress     EventID (EventLevelOp,    EvtNetDHCP, 28)
#define EvtNetDHCP_ServerAddressNotSelected EventID (EventLevelOp,    EvtNetDHCP, 29)
#define EvtNetDHCP_AssignedAddrNotRequested EventID (EventLevelOp,    EvtNetDHCP, 30)
#define EvtNetDHCP_NotDhcpAckMessage        EventID (EventLevelOp,    EvtNetDHCP, 31)
#define EvtNetDHCP_DhcpAckReceived          EventID (EventLevelOp,    EvtNetDHCP, 32)
#define EvtNetDHCP_ShowAssignedAddress      EventID (EventLevelOp,    EvtNetDHCP, 33)
#define EvtNetDHCP_DhcpNakReceived          EventID (EventLevelOp,    EvtNetDHCP, 34)
#define EvtNetDHCP_ShowServerId             EventID (EventLevelOp,    EvtNetDHCP, 35)
#define EvtNetDHCP_ShowNetMask              EventID (EventLevelOp,    EvtNetDHCP, 36)
#define EvtNetDHCP_ShowGatewayAddress       EventID (EventLevelOp,    EvtNetDHCP, 37)
#define EvtNetDHCP_ShowDnsServers           EventID (EventLevelOp,    EvtNetDHCP, 38)
#define EvtNetDHCP_ShowLeaseTime            EventID (EventLevelOp,    EvtNetDHCP, 39)
#define EvtNetDHCP_ShowTimeT1               EventID (EventLevelOp,    EvtNetDHCP, 40)
#define EvtNetDHCP_ShowTimeT2               EventID (EventLevelOp,    EvtNetDHCP, 41)
#define EvtNetDHCP_ShowBootfileName         EventID (EventLevelOp,    EvtNetDHCP, 42)
#define EvtNetDHCP_ShowNtpServerList        EventID (EventLevelOp,    EvtNetDHCP, 43)
#define EvtNetDHCP_SetOptionInvalidParam    EventID (EventLevelError, EvtNetDHCP, 44)
#define EvtNetDHCP_SetOptionInvalidState    EventID (EventLevelError, EvtNetDHCP, 45)
#define EvtNetDHCP_SetOptionClientIdDefault EventID (EventLevelAPI,   EvtNetDHCP, 46)
#define EvtNetDHCP_SetOptionClientId        EventID (EventLevelAPI,   EvtNetDHCP, 47)
#define EvtNetDHCP_UninitClient             EventID (EventLevelOp,    EvtNetDHCP, 48)
#endif

/**
  \brief  Event on DHCP client initialize (Op)
  \param  vcid          Vendor Class Identifier option flag
                         - 0:  not enabled
                         - >0: enabled
  \param  option_flags  DHCP client option flags
                         - 0x01: Opt.67 Bootfile name
                         - 0x02: Opt.42 NTP servers
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_InitClient(uint8_t vcid, uint8_t option_flags) {
    EventRecord2 (EvtNetDHCP_InitClient, vcid, option_flags);
  }
#else
  #define EvrNetDHCP_InitClient(vcid, option_flags)
#endif

/**
  \brief  Event on DHCP client failed to allocate UDP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_GetSocketFailed(void) {
    EventRecord2 (EvtNetDHCP_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetDHCP_GetSocketFailed()
#endif

/**
  \brief  Event on DHCP client start (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_StartClient(void) {
    EventRecord2 (EvtNetDHCP_StartClient, 0, 0);
  }
#else
  #define EvrNetDHCP_StartClient()
#endif

/**
  \brief  Event on DHCP client stop (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_StopClient(void) {
    EventRecord2 (EvtNetDHCP_StopClient, 0, 0);
  }
#else
  #define EvrNetDHCP_StopClient()
#endif

/**
  \brief  Event on DHCP client indication of client state (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ClientState(uint32_t state) {
    EventRecord2 (EvtNetDHCP_ClientState, state, 0);
  }
#else
  #define EvrNetDHCP_ClientState(state)
#endif

/**
  \brief  Event on DHCP client transition to next state (Op)
  \param  state         next state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_NextState(uint32_t state) {
    EventRecord2 (EvtNetDHCP_NextState, state, 0);
  }
#else
  #define EvrNetDHCP_NextState(state)
#endif

/**
  \brief  Event on DHCP client retransmit request in current state (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_StateRetransmit(uint32_t state) {
    EventRecord2 (EvtNetDHCP_StateRetransmit, state, 0);
  }
#else
  #define EvrNetDHCP_StateRetransmit(state)
#endif

/**
  \brief  Event on DHCP client state timeout and transit to next state (Op)
  \param  current       current state
  \param  next          next state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ChangeStateOnTimeout(uint32_t current, uint32_t next) {
    EventRecord2 (EvtNetDHCP_ChangeStateOnTimeout, current, next);
  }
#else
  #define EvrNetDHCP_ChangeStateOnTimeout(current, next)
#endif

/**
  \brief  Event on DHCP client address probe for AutoIP mode (Op)
  \param  ip4_addr      pointer to IP address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_AutoIpAddressProbe(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_AutoIpAddressProbe, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_AutoIpAddressProbe(ip4_addr)
#endif

/**
  \brief  Event on DHCP client assign AutoIP address success (Op)
  \param  ip4_addr      pointer to assigned AutoIP address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_AutoIpSuccess(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_AutoIpSuccess, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_AutoIpSuccess(ip4_addr)
#endif

/**
  \brief  Event on DHCP client timer T2 expired (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_T2Expired(uint32_t state) {
    EventRecord2 (EvtNetDHCP_T2Expired, state, 0);
  }
#else
  #define EvrNetDHCP_T2Expired(state)
#endif

/**
  \brief  Event on DHCP client address lease time expired (Error)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_LeaseExpired(uint32_t state) {
    EventRecord2 (EvtNetDHCP_LeaseExpired, state, 0);
  }
#else
  #define EvrNetDHCP_LeaseExpired(state)
#endif

/**
  \brief  Event on DHCP client send message (Op)
  \param  msg_type      message type
  \param  broadcast     broadcast the message
                         - 0: no
                         - 1: yes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_SendDhcpMessage(uint8_t msg_type, uint8_t broadcast) {
    EventRecord2 (EvtNetDHCP_SendDhcpMessage, msg_type, broadcast);
  }
#else
  #define EvrNetDHCP_SendDhcpMessage(msg_type, broadcast)
#endif

/**
  \brief  Event on DHCP client receive frame (Op)
  \param  ip4_addr      pointer to IPv4 address of the server
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ReceiveFrame(const uint8_t *ip4_addr, uint32_t length) {
    uint32_t tmp_addr;
    memcpy (&tmp_addr, ip4_addr, 4);
    EventRecord2 (EvtNetDHCP_ReceiveFrame, tmp_addr, length);
  }
#else
  #define EvrNetDHCP_ReceiveFrame(ip4_addr, length)
#endif

/**
  \brief  Event on DHCP client wrong server port (Error)
  \param  udp_port      wrong UDP port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_WrongServerPort(uint16_t udp_port) {
    EventRecord2 (EvtNetDHCP_WrongServerPort, udp_port, 0);
  }
#else
  #define EvrNetDHCP_WrongServerPort(udp_port)
#endif

/**
  \brief  Event on DHCP client received frame too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetDHCP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetDHCP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on DHCP client received misformed reply (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_MisformedReply(void) {
    EventRecord2 (EvtNetDHCP_MisformedReply, 0, 0);
  }
#else
  #define EvrNetDHCP_MisformedReply()
#endif

/**
  \brief  Event on DHCP client wrong transaction identifier (XID) received (Error)
  \param  xid           transaction identifier
  \param  xid_valid     expected/valid transaction identifier
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_WrongTransactionId(uint32_t xid, uint32_t xid_valid) {
    EventRecord2 (EvtNetDHCP_WrongTransactionId, xid, xid_valid);
  }
#else
  #define EvrNetDHCP_WrongTransactionId(xid, xid_valid)
#endif

/**
  \brief  Event on DHCP client wrong Client Hardware Address (CHADDR) received (Error)
  \param  chaddr        pointer to received chaddr address (MAC)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_WrongClientHwAddress(const uint8_t *chaddr) {
    EventRecordData (EvtNetDHCP_WrongClientHwAddress, chaddr, 6);
  }
#else
  #define EvrNetDHCP_WrongClientHwAddress(chaddr)
#endif

/**
  \brief  Event on DHCP client wrong BOOTP Magic Cookie received (Error)
  \param  magic_cookie  wrong magic cookie value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_WrongMagicCookie(uint32_t magic_cookie) {
    EventRecord2 (EvtNetDHCP_WrongMagicCookie, magic_cookie, 0);
  }
#else
  #define EvrNetDHCP_WrongMagicCookie(magic_cookie)
#endif

/**
  \brief  Event on DHCP client received message not DHCP_OFFER (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_NotDhcpOfferMessage(void) {
    EventRecord2 (EvtNetDHCP_NotDhcpOfferMessage, 0, 0);
  }
#else
  #define EvrNetDHCP_NotDhcpOfferMessage()
#endif

/**
  \brief  Event on DHCP client received DHCP_OFFER message (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_DhcpOfferReceived(void) {
    EventRecord2 (EvtNetDHCP_DhcpOfferReceived, 0, 0);
  }
#else
  #define EvrNetDHCP_DhcpOfferReceived()
#endif

/**
  \brief  Event on DHCP client received options overloaded to "File" field (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_FileOverloadedOptions(void) {
    EventRecord2 (EvtNetDHCP_FileOverloadedOptions, 0, 0);
  }
#else
  #define EvrNetDHCP_FileOverloadedOptions()
#endif

/**
  \brief  Event on DHCP client received options overloaded to "Sname" field (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_SnameOverloadedOptions(void) {
    EventRecord2 (EvtNetDHCP_SnameOverloadedOptions, 0, 0);
  }
#else
  #define EvrNetDHCP_SnameOverloadedOptions()
#endif

/**
  \brief  Event on DHCP client received DHCP_OFFER with no Server ID (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_DhcpOfferServerIdMissing(void) {
    EventRecord2 (EvtNetDHCP_DhcpOfferServerIdMissing, 0, 0);
  }
#else
  #define EvrNetDHCP_DhcpOfferServerIdMissing()
#endif

/**
  \brief  Event on DHCP client received forwarded message (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ForwardedMessage(void) {
    EventRecord2 (EvtNetDHCP_ForwardedMessage, 0, 0);
  }
#else
  #define EvrNetDHCP_ForwardedMessage()
#endif

/**
  \brief  Event on DHCP client received message from invalid Relay Agent address (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_RelayAgentAddressInvalid(void) {
    EventRecord2 (EvtNetDHCP_RelayAgentAddressInvalid, 0, 0);
  }
#else
  #define EvrNetDHCP_RelayAgentAddressInvalid()
#endif

/**
  \brief  Event on DHCP client display relay agent IP address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowRelayAgentAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_ShowRelayAgentAddress, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_ShowRelayAgentAddress(ip4_addr)
#endif

/**
  \brief  Event on DHCP client display offered IP address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowOfferedIpAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_ShowOfferedIpAddress, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_ShowOfferedIpAddress(ip4_addr)
#endif

/**
  \brief  Event on DHCP client received message not from selected server (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ServerAddressNotSelected(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_ServerAddressNotSelected, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_ServerAddressNotSelected(ip4_addr)
#endif

/**
  \brief  Event on DHCP client assigned IP address in DHCP_ACK not the same as requested (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_AssignedAddrNotRequested(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_AssignedAddrNotRequested, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_AssignedAddrNotRequested(ip4_addr)
#endif

/**
  \brief  Event on DHCP client received message not DHCP_ACK (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_NotDhcpAckMessage(void) {
    EventRecord2 (EvtNetDHCP_NotDhcpAckMessage, 0, 0);
  }
#else
  #define EvrNetDHCP_NotDhcpAckMessage()
#endif

/**
  \brief  Event on DHCP client received DHCP_ACK message (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_DhcpAckReceived(void) {
    EventRecord2 (EvtNetDHCP_DhcpAckReceived, 0, 0);
  }
#else
  #define EvrNetDHCP_DhcpAckReceived()
#endif

/**
  \brief  Event on DHCP client display assigned IP address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowAssignedAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_ShowAssignedAddress, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_ShowAssignedAddress(ip4_addr)
#endif

/**
  \brief  Event on DHCP client received DHCP_NAK message (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_DhcpNakReceived(void) {
    EventRecord2 (EvtNetDHCP_DhcpNakReceived, 0, 0);
  }
#else
  #define EvrNetDHCP_DhcpNakReceived()
#endif

/**
  \brief  Event on DHCP client display Server Identifier (IP address) (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowServerId(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_ShowServerId, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_ShowServerId(ip4_addr)
#endif

/**
  \brief  Event on DHCP client display Network Mask (Op)
  \param  net_mask      pointer to network mask
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowNetMask(const uint8_t *net_mask) {
    EventRecordData (EvtNetDHCP_ShowNetMask, net_mask, 4);
  }
#else
  #define EvrNetDHCP_ShowNetMask(net_mask)
#endif

/**
  \brief  Event on DHCP client display gateway IPv4 address (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowGatewayAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDHCP_ShowGatewayAddress, ip4_addr, 4);
  }
#else
  #define EvrNetDHCP_ShowGatewayAddress(ip4_addr)
#endif

/**
  \brief  Event on DHCP client display DNS server addresses (Op)
  \param  dns_list      pointer to DNS server address list
  \note   Displays primary and secondary DNS server address.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowDnsServers(const uint8_t *dns_list) {
    EventRecordData (EvtNetDHCP_ShowDnsServers, dns_list, 8);
  }
#else
  #define EvrNetDHCP_ShowDnsServers(dns_list)
#endif

/**
  \brief  Event on DHCP client display address lease time (Op)
  \param  lease_time    lease time in seconds
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowLeaseTime(uint32_t lease_time) {
    EventRecord2 (EvtNetDHCP_ShowLeaseTime, lease_time, 0);
  }
#else
  #define EvrNetDHCP_ShowLeaseTime(lease_time)
#endif

/**
  \brief  Event on DHCP client display renewal time T1 (Op)
  \param  t1_time       t1 time in seconds
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowTimeT1(uint32_t t1_time) {
    EventRecord2 (EvtNetDHCP_ShowTimeT1, t1_time, 0);
  }
#else
  #define EvrNetDHCP_ShowTimeT1(t1_time)
#endif

/**
  \brief  Event on DHCP client display rebinding time T2 (Op)
  \param  t2_time       t2 time in seconds
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowTimeT2(uint32_t t2_time) {
    EventRecord2 (EvtNetDHCP_ShowTimeT2, t2_time, 0);
  }
#else
  #define EvrNetDHCP_ShowTimeT2(t2_time)
#endif

/**
  \brief  Event on DHCP client display Bootfile Name (Op)
  \param  bootfile      bootfile name string
  \param  length        length of the bootfile name string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowBootfileName(const uint8_t *bootfile, uint32_t length) {
    if (length > 128) length = 128;
    EventRecordData (EvtNetDHCP_ShowBootfileName, bootfile, length);
  }
#else
  #define EvrNetDHCP_ShowBootfileName(bootfile, length)
#endif

/**
  \brief  Event on DHCP client display a list of NTP server addresses (Op)
  \param  ntp_list      pointer to an array of NTP server IP addresses
  \param  length        length of an array in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_ShowNtpServerList(const uint8_t *ntp_list, uint32_t length) {
    if (length > 64) length = 64;
    EventRecordData (EvtNetDHCP_ShowNtpServerList, ntp_list, length);
  }
#else
  #define EvrNetDHCP_ShowNtpServerList(ntp_list, length)
#endif

/**
  \brief  Event on \ref netDHCP_SetOption invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_SetOptionInvalidParam(void) {
    EventRecord2 (EvtNetDHCP_SetOptionInvalidParam, 0, 0);
  }
#else
  #define EvrNetDHCP_SetOptionInvalidParam()
#endif

/**
  \brief  Event on \ref netDHCP_SetOption invalid client state (Error)
  \param  state         DHCP client state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_SetOptionInvalidState(uint32_t state) {
    EventRecord2 (EvtNetDHCP_SetOptionInvalidState, state, 0);
  }
#else
  #define EvrNetDHCP_SetOptionInvalidState(state)
#endif

/**
  \brief  Event on \ref netDHCP_SetOption Client-identifier to default (API)
  \note   Default DHCP Client-identifier is ethernet MAC address.
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_SetOptionClientIdDefault(uint8_t type) {
    EventRecord2 (EvtNetDHCP_SetOptionClientIdDefault, type, 0);
  }
#else
  #define EvrNetDHCP_SetOptionClientIdDefault(type)
#endif

/**
  \brief  Event on \ref netDHCP_SetOption Client-identifier to user defined value (API)
  \param  type          hardware type
                         - 0:   Generic,
                         - 1:   Ethernet,
                         - 255: IANA.
  \param  length        length of DHCP option including hardware type
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_SetOptionClientId(uint8_t type, uint32_t length) {
    EventRecord2 (EvtNetDHCP_SetOptionClientId, type, length);
  }
#else
  #define EvrNetDHCP_SetOptionClientId(type, length)
#endif

/**
  \brief  Event on DHCP client de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP_UninitClient(void) {
    EventRecord2 (EvtNetDHCP_UninitClient, 0, 0);
  }
#else
  #define EvrNetDHCP_UninitClient()
#endif


// NetARP event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetARP_InitCache                 EventID (EventLevelOp,    EvtNetARP,  0)
#define EvtNetARP_ReceiveFrame              EventID (EventLevelOp,    EvtNetARP,  1)
#define EvtNetARP_FrameTooShort             EventID (EventLevelError, EvtNetARP,  2)
#define EvtNetARP_FrameCorrupted            EventID (EventLevelError, EvtNetARP,  3)
#define EvtNetARP_ShowFrameHeader           EventID (EventLevelDetail,EvtNetARP,  4)
#define EvtNetARP_SenderAddressInvalid      EventID (EventLevelOp,    EvtNetARP,  5)
#define EvtNetARP_CacheEntryUpdate          EventID (EventLevelOp,    EvtNetARP,  6)
#define EvtNetARP_ProbeResponseReceived     EventID (EventLevelOp,    EvtNetARP,  7)
#define EvtNetARP_WrongIpAddress            EventID (EventLevelOp,    EvtNetARP,  8)
#define EvtNetARP_WrongMacAddress           EventID (EventLevelOp,    EvtNetARP,  9)
#define EvtNetARP_OpcodeUnknown             EventID (EventLevelOp,    EvtNetARP, 10)
#define EvtNetARP_SendReply                 EventID (EventLevelOp,    EvtNetARP, 11)
#define EvtNetARP_CacheEntryRefreshed       EventID (EventLevelOp,    EvtNetARP, 12)
#define EvtNetARP_WrongResponse             EventID (EventLevelError, EvtNetARP, 13)
#define EvtNetARP_SendRequest               EventID (EventLevelOp,    EvtNetARP, 14)
#define EvtNetARP_CacheAllocFailed          EventID (EventLevelError, EvtNetARP, 15)
#define EvtNetARP_CacheIpRefreshed          EventID (EventLevelOp,    EvtNetARP, 16)
#define EvtNetARP_CacheAdd                  EventID (EventLevelOp,    EvtNetARP, 17)
#define EvtNetARP_GatewayUnknown            EventID (EventLevelError, EvtNetARP, 18)
#define EvtNetARP_CacheEntryAdded           EventID (EventLevelOp,    EvtNetARP, 19)
#define EvtNetARP_CacheEarly                EventID (EventLevelOp,    EvtNetARP, 20)
#define EvtNetARP_CacheFind                 EventID (EventLevelOp,    EvtNetARP, 21)
#define EvtNetARP_UsingGateway              EventID (EventLevelOp,    EvtNetARP, 22)
#define EvtNetARP_UnresolvedMacAddress      EventID (EventLevelError, EvtNetARP, 23)
#define EvtNetARP_EntryFound                EventID (EventLevelOp,    EvtNetARP, 24)
#define EvtNetARP_CacheIp                   EventID (EventLevelAPI,   EvtNetARP, 25)
#define EvtNetARP_CacheIpInvalidParameter   EventID (EventLevelError, EvtNetARP, 26)
#define EvtNetARP_CacheMac                  EventID (EventLevelAPI,   EvtNetARP, 27)
#define EvtNetARP_CacheMacInvalidParameter  EventID (EventLevelError, EvtNetARP, 28)
#define EvtNetARP_UnresolvedIpAddress       EventID (EventLevelError, EvtNetARP, 29)
#define EvtNetARP_GetIp                     EventID (EventLevelAPI,   EvtNetARP, 30)
#define EvtNetARP_GetIpInvalidParameter     EventID (EventLevelError, EvtNetARP, 31)
#define EvtNetARP_GetIpEntryNotFound        EventID (EventLevelOp,    EvtNetARP, 32)
#define EvtNetARP_GetMac                    EventID (EventLevelAPI,   EvtNetARP, 33)
#define EvtNetARP_GetMacInvalidParameter    EventID (EventLevelError, EvtNetARP, 34)
#define EvtNetARP_GetMacEntryNotFound       EventID (EventLevelOp,    EvtNetARP, 35)
#define EvtNetARP_Probe                     EventID (EventLevelAPI,   EvtNetARP, 36)
#define EvtNetARP_ProbeInvalidParameter     EventID (EventLevelError, EvtNetARP, 37)
#define EvtNetARP_ProbeBusy                 EventID (EventLevelError, EvtNetARP, 38)
#define EvtNetARP_ProbeRetransmit           EventID (EventLevelOp,    EvtNetARP, 39)
#define EvtNetARP_ProbeTimeout              EventID (EventLevelOp,    EvtNetARP, 40)
#define EvtNetARP_CacheEntryTimeout         EventID (EventLevelOp,    EvtNetARP, 41)
#define EvtNetARP_EntryReleased             EventID (EventLevelOp,    EvtNetARP, 42)
#define EvtNetARP_ResolveEntry              EventID (EventLevelOp,    EvtNetARP, 43)
#define EvtNetARP_RefreshEntry              EventID (EventLevelOp,    EvtNetARP, 44)
#define EvtNetARP_UninitCache               EventID (EventLevelOp,    EvtNetARP, 45)
#endif

/**
  \brief  Event on ARP cache initialize (Op)
  \param  num_entries   number of entries available in cache
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_InitCache(uint32_t num_entries) {
    EventRecord2 (EvtNetARP_InitCache, num_entries, 0);
  }
#else
  #define EvrNetARP_InitCache(num_entries)
#endif

/**
  \brief  Event on ARP receive frame (Op)
  \param  arp_opcode    operation code of received message
                         - 0x0001: ARP Request
                         - 0x0002: ARP Reply
                         - 0x0003: Reverse ARP Request
                         - 0x0004: Reverse ARP Reply
                         - 0x0008: Inverse ARP Request
                         - 0x0009: Inverse ARP Response
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_ReceiveFrame(uint16_t arp_opcode) {
    EventRecord2 (EvtNetARP_ReceiveFrame, arp_opcode, 0);
  }
#else
  #define EvrNetARP_ReceiveFrame(arp_opcode)
#endif

/**
  \brief  Event on ARP received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetARP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetARP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on ARP received frame corrupted (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_FrameCorrupted(void) {
    EventRecord2 (EvtNetARP_FrameCorrupted, 0, 0);
  }
#else
  #define EvrNetARP_FrameCorrupted()
#endif

/**
  \brief  Event on ARP display send/receive frame header (Detail)
  \param  arp_header    pointer to ARP frame header of 28 bytes
                         - HwType       (2 bytes)
                         - ProtType     (2 bytes)
                         - HwLen        (1 byte)
                         - ProtLen      (1 byte)
                         - OpCode       (2 bytes)
                         - SendHwAddr   (6 bytes)
                         - SendIpAddr   (4 bytes)
                         - TargHwAddr   (6 bytes)
                         - TargIpAddr   (4 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_ShowFrameHeader(const void *arp_header) {
    EventRecordData (EvtNetARP_ShowFrameHeader, arp_header, 28);
  }
#else
  #define EvrNetARP_ShowFrameHeader(arp_header)
#endif

/**
  \brief  Event on ARP invalid IP or MAC address of sender received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_SenderAddressInvalid(void) {
    EventRecord2 (EvtNetARP_SenderAddressInvalid, 0, 0);
  }
#else
  #define EvrNetARP_SenderAddressInvalid()
#endif

/**
  \brief  Event on ARP cache entry update (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheEntryUpdate(uint32_t entry) {
    EventRecord2 (EvtNetARP_CacheEntryUpdate, entry, 0);
  }
#else
  #define EvrNetARP_CacheEntryUpdate(entry)
#endif

/**
  \brief  Event on ARP probe response received (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_ProbeResponseReceived(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_ProbeResponseReceived, ip4_addr, 4);
  }
#else
  #define EvrNetARP_ProbeResponseReceived(ip4_addr)
#endif

/**
  \brief  Event on ARP wrong destination IPv4 address received (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_WrongIpAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_WrongIpAddress, ip4_addr, 4);
  }
#else
  #define EvrNetARP_WrongIpAddress(ip4_addr)
#endif

/**
  \brief  Event on ARP wrong destination MAC address received (Op)
  \param  mac_addr      pointer to MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_WrongMacAddress(const uint8_t *mac_addr) {
    EventRecordData (EvtNetARP_WrongMacAddress, mac_addr, 6);
  }
#else
  #define EvrNetARP_WrongMacAddress(mac_addr)
#endif

/**
  \brief  Event on ARP unknown opcode received (Op)
  \param  arp_opcode    unknown operation code value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_OpcodeUnknown(uint16_t arp_opcode) {
    EventRecord2 (EvtNetARP_OpcodeUnknown, arp_opcode, 0);
  }
#else
  #define EvrNetARP_OpcodeUnknown(arp_opcode)
#endif

/**
  \brief  Event on ARP send reply message (Op)
  \param  arp_opcode    operation code of sending message
                         - 0x0002: ARP Reply
                         - 0x0009: Inverse ARP Response
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_SendReply(uint16_t arp_opcode) {
    EventRecord2 (EvtNetARP_SendReply, arp_opcode, 0);
  }
#else
  #define EvrNetARP_SendReply(arp_opcode)
#endif

/**
  \brief  Event on ARP cache entry refreshed (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheEntryRefreshed(uint32_t entry) {
    EventRecord2 (EvtNetARP_CacheEntryRefreshed, entry, 0);
  }
#else
  #define EvrNetARP_CacheEntryRefreshed(entry)
#endif

/**
  \brief  Event on ARP wrong response received (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_WrongResponse(void) {
    EventRecord2 (EvtNetARP_WrongResponse, 0, 0);
  }
#else
  #define EvrNetARP_WrongResponse()
#endif

/**
  \brief  Event on ARP send request message (Op)
  \param  arp_opcode    operation code of sending message
                         - 0x0001: ARP Request
                         - 0x0008: Inverse ARP Request
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_SendRequest(uint16_t arp_opcode) {
    EventRecord2 (EvtNetARP_SendRequest, arp_opcode, 0);
  }
#else
  #define EvrNetARP_SendRequest(arp_opcode)
#endif

/**
  \brief  Event on ARP allocate free cache entry failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheAllocFailed(void) {
    EventRecord2 (EvtNetARP_CacheAllocFailed, 0, 0);
  }
#else
  #define EvrNetARP_CacheAllocFailed()
#endif

/**
  \brief  Event on ARP cached IP address refreshed (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheIpRefreshed(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_CacheIpRefreshed, ip4_addr, 4);
  }
#else
  #define EvrNetARP_CacheIpRefreshed(ip4_addr)
#endif

/**
  \brief  Event on ARP add host address to ARP cache (Op)
  \param  ip4_addr      pointer to IPv4 address
  \param  mac_addr      pointer to MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheAdd(const uint8_t *ip4_addr, const uint8_t *mac_addr) {
    memcpy (&evr_buf.u8[0], ip4_addr, 4);
    memcpy (&evr_buf.u8[4], mac_addr, 6);
    EventRecordData (EvtNetARP_CacheAdd, &evr_buf, 10);
  }
#else
  #define EvrNetARP_CacheAdd(ip4_addr, mac_addr)
#endif

/**
  \brief  Event on ARP cache add gateway unknown (Error)
  \param  ip4_addr      pointer to IPv4 address of a gateway
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_GatewayUnknown(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_GatewayUnknown, ip4_addr, 4);
  }
#else
  #define EvrNetARP_GatewayUnknown(ip4_addr)
#endif

/**
  \brief  Event on ARP cache entry added (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheEntryAdded(uint32_t entry) {
    EventRecord2 (EvtNetARP_CacheEntryAdded, entry, 0);
  }
#else
  #define EvrNetARP_CacheEntryAdded(entry)
#endif

/**
  \brief  Event on ARP early cache address of the sender (Op)
  \param  ip4_addr      pointer to IPv4 address
  \param  mac_addr      pointer to MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheEarly(const uint8_t *ip4_addr, const uint8_t *mac_addr) {
    memcpy (&evr_buf.u8[0], ip4_addr, 4);
    memcpy (&evr_buf.u8[4], mac_addr, 6);
    EventRecordData (EvtNetARP_CacheEarly, &evr_buf, 10);
  }
#else
  #define EvrNetARP_CacheEarly(ip4_addr, mac_addr)
#endif

/**
  \brief  Event on ARP find IP address in cache (Op)
  \param  ip4_addr      pointer to IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheFind(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_CacheFind, ip4_addr, 4);
  }
#else
  #define EvrNetARP_CacheFind(ip4_addr)
#endif

/**
  \brief  Event on ARP using gateway to access external address (Op)
  \param  ip4_addr      pointer to IPv4 address of a gateway
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_UsingGateway(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_UsingGateway, ip4_addr, 4);
  }
#else
  #define EvrNetARP_UsingGateway(ip4_addr)
#endif

/**
  \brief  Event on ARP unresolved MAC address in cache (Error)
  \param  ip4_addr      pointer to requested IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_UnresolvedMacAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_UnresolvedMacAddress, ip4_addr, 4);
  }
#else
  #define EvrNetARP_UnresolvedMacAddress(ip4_addr)
#endif

/**
  \brief  Event on ARP cache entry found (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_EntryFound(uint32_t entry) {
    EventRecord2 (EvtNetARP_EntryFound, entry, 0);
  }
#else
  #define EvrNetARP_EntryFound(entry)
#endif

/**
  \brief  Event on \ref netARP_CacheIP start (API)
  \param  ip4_addr      pointer to requested IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheIp(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_CacheIp, ip4_addr, 4);
  }
#else
  #define EvrNetARP_CacheIp(ip4_addr)
#endif

/**
  \brief  Event on \ref netARP_CacheIP invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheIpInvalidParameter(void) {
    EventRecord2 (EvtNetARP_CacheIpInvalidParameter, 0, 0);
  }
#else
  #define EvrNetARP_CacheIpInvalidParameter()
#endif

/**
  \brief  Event on \ref netARP_CacheMAC start (API)
  \param  mac_addr      pointer to requested MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheMac(const uint8_t *mac_addr) {
    EventRecordData (EvtNetARP_CacheMac, mac_addr, 6);
  }
#else
  #define EvrNetARP_CacheMac(mac_addr)
#endif

/**
  \brief  Event on \ref netARP_CacheMAC invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheMacInvalidParameter(void) {
    EventRecord2 (EvtNetARP_CacheMacInvalidParameter, 0, 0);
  }
#else
  #define EvrNetARP_CacheMacInvalidParameter()
#endif

/**
  \brief  Event on ARP unresolved IP address in cache (Op)
  \param  mac_addr      pointer to MAC cache address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_UnresolvedIpAddress(const uint8_t *mac_addr) {
    EventRecordData (EvtNetARP_UnresolvedIpAddress, mac_addr, 6);
  }
#else
  #define EvrNetARP_UnresolvedIpAddress(mac_addr)
#endif

/**
  \brief  Event on \ref netARP_GetIP resolving success (API)
  \param  mac_addr      pointer to requested MAC address
  \param  ip4_addr      pointer to resolved IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_GetIp(const uint8_t *mac_addr, const uint8_t *ip4_addr) {
    memcpy (&evr_buf.u8[0], ip4_addr, 4);
    memcpy (&evr_buf.u8[4], mac_addr, 6);
    EventRecordData (EvtNetARP_GetIp, &evr_buf, 10);
  }
#else
  #define EvrNetARP_GetIp(mac_addr, ip4_addr)
#endif

/**
  \brief  Event on \ref netARP_GetIP invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_GetIpInvalidParameter(void) {
    EventRecord2 (EvtNetARP_GetIpInvalidParameter, 0, 0);
  }
#else
  #define EvrNetARP_GetIpInvalidParameter()
#endif

/**
  \brief  Event on \ref netARP_GetIP entry with MAC address not found in cache (Op)
  \param  mac_addr      pointer to requested MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_GetIpEntryNotFound(const uint8_t *mac_addr) {
    EventRecordData (EvtNetARP_GetIpEntryNotFound, mac_addr, 6);
  }
#else
  #define EvrNetARP_GetIpEntryNotFound(mac_addr)
#endif

/**
  \brief  Event on \ref netARP_GetMAC resolving success (API)
  \param  ip4_addr      pointer to requested IPv4 address
  \param  mac_addr      pointer to resolved MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_GetMac(const uint8_t *ip4_addr, const uint8_t *mac_addr) {
    memcpy (&evr_buf.u8[0], ip4_addr, 4);
    memcpy (&evr_buf.u8[4], mac_addr, 6);
    EventRecordData (EvtNetARP_GetMac, &evr_buf, 10);
  }
#else
  #define EvrNetARP_GetMac(ip4_addr, mac_addr)
#endif

/**
  \brief  Event on \ref netARP_GetMAC invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_GetMacInvalidParameter(void) {
    EventRecord2 (EvtNetARP_GetMacInvalidParameter, 0, 0);
  }
#else
  #define EvrNetARP_GetMacInvalidParameter()
#endif

/**
  \brief  Event on \ref netARP_GetMAC entry with IP address not found in cache (Op)
  \param  ip4_addr      pointer to requested IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_GetMacEntryNotFound(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_GetMacEntryNotFound, ip4_addr, 4);
  }
#else
  #define EvrNetARP_GetMacEntryNotFound(ip4_addr)
#endif

/**
  \brief  Event on \ref netARP_Probe start (API)
  \param  ip4_addr      pointer to requested IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_Probe(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_Probe, ip4_addr, 4);
  }
#else
  #define EvrNetARP_Probe(ip4_addr)
#endif

/**
  \brief  Event on \ref netARP_Probe invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_ProbeInvalidParameter(void) {
    EventRecord2 (EvtNetARP_ProbeInvalidParameter, 0, 0);
  }
#else
  #define EvrNetARP_ProbeInvalidParameter()
#endif

/**
  \brief  Event on \ref netARP_Probe process busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_ProbeBusy(void) {
    EventRecord2 (EvtNetARP_ProbeBusy, 0, 0);
  }
#else
  #define EvrNetARP_ProbeBusy()
#endif

/**
  \brief  Event on ARP probe retransmit (Op)
  \param  ip4_addr      pointer to IPv4 address of probed host
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_ProbeRetransmit(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_ProbeRetransmit, ip4_addr, 4);
  }
#else
  #define EvrNetARP_ProbeRetransmit(ip4_addr)
#endif

/**
  \brief  Event on ARP probe timeout (Op)
  \param  ip4_addr      pointer to IPv4 address of probed host
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_ProbeTimeout(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetARP_ProbeTimeout, ip4_addr, 4);
  }
#else
  #define EvrNetARP_ProbeTimeout(ip4_addr)
#endif

/**
  \brief  Event on ARP cache entry timeout (Op)
  \param  entry         cache entry number
  \param  ip4_addr      pointer to IPv4 address
  \param  mac_addr      pointer to MAC address
  \param  type          cache entry type
                        - 0: Fixed IP
                        - 1: Temporary IP
                        - 2: In-use temporary IP
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_CacheEntryTimeout(uint32_t entry, const uint8_t *ip4_addr,
                                                   const uint8_t *mac_addr, uint32_t type) {
    evr_buf.u8[10] = (uint8_t)entry;
    evr_buf.u8[11] = (uint8_t)type;
    memcpy (&evr_buf.u8[0], ip4_addr, 4);
    memcpy (&evr_buf.u8[4], mac_addr, 6);
    EventRecordData (EvtNetARP_CacheEntryTimeout, &evr_buf, 12);
  }
#else
  #define EvrNetARP_CacheEntryTimeout(entry, type, ip4_addr, mac_addr)
#endif

/**
  \brief  Event on ARP cache entry released (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_EntryReleased(uint32_t entry) {
    EventRecord2 (EvtNetARP_EntryReleased, entry, 0);
  }
#else
  #define EvrNetARP_EntryReleased(entry)
#endif

/**
  \brief  Event on resolve ARP cache entry (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_ResolveEntry(uint32_t entry) {
    EventRecord2 (EvtNetARP_ResolveEntry, entry, 0);
  }
#else
  #define EvrNetARP_ResolveEntry(entry)
#endif

/**
  \brief  Event on refresh ARP cache entry (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_RefreshEntry(uint32_t entry) {
    EventRecord2 (EvtNetARP_RefreshEntry, entry, 0);
  }
#else
  #define EvrNetARP_RefreshEntry(entry)
#endif

/**
  \brief  Event on ARP cache de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetARP_UninitCache(void) {
    EventRecord2 (EvtNetARP_UninitCache, 0, 0);
  }
#else
  #define EvrNetARP_UninitCache()
#endif


// NetIP6 event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetIP6_InitCore                  EventID (EventLevelOp,    EvtNetIP6,  0)
#define EvtNetIP6_ReceiveFrame              EventID (EventLevelOp,    EvtNetIP6,  1)
#define EvtNetIP6_FrameTooShort             EventID (EventLevelError, EvtNetIP6,  2)
#define EvtNetIP6_InvalidIpVersion          EventID (EventLevelError, EvtNetIP6,  3)
#define EvtNetIP6_ShowFrameHeader           EventID (EventLevelDetail,EvtNetIP6,  4)
#define EvtNetIP6_WrongDestinationAddress   EventID (EventLevelOp,    EvtNetIP6,  5)
#define EvtNetIP6_InterfaceNotSupported     EventID (EventLevelError, EvtNetIP6,  6)
#define EvtNetIP6_WrongMulticastProtocol    EventID (EventLevelError, EvtNetIP6,  7)
#define EvtNetIP6_SendFrame                 EventID (EventLevelOp,    EvtNetIP6,  8)
#define EvtNetIP6_ProtocolUnknown           EventID (EventLevelError, EvtNetIP6,  9)
#define EvtNetIP6_SourceAddressNull         EventID (EventLevelError, EvtNetIP6, 10)
#define EvtNetIP6_DestinationAddressNull    EventID (EventLevelError, EvtNetIP6, 11)
#define EvtNetIP6_UninitCore                EventID (EventLevelOp,    EvtNetIP6, 12)
#endif

/**
  \brief  Event on IPv6 core initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_InitCore(void) {
    EventRecord2 (EvtNetIP6_InitCore, 0, 0);
  }
#else
  #define EvrNetIP6_InitCore()
#endif

/**
  \brief  Event on IPv6 receive frame (Op)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_ReceiveFrame(uint32_t length) {
    EventRecord2 (EvtNetIP6_ReceiveFrame, length, 0);
  }
#else
  #define EvrNetIP6_ReceiveFrame(length)
#endif

/**
  \brief  Event on IPv6 received frame too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetIP6_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetIP6_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on IPv6 invalid protocol version (Error)
  \param  ip_version    not IPv6 protocol version value
                         - 0..5
                         - 7..15
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_InvalidIpVersion(uint8_t ip_version) {
    EventRecord2 (EvtNetIP6_InvalidIpVersion, ip_version, 0);
  }
#else
  #define EvrNetIP6_InvalidIpVersion(ip_version)
#endif

/**
  \brief  Event on IPv6 display frame header (Detail)
  \param  ip6_header    pointer to IPv6 frame header of 40 bytes
                         - VerClass  (1 byte)
                         - ClassFlow (1 byte)
                         - Flow      (2 bytes)
                         - Len       (2 bytes)
                         - NextHdr   (1 byte)
                         - HopLim    (1 byte)
                         - SrcAddr   (16 bytes)
                         - DstAddr   (16 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_ShowFrameHeader(const void *ip6_header) {
    EventRecordData (EvtNetIP6_ShowFrameHeader, ip6_header, 40);
  }
#else
  #define EvrNetIP6_ShowFrameHeader(ip6_header)
#endif

/**
  \brief  Event on IPv6 wrong destination address (Op)
  \param  ip6_addr      pointer to IPv6 destination address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_WrongDestinationAddress(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetIP6_WrongDestinationAddress, ip6_addr, 16);
  }
#else
  #define EvrNetIP6_WrongDestinationAddress(ip6_addr)
#endif

/**
  \brief  Event on IPv6 network interface not supported (Error)
  \param  interface     network interface
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_InterfaceNotSupported(uint32_t interface) {
    EventRecord2 (EvtNetIP6_InterfaceNotSupported, interface, 0);
  }
#else
  #define EvrNetIP6_InterfaceNotSupported(interface)
#endif

/**
  \brief  Event on IPv6 wrong multicast protocol type (Error)
  \param  ip6_protocol  wrong protocol type value
  \note   Only UDP multicast frames are allowed!
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_WrongMulticastProtocol(uint8_t ip6_protocol) {
    EventRecord2 (EvtNetIP6_WrongMulticastProtocol, ip6_protocol, 0);
  }
#else
  #define EvrNetIP6_WrongMulticastProtocol(ip6_protocol)
#endif

/**
  \brief  Event on IP6 send frame (Op)
  \param  ip6_protocol  IPv6 protocol type
                         - 6:  TCP
                         - 17: UDP
                         - 58: ICMP6
  \param  length        frame length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_SendFrame(uint8_t ip6_protocol, uint32_t length) {
    EventRecord2 (EvtNetIP6_SendFrame, ip6_protocol, length);
  }
#else
  #define EvrNetIP6_SendFrame(ip6_protocol, length)
#endif

/**
  \brief  Event on IP6 unknown protocol type in sending frame (Error)
  \param  ip6_protocol  unknown IPv6 protocol type value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_ProtocolUnknown(uint8_t ip6_protocol) {
    EventRecord2 (EvtNetIP6_ProtocolUnknown, ip6_protocol, 0);
  }
#else
  #define EvrNetIP6_ProtocolUnknown(ip6_protocol)
#endif

/**
  \brief  Event on IP6 source address not provided (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_SourceAddressNull(void) {
    EventRecord2 (EvtNetIP6_SourceAddressNull, 0, 0);
  }
#else
  #define EvrNetIP6_SourceAddressNull()
#endif

/**
  \brief  Event on IP6 destination address not provided (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_DestinationAddressNull(void) {
    EventRecord2 (EvtNetIP6_DestinationAddressNull, 0, 0);
  }
#else
  #define EvrNetIP6_DestinationAddressNull()
#endif

/**
  \brief  Event on IPv6 core de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetIP6_UninitCore(void) {
    EventRecord2 (EvtNetIP6_UninitCore, 0, 0);
  }
#else
  #define EvrNetIP6_UninitCore()
#endif


// NetICMP6 event identifiers --------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetICMP6_ReceiveFrame            EventID (EventLevelOp,    EvtNetICMP6,  0)
#define EvtNetICMP6_FrameTooShort           EventID (EventLevelError, EvtNetICMP6,  1)
#define EvtNetICMP6_ChecksumFailed          EventID (EventLevelError, EvtNetICMP6,  2)
#define EvtNetICMP6_EchoRequestWrongCode    EventID (EventLevelError, EvtNetICMP6,  3)
#define EvtNetICMP6_EchoRequestReceived     EventID (EventLevelOp,    EvtNetICMP6,  4)
#define EvtNetICMP6_SendEchoReply           EventID (EventLevelOp,    EvtNetICMP6,  5)
#define EvtNetICMP6_EchoReplyReceived       EventID (EventLevelOp,    EvtNetICMP6,  6)
#define EvtNetICMP6_EchoReplyWrongState     EventID (EventLevelError, EvtNetICMP6,  7)
#define EvtNetICMP6_EchoReplyWrongCode      EventID (EventLevelError, EvtNetICMP6,  8)
#define EvtNetICMP6_EchoReplyWrongIpAddress EventID (EventLevelError, EvtNetICMP6,  9)
#define EvtNetICMP6_EchoReplyWrongId        EventID (EventLevelError, EvtNetICMP6, 10)
#define EvtNetICMP6_EchoReplyWrongPayload   EventID (EventLevelError, EvtNetICMP6, 11)
#define EvtNetICMP6_MessageDiscarded        EventID (EventLevelOp,    EvtNetICMP6, 12)
#define EvtNetICMP6_WrongHopLimit           EventID (EventLevelError, EvtNetICMP6, 13)
#define EvtNetICMP6_WrongCode               EventID (EventLevelError, EvtNetICMP6, 14)
#define EvtNetICMP6_MessageTypeUnknown      EventID (EventLevelOp,    EvtNetICMP6, 15)
#define EvtNetICMP6_SendFrame               EventID (EventLevelOp,    EvtNetICMP6, 16)
/* ICMP6-Ping client events */
#define EvtNetICMP6_PingEcho                EventID (EventLevelAPI,   EvtNetICMP6, 17)
#define EvtNetICMP6_PingTargetNotValid      EventID (EventLevelError, EvtNetICMP6, 18)
#define EvtNetICMP6_PingDnsError            EventID (EventLevelError, EvtNetICMP6, 19)
#define EvtNetICMP6_PingInvalidParameter    EventID (EventLevelError, EvtNetICMP6, 20)
#define EvtNetICMP6_PingClientBusy          EventID (EventLevelError, EvtNetICMP6, 21)
#define EvtNetICMP6_PingSendRequest         EventID (EventLevelOp,    EvtNetICMP6, 22)
#define EvtNetICMP6_PingRetransmitRequest   EventID (EventLevelOp,    EvtNetICMP6, 23)
#define EvtNetICMP6_PingTimeout             EventID (EventLevelOp,    EvtNetICMP6, 24)
#endif

/**
  \brief  Event on ICMP6 receive frame (Op)
  \param  icmp6_header  pointer to ICMP6 frame header of 8 bytes
                         - Type    (1 byte)
                         - Code    (1 byte)
                         - Chksum  (2 bytes)
                         - EchoId  (2 bytes)
                         - EchoSeq (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_ReceiveFrame(const void *icmp6_header) {
    EventRecordData (EvtNetICMP6_ReceiveFrame, icmp6_header, 8);
  }
#else
  #define EvrNetICMP6_ReceiveFrame(icmp6_header)
#endif

/**
  \brief  Event on ICMP6 received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetICMP6_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetICMP6_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on ICMP6 frame checksum check failed (Error)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_ChecksumFailed(uint32_t length) {
    EventRecord2 (EvtNetICMP6_ChecksumFailed, length, 0);
  }
#else
  #define EvrNetICMP6_ChecksumFailed(length)
#endif

/**
  \brief  Event on ICMP6 wrong code in echo request received (Error)
  \param  icmp6_code    wrong ICMP6 code value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_EchoRequestWrongCode(uint8_t icmp6_code) {
    EventRecord2 (EvtNetICMP6_EchoRequestWrongCode, icmp6_code, 0);
  }
#else
  #define EvrNetICMP6_EchoRequestWrongCode(icmp6_code)
#endif

/**
  \brief  Event on ICMP6 echo request received (Op)
  \param  data_length   length of the payload data
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_EchoRequestReceived(uint32_t data_length) {
    EventRecord2 (EvtNetICMP6_EchoRequestReceived, data_length, 0);
  }
#else
  #define EvrNetICMP6_EchoRequestReceived(data_length)
#endif

/**
  \brief  Event on ICMP6 send echo reply (Op)
  \param  icmp6_header  pointer to ICMP6 frame header of 8 bytes
                         - Type    (1 byte)
                         - Code    (1 byte)
                         - Chksum  (2 bytes)
                         - EchoId  (2 bytes)
                         - EchoSeq (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_SendEchoReply(const void *icmp6_header) {
    EventRecordData (EvtNetICMP6_SendEchoReply, icmp6_header, 8);
  }
#else
  #define EvrNetICMP6_SendEchoReply(icmp6_header)
#endif

/**
  \brief  Event on ICMP6 echo reply received (Op)
  \param  data_length   length of the payload data
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_EchoReplyReceived(uint32_t data_length) {
    EventRecord2 (EvtNetICMP6_EchoReplyReceived, data_length, 0);
  }
#else
  #define EvrNetICMP6_EchoReplyReceived(data_length)
#endif

/**
  \brief  Event on ICMP6 echo reply received in wrong ping process state (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_EchoReplyWrongState(void) {
    EventRecord2 (EvtNetICMP6_EchoReplyWrongState, 0, 0);
  }
#else
  #define EvrNetICMP6_EchoReplyWrongState()
#endif

/**
  \brief  Event on ICMP6 wrong code in echo reply received (Error)
  \param  icmp6_code    wrong ICMP code value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_EchoReplyWrongCode(uint8_t icmp6_code) {
    EventRecord2 (EvtNetICMP6_EchoReplyWrongCode, icmp6_code, 0);
  }
#else
  #define EvrNetICMP6_EchoReplyWrongCode(icmp6_code)
#endif

/**
  \brief  Event on ICMP6 wrong source IPv6 address of echo reply (Error)
  \param  ip6_addr      pointer to IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_EchoReplyWrongIpAddress(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetICMP6_EchoReplyWrongIpAddress, ip6_addr, 16);
  }
#else
  #define EvrNetICMP6_EchoReplyWrongIpAddress(ip6_addr)
#endif

/**
  \brief  Event on ICMP6 wrong echo identifier value in echo reply (Error)
  \param  echo_id       wrong echo identifier value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_EchoReplyWrongId(uint16_t echo_id) {
    EventRecord2 (EvtNetICMP6_EchoReplyWrongId, echo_id, 0);
  }
#else
  #define EvrNetICMP6_EchoReplyWrongId(echo_id)
#endif

/**
  \brief  Event on ICMP6 echo reply payload data invalid (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_EchoReplyWrongPayload(void) {
    EventRecord2 (EvtNetICMP6_EchoReplyWrongPayload, 0, 0);
  }
#else
  #define EvrNetICMP6_EchoReplyWrongPayload()
#endif

/**
  \brief  Event on ICMP6 router solicitation message discarded (Op)
  \param  icmp6_type    ICMP6 message type
  \note   Hosts must silently discard RS messages [RFC4861 page 38]
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_MessageDiscarded(uint8_t icmp6_type) {
    EventRecord2 (EvtNetICMP6_MessageDiscarded, icmp6_type, 0);
  }
#else
  #define EvrNetICMP6_MessageDiscarded(icmp6_type)
#endif

/**
  \brief  Event on ICMP6 wrong hop limit in discover message (Error)
  \param  hop_limit     hop limit value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_WrongHopLimit(uint8_t hop_limit) {
    EventRecord2 (EvtNetICMP6_WrongHopLimit, hop_limit, 0);
  }
#else
  #define EvrNetICMP6_WrongHopLimit(hop_limit)
#endif

/**
  \brief  Event on ICMP6 wrong code in discover message (Error)
  \param  icmp6_code    code value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_WrongCode(uint8_t icmp6_code) {
    EventRecord2 (EvtNetICMP6_WrongCode, icmp6_code, 0);
  }
#else
  #define EvrNetICMP6_WrongCode(icmp6_code)
#endif

/**
  \brief  Event on ICMP6 message type unknown (Op)
  \param  icmp6_type    ICMP6 message type
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_MessageTypeUnknown(uint8_t icmp6_type) {
    EventRecord2 (EvtNetICMP6_MessageTypeUnknown, icmp6_type, 0);
  }
#else
  #define EvrNetICMP6_MessageTypeUnknown(icmp6_type)
#endif

/**
  \brief  Event on ICMP6 send frame (Op)
  \param  icmp6_header  pointer to ICMP6 frame header of 8 bytes
                         - Type    (1 byte)
                         - Code    (1 byte)
                         - Chksum  (2 bytes)
                         - EchoId  (2 bytes)
                         - EchoSeq (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_SendFrame(const void *icmp6_header) {
    EventRecordData (EvtNetICMP6_SendFrame, icmp6_header, 8);
  }
#else
  #define EvrNetICMP6_SendFrame(icmp6_header)
#endif

/**
  \brief  Event on ICMP6 \ref netPing_Echo request (API)
  \param  ip6_addr      pointer to IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_PingEcho(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetICMP6_PingEcho, ip6_addr, 16);
  }
#else
  #define EvrNetICMP6_PingEcho(ip6_addr)
#endif

/**
  \brief  Event on \ref netPing_EchoX target name not valid (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_PingTargetNotValid(void) {
    EventRecord2 (EvtNetICMP6_PingTargetNotValid, 0, 0);
  }
#else
  #define EvrNetICMP6_PingTargetNotValid()
#endif

/**
  \brief  Event on \ref netPing_EchoX host name resolver error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_PingDnsError(void) {
    EventRecord2 (EvtNetICMP6_PingDnsError, 0, 0);
  }
#else
  #define EvrNetICMP6_PingDnsError()
#endif

/**
  \brief  Event on ICMP6 \ref netPing_Echo invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_PingInvalidParameter(void) {
    EventRecord2 (EvtNetICMP6_PingInvalidParameter, 0, 0);
  }
#else
  #define EvrNetICMP6_PingInvalidParameter()
#endif

/**
  \brief  Event on ICMP6 \ref netPing_Echo client busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_PingClientBusy(void) {
    EventRecord2 (EvtNetICMP6_PingClientBusy, 0, 0);
  }
#else
  #define EvrNetICMP6_PingClientBusy()
#endif

/**
  \brief  Event on ICMP6 \ref netPing_Echo send echo request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_PingSendRequest(void) {
    EventRecord2 (EvtNetICMP6_PingSendRequest, 0, 0);
  }
#else
  #define EvrNetICMP6_PingSendRequest()
#endif

/**
  \brief  Event on ICMP6 retransmit echo request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_PingRetransmitRequest(void) {
    EventRecord2 (EvtNetICMP6_PingRetransmitRequest, 0, 0);
  }
#else
  #define EvrNetICMP6_PingRetransmitRequest()
#endif

/**
  \brief  Event on ICMP6 timeout on ping send echo request (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetICMP6_PingTimeout(void) {
    EventRecord2 (EvtNetICMP6_PingTimeout, 0, 0);
  }
#else
  #define EvrNetICMP6_PingTimeout()
#endif


// NetDHCP6 event identifiers --------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetDHCP6_InitClient              EventID (EventLevelOp,    EvtNetDHCP6,  0)
#define EvtNetDHCP6_GetSocketFailed         EventID (EventLevelError, EvtNetDHCP6,  1)
#define EvtNetDHCP6_StartClient             EventID (EventLevelOp,    EvtNetDHCP6,  2)
#define EvtNetDHCP6_StopClient              EventID (EventLevelOp,    EvtNetDHCP6,  3)
#define EvtNetDHCP6_ClientState             EventID (EventLevelOp,    EvtNetDHCP6,  4)
#define EvtNetDHCP6_NextState               EventID (EventLevelOp,    EvtNetDHCP6,  5)
#define EvtNetDHCP6_NextStateDelay          EventID (EventLevelOp,    EvtNetDHCP6,  6)
#define EvtNetDHCP6_ChangeStateLinkDown     EventID (EventLevelOp,    EvtNetDHCP6,  7)
#define EvtNetDHCP6_NoAddressAvailable      EventID (EventLevelOp,    EvtNetDHCP6,  8)
#define EvtNetDHCP6_StateRetransmit         EventID (EventLevelOp,    EvtNetDHCP6,  9)
#define EvtNetDHCP6_StateTimeout            EventID (EventLevelOp,    EvtNetDHCP6, 10)
#define EvtNetDHCP6_ChangeStateOnTimeout    EventID (EventLevelOp,    EvtNetDHCP6, 11)
#define EvtNetDHCP6_T1Expired               EventID (EventLevelOp,    EvtNetDHCP6, 12)
#define EvtNetDHCP6_T2Expired               EventID (EventLevelOp,    EvtNetDHCP6, 13)
#define EvtNetDHCP6_SendDhcpMessage         EventID (EventLevelOp,    EvtNetDHCP6, 14)
#define EvtNetDHCP6_ReceiveFrame            EventID (EventLevelOp,    EvtNetDHCP6, 15)
#define EvtNetDHCP6_FrameTooShort           EventID (EventLevelError, EvtNetDHCP6, 16)
#define EvtNetDHCP6_WrongTransactionId      EventID (EventLevelError, EvtNetDHCP6, 17)
#define EvtNetDHCP6_UnknownMessageType      EventID (EventLevelError, EvtNetDHCP6, 18)
#define EvtNetDHCP6_ShowMessage             EventID (EventLevelOp,    EvtNetDHCP6, 19)
#define EvtNetDHCP6_InvalidMessageType      EventID (EventLevelOp,    EvtNetDHCP6, 20)
#define EvtNetDHCP6_WrongClientId           EventID (EventLevelOp,    EvtNetDHCP6, 21)
#define EvtNetDHCP6_ServerNotChosen         EventID (EventLevelOp,    EvtNetDHCP6, 22)
#define EvtNetDHCP6_StatusCode              EventID (EventLevelOp,    EvtNetDHCP6, 23)
#define EvtNetDHCP6_WrongServerId           EventID (EventLevelOp,    EvtNetDHCP6, 24)
#define EvtNetDHCP6_ShowDnsServers          EventID (EventLevelOp,    EvtNetDHCP6, 25)
#define EvtNetDHCP6_MissingServerId         EventID (EventLevelOp,    EvtNetDHCP6, 26)
#define EvtNetDHCP6_WrongIaid               EventID (EventLevelOp,    EvtNetDHCP6, 27)
#define EvtNetDHCP6_ShowIanaOffer           EventID (EventLevelOp,    EvtNetDHCP6, 28)
#define EvtNetDHCP6_OfferTimerCheckFailed   EventID (EventLevelError, EvtNetDHCP6, 29)
#define EvtNetDHCP6_UninitClient            EventID (EventLevelOp,    EvtNetDHCP6, 30)
#endif

/**
  \brief  Event on DHCP6 client initialize (Op)
  \param  option_flags  DHCPv6 client option flags
                         - 0x01: Opt.67 Vendor Class
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_InitClient(uint8_t option_flags) {
    EventRecord2 (EvtNetDHCP6_InitClient, option_flags, 0);
  }
#else
  #define EvrNetDHCP6_InitClient(option_flags)
#endif

/**
  \brief  Event on DHCP6 client failed to allocate UDP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_GetSocketFailed(void) {
    EventRecord2 (EvtNetDHCP6_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetDHCP6_GetSocketFailed()
#endif

/**
  \brief  Event on DHCP6 client start (Op)
  \param  mode          client mode
                         - 0: Stateless mode
                         - 1: Stateful mode
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_StartClient(uint32_t mode) {
    EventRecord2 (EvtNetDHCP6_StartClient, mode, 0);
  }
#else
  #define EvrNetDHCP6_StartClient(mode)
#endif

/**
  \brief  Event on DHCP6 client stop (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_StopClient(void) {
    EventRecord2 (EvtNetDHCP6_StopClient, 0, 0);
  }
#else
  #define EvrNetDHCP6_StopClient()
#endif

/**
  \brief  Event on DHCP6 client indication of client state (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_ClientState(uint32_t state) {
    EventRecord2 (EvtNetDHCP6_ClientState, state, 0);
  }
#else
  #define EvrNetDHCP6_ClientState(state)
#endif

/**
  \brief  Event on DHCP6 client transition to next state (Op)
  \param  state         next state

 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_NextState(uint32_t state) {
    EventRecord2 (EvtNetDHCP6_NextState, state, 0);
  }
#else
  #define EvrNetDHCP6_NextState(state)
#endif

/**
  \brief  Event on DHCP6 client transition to next state with random delay(Op)
  \param  state         next state
  \param  delay         start delay (in 100ms ticks)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_NextStateDelay(uint32_t state, uint32_t delay) {
    EventRecord2 (EvtNetDHCP6_NextStateDelay, state, delay);
  }
#else
  #define EvrNetDHCP6_NextStateDelay(state, delay)
#endif

/**
  \brief  Event on DHCP6 client change state on link-down (Op)
  \param  state         next state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_ChangeStateLinkDown(uint32_t state) {
    EventRecord2 (EvtNetDHCP6_ChangeStateLinkDown, state, 0);
  }
#else
  #define EvrNetDHCP6_ChangeStateLinkDown(state)
#endif

/**
  \brief  Event on DHCP6 client no address available status received (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_NoAddressAvailable(uint32_t state) {
    EventRecord2 (EvtNetDHCP6_NoAddressAvailable, state, 0);
  }
#else
  #define EvrNetDHCP6_NoAddressAvailable(state)
#endif

/**
  \brief  Event on DHCP6 client retransmit request in current state (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_StateRetransmit(uint32_t state) {
    EventRecord2 (EvtNetDHCP6_StateRetransmit, state, 0);
  }
#else
  #define EvrNetDHCP6_StateRetransmit(state)
#endif

/**
  \brief  Event on DHCP6 client timeout in current state (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_StateTimeout(uint32_t state) {
    EventRecord2 (EvtNetDHCP6_StateTimeout, state, 0);
  }
#else
  #define EvrNetDHCP6_StateTimeout(state)
#endif

/**
  \brief  Event on DHCP6 client state timeout and transit to next state (Op)
  \param  current       current state
  \param  next          next state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_ChangeStateOnTimeout(uint32_t current, uint32_t next) {
    EventRecord2 (EvtNetDHCP6_ChangeStateOnTimeout, current, next);
  }
#else
  #define EvrNetDHCP6_ChangeStateOnTimeout(current, next)
#endif

/**
  \brief  Event on DHCP6 client timer T1 expired (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_T1Expired(uint32_t state) {
    EventRecord2 (EvtNetDHCP6_T1Expired, state, 0);
  }
#else
  #define EvrNetDHCP6_T1Expired(state)
#endif

/**
  \brief  Event on DHCP6 client timer T2 expired (Op)
  \param  state         current state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_T2Expired(uint32_t state) {
    EventRecord2 (EvtNetDHCP6_T2Expired, state, 0);
  }
#else
  #define EvrNetDHCP6_T2Expired(state)
#endif

/**
  \brief  Event on DHCP6 client send message (Op)
  \param  msg_type      message type
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_SendDhcpMessage(uint8_t msg_type) {
    EventRecord2 (EvtNetDHCP6_SendDhcpMessage, msg_type, 0);
  }
#else
  #define EvrNetDHCP6_SendDhcpMessage(msg_type)
#endif

/**
  \brief  Event on DHCP6 client receive frame (Op)
  \param  length        message length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_ReceiveFrame(uint32_t length) {
    EventRecord2 (EvtNetDHCP6_ReceiveFrame, length, 0);
  }
#else
  #define EvrNetDHCP6_ReceiveFrame(length)
#endif

/**
  \brief  Event on DHCP6 client received frame too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetDHCP6_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetDHCP6_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on DHCP6 client wrong transaction identifier (XID) received (Error)
  \param  xid           transaction identifier
  \param  xid_valid     expected/valid transaction identifier
  \note   The size of transaction identifier (XID) is 3 bytes.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_WrongTransactionId(uint32_t xid, uint32_t xid_valid) {
    EventRecord2 (EvtNetDHCP6_WrongTransactionId, xid, xid_valid);
  }
#else
  #define EvrNetDHCP6_WrongTransactionId(xid, xid_valid)
#endif

/**
  \brief  Event on DHCP6 client unknown message type received (Error)
  \param  msg_type      message type
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_UnknownMessageType(uint8_t msg_type) {
    EventRecord2 (EvtNetDHCP6_UnknownMessageType, msg_type, 0);
  }
#else
  #define EvrNetDHCP6_UnknownMessageType(msg_type)
#endif

/**
  \brief  Event on DHCP6 client display message information (Op)
  \param  msg_type      message type
  \param  xid           transaction identifier
  \note   The size of transaction identifier (XID) is 3 bytes.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_ShowMessage(uint8_t msg_type, uint32_t xid) {
    EventRecord2 (EvtNetDHCP6_ShowMessage, msg_type, xid);
  }
#else
  #define EvrNetDHCP6_ShowMessage(msg_type, xid)
#endif

/**
  \brief  Event on DHCP6 client invalid message type received (Op)
  \param  msg_type      message type
  \note   Silently discard other message types.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_InvalidMessageType(uint8_t msg_type) {
    EventRecord2 (EvtNetDHCP6_InvalidMessageType, msg_type, 0);
  }
#else
  #define EvrNetDHCP6_InvalidMessageType(msg_type)
#endif

/**
  \brief  Event on DHCP6 client wrong client identifier received (Op)
  \note   Silently discard the message.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_WrongClientId(void) {
    EventRecord2 (EvtNetDHCP6_WrongClientId, 0, 0);
  }
#else
  #define EvrNetDHCP6_WrongClientId()
#endif

/**
  \brief  Event on DHCP6 client server not chosen in DISCOVER, REBIND state (Op)
  \note   Silently discard the message.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_ServerNotChosen(void) {
    EventRecord2 (EvtNetDHCP6_ServerNotChosen, 0, 0);
  }
#else
  #define EvrNetDHCP6_ServerNotChosen()
#endif

/**
  \brief  Event on DHCP6 client option status code received (Op)
  \param  status        status code
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_StatusCode(uint16_t status) {
    EventRecord2 (EvtNetDHCP6_StatusCode, status, 0);
  }
#else
  #define EvrNetDHCP6_StatusCode(status)
#endif

/**
  \brief  Event on DHCP6 client wrong server identifier received (Op)
  \note   Silently discard the message.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_WrongServerId(void) {
    EventRecord2 (EvtNetDHCP6_WrongServerId, 0, 0);
  }
#else
  #define EvrNetDHCP6_WrongServerId()
#endif

/**
  \brief  Event on DHCP6 client display DNS server addresses (Op)
  \param  dns_list      pointer to DNS server address list
  \note   Displays primary and secondary DNS server address.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_ShowDnsServers(const uint8_t *dns_list) {
    EventRecordData (EvtNetDHCP6_ShowDnsServers, dns_list, 32);
  }
#else
  #define EvrNetDHCP6_ShowDnsServers(dns_list)
#endif

/**
  \brief  Event on DHCP6 client missing server identifier option (Op)
  \note   Silently discard the message.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_MissingServerId(void) {
    EventRecord2 (EvtNetDHCP6_MissingServerId, 0, 0);
  }
#else
  #define EvrNetDHCP6_MissingServerId()
#endif

/**
  \brief  Event on DHCP6 client wrong Identity Association Identifier (IAID) received (Op)
  \param  iaid          wrong IAID value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_WrongIaid(uint32_t iaid) {
    EventRecord2 (EvtNetDHCP6_WrongIaid, iaid, 0);
  }
#else
  #define EvrNetDHCP6_WrongIaid(iaid)
#endif

/**
  \brief  Event on DHCP6 client display IANA offer (Op)
  \param  iana_offer    pointer to IANA structure
  \note   IANA: Identity Association for non-temporary Address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_ShowIanaOffer(const void *iana_offer) {
    EventRecordData (EvtNetDHCP6_ShowIanaOffer, iana_offer, 32);
  }
#else
  #define EvrNetDHCP6_ShowIanaOffer(iana_offer)
#endif

/**
  \brief  Event on DHCP6 client offer address timer check failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_OfferTimerCheckFailed(void) {
    EventRecord2 (EvtNetDHCP6_OfferTimerCheckFailed, 0, 0);
  }
#else
  #define EvrNetDHCP6_OfferTimerCheckFailed()
#endif

/**
  \brief  Event on DHCP6 client de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDHCP6_UninitClient(void) {
    EventRecord2 (EvtNetDHCP6_UninitClient, 0, 0);
  }
#else
  #define EvrNetDHCP6_UninitClient()
#endif


// NetNDP event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetNDP_InitCache                 EventID (EventLevelOp,    EvtNetNDP,  0)
#define EvtNetNDP_FrameCheckFailed          EventID (EventLevelError, EvtNetNDP,  1)
#define EvtNetNDP_AddressNotLinkLocal       EventID (EventLevelOp,    EvtNetNDP,  2)
#define EvtNetNDP_RouterAdvertisement       EventID (EventLevelOp,    EvtNetNDP,  3)
#define EvtNetNDP_OptionMtu                 EventID (EventLevelOp,    EvtNetNDP,  4)
#define EvtNetNDP_OptionSourceMacAddress    EventID (EventLevelOp,    EvtNetNDP,  5)
#define EvtNetNDP_OptionPrefixInfo          EventID (EventLevelOp,    EvtNetNDP,  6)
#define EvtNetNDP_PrefixInfoNotValid        EventID (EventLevelOp,    EvtNetNDP,  7)
#define EvtNetNDP_WrongTargetAddress        EventID (EventLevelOp,    EvtNetNDP,  8)
#define EvtNetNDP_NeighborSolicitation      EventID (EventLevelOp,    EvtNetNDP,  9)
#define EvtNetNDP_ProbeRequest              EventID (EventLevelOp,    EvtNetNDP, 10)
#define EvtNetNDP_NeighborAdvertisement     EventID (EventLevelOp,    EvtNetNDP, 11)
#define EvtNetNDP_ProbeResponse             EventID (EventLevelOp,    EvtNetNDP, 12)
#define EvtNetNDP_WrongDestinationAddress   EventID (EventLevelOp,    EvtNetNDP, 13)
#define EvtNetNDP_SendReply                 EventID (EventLevelOp,    EvtNetNDP, 14)
#define EvtNetNDP_ShowFlags                 EventID (EventLevelOp,    EvtNetNDP, 15)
#define EvtNetNDP_SendRequest               EventID (EventLevelOp,    EvtNetNDP, 16)
#define EvtNetNDP_ShowTargetAddress         EventID (EventLevelOp,    EvtNetNDP, 17)
#define EvtNetNDP_OptionTargetMacAddress    EventID (EventLevelOp,    EvtNetNDP, 18)
#define EvtNetNDP_CacheEntryUpdate          EventID (EventLevelOp,    EvtNetNDP, 19)
#define EvtNetNDP_WrongResponse             EventID (EventLevelError, EvtNetNDP, 20)
#define EvtNetNDP_CreateTempAddress         EventID (EventLevelOp,    EvtNetNDP, 21)
#define EvtNetNDP_CacheIpRefreshed          EventID (EventLevelOp,    EvtNetNDP, 22)
#define EvtNetNDP_CacheAdd                  EventID (EventLevelOp,    EvtNetNDP, 23)
#define EvtNetNDP_RouterListFull            EventID (EventLevelOp,    EvtNetNDP, 24)
#define EvtNetNDP_CacheAllocFailed          EventID (EventLevelError, EvtNetNDP, 25)
#define EvtNetNDP_CacheEntryAdded           EventID (EventLevelOp,    EvtNetNDP, 26)
#define EvtNetNDP_CacheEntryDeleted         EventID (EventLevelOp,    EvtNetNDP, 27)
#define EvtNetNDP_CacheEntryRefreshed       EventID (EventLevelOp,    EvtNetNDP, 28)
#define EvtNetNDP_CacheFind                 EventID (EventLevelOp,    EvtNetNDP, 29)
#define EvtNetNDP_GatewayUnknown            EventID (EventLevelError, EvtNetNDP, 30)
#define EvtNetNDP_UsingGateway              EventID (EventLevelOp,    EvtNetNDP, 31)
#define EvtNetNDP_UnresolvedMacAddress      EventID (EventLevelError, EvtNetNDP, 32)
#define EvtNetNDP_EntryFound                EventID (EventLevelOp,    EvtNetNDP, 33)
#define EvtNetNDP_CacheIp                   EventID (EventLevelAPI,   EvtNetNDP, 34)
#define EvtNetNDP_CacheIpInvalidParameter   EventID (EventLevelError, EvtNetNDP, 35)
#define EvtNetNDP_GetIp                     EventID (EventLevelAPI,   EvtNetNDP, 36)
#define EvtNetNDP_GetIpInvalidParameter     EventID (EventLevelError, EvtNetNDP, 37)
#define EvtNetNDP_GetIpEntryNotFound        EventID (EventLevelOp,    EvtNetNDP, 38)
#define EvtNetNDP_GetMac                    EventID (EventLevelAPI,   EvtNetNDP, 39)
#define EvtNetNDP_GetMacInvalidParameter    EventID (EventLevelError, EvtNetNDP, 40)
#define EvtNetNDP_GetMacEntryNotFound       EventID (EventLevelOp,    EvtNetNDP, 41)
#define EvtNetNDP_Probe                     EventID (EventLevelAPI,   EvtNetNDP, 42)
#define EvtNetNDP_ProbeInvalidParameter     EventID (EventLevelError, EvtNetNDP, 43)
#define EvtNetNDP_ProbeBusy                 EventID (EventLevelError, EvtNetNDP, 44)
#define EvtNetNDP_ProbeRetransmit           EventID (EventLevelOp,    EvtNetNDP, 45)
#define EvtNetNDP_ProbeTimeout              EventID (EventLevelOp,    EvtNetNDP, 46)
#define EvtNetNDP_CacheEntryTimeout         EventID (EventLevelOp,    EvtNetNDP, 47)
#define EvtNetNDP_EntryReleased             EventID (EventLevelOp,    EvtNetNDP, 48)
#define EvtNetNDP_ResolveEntry              EventID (EventLevelOp,    EvtNetNDP, 49)
#define EvtNetNDP_RefreshEntry              EventID (EventLevelOp,    EvtNetNDP, 50)
#define EvtNetNDP_UninitCache               EventID (EventLevelOp,    EvtNetNDP, 51)
#endif

/**
  \brief  Event on NDP cache initialize (Op)
  \param  num_entries   number of entries available in NDP cache
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_InitCache(uint32_t num_entries) {
    EventRecord2 (EvtNetNDP_InitCache, num_entries, 0);
  }
#else
  #define EvrNetNDP_InitCache(num_entries)
#endif

/**
  \brief  Event on NDP discovery frame consistency check failed (Error)
  \param  msg_type      discovery message type
                         - 134: Router Advertisement
                         - 135: Neighbor Solicitation
                         - 136: Neighbor Advertisement
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_FrameCheckFailed(uint8_t msg_type) {
    EventRecord2 (EvtNetNDP_FrameCheckFailed, msg_type, 0);
  }
#else
  #define EvrNetNDP_FrameCheckFailed(msg_type)
#endif

/**
  \brief  Event on NDP received address not link-local (Op)
  \param  ip6_addr      pointer to IPv6 source address
  \note   Nodes must silently discard RA messages if src IP not link-local.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_AddressNotLinkLocal(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_AddressNotLinkLocal, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_AddressNotLinkLocal(ip6_addr)
#endif

/**
  \brief  Event on NDP Router Advertisement (RA) message received (Op)
  \param  hop_limit     current hop limit
  \param  flags         advertisement flags
                         - 0x80: M-flag (Managed address configuration)
                         - 0x40: O-flag (Other configuration)
  \param  lifetime      router lifetime in seconds
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_RouterAdvertisement(uint8_t hop_limit, uint8_t flags, uint16_t lifetime) {
    EventRecord4 (EvtNetNDP_RouterAdvertisement, hop_limit, flags, lifetime, 0);
  }
#else
  #define EvrNetNDP_RouterAdvertisement(hop_limit, flags, lifetime)
#endif

/**
  \brief  Event on NDP option Maximum transmission unit (MTU) (Op)
  \param  mtu           maximum transmission unit
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_OptionMtu(uint32_t mtu) {
    EventRecord2 (EvtNetNDP_OptionMtu, mtu, 0);
  }
#else
  #define EvrNetNDP_OptionMtu(mtu)
#endif

/**
  \brief  Event on NDP option Source link-layer (MAC) address (Op)
  \param  src_addr      source link-layer address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_OptionSourceMacAddress(const uint8_t *src_addr) {
    EventRecordData (EvtNetNDP_OptionSourceMacAddress, src_addr, 6);
  }
#else
  #define EvrNetNDP_OptionSourceMacAddress(src_addr)
#endif

/**
  \brief  Event on NDP RA-option address prefix info (Op)
  \param  opt_prefix    pointer to RA prefix option structure (32 bytes)
                         - Type:      1 byte
                         - Length:    1 byte
                         - PrefixLen: 1 byte
                         - Flags:     1 byte
                         - Valid:     4 bytes
                         - Preferred: 4 bytes
                         - Reserved:  4 bytes
                         - Prefix:   16 bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_OptionPrefixInfo(const uint8_t *opt_prefix) {
    EventRecordData (EvtNetNDP_OptionPrefixInfo, opt_prefix, 32);
  }
#else
  #define EvrNetNDP_OptionPrefixInfo(opt_prefix)
#endif

/**
  \brief  Event on NDP prefix information option not valid (Op)
  \note   Nodes must silently discard not valid RA messages.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_PrefixInfoNotValid(void) {
    EventRecord2 (EvtNetNDP_PrefixInfoNotValid, 0, 0);
  }
#else
  #define EvrNetNDP_PrefixInfoNotValid()
#endif

/**
  \brief  Event on NDP wrong target address received (Op)
  \param  targ_addr     pointer to target address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_WrongTargetAddress(const uint8_t *targ_addr) {
    EventRecordData (EvtNetNDP_WrongTargetAddress, targ_addr, 16);
  }
#else
  #define EvrNetNDP_WrongTargetAddress(targ_addr)
#endif

/**
  \brief  Event on NDP Neighbor Solicitation (NS) message received (Op)
  \param  targ_addr     pointer to target address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_NeighborSolicitation(const uint8_t *targ_addr) {
    EventRecordData (EvtNetNDP_NeighborSolicitation, targ_addr, 16);
  }
#else
  #define EvrNetNDP_NeighborSolicitation(targ_addr)
#endif

/**
  \brief  Event on NDP probe request received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ProbeRequest(void) {
    EventRecord2 (EvtNetNDP_ProbeRequest, 0, 0);
  }
#else
  #define EvrNetNDP_ProbeRequest()
#endif

/**
  \brief  Event on NDP Neighbor Advertisement (NA) message received (Op)
  \param  targ_addr     pointer to target address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_NeighborAdvertisement(const uint8_t *targ_addr) {
    EventRecordData (EvtNetNDP_NeighborAdvertisement, targ_addr, 16);
  }
#else
  #define EvrNetNDP_NeighborAdvertisement(targ_addr)
#endif

/**
  \brief  Event on NDP probe response received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ProbeResponse(void) {
    EventRecord2 (EvtNetNDP_ProbeResponse, 0, 0);
  }
#else
  #define EvrNetNDP_ProbeResponse()
#endif

/**
  \brief  Event on NDP wrong destination address received (Op)
  \param  dst_addr      pointer to destination address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_WrongDestinationAddress(const uint8_t *dst_addr) {
    EventRecordData (EvtNetNDP_WrongDestinationAddress, dst_addr, 16);
  }
#else
  #define EvrNetNDP_WrongDestinationAddress(dst_addr)
#endif

/**
  \brief  Event on NDP send advertisement reply to solicitation message (Op)
  \param  targ_addr     pointer to target address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_SendReply(const uint8_t *targ_addr) {
    EventRecordData (EvtNetNDP_SendReply, targ_addr, 16);
  }
#else
  #define EvrNetNDP_SendReply(targ_addr)
#endif

/**
  \brief  Event on NDP display advertisement message flags (Op)
  \param  flags         message flags
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ShowFlags(uint32_t flags) {
    EventRecord2 (EvtNetNDP_ShowFlags, flags, 0);
  }
#else
  #define EvrNetNDP_ShowFlags(flags)
#endif

/**
  \brief  Event on NDP send solicitation request (Op)
  \param  type          request type
                         - 133: Router Solicitation
                         - 135: Neighbor Solicitation
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_SendRequest(uint32_t type) {
    if (type == 255) type = 135;
    EventRecord2 (EvtNetNDP_SendRequest, type, 0);
  }
#else
  #define EvrNetNDP_SendRequest(type)
#endif

/**
  \brief  Event on NDP display target address (Op)
  \param  targ_addr     pointer to target address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ShowTargetAddress(const uint8_t *targ_addr) {
    EventRecordData (EvtNetNDP_ShowTargetAddress, targ_addr, 16);
  }
#else
  #define EvrNetNDP_ShowTargetAddress(targ_addr)
#endif

/**
  \brief  Event on NDP option Target link-layer (MAC) address (Op)
  \param  targ_addr     target link-layer address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_OptionTargetMacAddress(const uint8_t *targ_addr) {
    EventRecordData (EvtNetNDP_OptionTargetMacAddress, targ_addr, 6);
  }
#else
  #define EvrNetNDP_OptionTargetMacAddress(targ_addr)
#endif

/**
  \brief  Event on NDP cache entry update (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheEntryUpdate(uint32_t entry) {
    EventRecord2 (EvtNetNDP_CacheEntryUpdate, entry, 0);
  }
#else
  #define EvrNetNDP_CacheEntryUpdate(entry)
#endif

/**
  \brief  Event on NDP wrong response received (Error)
  \param  msg_type      discovery message type
                         - 136: Neighbor Advertisement
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_WrongResponse(uint8_t msg_type) {
    EventRecord2 (EvtNetNDP_WrongResponse, msg_type, 0);
  }
#else
  #define EvrNetNDP_WrongResponse(msg_type)
#endif

/**
  \brief  Event on NDP create temporary address (SLAAC) using prefix info (Op)
  \param  temp_addr     pointer to temporary address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CreateTempAddress(const uint8_t *temp_addr) {
    EventRecordData (EvtNetNDP_CreateTempAddress, temp_addr, 16);
  }
#else
  #define EvrNetNDP_CreateTempAddress(temp_addr)
#endif

/**
  \brief  Event on NDP cached IP address refreshed (Op)
  \param  ip6_addr      pointer to IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheIpRefreshed(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_CacheIpRefreshed, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_CacheIpRefreshed(ip6_addr)
#endif

/**
  \brief  Event on NDP add entry to NDP cache (Op)
  \param  ip6_addr      pointer to IPv6 address
  \param  mac_addr      pointer to MAC address
  \param  type          cache entry type:
                         - 0x01: Host
                         - 0x02: Router
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheAdd(const uint8_t *ip6_addr, const uint8_t *mac_addr, uint8_t type) {
    evr_buf.u8[22] = 0;
    evr_buf.u8[23] = type;
    memcpy (&evr_buf.u8[0],  ip6_addr, 16);
    memcpy (&evr_buf.u8[16], mac_addr, 6);
    EventRecordData (EvtNetNDP_CacheAdd, &evr_buf, 24);
  }
#else
  #define EvrNetNDP_CacheAdd(ip6_addr, mac_addr, type)
#endif

/**
  \brief  Event on NDP cache add router list full (Op)
  \param  max_count     maximum number of routers cached
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_RouterListFull(uint32_t max_count) {
    EventRecord2 (EvtNetNDP_RouterListFull, max_count, 0);
  }
#else
  #define EvrNetNDP_RouterListFull(max_count)
#endif

/**
  \brief  Event on NDP allocate free cache entry failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheAllocFailed(void) {
    EventRecord2 (EvtNetNDP_CacheAllocFailed, 0, 0);
  }
#else
  #define EvrNetNDP_CacheAllocFailed()
#endif

/**
  \brief  Event on NDP cache entry added (Op)
  \param  entry         cache entry number
  \param  timeout       entry timeout in seconds
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheEntryAdded(uint32_t entry, uint32_t timeout) {
    EventRecord2 (EvtNetNDP_CacheEntryAdded, entry, timeout);
  }
#else
  #define EvrNetNDP_CacheEntryAdded(entry, timeout)
#endif

/**
  \brief  Event on NDP cache entry deleted (Op)
  \param  entry         cache entry number
  \param  ip6_addr      pointer to IPv6 address
  \param  type          cache entry type:
                         - 0x01: Host
                         - 0x02: Router
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheEntryDeleted(uint32_t entry, const uint8_t *ip6_addr, uint8_t type) {
    evr_buf.u8[22] = (uint8_t)entry;
    evr_buf.u8[23] = (uint8_t)type;
    memcpy (&evr_buf.u8[0], ip6_addr, 16);
    EventRecordData (EvtNetNDP_CacheEntryDeleted, &evr_buf, 24);
  }
#else
  #define EvrNetNDP_CacheEntryDeleted(entry, ip6_addr, type)
#endif

/**
  \brief  Event on NDP cache entry refreshed (Op)
  \param  entry         cache entry number
  \param  ip6_addr      pointer to IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheEntryRefreshed(uint32_t entry, const uint8_t *ip6_addr) {
    evr_buf.u8[22] = (uint8_t)entry;
    memcpy (&evr_buf.u8[0], ip6_addr, 16);
    EventRecordData (EvtNetNDP_CacheEntryRefreshed, &evr_buf, 24);
  }
#else
  #define EvrNetNDP_CacheEntryRefreshed(entry, ip6_addr)
#endif

/**
  \brief  Event on NDP find IP address in cache (Op)
  \param  ip6_addr      pointer to IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheFind(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_CacheFind, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_CacheFind(ip6_addr)
#endif

/**
  \brief  Event on NDP cache add gateway unknown (Error)
  \param  ip6_addr      pointer to IPv6 address of a gateway
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_GatewayUnknown(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_GatewayUnknown, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_GatewayUnknown(ip6_addr)
#endif

/**
  \brief  Event on NDP using gateway to access external address (Op)
  \param  ip6_addr      pointer to IPv6 address of a gateway
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_UsingGateway(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_UsingGateway, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_UsingGateway(ip6_addr)
#endif

/**
  \brief  Event on NDP unresolved MAC address in cache (Error)
  \param  ip6_addr      pointer to requested IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_UnresolvedMacAddress(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_UnresolvedMacAddress, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_UnresolvedMacAddress(ip6_addr)
#endif

/**
  \brief  Event on NDP cache entry found (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_EntryFound(uint32_t entry) {
    EventRecord2 (EvtNetNDP_EntryFound, entry, 0);
  }
#else
  #define EvrNetNDP_EntryFound(entry)
#endif

/**
  \brief  Event on \ref netNDP_CacheIP start (API)
  \param  ip6_addr      pointer to requested IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheIp(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_CacheIp, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_CacheIp(ip6_addr)
#endif

/**
  \brief  Event on \ref netNDP_CacheIP invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheIpInvalidParameter(void) {
    EventRecord2 (EvtNetNDP_CacheIpInvalidParameter, 0, 0);
  }
#else
  #define EvrNetNDP_CacheIpInvalidParameter()
#endif

/**
  \brief  Event on \ref netNDP_GetIP resolving success (API)
  \param  mac_addr      pointer to requested MAC address
  \param  ip6_addr      pointer to resolved IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_GetIp(const uint8_t *mac_addr, const uint8_t *ip6_addr) {
    memcpy (&evr_buf.u8[0], ip6_addr, 16);
    memcpy (&evr_buf.u8[16], mac_addr, 6);
    EventRecordData (EvtNetNDP_GetIp, &evr_buf, 22);
  }
#else
  #define EvrNetNDP_GetIp(mac_addr, ip6_addr)
#endif

/**
  \brief  Event on \ref netNDP_GetIP invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_GetIpInvalidParameter(void) {
    EventRecord2 (EvtNetNDP_GetIpInvalidParameter, 0, 0);
  }
#else
  #define EvrNetNDP_GetIpInvalidParameter()
#endif

/**
  \brief  Event on \ref netNDP_GetIP entry with MAC address not found in cache (Op)
  \param  mac_addr      pointer to requested MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_GetIpEntryNotFound(const uint8_t *mac_addr) {
    EventRecordData (EvtNetNDP_GetIpEntryNotFound, mac_addr, 6);
  }
#else
  #define EvrNetNDP_GetIpEntryNotFound(mac_addr)
#endif

/**
  \brief  Event on \ref netNDP_GetMAC resolving success (API)
  \param  ip6_addr      pointer to requested IPv6 address
  \param  mac_addr      pointer to resolved MAC address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_GetMac(const uint8_t *ip6_addr, const uint8_t *mac_addr) {
    memcpy (&evr_buf.u8[0], ip6_addr, 16);
    memcpy (&evr_buf.u8[16], mac_addr, 6);
    EventRecordData (EvtNetNDP_GetMac, &evr_buf, 22);
  }
#else
  #define EvrNetNDP_GetMac(ip6_addr, mac_addr)
#endif

/**
  \brief  Event on \ref netNDP_GetMAC invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_GetMacInvalidParameter(void) {
    EventRecord2 (EvtNetNDP_GetMacInvalidParameter, 0, 0);
  }
#else
  #define EvrNetNDP_GetMacInvalidParameter()
#endif

/**
  \brief  Event on \ref netNDP_GetMAC entry with IP address not found in cache (Op)
  \param  ip6_addr      pointer to requested IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_GetMacEntryNotFound(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_GetMacEntryNotFound, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_GetMacEntryNotFound(ip6_addr)
#endif

/**
  \brief  Event on \ref netNDP_Probe start (API)
  \param  ip6_addr      pointer to requested IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_Probe(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_Probe, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_Probe(ip6_addr)
#endif

/**
  \brief  Event on \ref netNDP_Probe invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ProbeInvalidParameter(void) {
    EventRecord2 (EvtNetNDP_ProbeInvalidParameter, 0, 0);
  }
#else
  #define EvrNetNDP_ProbeInvalidParameter()
#endif

/**
  \brief  Event on \ref netNDP_Probe process busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ProbeBusy(void) {
    EventRecord2 (EvtNetNDP_ProbeBusy, 0, 0);
  }
#else
  #define EvrNetNDP_ProbeBusy()
#endif

/**
  \brief  Event on NDP probe retransmit (Op)
  \param  ip6_addr      pointer to IPv6 address of probed host
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ProbeRetransmit(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_ProbeRetransmit, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_ProbeRetransmit(ip6_addr)
#endif

/**
  \brief  Event on NDP probe timeout (Op)
  \param  ip6_addr      pointer to IPv6 address of probed host
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ProbeTimeout(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetNDP_ProbeTimeout, ip6_addr, 16);
  }
#else
  #define EvrNetNDP_ProbeTimeout(ip6_addr)
#endif

/**
  \brief  Event on NDP cache entry timeout (Op)
  \param  entry         cache entry number
  \param  ip6_addr      pointer to IPv6 address
  \param  mac_addr      pointer to MAC address
  \param  type          cache entry type:
                         - 0x01: Host
                         - 0x02: Router
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_CacheEntryTimeout(uint32_t entry, const uint8_t *ip6_addr,
                                                   const uint8_t *mac_addr, uint32_t type) {
    evr_buf.u8[22] = (uint8_t)entry;
    evr_buf.u8[23] = (uint8_t)type & 0x03;
    memcpy (&evr_buf.u8[0], ip6_addr, 16);
    memcpy (&evr_buf.u8[16], mac_addr, 6);
    EventRecordData (EvtNetNDP_CacheEntryTimeout, &evr_buf, 24);
  }
#else
  #define EvrNetNDP_CacheEntryTimeout(entry, type, ip6_addr, mac_addr)
#endif

/**
  \brief  Event on NDP cache entry released (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_EntryReleased(uint32_t entry) {
    EventRecord2 (EvtNetNDP_EntryReleased, entry, 0);
  }
#else
  #define EvrNetNDP_EntryReleased(entry)
#endif

/**
  \brief  Event on resolve NDP cache entry (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_ResolveEntry(uint32_t entry) {
    EventRecord2 (EvtNetNDP_ResolveEntry, entry, 0);
  }
#else
  #define EvrNetNDP_ResolveEntry(entry)
#endif

/**
  \brief  Event on refresh NDP cache entry (Op)
  \param  entry         cache entry number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_RefreshEntry(uint32_t entry) {
    EventRecord2 (EvtNetNDP_RefreshEntry, entry, 0);
  }
#else
  #define EvrNetNDP_RefreshEntry(entry)
#endif

/**
  \brief  Event on NDP cache de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetNDP_UninitCache(void) {
    EventRecord2 (EvtNetNDP_UninitCache, 0, 0);
  }
#else
  #define EvrNetNDP_UninitCache()
#endif


// NetUDP event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetUDP_InitSockets               EventID (EventLevelOp,    EvtNetUDP,  0)
#define EvtNetUDP_GetSocketInvalidParameter EventID (EventLevelError, EvtNetUDP,  1)
#define EvtNetUDP_SocketAllocated           EventID (EventLevelOp,    EvtNetUDP,  2)
#define EvtNetUDP_NoSocketsAvailable        EventID (EventLevelError, EvtNetUDP,  3)
#define EvtNetUDP_ReleaseSocket             EventID (EventLevelOp,    EvtNetUDP,  4)
#define EvtNetUDP_ReleaseSocketNotValid     EventID (EventLevelError, EvtNetUDP,  5)
#define EvtNetUDP_ReleaseSocketWrongState   EventID (EventLevelError, EvtNetUDP,  6)
#define EvtNetUDP_OpenSocket                EventID (EventLevelOp,    EvtNetUDP,  7)
#define EvtNetUDP_OpenSocketNotValid        EventID (EventLevelError, EvtNetUDP,  8)
#define EvtNetUDP_OpenSocketWrongState      EventID (EventLevelError, EvtNetUDP,  9)
#define EvtNetUDP_OpenLocalPortAssigned     EventID (EventLevelOp,    EvtNetUDP, 10)
#define EvtNetUDP_OpenLocalPortInUse        EventID (EventLevelError, EvtNetUDP, 11)
#define EvtNetUDP_CloseSocket               EventID (EventLevelOp,    EvtNetUDP, 12)
#define EvtNetUDP_CloseSocketNotValid       EventID (EventLevelError, EvtNetUDP, 13)
#define EvtNetUDP_SetOptionSocket           EventID (EventLevelOp,    EvtNetUDP, 14)
#define EvtNetUDP_SetOptionSocketNotValid   EventID (EventLevelError, EvtNetUDP, 15)
#define EvtNetUDP_SetOptionSocketWrongState EventID (EventLevelError, EvtNetUDP, 16)
#define EvtNetUDP_SetOptionTos              EventID (EventLevelOp,    EvtNetUDP, 17)
#define EvtNetUDP_SetOptionTtl              EventID (EventLevelOp,    EvtNetUDP, 18)
#define EvtNetUDP_SetOptionTclass           EventID (EventLevelOp,    EvtNetUDP, 19)
#define EvtNetUDP_SetOptionHopLimit         EventID (EventLevelOp,    EvtNetUDP, 20)
#define EvtNetUDP_SetOptionChecksum         EventID (EventLevelOp,    EvtNetUDP, 21)
#define EvtNetUDP_SetOptionWrongOption      EventID (EventLevelError, EvtNetUDP, 22)
#define EvtNetUDP_SetOptionWrongValue       EventID (EventLevelError, EvtNetUDP, 23)
#define EvtNetUDP_GetBufferFailed           EventID (EventLevelError, EvtNetUDP, 24)
#define EvtNetUDP_SendFrame                 EventID (EventLevelOp,    EvtNetUDP, 25)
#define EvtNetUDP_SendBufferInvalid         EventID (EventLevelError, EvtNetUDP, 26)
#define EvtNetUDP_SendSocketNotValid        EventID (EventLevelError, EvtNetUDP, 27)
#define EvtNetUDP_SendAddressUnspecified    EventID (EventLevelError, EvtNetUDP, 28)
#define EvtNetUDP_SendPortUndefined         EventID (EventLevelError, EvtNetUDP, 29)
#define EvtNetUDP_SendSocketNotOpen         EventID (EventLevelError, EvtNetUDP, 30)
#define EvtNetUDP_SendZeroLengthFrame       EventID (EventLevelOp,    EvtNetUDP, 31)
#define EvtNetUDP_SendSizeTruncated         EventID (EventLevelOp,    EvtNetUDP, 32)
#define EvtNetUDP_ShowFrameHeader           EventID (EventLevelDetail,EvtNetUDP, 33)
#define EvtNetUDP_ReceiveFrame              EventID (EventLevelOp,    EvtNetUDP, 34)
#define EvtNetUDP_FrameTooShort             EventID (EventLevelError, EvtNetUDP, 35)
#define EvtNetUDP_MappedToSocket            EventID (EventLevelOp,    EvtNetUDP, 36)
#define EvtNetUDP_FrameNotMapped            EventID (EventLevelOp,    EvtNetUDP, 37)
#define EvtNetUDP_ChecksumFailed            EventID (EventLevelError, EvtNetUDP, 38)
#define EvtNetUDP_UninitSockets             EventID (EventLevelOp,    EvtNetUDP, 39)
#endif

/**
  \brief  Event on UDP socket initialize (Op)
  \param  num_sockets   number of available UDP sockets
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_InitSockets(uint32_t num_sockets) {
    EventRecord2 (EvtNetUDP_InitSockets, num_sockets, 0);
  }
#else
  #define EvrNetUDP_InitSockets(num_sockets)
#endif

/**
  \brief  Event on UDP get socket failed, invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_GetSocketInvalidParameter(void) {
    EventRecord2 (EvtNetUDP_GetSocketInvalidParameter, 0, 0);
  }
#else
  #define EvrNetUDP_GetSocketInvalidParameter()
#endif

/**
  \brief  Event on UDP free socket allocated (Op)
  \param  socket        allocated socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SocketAllocated(int32_t socket) {
    EventRecord2 (EvtNetUDP_SocketAllocated, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SocketAllocated(socket)
#endif

/**
  \brief  Event on UDP get socket failed, no free sockets available (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_NoSocketsAvailable(void) {
    EventRecord2 (EvtNetUDP_NoSocketsAvailable, 0, 0);
  }
#else
  #define EvrNetUDP_NoSocketsAvailable()
#endif

/**
  \brief  Event on UDP release socket (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_ReleaseSocket(int32_t socket) {
    EventRecord2 (EvtNetUDP_ReleaseSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_ReleaseSocket(socket)
#endif

/**
  \brief  Event on UDP release failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_ReleaseSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetUDP_ReleaseSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_ReleaseSocketNotValid(socket)
#endif

/**
  \brief  Event on UDP release failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_ReleaseSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetUDP_ReleaseSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_ReleaseSocketWrongState(socket)
#endif

/**
  \brief  Event on UDP open socket for communication (Op)
  \param  socket        socket handle
  \param  port          local port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_OpenSocket(int32_t socket, uint16_t port) {
    EventRecord2 (EvtNetUDP_OpenSocket, (uint32_t)socket, port);
  }
#else
  #define EvrNetUDP_OpenSocket(socket, port)
#endif

/**
  \brief  Event on UDP open failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_OpenSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetUDP_OpenSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_OpenSocketNotValid(socket)
#endif

/**
  \brief  Event on UDP open socket failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_OpenSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetUDP_OpenSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_OpenSocketWrongState(socket)
#endif

/**
  \brief  Event on UDP local port assigned in open socket request (Op)
  \param  local_port    assigned local port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_OpenLocalPortAssigned(uint16_t local_port) {
    EventRecord2 (EvtNetUDP_OpenLocalPortAssigned, local_port, 0);
  }
#else
  #define EvrNetUDP_OpenLocalPortAssigned(local_port)
#endif

/**
  \brief  Event on UDP open socket failed, requested port already used (Error)
  \param  socket        socket handle
  \param  port          local port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_OpenLocalPortInUse(int32_t socket, uint16_t port) {
    EventRecord2 (EvtNetUDP_OpenLocalPortInUse, (uint32_t)socket, port);
  }
#else
  #define EvrNetUDP_OpenLocalPortInUse(socket, port)
#endif

/**
  \brief  Event on UDP close socket (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_CloseSocket(int32_t socket) {
    EventRecord2 (EvtNetUDP_CloseSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_CloseSocket(socket)
#endif

/**
  \brief  Event on UDP close socket failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_CloseSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetUDP_CloseSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_CloseSocketNotValid(socket)
#endif

/**
  \brief  Event on UDP set socket option (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionSocket(int32_t socket) {
    EventRecord2 (EvtNetUDP_SetOptionSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SetOptionSocket(socket)
#endif

/**
  \brief  Event on UDP set socket option failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetUDP_SetOptionSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SetOptionSocketNotValid(socket)
#endif

/**
  \brief  Event on UDP set socket option failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetUDP_SetOptionSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SetOptionSocketWrongState(socket)
#endif

/**
  \brief  Event on UDP set socket option Type Of Service for IPv4 (Op)
  \param  ip4_tos       type of service value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionTos(uint8_t ip4_tos) {
    EventRecord2 (EvtNetUDP_SetOptionTos, ip4_tos, 0);
  }
#else
  #define EvrNetUDP_SetOptionTos(ip4_tos)
#endif

/**
  \brief  Event on UDP set socket option Time To Live for IPv4 (Op)
  \param  ip4_ttl       time to live value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionTtl(uint8_t ip4_ttl) {
    EventRecord2 (EvtNetUDP_SetOptionTtl, ip4_ttl, 0);
  }
#else
  #define EvrNetUDP_SetOptionTtl(ip4_ttl)
#endif

/**
  \brief  Event on UDP set socket option Traffic Class for IPv6 (Op)
  \param  ip6_tclass    traffic class value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionTclass(uint8_t ip6_tclass) {
    EventRecord2 (EvtNetUDP_SetOptionTclass, ip6_tclass, 0);
  }
#else
  #define EvrNetUDP_SetOptionTclass(ip6_tclass)
#endif

/**
  \brief  Event on UDP set socket option Hop Limit for IPv6 (Op)
  \param  ip6_hoplimit  hop limit value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionHopLimit(uint8_t ip6_hoplimit) {
    EventRecord2 (EvtNetUDP_SetOptionHopLimit, ip6_hoplimit, 0);
  }
#else
  #define EvrNetUDP_SetOptionHopLimit(ip6_hoplimit)
#endif

/**
  \brief  Event on UDP set socket option checksum calculation options (Op)
  \param  checksum      checksum calculation options bitmap
                         - 0x01: calculate checksum for send frames
                         - 0x02: verify checksum for received frames
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionChecksum(uint8_t checksum) {
    EventRecord2 (EvtNetUDP_SetOptionChecksum, checksum & 1, (checksum >> 1) & 1);
  }
#else
  #define EvrNetUDP_SetOptionChecksum(checksum)
#endif

/**
  \brief  Event on UDP wrong set socket option (Error)
  \param  socket        socket handle
  \param  udp_option    wrong set option value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionWrongOption(int32_t socket, uint8_t udp_option) {
    EventRecord2 (EvtNetUDP_SetOptionWrongOption, (uint32_t)socket, udp_option);
  }
#else
  #define EvrNetUDP_SetOptionWrongOption(socket, udp_option)
#endif

/**
  \brief  Event on UDP wrong value for set socket option (Error)
  \param  socket        socket handle
  \param  opt_value     wrong value for set option
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SetOptionWrongValue(int32_t socket, uint8_t opt_value) {
    EventRecord2 (EvtNetUDP_SetOptionWrongValue, (uint32_t)socket, opt_value);
  }
#else
  #define EvrNetUDP_SetOptionWrongValue(socket, opt_value)
#endif

/**
  \brief  Event on UDP get buffer failed, out of memory (Error)
  \param  mem_size      requested memory size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_GetBufferFailed(uint16_t mem_size) {
    EventRecord2 (EvtNetUDP_GetBufferFailed, mem_size, 0);
  }
#else
  #define EvrNetUDP_GetBufferFailed(mem_size)
#endif

/**
  \brief  Event on UDP send frame (Op)
  \param  socket        socket handle
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SendFrame(int32_t socket, uint32_t length) {
    EventRecord2 (EvtNetUDP_SendFrame, (uint32_t)socket, length);
  }
#else
  #define EvrNetUDP_SendFrame(socket, length)
#endif

/**
  \brief  Event on UDP send buffer is invalid (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SendBufferInvalid(int32_t socket) {
    EventRecord2 (EvtNetUDP_SendBufferInvalid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SendBufferInvalid(socket)
#endif

/**
  \brief  Event on UDP send socket handle is not valid (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SendSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetUDP_SendSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SendSocketNotValid(socket)
#endif

/**
  \brief  Event on UDP send destination address not specified (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SendAddressUnspecified(int32_t socket) {
    EventRecord2 (EvtNetUDP_SendAddressUnspecified, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SendAddressUnspecified(socket)
#endif

/**
  \brief  Event on UDP send port undefined (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SendPortUndefined(int32_t socket) {
    EventRecord2 (EvtNetUDP_SendPortUndefined, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SendPortUndefined(socket)
#endif

/**
  \brief  Event on UDP send socket not open, invalid socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SendSocketNotOpen(int32_t socket) {
    EventRecord2 (EvtNetUDP_SendSocketNotOpen, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SendSocketNotOpen(socket)
#endif

/**
  \brief  Event on UDP send 0-length frame, frame is dumped (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SendZeroLengthFrame(int32_t socket) {
    EventRecord2 (EvtNetUDP_SendZeroLengthFrame, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_SendZeroLengthFrame(socket)
#endif

/**
  \brief  Event on UDP send truncated over-size packet (Op)
  \param  new_length    truncated frame length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_SendSizeTruncated(uint32_t new_length) {
    EventRecord2 (EvtNetUDP_SendSizeTruncated, new_length, 0);
  }
#else
  #define EvrNetUDP_SendSizeTruncated(new_length)
#endif

/**
  \brief  Event on UDP display send/receive frame header (Detail)
  \param  udp_header    pointer to UDP frame header of 8 bytes
                         - SrcPort (2 bytes)
                         - DstPort (2 bytes)
                         - Len     (2 bytes)
                         - Chksum  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_ShowFrameHeader(const void *udp_header) {
    EventRecordData (EvtNetUDP_ShowFrameHeader, udp_header, 8);
  }
#else
  #define EvrNetUDP_ShowFrameHeader(udp_header)
#endif

/**
  \brief  Event on UDP receive frame (Op)
  \param  length        frame length in bytes
  \param  ip_version    internet protocol version
                         - 0: IPv4
                         - 1: IPv6
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_ReceiveFrame(uint32_t length, uint8_t ip_version) {
    EventRecord2 (EvtNetUDP_ReceiveFrame, length, ip_version);
  }
#else
  #define EvrNetUDP_ReceiveFrame(length, ip_version)
#endif

/**
  \brief  Event on UDP received frame too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetUDP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetUDP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on UDP received frame mapped to a socket (Op)
  \param  socket        mapped socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_MappedToSocket(int32_t socket) {
    EventRecord2 (EvtNetUDP_MappedToSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_MappedToSocket(socket)
#endif

/**
  \brief  Event on UDP received frame not mapped, no open sockets found (Op)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_FrameNotMapped(uint32_t length) {
    EventRecord2 (EvtNetUDP_FrameNotMapped, length, 0);
  }
#else
  #define EvrNetUDP_FrameNotMapped(length)
#endif

/**
  \brief  Event on UDP frame checksum check failed (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_ChecksumFailed(int32_t socket) {
    EventRecord2 (EvtNetUDP_ChecksumFailed, (uint32_t)socket, 0);
  }
#else
  #define EvrNetUDP_ChecksumFailed(socket)
#endif

/**
  \brief  Event on UDP socket de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetUDP_UninitSockets(void) {
    EventRecord2 (EvtNetUDP_UninitSockets, 0, 0);
  }
#else
  #define EvrNetUDP_UninitSockets()
#endif


// NetTCP event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetTCP_InitSockets               EventID (EventLevelOp,    EvtNetTCP,  0)
#define EvtNetTCP_GetSocketInvalidParameter EventID (EventLevelError, EvtNetTCP,  1)
#define EvtNetTCP_SocketAllocated           EventID (EventLevelOp,    EvtNetTCP,  2)
#define EvtNetTCP_NoSocketsAvailable        EventID (EventLevelError, EvtNetTCP,  3)
#define EvtNetTCP_ReleaseSocket             EventID (EventLevelOp,    EvtNetTCP,  4)
#define EvtNetTCP_ReleaseSocketNotValid     EventID (EventLevelError, EvtNetTCP,  5)
#define EvtNetTCP_ReleaseSocketWrongState   EventID (EventLevelError, EvtNetTCP,  6)
#define EvtNetTCP_ListenSocket              EventID (EventLevelOp,    EvtNetTCP,  7)
#define EvtNetTCP_ListenSocketNotValid      EventID (EventLevelError, EvtNetTCP,  8)
#define EvtNetTCP_ListenPortUndefined       EventID (EventLevelError, EvtNetTCP,  9)
#define EvtNetTCP_ListenSocketWrongState    EventID (EventLevelError, EvtNetTCP, 10)
#define EvtNetTCP_ConnectSocket             EventID (EventLevelOp,    EvtNetTCP, 11)
#define EvtNetTCP_ConnectSocketNotValid     EventID (EventLevelError, EvtNetTCP, 12)
#define EvtNetTCP_ConnectAddressUnspecified EventID (EventLevelError, EvtNetTCP, 13)
#define EvtNetTCP_ConnectPortUndefined      EventID (EventLevelError, EvtNetTCP, 14)
#define EvtNetTCP_ShowNetAddressIp4         EventID (EventLevelOp,    EvtNetTCP, 15)
#define EvtNetTCP_ShowNetAddressIp6         EventID (EventLevelOp,    EvtNetTCP, 16)
#define EvtNetTCP_ConnectLocalPortInvalid   EventID (EventLevelError, EvtNetTCP, 17)
#define EvtNetTCP_ConnectLocalPortAssigned  EventID (EventLevelOp,    EvtNetTCP, 18)
#define EvtNetTCP_ConnectSocketWrongState   EventID (EventLevelError, EvtNetTCP, 19)
#define EvtNetTCP_ShowRttVariables          EventID (EventLevelDetail,EvtNetTCP, 20)
#define EvtNetTCP_GetBufferFailed           EventID (EventLevelError, EvtNetTCP, 21)
#define EvtNetTCP_SendFrame                 EventID (EventLevelOp,    EvtNetTCP, 22)
#define EvtNetTCP_SendBufferInvalid         EventID (EventLevelError, EvtNetTCP, 23)
#define EvtNetTCP_SendSocketNotValid        EventID (EventLevelError, EvtNetTCP, 24)
#define EvtNetTCP_SendSocketNotConnected    EventID (EventLevelError, EvtNetTCP, 25)
#define EvtNetTCP_SendSocketClosing         EventID (EventLevelError, EvtNetTCP, 26)
#define EvtNetTCP_SendReenteredCall         EventID (EventLevelError, EvtNetTCP, 27)
#define EvtNetTCP_SendDataUnacked           EventID (EventLevelError, EvtNetTCP, 28)
#define EvtNetTCP_SendMssExceeded           EventID (EventLevelError, EvtNetTCP, 29)
#define EvtNetTCP_SendZeroLengthFrame       EventID (EventLevelOp,    EvtNetTCP, 30)
#define EvtNetTCP_CloseSocket               EventID (EventLevelOp,    EvtNetTCP, 31)
#define EvtNetTCP_CloseSocketNotValid       EventID (EventLevelError, EvtNetTCP, 32)
#define EvtNetTCP_CloseDataUnacked          EventID (EventLevelOp,    EvtNetTCP, 33)
#define EvtNetTCP_CloseSocketWrongState     EventID (EventLevelError, EvtNetTCP, 34)
#define EvtNetTCP_AbortSocket               EventID (EventLevelOp,    EvtNetTCP, 35)
#define EvtNetTCP_AbortSocketNotValid       EventID (EventLevelError, EvtNetTCP, 36)
#define EvtNetTCP_AbortSocketWrongState     EventID (EventLevelError, EvtNetTCP, 37)
#define EvtNetTCP_SendReadySocketNotValid   EventID (EventLevelError, EvtNetTCP, 38)
#define EvtNetTCP_SendReadyReenteredCall    EventID (EventLevelError, EvtNetTCP, 39)
#define EvtNetTCP_ResetWindowSocketNotValid EventID (EventLevelError, EvtNetTCP, 40)
#define EvtNetTCP_ResetWindowNotConnected   EventID (EventLevelError, EvtNetTCP, 41)
#define EvtNetTCP_ResetWindowNoFlowControl  EventID (EventLevelError, EvtNetTCP, 42)
#define EvtNetTCP_ResetWindowUpdate         EventID (EventLevelOp,    EvtNetTCP, 43)
#define EvtNetTCP_SetOptionSocket           EventID (EventLevelOp,    EvtNetTCP, 44)
#define EvtNetTCP_SetOptionSocketNotValid   EventID (EventLevelError, EvtNetTCP, 45)
#define EvtNetTCP_SetOptionSocketWrongState EventID (EventLevelError, EvtNetTCP, 46)
#define EvtNetTCP_SetOptionTos              EventID (EventLevelOp,    EvtNetTCP, 47)
#define EvtNetTCP_SetOptionTclass           EventID (EventLevelOp,    EvtNetTCP, 48)
#define EvtNetTCP_SetOptionTimeout          EventID (EventLevelOp,    EvtNetTCP, 49)
#define EvtNetTCP_SetOptionKeepAlive        EventID (EventLevelOp,    EvtNetTCP, 50)
#define EvtNetTCP_SetOptionFlowControl      EventID (EventLevelOp,    EvtNetTCP, 51)
#define EvtNetTCP_SetOptionDelayedAck       EventID (EventLevelOp,    EvtNetTCP, 52)
#define EvtNetTCP_SetOptionWrongOption      EventID (EventLevelError, EvtNetTCP, 53)
#define EvtNetTCP_SetOptionWrongValue       EventID (EventLevelError, EvtNetTCP, 54)
#define EvtNetTCP_SendDelayedAck            EventID (EventLevelOp,    EvtNetTCP, 55)
#define EvtNetTCP_SendKeepAliveProbe        EventID (EventLevelOp,    EvtNetTCP, 56)
#define EvtNetTCP_KeepAliveTimeoutClosing   EventID (EventLevelOp,    EvtNetTCP, 57)
#define EvtNetTCP_CallbackEventAck          EventID (EventLevelOp,    EvtNetTCP, 58)
#define EvtNetTCP_ResendOnTimeout           EventID (EventLevelOp,    EvtNetTCP, 59)
#define EvtNetTCP_ShowCongestionVariables   EventID (EventLevelDetail,EvtNetTCP, 60)
#define EvtNetTCP_TimeoutInState            EventID (EventLevelOp,    EvtNetTCP, 61)
#define EvtNetTCP_TwaitTimeoutClosing       EventID (EventLevelOp,    EvtNetTCP, 62)
#define EvtNetTCP_ClosingTimeout            EventID (EventLevelOp,    EvtNetTCP, 63)
#define EvtNetTCP_NoRetriesLeft             EventID (EventLevelError, EvtNetTCP, 64)
#define EvtNetTCP_ReceiveFrame              EventID (EventLevelOp,    EvtNetTCP, 65)
#define EvtNetTCP_FrameTooShort             EventID (EventLevelError, EvtNetTCP, 66)
#define EvtNetTCP_MappedToSocket            EventID (EventLevelOp,    EvtNetTCP, 67)
#define EvtNetTCP_FrameNotMapped            EventID (EventLevelOp,    EvtNetTCP, 68)
#define EvtNetTCP_ChecksumFailed            EventID (EventLevelError, EvtNetTCP, 69)
#define EvtNetTCP_ShowFrameHeader           EventID (EventLevelDetail,EvtNetTCP, 70)
#define EvtNetTCP_RstInWindow               EventID (EventLevelOp,    EvtNetTCP, 71)
#define EvtNetTCP_RstNotValid               EventID (EventLevelOp,    EvtNetTCP, 72)
#define EvtNetTCP_RepeatedSynAck            EventID (EventLevelOp,    EvtNetTCP, 73)
#define EvtNetTCP_AckNotSet                 EventID (EventLevelOp,    EvtNetTCP, 74)
#define EvtNetTCP_ShowSendWindow            EventID (EventLevelOp,    EvtNetTCP, 75)
#define EvtNetTCP_KeepAliveSegment          EventID (EventLevelOp,    EvtNetTCP, 76)
#define EvtNetTCP_RetransmittedSegment      EventID (EventLevelOp,    EvtNetTCP, 77)
#define EvtNetTCP_OutOfRangeSegment         EventID (EventLevelError, EvtNetTCP, 78)
#define EvtNetTCP_ZeroWindowProbe           EventID (EventLevelOp,    EvtNetTCP, 79)
#define EvtNetTCP_RemotePeerClosing         EventID (EventLevelOp,    EvtNetTCP, 80)
#define EvtNetTCP_ShowReceiveWindow         EventID (EventLevelOp,    EvtNetTCP, 81)
#define EvtNetTCP_InvalidAck                EventID (EventLevelError, EvtNetTCP, 82)
#define EvtNetTCP_SynNotSet                 EventID (EventLevelError, EvtNetTCP, 83)
#define EvtNetTCP_UserConnectionReject      EventID (EventLevelOp,    EvtNetTCP, 84)
#define EvtNetTCP_NextState                 EventID (EventLevelOp,    EvtNetTCP, 85)
#define EvtNetTCP_WrongSynAck               EventID (EventLevelError, EvtNetTCP, 86)
#define EvtNetTCP_WrongAckNumber            EventID (EventLevelError, EvtNetTCP, 87)
#define EvtNetTCP_WrongSeqNumber            EventID (EventLevelError, EvtNetTCP, 88)
#define EvtNetTCP_RepeatedSyn               EventID (EventLevelOp,    EvtNetTCP, 89)
#define EvtNetTCP_FrameUnrecognised         EventID (EventLevelError, EvtNetTCP, 90)
#define EvtNetTCP_SimultOpenNextState       EventID (EventLevelOp,    EvtNetTCP, 91)
#define EvtNetTCP_WrongFinAck               EventID (EventLevelError, EvtNetTCP, 92)
#define EvtNetTCP_FinAckNextState           EventID (EventLevelOp,    EvtNetTCP, 93)
#define EvtNetTCP_SimultCloseNextState      EventID (EventLevelOp,    EvtNetTCP, 94)
#define EvtNetTCP_AckNextState              EventID (EventLevelOp,    EvtNetTCP, 95)
#define EvtNetTCP_FinNextState              EventID (EventLevelOp,    EvtNetTCP, 96)
#define EvtNetTCP_PshAckInHalfOpen          EventID (EventLevelError, EvtNetTCP, 97)
#define EvtNetTCP_RepeatedFin               EventID (EventLevelOp,    EvtNetTCP, 98)
#define EvtNetTCP_LastAckNextState          EventID (EventLevelOp,    EvtNetTCP, 99)
#define EvtNetTCP_RstReceived               EventID (EventLevelError, EvtNetTCP,100)
#define EvtNetTCP_InvalidState              EventID (EventLevelError, EvtNetTCP,101)
#define EvtNetTCP_SendData                  EventID (EventLevelOp,    EvtNetTCP,102)
#define EvtNetTCP_SendControl               EventID (EventLevelOp,    EvtNetTCP,103)
#define EvtNetTCP_SendReset                 EventID (EventLevelOp,    EvtNetTCP,104)
#define EvtNetTCP_ParseHeaderOptions        EventID (EventLevelOp,    EvtNetTCP,105)
#define EvtNetTCP_OptionMss                 EventID (EventLevelOp,    EvtNetTCP,106)
#define EvtNetTCP_DuplicateAck              EventID (EventLevelOp,    EvtNetTCP,107)
#define EvtNetTCP_FastRetransmit            EventID (EventLevelOp,    EvtNetTCP,108)
#define EvtNetTCP_DataAcked                 EventID (EventLevelOp,    EvtNetTCP,109)
#define EvtNetTCP_ResendData                EventID (EventLevelOp,    EvtNetTCP,110)
#define EvtNetTCP_MapSocketWrongFlagsSet    EventID (EventLevelError, EvtNetTCP,111)
#define EvtNetTCP_MapSocketSynNotSet        EventID (EventLevelError, EvtNetTCP,112)
#define EvtNetTCP_MapSocketNoListenSocket   EventID (EventLevelError, EvtNetTCP,113)
#define EvtNetTCP_UninitSockets             EventID (EventLevelOp,    EvtNetTCP,114)
#endif

/**
  \brief  Event on TCP socket initialize (Op)
  \param  num_sockets   number of available TCP sockets
  \param  max_segsize   maximum segment size in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_InitSockets(uint32_t num_sockets, uint16_t max_segsize) {
    EventRecord2 (EvtNetTCP_InitSockets, num_sockets, max_segsize);
  }
#else
  #define EvrNetTCP_InitSockets(num_sockets, max_segsize)
#endif

/**
  \brief  Event on TCP get socket failed, invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_GetSocketInvalidParameter(void) {
    EventRecord2 (EvtNetTCP_GetSocketInvalidParameter, 0, 0);
  }
#else
  #define EvrNetTCP_GetSocketInvalidParameter()
#endif

/**
  \brief  Event on TCP free socket allocated (Op)
  \param  socket        allocated socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SocketAllocated(int32_t socket) {
    EventRecord2 (EvtNetTCP_SocketAllocated, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SocketAllocated(socket)
#endif

/**
  \brief  Event on TCP get socket failed, no free sockets available (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_NoSocketsAvailable(void) {
    EventRecord2 (EvtNetTCP_NoSocketsAvailable, 0, 0);
  }
#else
  #define EvrNetTCP_NoSocketsAvailable()
#endif

/**
  \brief  Event on TCP release socket (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ReleaseSocket(int32_t socket) {
    EventRecord2 (EvtNetTCP_ReleaseSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ReleaseSocket(socket)
#endif

/**
  \brief  Event on TCP release failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ReleaseSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_ReleaseSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ReleaseSocketNotValid(socket)
#endif

/**
  \brief  Event on TCP release failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ReleaseSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetTCP_ReleaseSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ReleaseSocketWrongState(socket)
#endif

/**
  \brief  Event on TCP listen socket request (Op)
  \param  socket        socket handle
  \param  port          local port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ListenSocket(int32_t socket, uint16_t port) {
    EventRecord2 (EvtNetTCP_ListenSocket, (uint32_t)socket, port);
  }
#else
  #define EvrNetTCP_ListenSocket(socket, port)
#endif

/**
  \brief  Event on TCP listen failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ListenSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_ListenSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ListenSocketNotValid(socket)
#endif

/**
  \brief  Event on TCP listen failed, listening port undefined (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ListenPortUndefined(int32_t socket) {
    EventRecord2 (EvtNetTCP_ListenPortUndefined, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ListenPortUndefined(socket)
#endif

/**
  \brief  Event on TCP listen failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ListenSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetTCP_ListenSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ListenSocketWrongState(socket)
#endif

/**
  \brief  Event on TCP connect socket request (Op)
  \param  socket        socket handle
  \param  local_port    local port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ConnectSocket(int32_t socket, uint16_t local_port) {
    EventRecord2 (EvtNetTCP_ConnectSocket, (uint32_t)socket, local_port);
  }
#else
  #define EvrNetTCP_ConnectSocket(socket, local_port)
#endif

/**
  \brief  Event on TCP connect failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ConnectSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_ConnectSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ConnectSocketNotValid(socket)
#endif

/**
  \brief  Event on TCP connect failed, address unspecified (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ConnectAddressUnspecified(int32_t socket) {
    EventRecord2 (EvtNetTCP_ConnectAddressUnspecified, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ConnectAddressUnspecified(socket)
#endif

/**
  \brief  Event on TCP connect failed, port undefined (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ConnectPortUndefined(int32_t socket) {
    EventRecord2 (EvtNetTCP_ConnectPortUndefined, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ConnectPortUndefined(socket)
#endif

/**
  \brief  Event on TCP display net address IPv4 (Op)
  \param  net_addr      pointer to \ref NET_ADDR4 address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ShowNetAddressIp4(const void *net_addr) {
    EventRecordData (EvtNetTCP_ShowNetAddressIp4, net_addr, 8);
  }
#else
  #define EvrNetTCP_ShowNetAddressIp4(net_addr)
#endif

/**
  \brief  Event on TCP display net address IPv6 (Op)
  \param  net_addr      pointer to \ref NET_ADDR address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ShowNetAddressIp6(const void *net_addr) {
    EventRecordData (EvtNetTCP_ShowNetAddressIp6, net_addr, 20);
  }
#else
  #define EvrNetTCP_ShowNetAddressIp6(net_addr)
#endif

/**
  \brief  Event on TCP connect failed, local port invalid (Error)
  \param  socket        socket handle
  \param  local_port    local port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ConnectLocalPortInvalid(int32_t socket, uint16_t local_port) {
    EventRecord2 (EvtNetTCP_ConnectLocalPortInvalid, (uint32_t)socket, local_port);
  }
#else
  #define EvrNetTCP_ConnectLocalPortInvalid(socket, local_port)
#endif

/**
  \brief  Event on TCP local port assigned in socket connect request (Op)
  \param  local_port    assigned local port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ConnectLocalPortAssigned(uint16_t local_port) {
    EventRecord2 (EvtNetTCP_ConnectLocalPortAssigned, local_port, 0);
  }
#else
  #define EvrNetTCP_ConnectLocalPortAssigned(local_port)
#endif

/**
  \brief  Event on TCP connect failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ConnectSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetTCP_ConnectSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ConnectSocketWrongState(socket)
#endif

/**
  \brief  Event on TCP display RTT (round trip time) estimation variables (Detail)
  \param  sa            rtt average estimator
  \param  sv            rtt deviation estimator
  \note   Those variables are taken from VJs original code in his paper
          (Congestion avoidance and control, page 20)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ShowRttVariables(int16_t sa, int16_t sv) {
    int32_t rto = ((sa>>3) + sv) * 100;
    EventRecord4 (EvtNetTCP_ShowRttVariables, (uint32_t)rto, (uint32_t)sa, (uint32_t)sv, 0);
  }
#else
  #define EvrNetTCP_ShowRttVariables(sa, sv)
#endif

/**
  \brief  Event on TCP get buffer failed, out of memory (Error)
  \param  mem_size      requested memory size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_GetBufferFailed(uint16_t mem_size) {
    EventRecord2 (EvtNetTCP_GetBufferFailed, mem_size, 0);
  }
#else
  #define EvrNetTCP_GetBufferFailed(mem_size)
#endif

/**
  \brief  Event on TCP send frame (Op)
  \param  socket        socket handle
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendFrame(int32_t socket, uint32_t length) {
    EventRecord2 (EvtNetTCP_SendFrame, (uint32_t)socket, length);
  }
#else
  #define EvrNetTCP_SendFrame(socket, length)
#endif

/**
  \brief  Event on TCP send failed, invalid buffer (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendBufferInvalid(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendBufferInvalid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendBufferInvalid(socket)
#endif

/**
  \brief  Event on TCP send failed, socket handle not valid (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendSocketNotValid(socket)
#endif

/**
  \brief  Event on TCP send failed, socket not connected (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendSocketNotConnected(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendSocketNotConnected, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendSocketNotConnected(socket)
#endif

/**
  \brief  Event on TCP send failed, socket closing (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendSocketClosing(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendSocketClosing, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendSocketClosing(socket)
#endif

/**
  \brief  Event on TCP send failed, called from a callback function (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendReenteredCall(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendReenteredCall, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendReenteredCall(socket)
#endif

/**
  \brief  Event on TCP send failed, unacked data pending (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendDataUnacked(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendDataUnacked, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendDataUnacked(socket)
#endif

/**
  \brief  Event on TCP send failed, Maximum Segment Size exceeded (Error)
  \param  socket        socket handle
  \note   Frame length is larger than MSS.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendMssExceeded(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendMssExceeded, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendMssExceeded(socket)
#endif

/**
  \brief  Event on TCP send 0-length frame, frame dumped (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendZeroLengthFrame(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendZeroLengthFrame, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendZeroLengthFrame(socket)
#endif

/**
  \brief  Event on TCP close socket (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_CloseSocket(int32_t socket) {
    EventRecord2 (EvtNetTCP_CloseSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_CloseSocket(socket)
#endif

/**
  \brief  Event on TCP close socket failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_CloseSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_CloseSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_CloseSocketNotValid(socket)
#endif

/**
  \brief  Event on TCP close pended, unacked data exist (Op)
  \param  unack_length  length of unacked data
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_CloseDataUnacked(uint32_t unack_length) {
    EventRecord2 (EvtNetTCP_CloseDataUnacked, unack_length, 0);
  }
#else
  #define EvrNetTCP_CloseDataUnacked(unack_length)
#endif

/**
  \brief  Event on TCP close failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_CloseSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetTCP_CloseSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_CloseSocketWrongState(socket)
#endif

/**
  \brief  Event on TCP abort socket (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_AbortSocket(int32_t socket) {
    EventRecord2 (EvtNetTCP_AbortSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_AbortSocket(socket)
#endif

/**
  \brief  Event on TCP abort failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_AbortSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_AbortSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_AbortSocketNotValid(socket)
#endif

/**
  \brief  Event on TCP abort failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_AbortSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetTCP_AbortSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_AbortSocketWrongState(socket)
#endif

/**
  \brief  Event on TCP send ready failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendReadySocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendReadySocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendReadySocketNotValid(socket)
#endif

/**
  \brief  Event on TCP send ready failed, called from a callback function (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendReadyReenteredCall(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendReadyReenteredCall, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendReadyReenteredCall(socket)
#endif

/**
  \brief  Event on TCP reset window failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ResetWindowSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_ResetWindowSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ResetWindowSocketNotValid(socket)
#endif

/**
  \brief  Event on TCP reset window failed, invalid socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ResetWindowNotConnected(int32_t socket) {
    EventRecord2 (EvtNetTCP_ResetWindowNotConnected, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ResetWindowNotConnected(socket)
#endif

/**
  \brief  Event on TCP reset window failed, flow-control not enabled (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ResetWindowNoFlowControl(int32_t socket) {
    EventRecord2 (EvtNetTCP_ResetWindowNoFlowControl, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ResetWindowNoFlowControl(socket)
#endif

/**
  \brief  Event on TCP reset window update (Error)
  \param  socket        socket handle
  \param  window_size   updated receive window size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ResetWindowUpdate(int32_t socket, uint16_t window_size) {
    EventRecord2 (EvtNetTCP_ResetWindowUpdate, (uint32_t)socket, window_size);
  }
#else
  #define EvrNetTCP_ResetWindowUpdate(socket, window_size)
#endif

/**
  \brief  Event on TCP set socket option (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionSocket(int32_t socket) {
    EventRecord2 (EvtNetTCP_SetOptionSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SetOptionSocket(socket)
#endif

/**
  \brief  Event on TCP set socket option failed, not valid socket (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionSocketNotValid(int32_t socket) {
    EventRecord2 (EvtNetTCP_SetOptionSocketNotValid, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SetOptionSocketNotValid(socket)
#endif

/**
  \brief  Event on TCP set socket option failed, wrong socket state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionSocketWrongState(int32_t socket) {
    EventRecord2 (EvtNetTCP_SetOptionSocketWrongState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SetOptionSocketWrongState(socket)
#endif

/**
  \brief  Event on TCP set socket option Type Of Service for IPv4 (Op)
  \param  ip4_tos       type of service value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionTos(uint8_t ip4_tos) {
    EventRecord2 (EvtNetTCP_SetOptionTos, ip4_tos, 0);
  }
#else
  #define EvrNetTCP_SetOptionTos(ip4_tos)
#endif

/**
  \brief  Event on TCP set socket option Traffic Class for IPv6 (Op)
  \param  ip6_tclass    traffic class value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionTclass(uint8_t ip6_tclass) {
    EventRecord2 (EvtNetTCP_SetOptionTclass, ip6_tclass, 0);
  }
#else
  #define EvrNetTCP_SetOptionTclass(ip6_tclass)
#endif

/**
  \brief  Event on TCP set socket option connection timeout (Op)
  \param  timeout       connection or keep-alive timeout in seconds
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionTimeout(uint16_t timeout) {
    EventRecord2 (EvtNetTCP_SetOptionTimeout, timeout, 0);
  }
#else
  #define EvrNetTCP_SetOptionTimeout(timeout)
#endif

/**
  \brief  Event on TCP set socket keep-alive option (Op)
  \param  enable        keep-alive state
                         - 0: disable
                         - 1: enable
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionKeepAlive(uint32_t enable) {
    EventRecord2 (EvtNetTCP_SetOptionKeepAlive, enable, 0);
  }
#else
  #define EvrNetTCP_SetOptionKeepAlive(enable)
#endif

/**
  \brief  Event on TCP set socket flow-control option (Op)
  \param  enable        flow-control state
                         - 0: disable
                         - 1: enable
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionFlowControl(uint32_t enable) {
    EventRecord2 (EvtNetTCP_SetOptionFlowControl, enable, 0);
  }
#else
  #define EvrNetTCP_SetOptionFlowControl(enable)
#endif

/**
  \brief  Event on TCP set socket delayed-acknowledge option (Op)
  \param  enable        delayed-acknowledge state
                         - 0: disable
                         - 1: enable
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionDelayedAck(uint32_t enable) {
    EventRecord2 (EvtNetTCP_SetOptionDelayedAck, enable, 0);
  }
#else
  #define EvrNetTCP_SetOptionDelayedAck(enable)
#endif

/**
  \brief  Event on TCP wrong set socket option (Error)
  \param  socket        socket handle
  \param  tcp_option    wrong set option value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionWrongOption(int32_t socket, uint8_t tcp_option) {
    EventRecord2 (EvtNetTCP_SetOptionWrongOption, (uint32_t)socket, tcp_option);
  }
#else
  #define EvrNetTCP_SetOptionWrongOption(socket, tcp_option)
#endif

/**
  \brief  Event on TCP wrong value for set socket option (Error)
  \param  socket        socket handle
  \param  opt_value     wrong value for set option
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SetOptionWrongValue(int32_t socket, uint8_t opt_value) {
    EventRecord2 (EvtNetTCP_SetOptionWrongValue, (uint32_t)socket, opt_value);
  }
#else
  #define EvrNetTCP_SetOptionWrongValue(socket, opt_value)
#endif

/**
  \brief  Event on TCP send delayed-acknowledge (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendDelayedAck(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendDelayedAck, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendDelayedAck(socket)
#endif

/**
  \brief  Event on TCP send keep-alive probe (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendKeepAliveProbe(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendKeepAliveProbe, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendKeepAliveProbe(socket)
#endif

/**
  \brief  Event on TCP keep-alive timeout no response, close socket (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_KeepAliveTimeoutClosing(int32_t socket) {
    EventRecord2 (EvtNetTCP_KeepAliveTimeoutClosing, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_KeepAliveTimeoutClosing(socket)
#endif

/**
  \brief  Event on TCP acknowledge callback notification to the user (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_CallbackEventAck(int32_t socket) {
    EventRecord2 (EvtNetTCP_CallbackEventAck, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_CallbackEventAck(socket)
#endif

/**
  \brief  Event on TCP resend data on timeout (Op)
  \param  socket        socket handle
  \param  length        length of data to resend
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ResendOnTimeout(int32_t socket, uint32_t length) {
    EventRecord2 (EvtNetTCP_ResendOnTimeout, (uint32_t)socket, length);
  }
#else
  #define EvrNetTCP_ResendOnTimeout(socket, length)
#endif

/**
  \brief  Event on TCP display congestion avoidance/control variables (Detail)
  \param  cwnd          congestion window
  \param  ssth          slow start threshold
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ShowCongestionVariables(uint16_t cwnd, uint16_t ssth) {
    EventRecord2 (EvtNetTCP_ShowCongestionVariables, cwnd, ssth);
  }
#else
  #define EvrNetTCP_ShowCongestionVariables(cwnd, ssth)
#endif

/**
  \brief  Event on TCP timeout expired in current state (Op)
  \param  socket        socket handle
  \param  state         current socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_TimeoutInState(int32_t socket, uint32_t state) {
    EventRecord2 (EvtNetTCP_TimeoutInState, (uint32_t)socket, state);
  }
#else
  #define EvrNetTCP_TimeoutInState(socket, state)
#endif

/**
  \brief  Event on TCP timeout in TIME_WAIT state, closing socket (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_TwaitTimeoutClosing(int32_t socket) {
    EventRecord2 (EvtNetTCP_TwaitTimeoutClosing, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_TwaitTimeoutClosing(socket)
#endif

/**
  \brief  Event on TCP timeout in CLOSING, FIN_WAIT_1 or LAST_ACK state (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ClosingTimeout(int32_t socket) {
    EventRecord2 (EvtNetTCP_ClosingTimeout, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ClosingTimeout(socket)
#endif

/**
  \brief  Event on TCP no retries left (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_NoRetriesLeft(int32_t socket) {
    EventRecord2 (EvtNetTCP_NoRetriesLeft, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_NoRetriesLeft(socket)
#endif

/**
  \brief  Event on TCP receive frame (Op)
  \param  length        frame length in bytes
  \param  ip_version    internet protocol version
                         - 0: IPv4
                         - 1: IPv6
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ReceiveFrame(uint32_t length, uint8_t ip_version) {
    EventRecord2 (EvtNetTCP_ReceiveFrame, length, ip_version);
  }
#else
  #define EvrNetTCP_ReceiveFrame(length, ip_version)
#endif

/**
  \brief  Event on TCP received frame too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetTCP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetTCP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on TCP received frame not mapped, no active socket found (Op)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_FrameNotMapped(uint32_t length) {
    EventRecord2 (EvtNetTCP_FrameNotMapped, length, 0);
  }
#else
  #define EvrNetTCP_FrameNotMapped(length)
#endif

/**
  \brief  Event on TCP received frame mapped to a socket (Op)
  \param  socket        mapped socket handle
  \param  state         current socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_MappedToSocket(int32_t socket, uint32_t state) {
    EventRecord2 (EvtNetTCP_MappedToSocket, (uint32_t)socket, state);
  }
#else
  #define EvrNetTCP_MappedToSocket(socket, state)
#endif

/**
  \brief  Event on TCP frame checksum check failed (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ChecksumFailed(int32_t socket) {
    EventRecord2 (EvtNetTCP_ChecksumFailed, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_ChecksumFailed(socket)
#endif

/**
  \brief  Event on TCP display send/receive frame header (Detail)
  \param  tcp_header    pointer to TCP frame header of 20 bytes
                         - SrcPort (2 bytes)
                         - DstPort (2 bytes)
                         - SeqNr   (4 bytes)
                         - AckNr   (4 bytes)
                         - HLen    (1 byte)
                         - Flags   (1 byte)
                         - Window  (2 bytes)
                         - Chksum  (2 bytes)
                         - UrgPtr  (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ShowFrameHeader(const void *tcp_header) {
    EventRecordData (EvtNetTCP_ShowFrameHeader, tcp_header, 20);
  }
#else
  #define EvrNetTCP_ShowFrameHeader(tcp_header)
#endif

/**
  \brief  Event on TCP RST-flag received within current receive window (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_RstInWindow(void) {
    EventRecord2 (EvtNetTCP_RstInWindow, 0, 0);
  }
#else
  #define EvrNetTCP_RstInWindow()
#endif

/**
  \brief  Event on TCP RST-flag received outside current receive window (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_RstNotValid(void) {
    EventRecord2 (EvtNetTCP_RstNotValid, 0, 0);
  }
#else
  #define EvrNetTCP_RstNotValid()
#endif

/**
  \brief  Event on TCP repeated SYN+ACK-flags received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_RepeatedSynAck(void) {
    EventRecord2 (EvtNetTCP_RepeatedSynAck, 0, 0);
  }
#else
  #define EvrNetTCP_RepeatedSynAck()
#endif

/**
  \brief  Event on TCP ACK-flag not set in received frame (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_AckNotSet(void) {
    EventRecord2 (EvtNetTCP_AckNotSet, 0, 0);
  }
#else
  #define EvrNetTCP_AckNotSet()
#endif

/**
  \brief  Event on TCP display current sending window size (Op)
  \param  send_window   current sending window size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ShowSendWindow(uint16_t send_window) {
    EventRecord2 (EvtNetTCP_ShowSendWindow, send_window, 0);
  }
#else
  #define EvrNetTCP_ShowSendWindow(send_window)
#endif

/**
  \brief  Event on TCP keep-alive frame received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_KeepAliveSegment(void) {
    EventRecord2 (EvtNetTCP_KeepAliveSegment, 0, 0);
  }
#else
  #define EvrNetTCP_KeepAliveSegment()
#endif

/**
  \brief  Event on TCP retransmitted frame received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_RetransmittedSegment(void) {
    EventRecord2 (EvtNetTCP_RetransmittedSegment, 0, 0);
  }
#else
  #define EvrNetTCP_RetransmittedSegment()
#endif

/**
  \brief  Event on TCP out of range segment received (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_OutOfRangeSegment(int32_t socket) {
    EventRecord2 (EvtNetTCP_OutOfRangeSegment, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_OutOfRangeSegment(socket)
#endif

/**
  \brief  Event on TCP zero-window probe received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ZeroWindowProbe(void) {
    EventRecord2 (EvtNetTCP_ZeroWindowProbe, 0, 0);
  }
#else
  #define EvrNetTCP_ZeroWindowProbe()
#endif

/**
  \brief  Event on TCP FIN-flag received, remote peer wants to close (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_RemotePeerClosing(void) {
    EventRecord2 (EvtNetTCP_RemotePeerClosing, 0, 0);
  }
#else
  #define EvrNetTCP_RemotePeerClosing()
#endif

/**
  \brief  Event on TCP display current receiving window size (Op)
  \param  rec_window    current receiving window size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ShowReceiveWindow(uint16_t rec_window) {
    EventRecord2 (EvtNetTCP_ShowReceiveWindow, rec_window, 0);
  }
#else
  #define EvrNetTCP_ShowReceiveWindow(rec_window)
#endif

/**
  \brief  Event on TCP ACK-flag set received, invalid in state LISTEN (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_InvalidAck(int32_t socket) {
    EventRecord2 (EvtNetTCP_InvalidAck, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_InvalidAck(socket)
#endif

/**
  \brief  Event on TCP SYN-flag not set in received frame (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SynNotSet(int32_t socket) {
    EventRecord2 (EvtNetTCP_SynNotSet, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SynNotSet(socket)
#endif

/**
  \brief  Event on TCP user rejected inbound connection (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_UserConnectionReject(void) {
    EventRecord2 (EvtNetTCP_UserConnectionReject, 0, 0);
  }
#else
  #define EvrNetTCP_UserConnectionReject()
#endif

/**
  \brief  Event on TCP transition to next state (Op)
  \param  state         next socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_NextState(uint32_t state) {
    EventRecord2 (EvtNetTCP_NextState, state, 0);
  }
#else
  #define EvrNetTCP_NextState(state)
#endif

/**
  \brief  Event on TCP SYN+ACK-flags set but wrong ack number (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_WrongSynAck(int32_t socket) {
    EventRecord2 (EvtNetTCP_WrongSynAck, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_WrongSynAck(socket)
#endif

/**
  \brief  Event on TCP ACK-flag set but wrong ack number (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_WrongAckNumber(int32_t socket) {
    EventRecord2 (EvtNetTCP_WrongAckNumber, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_WrongAckNumber(socket)
#endif

/**
  \brief  Event on TCP ACK-flag set but wrong seq number (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_WrongSeqNumber(int32_t socket) {
    EventRecord2 (EvtNetTCP_WrongSeqNumber, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_WrongSeqNumber(socket)
#endif

/**
  \brief  Event on TCP repeated SYN-flag set frame received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_RepeatedSyn(void) {
    EventRecord2 (EvtNetTCP_RepeatedSyn, 0, 0);
  }
#else
  #define EvrNetTCP_RepeatedSyn()
#endif

/**
  \brief  Event on TCP unrecognised frame received (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_FrameUnrecognised(int32_t socket) {
    EventRecord2 (EvtNetTCP_FrameUnrecognised, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_FrameUnrecognised(socket)
#endif

/**
  \brief  Event on TCP SYN-flag set received in simultaneous open (Op)
  \param  state         next socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SimultOpenNextState(uint32_t state) {
    EventRecord2 (EvtNetTCP_SimultOpenNextState, state, 0);
  }
#else
  #define EvrNetTCP_SimultOpenNextState(state)
#endif

/**
  \brief  Event on TCP frame with FIN+ACK-flags set but wrong ack number (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_WrongFinAck(int32_t socket) {
    EventRecord2 (EvtNetTCP_WrongFinAck, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_WrongFinAck(socket)
#endif

/**
  \brief  Event on TCP frame with FIN+ACK-flags received (Op)
  \param  state         next socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_FinAckNextState(uint32_t state) {
    EventRecord2 (EvtNetTCP_FinAckNextState, state, 0);
  }
#else
  #define EvrNetTCP_FinAckNextState(state)
#endif

/**
  \brief  Event on TCP frame with FIN-flag received in simultaneous close (Op)
  \param  state         next socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SimultCloseNextState(uint32_t state) {
    EventRecord2 (EvtNetTCP_SimultCloseNextState, state, 0);
  }
#else
  #define EvrNetTCP_SimultCloseNextState(state)
#endif

/**
  \brief  Event on TCP frame with ACK-flag received (Op)
  \param  state         next socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_AckNextState(uint32_t state) {
    EventRecord2 (EvtNetTCP_AckNextState, state, 0);
  }
#else
  #define EvrNetTCP_AckNextState(state)
#endif

/**
  \brief  Event on TCP frame with FIN-flag received (Op)
  \param  state         next socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_FinNextState(uint32_t state) {
    EventRecord2 (EvtNetTCP_FinNextState, state, 0);
  }
#else
  #define EvrNetTCP_FinNextState(state)
#endif

/**
  \brief  Event on TCP PSH+ACK-flags received in half open state FIN_WAIT_2 (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_PshAckInHalfOpen(int32_t socket) {
    EventRecord2 (EvtNetTCP_PshAckInHalfOpen, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_PshAckInHalfOpen(socket)
#endif

/**
  \brief  Event on TCP repeated FIN-flag received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_RepeatedFin(void) {
    EventRecord2 (EvtNetTCP_RepeatedFin, 0, 0);
  }
#else
  #define EvrNetTCP_RepeatedFin()
#endif

/**
  \brief  Event on TCP last ACK-flag received (Op)
  \param  state         next socket state
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_LastAckNextState(uint32_t state) {
    EventRecord2 (EvtNetTCP_LastAckNextState, state, 0);
  }
#else
  #define EvrNetTCP_LastAckNextState(state)
#endif

/**
  \brief  Event on TCP RST-flag received (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_RstReceived(int32_t socket) {
    EventRecord2 (EvtNetTCP_RstReceived, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_RstReceived(socket)
#endif

/**
  \brief  Event on TCP socket in invalid state (Error)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_InvalidState(int32_t socket) {
    EventRecord2 (EvtNetTCP_InvalidState, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_InvalidState(socket)
#endif

/**
  \brief  Event on TCP send data frame (Op)
  \param  socket        socket handle
  \param  length        data length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendData(int32_t socket, uint16_t length) {
    EventRecord2 (EvtNetTCP_SendData, (uint32_t)socket, length);
  }
#else
  #define EvrNetTCP_SendData(socket, length)
#endif

/**
  \brief  Event on TCP send control frame (Op)
  \param  socket        socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendControl(int32_t socket) {
    EventRecord2 (EvtNetTCP_SendControl, (uint32_t)socket, 0);
  }
#else
  #define EvrNetTCP_SendControl(socket)
#endif

/**
  \brief  Event on TCP send reset to remote host for unmapped frame (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_SendReset(void) {
    EventRecord2 (EvtNetTCP_SendReset, 0, 0);
  }
#else
  #define EvrNetTCP_SendReset()
#endif

/**
  \brief  Event on TCP parse header options (Op)
  \param  opt_length    length of header options
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ParseHeaderOptions(uint32_t opt_length) {
    EventRecord2 (EvtNetTCP_ParseHeaderOptions, opt_length, 0);
  }
#else
  #define EvrNetTCP_ParseHeaderOptions(opt_length)
#endif

/**
  \brief  Event on TCP header option MSS (maximum segment size) (Op)
  \param  mss           maximum segment size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_OptionMss(uint32_t mss) {
    EventRecord2 (EvtNetTCP_OptionMss, mss, 0);
  }
#else
  #define EvrNetTCP_OptionMss(mss)
#endif

/**
  \brief  Event on TCP duplicate acknowledge processing (Op)
  \param  dup_acks      number of duplicate acks
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_DuplicateAck(uint32_t dup_acks) {
    EventRecord2 (EvtNetTCP_DuplicateAck, dup_acks, 0);
  }
#else
  #define EvrNetTCP_DuplicateAck(dup_acks)
#endif

/**
  \brief  Event on TCP fast retransmit, received 3 duplicate acks (Op)
  \param  data_length   length of data to fast retransmit
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_FastRetransmit(uint32_t data_length) {
    EventRecord2 (EvtNetTCP_FastRetransmit, data_length, 0);
  }
#else
  #define EvrNetTCP_FastRetransmit(data_length)
#endif

/**
  \brief  Event on TCP data acknowledge processing (Op)
  \param  ack_length    acknowledged data length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_DataAcked(uint32_t ack_length) {
    EventRecord2 (EvtNetTCP_DataAcked, ack_length, 0);
  }
#else
  #define EvrNetTCP_DataAcked(ack_length)
#endif

/**
  \brief  Event on TCP resend data from transmit queue (Op)
  \param  socket        socket handle
  \param  length        length of the data to resend
  \param  tout          retry timeout in ms
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_ResendData(int32_t socket, uint32_t length, uint32_t tout) {
    EventRecord4 (EvtNetTCP_ResendData, (uint32_t)socket, length, tout, 0);
  }
#else
  #define EvrNetTCP_ResendData(socket, length, tout)
#endif

/**
  \brief  Event on TCP map socket failed, wrong TCP flags set (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_MapSocketWrongFlagsSet(void) {
    EventRecord2 (EvtNetTCP_MapSocketWrongFlagsSet, 0, 0);
  }
#else
  #define EvrNetTCP_MapSocketWrongFlagsSet()
#endif

/**
  \brief  Event on TCP map socket failed, SYN-flag not set (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_MapSocketSynNotSet(void) {
    EventRecord2 (EvtNetTCP_MapSocketSynNotSet, 0, 0);
  }
#else
  #define EvrNetTCP_MapSocketSynNotSet()
#endif

/**
  \brief  Event on TCP map socket failed, no listening sockets found (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_MapSocketNoListenSocket(void) {
    EventRecord2 (EvtNetTCP_MapSocketNoListenSocket, 0, 0);
  }
#else
  #define EvrNetTCP_MapSocketNoListenSocket()
#endif

/**
  \brief  Event on TCP socket de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTCP_UninitSockets(void) {
    EventRecord2 (EvtNetTCP_UninitSockets, 0, 0);
  }
#else
  #define EvrNetTCP_UninitSockets()
#endif


// NetBSD event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetBSD_InitSockets               EventID (EventLevelOp,    EvtNetBSD,  0)
#define EvtNetBSD_SocketInvalidParameter    EventID (EventLevelError, EvtNetBSD,  1)
#define EvtNetBSD_SocketAllocated           EventID (EventLevelOp,    EvtNetBSD,  2)
#define EvtNetBSD_NoSocketsAvailable        EventID (EventLevelError, EvtNetBSD,  3)
#define EvtNetBSD_BindSocket                EventID (EventLevelAPI,   EvtNetBSD,  4)
#define EvtNetBSD_BindSocketNotValid        EventID (EventLevelError, EvtNetBSD,  5)
#define EvtNetBSD_BindInvalidParameter      EventID (EventLevelError, EvtNetBSD,  6)
#define EvtNetBSD_BindSocketNotCreated      EventID (EventLevelError, EvtNetBSD,  7)
#define EvtNetBSD_BindSocketAlreadyBound    EventID (EventLevelError, EvtNetBSD,  8)
#define EvtNetBSD_BindSocketConnected       EventID (EventLevelError, EvtNetBSD,  9)
#define EvtNetBSD_BindPortInUse             EventID (EventLevelError, EvtNetBSD, 10)
#define EvtNetBSD_ShowNetAddressIp4         EventID (EventLevelOp,    EvtNetBSD, 11)
#define EvtNetBSD_ShowNetAddressIp6         EventID (EventLevelOp,    EvtNetBSD, 12)
#define EvtNetBSD_ListenSocket              EventID (EventLevelAPI,   EvtNetBSD, 13)
#define EvtNetBSD_ListenSocketNotValid      EventID (EventLevelError, EvtNetBSD, 14)
#define EvtNetBSD_ListenInvalidParameter    EventID (EventLevelError, EvtNetBSD, 15)
#define EvtNetBSD_ListenSocketNotBound      EventID (EventLevelError, EvtNetBSD, 16)
#define EvtNetBSD_ListenSocketNotStreamType EventID (EventLevelError, EvtNetBSD, 17)
#define EvtNetBSD_ListenSockAlreadyListens  EventID (EventLevelError, EvtNetBSD, 18)
#define EvtNetBSD_ListenBacklogCreateFailed EventID (EventLevelError, EvtNetBSD, 19)
#define EvtNetBSD_AcceptSocket              EventID (EventLevelAPI,   EvtNetBSD, 20)
#define EvtNetBSD_AcceptSocketNotValid      EventID (EventLevelError, EvtNetBSD, 21)
#define EvtNetBSD_AcceptSocketNotListening  EventID (EventLevelError, EvtNetBSD, 22)
#define EvtNetBSD_AcceptSocketNotStreamType EventID (EventLevelError, EvtNetBSD, 23)
#define EvtNetBSD_AcceptSocketLocked        EventID (EventLevelError, EvtNetBSD, 24)
#define EvtNetBSD_AcceptChildSockConnected  EventID (EventLevelOp,    EvtNetBSD, 25)
#define EvtNetBSD_AcceptSocketClosed        EventID (EventLevelError, EvtNetBSD, 26)
#define EvtNetBSD_AcceptSocketKilled        EventID (EventLevelError, EvtNetBSD, 27)
#define EvtNetBSD_ConnectSocket             EventID (EventLevelAPI,   EvtNetBSD, 28)
#define EvtNetBSD_ConnectSocketNotValid     EventID (EventLevelError, EvtNetBSD, 29)
#define EvtNetBSD_ConnectInvalidParameter   EventID (EventLevelError, EvtNetBSD, 30)
#define EvtNetBSD_ConnectSocketLocked       EventID (EventLevelError, EvtNetBSD, 31)
#define EvtNetBSD_ConnectSocketNotCreated   EventID (EventLevelError, EvtNetBSD, 32)
#define EvtNetBSD_ConnectSocketWrongState   EventID (EventLevelError, EvtNetBSD, 33)
#define EvtNetBSD_ConnectDatagramSuccess    EventID (EventLevelOp,    EvtNetBSD, 34)
#define EvtNetBSD_ConnectStreamSuccess      EventID (EventLevelOp,    EvtNetBSD, 35)
#define EvtNetBSD_ConnectStreamTimeout      EventID (EventLevelError, EvtNetBSD, 36)
#define EvtNetBSD_ConnectStreamRefused      EventID (EventLevelError, EvtNetBSD, 37)
#define EvtNetBSD_ConnectSocketKilled       EventID (EventLevelError, EvtNetBSD, 38)
#define EvtNetBSD_SendSocket                EventID (EventLevelAPI,   EvtNetBSD, 39)
#define EvtNetBSD_SendSocketNotValid        EventID (EventLevelError, EvtNetBSD, 40)
#define EvtNetBSD_SendInvalidParameter      EventID (EventLevelError, EvtNetBSD, 41)
#define EvtNetBSD_SendSocketNotCreated      EventID (EventLevelError, EvtNetBSD, 42)
#define EvtNetBSD_SendSocketLocked          EventID (EventLevelError, EvtNetBSD, 43)
#define EvtNetBSD_SendSocketNotConnected    EventID (EventLevelError, EvtNetBSD, 44)
#define EvtNetBSD_SendSocketWrongState      EventID (EventLevelError, EvtNetBSD, 45)
#define EvtNetBSD_SendSocketClosed          EventID (EventLevelError, EvtNetBSD, 46)
#define EvtNetBSD_SendCompleteNonBlocking   EventID (EventLevelOp,    EvtNetBSD, 47)
#define EvtNetBSD_SendSocketKilled          EventID (EventLevelError, EvtNetBSD, 48)
#define EvtNetBSD_SendStreamError           EventID (EventLevelError, EvtNetBSD, 49)
#define EvtNetBSD_SendtoInvalidParameter    EventID (EventLevelError, EvtNetBSD, 50)
#define EvtNetBSD_SendDestAddressUndefined  EventID (EventLevelError, EvtNetBSD, 51)
#define EvtNetBSD_SendDatagramError         EventID (EventLevelError, EvtNetBSD, 52)
#define EvtNetBSD_SendCompleteBlocking      EventID (EventLevelOp,    EvtNetBSD, 53)
#define EvtNetBSD_SendTimeoutBlocking       EventID (EventLevelOp,    EvtNetBSD, 54)
#define EvtNetBSD_SendNoMemory              EventID (EventLevelOp,    EvtNetBSD, 55)
#define EvtNetBSD_RecvSocket                EventID (EventLevelAPI,   EvtNetBSD, 56)
#define EvtNetBSD_RecvSocketNotValid        EventID (EventLevelError, EvtNetBSD, 57)
#define EvtNetBSD_RecvInvalidParameter      EventID (EventLevelError, EvtNetBSD, 58)
#define EvtNetBSD_RecvSocketNotCreated      EventID (EventLevelError, EvtNetBSD, 59)
#define EvtNetBSD_RecvSocketNotConnected    EventID (EventLevelError, EvtNetBSD, 60)
#define EvtNetBSD_RecvSocketWrongState      EventID (EventLevelError, EvtNetBSD, 61)
#define EvtNetBSD_RecvSocketLocked          EventID (EventLevelError, EvtNetBSD, 62)
#define EvtNetBSD_RecvSocketClosed          EventID (EventLevelError, EvtNetBSD, 63)
#define EvtNetBSD_RecvTimeout               EventID (EventLevelOp,    EvtNetBSD, 64)
#define EvtNetBSD_RecvSocketKilled          EventID (EventLevelError, EvtNetBSD, 65)
#define EvtNetBSD_RecvQueueFree             EventID (EventLevelOp,    EvtNetBSD, 66)
#define EvtNetBSD_RecvComplete              EventID (EventLevelOp,    EvtNetBSD, 67)
#define EvtNetBSD_Closesocket               EventID (EventLevelAPI,   EvtNetBSD, 68)
#define EvtNetBSD_CloseSocketNotValid       EventID (EventLevelError, EvtNetBSD, 69)
#define EvtNetBSD_Select                    EventID (EventLevelAPI,   EvtNetBSD, 70)
#define EvtNetBSD_SelectInvalidParameter    EventID (EventLevelError, EvtNetBSD, 71)
#define EvtNetBSD_SelectSuspendFailed       EventID (EventLevelError, EvtNetBSD, 72)
#define EvtNetBSD_SelectComplete            EventID (EventLevelOp,    EvtNetBSD, 73)
#define EvtNetBSD_SelectCompleteBlocking    EventID (EventLevelOp,    EvtNetBSD, 74)
#define EvtNetBSD_Getpeername               EventID (EventLevelAPI,   EvtNetBSD, 75)
#define EvtNetBSD_GetpeerSocketNotValid     EventID (EventLevelError, EvtNetBSD, 76)
#define EvtNetBSD_GetpeerInvalidParameter   EventID (EventLevelError, EvtNetBSD, 77)
#define EvtNetBSD_GetpeerSocketNotConnected EventID (EventLevelError, EvtNetBSD, 78)
#define EvtNetBSD_Getsockname               EventID (EventLevelAPI,   EvtNetBSD, 79)
#define EvtNetBSD_GetsockSocketNotValid     EventID (EventLevelError, EvtNetBSD, 80)
#define EvtNetBSD_GetsockInvalidParameter   EventID (EventLevelError, EvtNetBSD, 81)
#define EvtNetBSD_GetsockSocketNotBound     EventID (EventLevelError, EvtNetBSD, 82)
#define EvtNetBSD_Setsockopt                EventID (EventLevelAPI,   EvtNetBSD, 83)
#define EvtNetBSD_SetoptSocketNotValid      EventID (EventLevelError, EvtNetBSD, 84)
#define EvtNetBSD_SetoptInvalidParameter    EventID (EventLevelError, EvtNetBSD, 85)
#define EvtNetBSD_SetoptSocketNotCreated    EventID (EventLevelError, EvtNetBSD, 86)
#define EvtNetBSD_SetoptOptionNotSupported  EventID (EventLevelError, EvtNetBSD, 87)
#define EvtNetBSD_SetoptKeepAlive           EventID (EventLevelOp,    EvtNetBSD, 88)
#define EvtNetBSD_SetoptRecvTimeout         EventID (EventLevelOp,    EvtNetBSD, 89)
#define EvtNetBSD_SetoptSendTimeout         EventID (EventLevelOp,    EvtNetBSD, 90)
#define EvtNetBSD_SetoptTos                 EventID (EventLevelOp,    EvtNetBSD, 91)
#define EvtNetBSD_SetoptTtl                 EventID (EventLevelOp,    EvtNetBSD, 92)
#define EvtNetBSD_SetoptTclass              EventID (EventLevelOp,    EvtNetBSD, 93)
#define EvtNetBSD_SetoptHopLimit            EventID (EventLevelOp,    EvtNetBSD, 94)
#define EvtNetBSD_Getsockopt                EventID (EventLevelAPI,   EvtNetBSD, 95)
#define EvtNetBSD_GetoptSocketNotValid      EventID (EventLevelError, EvtNetBSD, 96)
#define EvtNetBSD_GetoptInvalidParameter    EventID (EventLevelError, EvtNetBSD, 97)
#define EvtNetBSD_GetoptSocketNotCreated    EventID (EventLevelError, EvtNetBSD, 98)
#define EvtNetBSD_GetoptOptionNotSupported  EventID (EventLevelError, EvtNetBSD, 99)
#define EvtNetBSD_Ioctlsocket               EventID (EventLevelAPI,   EvtNetBSD,100)
#define EvtNetBSD_IoctlSocketNotValid       EventID (EventLevelError, EvtNetBSD,101)
#define EvtNetBSD_IoctlInvalidParameter     EventID (EventLevelError, EvtNetBSD,102)
#define EvtNetBSD_IoctlSocketNotCreated     EventID (EventLevelError, EvtNetBSD,103)
#define EvtNetBSD_IoctlSocketNotStreamType  EventID (EventLevelError, EvtNetBSD,104)
#define EvtNetBSD_IoctlNonBlocking          EventID (EventLevelOp,    EvtNetBSD,105)
#define EvtNetBSD_IoctlDelayAck             EventID (EventLevelOp,    EvtNetBSD,106)
#define EvtNetBSD_IoctlKeepAlive            EventID (EventLevelOp,    EvtNetBSD,107)
#define EvtNetBSD_IoctlFlowControl          EventID (EventLevelOp,    EvtNetBSD,108)
#define EvtNetBSD_CbfuncTcpEvent            EventID (EventLevelOp,    EvtNetBSD,109)
#define EvtNetBSD_CbfuncTcpQueueAdd         EventID (EventLevelOp,    EvtNetBSD,110)
#define EvtNetBSD_CbfuncTcpNoMemory         EventID (EventLevelError, EvtNetBSD,111)
#define EvtNetBSD_CbfuncUdpQueueAdd         EventID (EventLevelOp,    EvtNetBSD,112)
#define EvtNetBSD_CbfuncUdpDumpData         EventID (EventLevelError, EvtNetBSD,113)
#define EvtNetBSD_UninitSockets             EventID (EventLevelOp,    EvtNetBSD,114)
/* BSD-Host resolver events */
#define EvtNetBSD_GetHostInit               EventID (EventLevelOp,    EvtNetBSD,115)
#define EvtNetBSD_GetHostByName             EventID (EventLevelAPI,   EvtNetBSD,116)
#define EvtNetBSD_GetHostNameResolved       EventID (EventLevelOp,    EvtNetBSD,117)
#define EvtNetBSD_GetHostNameNotExisting    EventID (EventLevelOp,    EvtNetBSD,118)
#define EvtNetBSD_GetHostResolverTimeout    EventID (EventLevelOp,    EvtNetBSD,119)
#define EvtNetBSD_GetHostResolverError      EventID (EventLevelError, EvtNetBSD,120)
#define EvtNetBSD_GetHostResolverBusy       EventID (EventLevelError, EvtNetBSD,121)
#define EvtNetBSD_GetHostInvalidParameter   EventID (EventLevelError, EvtNetBSD,122)
#define EvtNetBSD_GetHostUninit             EventID (EventLevelOp,    EvtNetBSD,123)
#endif

/**
  \brief  Event on BSD sockets API initialize (Op)
  \param  num_socks     number of available BSD sockets
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_InitSockets(uint32_t num_socks) {
    EventRecord2 (EvtNetBSD_InitSockets, num_socks, 0);
  }
#else
  #define EvrNetBSD_InitSockets(num_socks)
#endif

/**
  \brief  Event on BSD \ref socket failed, invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SocketInvalidParameter(void) {
    EventRecord2 (EvtNetBSD_SocketInvalidParameter, 0, 0);
  }
#else
  #define EvrNetBSD_SocketInvalidParameter()
#endif

/**
  \brief  Event on BSD \ref socket success (Op)
  \param  sock          allocated socket handle
  \param  type          socket type
                         - 1: SOCK_STREAM
                         - 2: SOCK_DGRAM
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SocketAllocated(int32_t sock, int32_t type) {
    EventRecord2 (EvtNetBSD_SocketAllocated, (uint32_t)sock, (uint32_t)type);
  }
#else
  #define EvrNetBSD_SocketAllocated(sock, type)
#endif

/**
  \brief  Event on BSD \ref socket failed, no free sockets available (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_NoSocketsAvailable(void) {
    EventRecord2 (EvtNetBSD_NoSocketsAvailable, 0, 0);
  }
#else
  #define EvrNetBSD_NoSocketsAvailable()
#endif

/**
  \brief  Event on BSD \ref bind socket (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_BindSocket(int32_t sock) {
    EventRecord2 (EvtNetBSD_BindSocket, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_BindSocket(sock)
#endif

/**
  \brief  Event on BSD \ref bind failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_BindSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_BindSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_BindSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref bind failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_BindInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_BindInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_BindInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref bind failed, socket not created (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_BindSocketNotCreated(int32_t sock) {
    EventRecord2 (EvtNetBSD_BindSocketNotCreated, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_BindSocketNotCreated(sock)
#endif

/**
  \brief  Event on BSD \ref bind failed, socket already bound (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_BindSocketAlreadyBound(int32_t sock) {
    EventRecord2 (EvtNetBSD_BindSocketAlreadyBound, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_BindSocketAlreadyBound(sock)
#endif

/**
  \brief  Event on BSD \ref bind failed, socket already connected (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_BindSocketConnected(int32_t sock) {
    EventRecord2 (EvtNetBSD_BindSocketConnected, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_BindSocketConnected(sock)
#endif

/**
  \brief  Event on BSD \ref bind failed, port in use (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_BindPortInUse(int32_t sock) {
    EventRecord2 (EvtNetBSD_BindPortInUse, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_BindPortInUse(sock)
#endif

/**
  \brief  Event on BSD display net address IPv4 (Op)
  \param  net_addr      pointer to \ref NET_ADDR4 address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ShowNetAddressIp4(const void *net_addr) {
    EventRecordData (EvtNetBSD_ShowNetAddressIp4, net_addr, 8);
  }
#else
  #define EvrNetBSD_ShowNetAddressIp4(net_addr)
#endif

/**
  \brief  Event on BSD display net address IPv6 (Op)
  \param  net_addr      pointer to \ref NET_ADDR address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ShowNetAddressIp6(const void *net_addr) {
    EventRecordData (EvtNetBSD_ShowNetAddressIp6, net_addr, 20);
  }
#else
  #define EvrNetBSD_ShowNetAddressIp6(net_addr)
#endif

/**
  \brief  Event on BSD \ref listen socket (API)
  \param  sock          socket handle
  \param  backlog       max number of queued connection requests
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ListenSocket(int32_t sock, int32_t backlog) {
    EventRecord2 (EvtNetBSD_ListenSocket, (uint32_t)sock, (uint32_t)backlog);
  }
#else
  #define EvrNetBSD_ListenSocket(sock, backlog)
#endif

/**
  \brief  Event on BSD \ref listen failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ListenSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_ListenSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ListenSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref listen failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ListenInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_ListenInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ListenInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref listen failed, socket not bound (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ListenSocketNotBound(int32_t sock) {
    EventRecord2 (EvtNetBSD_ListenSocketNotBound, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ListenSocketNotBound(sock)
#endif

/**
  \brief  Event on BSD \ref listen failed, socket not stream type (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ListenSocketNotStreamType(int32_t sock) {
    EventRecord2 (EvtNetBSD_ListenSocketNotStreamType, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ListenSocketNotStreamType(sock)
#endif

/**
  \brief  Event on BSD \ref listen failed, socket already listens (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ListenSockAlreadyListens(int32_t sock) {
    EventRecord2 (EvtNetBSD_ListenSockAlreadyListens, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ListenSockAlreadyListens(sock)
#endif

/**
  \brief  Event on BSD \ref listen failed, creating socket backlog failed (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ListenBacklogCreateFailed(int32_t sock) {
    EventRecord2 (EvtNetBSD_ListenBacklogCreateFailed, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ListenBacklogCreateFailed(sock)
#endif

/**
  \brief  Event on BSD \ref accept socket (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_AcceptSocket(int32_t sock) {
    EventRecord2 (EvtNetBSD_AcceptSocket, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_AcceptSocket(sock)
#endif

/**
  \brief  Event on BSD \ref accept failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_AcceptSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_AcceptSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_AcceptSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref accept failed, socket not listening (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_AcceptSocketNotListening(int32_t sock) {
    EventRecord2 (EvtNetBSD_AcceptSocketNotListening, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_AcceptSocketNotListening(sock)
#endif

/**
  \brief  Event on BSD \ref accept failed, socket not stream type (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_AcceptSocketNotStreamType(int32_t sock) {
    EventRecord2 (EvtNetBSD_AcceptSocketNotStreamType, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_AcceptSocketNotStreamType(sock)
#endif

/**
  \brief  Event on BSD \ref accept failed, socket locked by another thread (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_AcceptSocketLocked(int32_t sock) {
    EventRecord2 (EvtNetBSD_AcceptSocketLocked, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_AcceptSocketLocked(sock)
#endif

/**
  \brief  Event on BSD \ref accept success, child socket connected (Op)
  \param  child_sock    child socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_AcceptChildSockConnected(int32_t child_sock) {
    EventRecord2 (EvtNetBSD_AcceptChildSockConnected, (uint32_t)child_sock, 0);
  }
#else
  #define EvrNetBSD_AcceptChildSockConnected(child_sock)
#endif

/**
  \brief  Event on BSD \ref accept failed, socket closed by peer (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_AcceptSocketClosed(int32_t sock) {
    EventRecord2 (EvtNetBSD_AcceptSocketClosed, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_AcceptSocketClosed(sock)
#endif

/**
  \brief  Event on BSD \ref accept failed, socket killed locally (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_AcceptSocketKilled(int32_t sock) {
    EventRecord2 (EvtNetBSD_AcceptSocketKilled, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_AcceptSocketKilled(sock)
#endif

/**
  \brief  Event on BSD \ref connect socket (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectSocket(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectSocket, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectSocket(sock)
#endif

/**
  \brief  Event on BSD \ref connect failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref connect failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref connect failed, socket locked by another thread (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectSocketLocked(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectSocketLocked, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectSocketLocked(sock)
#endif

/**
  \brief  Event on BSD \ref connect failed, socket not created (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectSocketNotCreated(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectSocketNotCreated, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectSocketNotCreated(sock)
#endif

/**
  \brief  Event on BSD \ref connect failed, wrong socket state (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectSocketWrongState(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectSocketWrongState, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectSocketWrongState(sock)
#endif

/**
  \brief  Event on BSD \ref connect success, datagram socket connected (Op)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectDatagramSuccess(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectDatagramSuccess, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectDatagramSuccess(sock)
#endif

/**
  \brief  Event on BSD \ref connect success, stream socket connected (Op)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectStreamSuccess(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectStreamSuccess, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectStreamSuccess(sock)
#endif

/**
  \brief  Event on BSD \ref connect failed, connect request has timed out (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectStreamTimeout(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectStreamTimeout, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectStreamTimeout(sock)
#endif

/**
  \brief  Event on BSD \ref connect failed, remote host refused connection (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectStreamRefused(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectStreamRefused, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectStreamRefused(sock)
#endif

/**
  \brief  Event on BSD \ref connect failed, socket killed locally (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_ConnectSocketKilled(int32_t sock) {
    EventRecord2 (EvtNetBSD_ConnectSocketKilled, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_ConnectSocketKilled(sock)
#endif

/**
  \brief  Event on BSD \ref send socket (API)
  \param  sock          socket handle
  \param  length        length of data to send
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendSocket(int32_t sock, uint32_t length) {
    EventRecord2 (EvtNetBSD_SendSocket, (uint32_t)sock, length);
  }
#else
  #define EvrNetBSD_SendSocket(sock, length)
#endif

/**
  \brief  Event on BSD \ref send failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, socket not created (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendSocketNotCreated(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendSocketNotCreated, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendSocketNotCreated(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, socket locked by another thread (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendSocketLocked(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendSocketLocked, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendSocketLocked(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, socket not connected (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendSocketNotConnected(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendSocketNotConnected, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendSocketNotConnected(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, wrong socket state (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendSocketWrongState(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendSocketWrongState, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendSocketWrongState(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, socket closed by peer (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendSocketClosed(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendSocketClosed, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendSocketClosed(sock)
#endif

/**
  \brief  Event on BSD \ref send complete non-blocking mode (Op)
  \param  num           number of bytes successfully sent
  \param  length        length of data requested to send
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendCompleteNonBlocking(uint32_t num, uint32_t length) {
    EventRecord2 (EvtNetBSD_SendCompleteNonBlocking, num, length);
  }
#else
  #define EvrNetBSD_SendCompleteNonBlocking(num, length)
#endif

/**
  \brief  Event on BSD \ref send failed, socket killed locally (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendSocketKilled(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendSocketKilled, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendSocketKilled(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, stream socket unspecified error (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendStreamError(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendStreamError, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendStreamError(sock)
#endif

/**
  \brief  Event on BSD \ref sendto failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendtoInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendtoInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendtoInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, destination address unspecified (Error)
  \param  sock          socket handle
  \note   The \ref connect function was not previously called for datagram socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendDestAddressUndefined(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendDestAddressUndefined, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendDestAddressUndefined(sock)
#endif

/**
  \brief  Event on BSD \ref send failed, datagram socket unspecified error (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendDatagramError(int32_t sock) {
    EventRecord2 (EvtNetBSD_SendDatagramError, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SendDatagramError(sock)
#endif

/**
  \brief  Event on BSD \ref send complete in blocking mode (Op)
  \param  num           number of bytes successfully sent
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendCompleteBlocking(uint32_t num) {
    EventRecord2 (EvtNetBSD_SendCompleteBlocking, num, 0);
  }
#else
  #define EvrNetBSD_SendCompleteBlocking(num)
#endif

/**
  \brief  Event on BSD \ref send timeout in blocking mode (Op)
  \param  num           number of bytes successfully sent
  \param  length        length of data requested to send
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendTimeoutBlocking(uint32_t num, uint32_t length) {
    EventRecord2 (EvtNetBSD_SendTimeoutBlocking, num, length);
  }
#else
  #define EvrNetBSD_SendTimeoutBlocking(num, length)
#endif

/**
  \brief  Event on BSD \ref send out of memory (Op)
  \param  num           number of bytes successfully sent
  \param  length        length of data requested to send
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SendNoMemory(uint32_t num, uint32_t length) {
    EventRecord2 (EvtNetBSD_SendNoMemory, num, length);
  }
#else
  #define EvrNetBSD_SendNoMemory(num, length)
#endif

/**
  \brief  Event on BSD \ref recv socket (API)
  \param  sock          socket handle
  \param  length        length of data to receive
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvSocket(int32_t sock, uint32_t length) {
    EventRecord2 (EvtNetBSD_RecvSocket, (uint32_t)sock, length);
  }
#else
  #define EvrNetBSD_RecvSocket(sock, length)
#endif

/**
  \brief  Event on BSD \ref recv failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref recv failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref recv failed, socket not created (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvSocketNotCreated(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvSocketNotCreated, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvSocketNotCreated(sock)
#endif

/**
  \brief  Event on BSD \ref recv failed, socket not connected (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvSocketNotConnected(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvSocketNotConnected, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvSocketNotConnected(sock)
#endif

/**
  \brief  Event on BSD \ref recv failed, wrong socket state (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvSocketWrongState(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvSocketWrongState, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvSocketWrongState(sock)
#endif

/**
  \brief  Event on BSD \ref recv failed, socket locked by another thread (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvSocketLocked(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvSocketLocked, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvSocketLocked(sock)
#endif

/**
  \brief  Event on BSD \ref recv failed, socket closed by peer (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvSocketClosed(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvSocketClosed, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvSocketClosed(sock)
#endif

/**
  \brief  Event on BSD \ref recv timeout (Op)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvTimeout(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvTimeout, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvTimeout(sock)
#endif

/**
  \brief  Event on BSD \ref recv failed, socket killed locally (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvSocketKilled(int32_t sock) {
    EventRecord2 (EvtNetBSD_RecvSocketKilled, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_RecvSocketKilled(sock)
#endif

/**
  \brief  Event on BSD \ref recv queue free (Op)
  \param  sock          socket handle
  \param  length        length of data to release
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvQueueFree(int32_t sock, uint32_t length) {
    EventRecord2 (EvtNetBSD_RecvQueueFree, (uint32_t)sock, length);
  }
#else
  #define EvrNetBSD_RecvQueueFree(sock, length)
#endif

/**
  \brief  Event on BSD \ref recv complete (Op)
  \param  sock          socket handle
  \param  length        length of data received
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_RecvComplete(int32_t sock, uint32_t length) {
    EventRecord2 (EvtNetBSD_RecvComplete, (uint32_t)sock, length);
  }
#else
  #define EvrNetBSD_RecvComplete(sock, length)
#endif

/**
  \brief  Event on BSD \ref closesocket (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_Closesocket(int32_t sock) {
    EventRecord2 (EvtNetBSD_Closesocket, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_Closesocket(sock)
#endif

/**
  \brief  Event on BSD \ref closesocket failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_CloseSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_CloseSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_CloseSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref select (API)
  \param  nfds          range of sockets to be tested
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_Select(int32_t nfds) {
    EventRecord2 (EvtNetBSD_Select, (uint32_t)nfds, 0);
  }
#else
  #define EvrNetBSD_Select(nfds)
#endif


/**
  \brief  Event on BSD \ref select failed, invalid parameter (Error)
  \param  nfds          range of sockets to be tested
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SelectInvalidParameter(int32_t nfds) {
    EventRecord2 (EvtNetBSD_SelectInvalidParameter, (uint32_t)nfds, 0);
  }
#else
  #define EvrNetBSD_SelectInvalidParameter(nfds)
#endif

/**
  \brief  Event on BSD \ref select suspend failed (Error)
  \param  nfds          range of sockets to be subscribed to resume events
  \note   Subscription to resume events has failed, no sockets subscribed.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SelectSuspendFailed(int32_t nfds) {
    EventRecord2 (EvtNetBSD_SelectSuspendFailed, (uint32_t)nfds, 0);
  }
#else
  #define EvrNetBSD_SelectSuspendFailed(nfds)
#endif

/**
  \brief  Event on BSD \ref select complete non-blocking (Op)
  \param  n_ready       number of ready sockets
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SelectComplete(int32_t n_ready) {
    EventRecord2 (EvtNetBSD_SelectComplete, (uint32_t)n_ready, 0);
  }
#else
  #define EvrNetBSD_SelectComplete(n_ready)
#endif

/**
  \brief  Event on BSD \ref select complete blocking (Op)
  \param  n_ready       number of ready sockets
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SelectCompleteBlocking(int32_t n_ready) {
    EventRecord2 (EvtNetBSD_SelectCompleteBlocking, (uint32_t)n_ready, 0);
  }
#else
  #define EvrNetBSD_SelectCompleteBlocking(n_ready)
#endif

/**
  \brief  Event on BSD \ref getpeername (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_Getpeername(int32_t sock) {
    EventRecord2 (EvtNetBSD_Getpeername, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_Getpeername(sock)
#endif

/**
  \brief  Event on BSD \ref getpeername failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetpeerSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetpeerSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetpeerSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref getpeername failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetpeerInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetpeerInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetpeerInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref getpeername failed, socket not connected (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetpeerSocketNotConnected(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetpeerSocketNotConnected, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetpeerSocketNotConnected(sock)
#endif

/**
  \brief  Event on BSD \ref getsockname (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_Getsockname(int32_t sock) {
    EventRecord2 (EvtNetBSD_Getsockname, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_Getsockname(sock)
#endif

/**
  \brief  Event on BSD \ref getsockname failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetsockSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetsockSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetsockSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref getsockname failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetsockInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetsockInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetsockInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref getsockname failed, socket not bound (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetsockSocketNotBound(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetsockSocketNotBound, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetsockSocketNotBound(sock)
#endif

/**
  \brief  Event on BSD \ref setsockopt (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_Setsockopt(int32_t sock) {
    EventRecord2 (EvtNetBSD_Setsockopt, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_Setsockopt(sock)
#endif

/**
  \brief  Event on BSD \ref setsockopt failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_SetoptSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SetoptSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref setsockopt failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_SetoptInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SetoptInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref setsockopt failed, socket not created (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptSocketNotCreated(int32_t sock) {
    EventRecord2 (EvtNetBSD_SetoptSocketNotCreated, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SetoptSocketNotCreated(sock)
#endif

/**
  \brief  Event on BSD \ref setsockopt failed, option not supported by the protocol (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptOptionNotSupported(int32_t sock) {
    EventRecord2 (EvtNetBSD_SetoptOptionNotSupported, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_SetoptOptionNotSupported(sock)
#endif

/**
  \brief  Event on BSD \ref setsockopt keep-alive option (Op)
  \param  enable        keep-alive state
                         - 0: disable
                         - 1: enable
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptKeepAlive(uint32_t enable) {
    EventRecord2 (EvtNetBSD_SetoptKeepAlive, enable, 0);
  }
#else
  #define EvrNetBSD_SetoptKeepAlive(enable)
#endif

/**
  \brief  Event on BSD \ref setsockopt receive timeout (Op)
  \param  timeout       receive timeout in ms
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptRecvTimeout(uint32_t timeout) {
    EventRecord2 (EvtNetBSD_SetoptRecvTimeout, timeout, 0);
  }
#else
  #define EvrNetBSD_SetoptRecvTimeout(timeout)
#endif

/**
  \brief  Event on BSD \ref setsockopt send timeout (Op)
  \param  timeout       send timeout in ms
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptSendTimeout(uint32_t timeout) {
    EventRecord2 (EvtNetBSD_SetoptSendTimeout, timeout, 0);
  }
#else
  #define EvrNetBSD_SetoptSendTimeout(timeout)
#endif

/**
  \brief  Event on BSD \ref setsockopt TOS for IPv4 (Op)
  \param  ip4_tos       type of service value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptTos(uint8_t ip4_tos) {
    EventRecord2 (EvtNetBSD_SetoptTos, ip4_tos, 0);
  }
#else
  #define EvrNetBSD_SetoptTos(ip4_tos)
#endif

/**
  \brief  Event on BSD \ref setsockopt TTL for IPv4 (Op)
  \param  ip4_ttl       time to live value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptTtl(uint8_t ip4_ttl) {
    EventRecord2 (EvtNetBSD_SetoptTtl, ip4_ttl, 0);
  }
#else
  #define EvrNetBSD_SetoptTtl(ip4_ttl)
#endif

/**
  \brief  Event on BSD \ref setsockopt Traffic Class for IPv6 (Op)
  \param  ip6_tclass    traffic class value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptTclass(uint8_t ip6_tclass) {
    EventRecord2 (EvtNetBSD_SetoptTclass, ip6_tclass, 0);
  }
#else
  #define EvrNetBSD_SetoptTclass(ip6_tclass)
#endif

/**
  \brief  Event on BSD \ref setsockopt Hop Limit for IPv6 (Op)
  \param  ip6_hoplimit  hop limit value
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_SetoptHopLimit(uint8_t ip6_hoplimit) {
    EventRecord2 (EvtNetBSD_SetoptHopLimit, ip6_hoplimit, 0);
  }
#else
  #define EvrNetBSD_SetoptHopLimit(ip6_hoplimit)
#endif

/**
  \brief  Event on BSD \ref getsockopt (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_Getsockopt(int32_t sock) {
    EventRecord2 (EvtNetBSD_Getsockopt, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_Getsockopt(sock)
#endif

/**
  \brief  Event on BSD \ref getsockopt failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetoptSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetoptSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetoptSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref getsockopt failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetoptInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetoptInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetoptInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref getsockopt failed, socket not created (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetoptSocketNotCreated(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetoptSocketNotCreated, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetoptSocketNotCreated(sock)
#endif

/**
  \brief  Event on BSD \ref getsockopt failed, option not supported by the protocol (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetoptOptionNotSupported(int32_t sock) {
    EventRecord2 (EvtNetBSD_GetoptOptionNotSupported, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_GetoptOptionNotSupported(sock)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket (API)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_Ioctlsocket(int32_t sock) {
    EventRecord2 (EvtNetBSD_Ioctlsocket, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_Ioctlsocket(sock)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket failed, socket not valid (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_IoctlSocketNotValid(int32_t sock) {
    EventRecord2 (EvtNetBSD_IoctlSocketNotValid, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_IoctlSocketNotValid(sock)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket failed, invalid parameter (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_IoctlInvalidParameter(int32_t sock) {
    EventRecord2 (EvtNetBSD_IoctlInvalidParameter, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_IoctlInvalidParameter(sock)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket failed, socket not created (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_IoctlSocketNotCreated(int32_t sock) {
    EventRecord2 (EvtNetBSD_IoctlSocketNotCreated, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_IoctlSocketNotCreated(sock)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket failed, socket not stream type (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_IoctlSocketNotStreamType(int32_t sock) {
    EventRecord2 (EvtNetBSD_IoctlSocketNotStreamType, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_IoctlSocketNotStreamType(sock)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket enable non-blocking mode (Op)
  \param  enable        non-blocking state
                         - 0: disable
                         - 1: enable
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_IoctlNonBlocking(uint32_t enable) {
    if (enable) enable = 1;
    EventRecord2 (EvtNetBSD_IoctlNonBlocking, enable, 0);
  }
#else
  #define EvrNetBSD_IoctlNonBlocking(enable)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket enable TCP delay-ack mode (Op)
  \param  enable        delay-ack state
                         - 0: disable
                         - 1: enable
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_IoctlDelayAck(uint32_t enable) {
    if (enable) enable = 1;
    EventRecord2 (EvtNetBSD_IoctlDelayAck, enable, 0);
  }
#else
  #define EvrNetBSD_IoctlDelayAck(enable)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket enable TCP keep-alive mode (Op)
  \param  enable        keep-alive state
                         - 0: disable
                         - 1: enable
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_IoctlKeepAlive(uint32_t enable) {
    if (enable) enable = 1;
    EventRecord2 (EvtNetBSD_IoctlKeepAlive, enable, 0);
  }
#else
  #define EvrNetBSD_IoctlKeepAlive(enable)
#endif

/**
  \brief  Event on BSD \ref ioctlsocket enable TCP flow-control mode (Op)
  \param  enable        flow-control state
                         - 0: disable
                         - 1: enable
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_IoctlFlowControl(uint32_t enable) {
    if (enable) enable = 1;
    EventRecord2 (EvtNetBSD_IoctlFlowControl, enable, 0);
  }
#else
  #define EvrNetBSD_IoctlFlowControl(enable)
#endif

/**
  \brief  Event on BSD callback TCP function events (Op)
  \param  sock          socket handle
  \param  tcp_event     TCP socket event
                         - 0: netTCP_EventConnect
                         - 1: netTCP_EventEstablished
                         - 2: netTCP_EventClosed
                         - 3: netTCP_EventAborted
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_CbfuncTcpEvent(int32_t sock, int32_t tcp_event) {
    EventRecord2 (EvtNetBSD_CbfuncTcpEvent, (uint32_t)sock, (uint32_t)tcp_event);
  }
#else
  #define EvrNetBSD_CbfuncTcpEvent(sock, tcp_event)
#endif

/**
  \brief  Event on BSD callback TCP data received (Op)
  \param  sock          socket handle
  \param  length        length of data received
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_CbfuncTcpQueueAdd(int32_t sock, uint32_t length) {
    EventRecord2 (EvtNetBSD_CbfuncTcpQueueAdd, (uint32_t)sock, length);
  }
#else
  #define EvrNetBSD_CbfuncTcpQueueAdd(sock, length)
#endif

/**
  \brief  Event on BSD callback no memory for TCP data (Error)
  \param  sock          socket handle
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_CbfuncTcpNoMemory(int32_t sock) {
    EventRecord2 (EvtNetBSD_CbfuncTcpNoMemory, (uint32_t)sock, 0);
  }
#else
  #define EvrNetBSD_CbfuncTcpNoMemory(enable)
#endif

/**
  \brief  Event on BSD callback UDP data received (Op)
  \param  sock          socket handle
  \param  length        length of data received
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_CbfuncUdpQueueAdd(int32_t sock, uint32_t length) {
    EventRecord2 (EvtNetBSD_CbfuncUdpQueueAdd, (uint32_t)sock, length);
  }
#else
  #define EvrNetBSD_CbfuncUdpQueueAdd(sock, length)
#endif

/**
  \brief  Event on BSD callback no memory for UDP data (Error)
  \param  sock          socket handle
  \param  length        length of dumped data
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_CbfuncUdpDumpData(int32_t sock, uint32_t length) {
    EventRecord2 (EvtNetBSD_CbfuncUdpDumpData, (uint32_t)sock, length);
  }
#else
  #define EvrNetBSD_CbfuncUdpDumpData(sock, length)
#endif

/**
  \brief  Event on BSD sockets API de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_UninitSockets(void) {
    EventRecord2 (EvtNetBSD_UninitSockets, 0, 0);
  }
#else
  #define EvrNetBSD_UninitSockets()
#endif

// BSD-Host resolver event recorder functions ----------------------------------

/**
  \brief  Event on BSD hostname resolver initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostInit(void) {
    EventRecord2 (EvtNetBSD_GetHostInit, 0, 0);
  }
#else
  #define EvrNetBSD_GetHostInit()
#endif

/**
  \brief  Event on BSD \ref gethostbyname (API)
  \param  hostname      name of the host to resolve
  \param  length        length of the hostname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostByName(const char *hostname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetBSD_GetHostByName, hostname, length);
  }
#else
  #define EvrNetBSD_GetHostByName(hostname, length)
#endif

/**
  \brief  Event on BSD hostname resolved successfully (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostNameResolved(void) {
    EventRecord2 (EvtNetBSD_GetHostNameResolved, 0, 0);
  }
#else
  #define EvrNetBSD_GetHostNameResolved()
#endif

/**
  \brief  Event on BSD requested hostname not existing (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostNameNotExisting(void) {
    EventRecord2 (EvtNetBSD_GetHostNameNotExisting, 0, 0);
  }
#else
  #define EvrNetBSD_GetHostNameNotExisting()
#endif

/**
  \brief  Event on BSD resolver timeout expired (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostResolverTimeout(void) {
    EventRecord2 (EvtNetBSD_GetHostResolverTimeout, 0, 0);
  }
#else
  #define EvrNetBSD_GetHostResolverTimeout()
#endif

/**
  \brief  Event on BSD resolver protocol error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostResolverError(void) {
    EventRecord2 (EvtNetBSD_GetHostResolverError, 0, 0);
  }
#else
  #define EvrNetBSD_GetHostResolverError()
#endif

/**
  \brief  Event on BSD resolver busy error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostResolverBusy(void) {
    EventRecord2 (EvtNetBSD_GetHostResolverBusy, 0, 0);
  }
#else
  #define EvrNetBSD_GetHostResolverBusy()
#endif

/**
  \brief  Event on BSD \ref gethostbyname invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostInvalidParameter(void) {
    EventRecord2 (EvtNetBSD_GetHostInvalidParameter, 0, 0);
  }
#else
  #define EvrNetBSD_GetHostInvalidParameter()
#endif

/**
  \brief  Event on BSD hostname resolver de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetBSD_GetHostUninit(void) {
    EventRecord2 (EvtNetBSD_GetHostUninit, 0, 0);
  }
#else
  #define EvrNetBSD_GetHostUninit()
#endif


// NetHTTPs event identifiers --------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetHTTPs_InitServer              EventID (EventLevelOp,    EvtNetHTTPs,  0)
#define EvtNetHTTPs_ShowRootFolder          EventID (EventLevelOp,    EvtNetHTTPs,  1)
#define EvtNetHTTPs_GetSocketFailed         EventID (EventLevelError, EvtNetHTTPs,  2)
#define EvtNetHTTPs_SetRootPath             EventID (EventLevelAPI,   EvtNetHTTPs,  3)
#define EvtNetHTTPs_SetUsername             EventID (EventLevelAPI,   EvtNetHTTPs,  4)
#define EvtNetHTTPs_SetPassword             EventID (EventLevelAPI,   EvtNetHTTPs,  5)
#define EvtNetHTTPs_StartService            EventID (EventLevelOp,    EvtNetHTTPs,  6)
#define EvtNetHTTPs_StopService             EventID (EventLevelOp,    EvtNetHTTPs,  7)
#define EvtNetHTTPs_FileNotFound            EventID (EventLevelError, EvtNetHTTPs,  8)
#define EvtNetHTTPs_FileAccessForbidden     EventID (EventLevelError, EvtNetHTTPs,  9)
#define EvtNetHTTPs_FileCached              EventID (EventLevelOp,    EvtNetHTTPs, 10)
#define EvtNetHTTPs_CloseSession            EventID (EventLevelOp,    EvtNetHTTPs, 11)
#define EvtNetHTTPs_SendFile                EventID (EventLevelOp,    EvtNetHTTPs, 12)
#define EvtNetHTTPs_CgiSendFile             EventID (EventLevelOp,    EvtNetHTTPs, 13)
#define EvtNetHTTPs_CgiIncludeFile          EventID (EventLevelOp,    EvtNetHTTPs, 14)
#define EvtNetHTTPs_CgiStopEngine           EventID (EventLevelOp,    EvtNetHTTPs, 15)
#define EvtNetHTTPs_CgiScriptError          EventID (EventLevelError, EvtNetHTTPs, 16)
#define EvtNetHTTPs_UnauthorizedAccess      EventID (EventLevelError, EvtNetHTTPs, 17)
#define EvtNetHTTPs_MethodNotImplemented    EventID (EventLevelError, EvtNetHTTPs, 18)
#define EvtNetHTTPs_UserAccessDeniedIp4     EventID (EventLevelError, EvtNetHTTPs, 19)
#define EvtNetHTTPs_UserAccessDeniedIp6     EventID (EventLevelError, EvtNetHTTPs, 20)
#define EvtNetHTTPs_SessionOpen             EventID (EventLevelOp,    EvtNetHTTPs, 21)
#define EvtNetHTTPs_SocketAborted           EventID (EventLevelOp,    EvtNetHTTPs, 22)
#define EvtNetHTTPs_SocketClosed            EventID (EventLevelOp,    EvtNetHTTPs, 23)
#define EvtNetHTTPs_ReceiveFrame            EventID (EventLevelOp,    EvtNetHTTPs, 24)
#define EvtNetHTTPs_FrameTooShort           EventID (EventLevelError, EvtNetHTTPs, 25)
#define EvtNetHTTPs_RequestMethod           EventID (EventLevelOp,    EvtNetHTTPs, 26)
#define EvtNetHTTPs_CgiStartEngine          EventID (EventLevelOp,    EvtNetHTTPs, 27)
#define EvtNetHTTPs_RequestedFile           EventID (EventLevelOp,    EvtNetHTTPs, 28)
#define EvtNetHTTPs_FileOpenDrive           EventID (EventLevelOp,    EvtNetHTTPs, 29)
#define EvtNetHTTPs_ShowFileStatus          EventID (EventLevelOp,    EvtNetHTTPs, 30)
#define EvtNetHTTPs_ShowRedirectionUrl      EventID (EventLevelOp,    EvtNetHTTPs, 31)
#define EvtNetHTTPs_ShowUserCredentials     EventID (EventLevelOp,    EvtNetHTTPs, 32)
#define EvtNetHTTPs_ShowAcceptLanguage      EventID (EventLevelOp,    EvtNetHTTPs, 33)
#define EvtNetHTTPs_ShowXmlType             EventID (EventLevelOp,    EvtNetHTTPs, 34)
#define EvtNetHTTPs_PostXmlEncoded          EventID (EventLevelOp,    EvtNetHTTPs, 35)
#define EvtNetHTTPs_PostCompleteMultipacket EventID (EventLevelOp,    EvtNetHTTPs, 36)
#define EvtNetHTTPs_PostFormUrlEncoded      EventID (EventLevelOp,    EvtNetHTTPs, 37)
#define EvtNetHTTPs_PostDataTruncated       EventID (EventLevelError, EvtNetHTTPs, 38)
#define EvtNetHTTPs_FileUploadRequested     EventID (EventLevelOp,    EvtNetHTTPs, 39)
#define EvtNetHTTPs_FileUploadDataReceived  EventID (EventLevelOp,    EvtNetHTTPs, 40)
#define EvtNetHTTPs_FileUploadRemaining     EventID (EventLevelOp,    EvtNetHTTPs, 41)
#define EvtNetHTTPs_FileUploadComplete      EventID (EventLevelOp,    EvtNetHTTPs, 42)
#define EvtNetHTTPs_TlsGetContextFailed     EventID (EventLevelError, EvtNetHTTPs, 43)
#define EvtNetHTTPs_UninitServer            EventID (EventLevelOp,    EvtNetHTTPs, 44)
#endif

/**
  \brief  Event on HTTP server initialize (Op)
  \param  num_sessions  number of available HTTP sessions
  \param  port          listening port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_InitServer(uint32_t num_sessions, uint16_t port) {
    EventRecord2 (EvtNetHTTPs_InitServer, num_sessions, port);
  }
#else
  #define EvrNetHTTPs_InitServer(num_sessions, port)
#endif

/**
  \brief  Event on HTTP server display root folder (Op)
  \param  root_folder   path to server root folder
  \param  length        length of the root_folder string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_ShowRootFolder(const char *root_folder, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetHTTPs_ShowRootFolder, root_folder, length);
  }
#else
  #define EvrNetHTTPs_ShowRootFolder(root_folder, length)
#endif

/**
  \brief  Event on HTTP server failed to allocate TCP socket (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_GetSocketFailed(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_GetSocketFailed, session, 0);
  }
#else
  #define EvrNetHTTPs_GetSocketFailed(session)
#endif

/**
  \brief  Event on HTTP server \ref netHTTPs_SetRootPath (API)
  \param  path          path to server root folder
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_SetRootPath(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetHTTPs_SetRootPath, path, length);
  }
#else
  #define EvrNetHTTPs_SetRootPath(path, length)
#endif

/**
  \brief  Event on HTTP server \ref netHTTPs_SetUsername (API)
  \param  username      new username
  \param  length        length of the username string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_SetUsername(const char *username, uint32_t length) {
    if (length > 16) length = 16;
    EventRecordData (EvtNetHTTPs_SetUsername, username, length);
  }
#else
  #define EvrNetHTTPs_SetUsername(username, length)
#endif

/**
  \brief  Event on HTTP server \ref netHTTPs_SetPassword (API)
  \param  password      new password
  \param  length        length of the password string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_SetPassword(const char *password, uint32_t length) {
    if (length > 16) length = 16;
    EventRecordData (EvtNetHTTPs_SetPassword, password, length);
  }
#else
  #define EvrNetHTTPs_SetPassword(password, length)
#endif

/**
  \brief  Event on HTTP server start service (Op)
  \param  port          listening port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_StartService(uint16_t port) {
    EventRecord2 (EvtNetHTTPs_StartService, port, 0);
  }
#else
  #define EvrNetHTTPs_StartService(port)
#endif

/**
  \brief  Event on HTTP server stop service (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_StopService(void) {
    EventRecord2 (EvtNetHTTPs_StopService, 0, 0);
  }
#else
  #define EvrNetHTTPs_StopService()
#endif

/**
  \brief  Event on HTTP server requested resource file not found (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FileNotFound(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_FileNotFound, session, 0);
  }
#else
  #define EvrNetHTTPs_FileNotFound(session)
#endif

/**
  \brief  Event on HTTP server access to requested resource not allowed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FileAccessForbidden(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_FileAccessForbidden, session, 0);
  }
#else
  #define EvrNetHTTPs_FileAccessForbidden(session)
#endif

/**
  \brief  Event on HTTP server requested resource file cached on the client (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FileCached(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_FileCached, session, 0);
  }
#else
  #define EvrNetHTTPs_FileCached(session)
#endif

/**
  \brief  Event on HTTP server session close (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_CloseSession(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_CloseSession, session, 0);
  }
#else
  #define EvrNetHTTPs_CloseSession(session)
#endif

/**
  \brief  Event on HTTP server send a resource file (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_SendFile(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_SendFile, session, 0);
  }
#else
  #define EvrNetHTTPs_SendFile(session)
#endif

/**
  \brief  Event on HTTP server CGI send dynamic resource file (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_CgiSendFile(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_CgiSendFile, session, 0);
  }
#else
  #define EvrNetHTTPs_CgiSendFile(session)
#endif

/**
  \brief  Event on HTTP server CGI include a file (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_CgiIncludeFile(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_CgiIncludeFile, session, 0);
  }
#else
  #define EvrNetHTTPs_CgiIncludeFile(session)
#endif

/**
  \brief  Event on HTTP server stop CGI script interpreter engine (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_CgiStopEngine(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_CgiStopEngine, session, 0);
  }
#else
  #define EvrNetHTTPs_CgiStopEngine(session)
#endif

/**
  \brief  Event on HTTP server CGI script error found (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_CgiScriptError(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_CgiScriptError, session, 0);
  }
#else
  #define EvrNetHTTPs_CgiScriptError(session)
#endif

/**
  \brief  Event on HTTP server unauthorized access (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_UnauthorizedAccess(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_UnauthorizedAccess, session, 0);
  }
#else
  #define EvrNetHTTPs_UnauthorizedAccess(session)
#endif

/**
  \brief  Event on HTTP server requested method not implemented (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_MethodNotImplemented(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_MethodNotImplemented, session, 0);
  }
#else
  #define EvrNetHTTPs_MethodNotImplemented(session)
#endif

/**
  \brief  Event on HTTP server user denied access for IPv4 client (Error)
  \param  net_addr      pointer to \ref NET_ADDR4 address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_UserAccessDeniedIp4(const void *net_addr) {
    EventRecordData (EvtNetHTTPs_UserAccessDeniedIp4, net_addr, 8);
  }
#else
  #define EvrNetHTTPs_UserAccessDeniedIp4(net_addr)
#endif

/**
  \brief  Event on HTTP server user denied access for IPv6 client (Error)
  \param  net_addr      pointer to \ref NET_ADDR address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_UserAccessDeniedIp6(const void *net_addr) {
    EventRecordData (EvtNetHTTPs_UserAccessDeniedIp6, net_addr, 20);
  }
#else
  #define EvrNetHTTPs_UserAccessDeniedIp6(net_addr)
#endif

/**
  \brief  Event on HTTP server session open (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_SessionOpen(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_SessionOpen, session, 0);
  }
#else
  #define EvrNetHTTPs_SessionOpen(session)
#endif

/**
  \brief  Event on HTTP server socket aborted (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_SocketAborted(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_SocketAborted, session, 0);
  }
#else
  #define EvrNetHTTPs_SocketAborted(session)
#endif

/**
  \brief  Event on HTTP server socket closed (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_SocketClosed(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_SocketClosed, session, 0);
  }
#else
  #define EvrNetHTTPs_SocketClosed(session)
#endif

/**
  \brief  Event on HTTP server receive frame (Op)
  \param  session       session number
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_ReceiveFrame(uint8_t session, uint32_t length) {
    EventRecord2 (EvtNetHTTPs_ReceiveFrame, session, length);
  }
#else
  #define EvrNetHTTPs_ReceiveFrame(session, length)
#endif

/**
  \brief  Event on HTTP server received frame too short (Error)
  \param  session       session number
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FrameTooShort(uint8_t session, uint32_t length, uint32_t min_length) {
    uint32_t val2 = (min_length << 16) | length;
    EventRecord2 (EvtNetHTTPs_FrameTooShort, session, val2);
  }
#else
  #define EvrNetHTTPs_FrameTooShort(session, length, min_length)
#endif

/**
  \brief  Event on HTTP server display requested method (Op)
  \param  session       session number
  \param  method        http request method
                         - 0: HEAD
                         - 1: GET
                         - 2: POST
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_RequestMethod(uint8_t session, uint8_t method) {
    EventRecord2 (EvtNetHTTPs_RequestMethod, session, method);
  }
#else
  #define EvrNetHTTPs_RequestMethod(session, method)
#endif

/**
  \brief  Event on HTTP server start CGI script interpreter engine (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_CgiStartEngine(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_CgiStartEngine, session, 0);
  }
#else
  #define EvrNetHTTPs_CgiStartEngine(session)
#endif

/**
  \brief  Event on HTTP server requested resource file (Op)
  \param  fname         name of the requested file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_RequestedFile(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetHTTPs_RequestedFile, fname, length);
  }
#else
  #define EvrNetHTTPs_RequestedFile(fname, length)
#endif

/**
  \brief  Event on HTTP server resource file location (Op)
  \param  drive         where the resource file is stored
                         - 0: ROM Image
                         - 1: External Drive
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FileOpenDrive(uint32_t drive) {
    EventRecord2 (EvtNetHTTPs_FileOpenDrive, drive, 0);
  }
#else
  #define EvrNetHTTPs_FileOpenDrive(drive)
#endif

/**
  \brief  Event on HTTP server display open file status (Op)
  \param  size          size of the open file
  \param  lm_time       last-modified time of the file
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_ShowFileStatus(uint32_t size, uint32_t lm_time) {
    EventRecord2 (EvtNetHTTPs_ShowFileStatus, size, lm_time);
  }
#else
  #define EvrNetHTTPs_ShowFileStatus(size, lm_time)
#endif

/**
  \brief  Event on HTTP server display resource file redirection URL (Op)
  \param  url           redirection url in text format
  \param  length        length of the url string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_ShowRedirectionUrl(const char *url, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetHTTPs_ShowRedirectionUrl, url, length);
  }
#else
  #define EvrNetHTTPs_ShowRedirectionUrl(url, length)
#endif

/**
  \brief  Event on HTTP server display user credentials (Op)
  \param  credentials   user credentials in the form of: "username:password"
  \param  length        length of the credentials string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_ShowUserCredentials(const char *credentials, uint32_t length) {
    if (length > 40) length = 40;
    EventRecordData (EvtNetHTTPs_ShowUserCredentials, credentials, length);
  }
#else
  #define EvrNetHTTPs_ShowUserCredentials(credentials, length)
#endif

/**
  \brief  Event on HTTP server display accept language http header (Op)
  \param  accept_lang   accept language header content
  \param  length        length of the accept_lang string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_ShowAcceptLanguage(const char *accept_lang, uint32_t length) {
    if (length > 8) length = 8;
    EventRecordData (EvtNetHTTPs_ShowAcceptLanguage, accept_lang, length);
  }
#else
  #define EvrNetHTTPs_ShowAcceptLanguage(accept_lang, length)
#endif

/**
  \brief  Event on HTTP server display xml content type header (Op)
  \param  xml_type      xml content type in text format
  \param  length        length of the xml_type string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_ShowXmlType(const char *xml_type, uint32_t length) {
    if (length > 64) length = 64;
    EventRecordData (EvtNetHTTPs_ShowXmlType, xml_type, length);
  }
#else
  #define EvrNetHTTPs_ShowXmlType(xml_type, length)
#endif

/**
  \brief  Event on HTTP server POST type: xml-encoded (Op)
  \param  length        content-length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_PostXmlEncoded(uint32_t length) {
    EventRecord2 (EvtNetHTTPs_PostXmlEncoded, length, 0);
  }
#else
  #define EvrNetHTTPs_PostXmlEncoded(length)
#endif

/**
  \brief  Event on HTTP server POST complete for multipacket POST (Op)
  \param  length        data length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_PostCompleteMultipacket(uint32_t length) {
    EventRecord2 (EvtNetHTTPs_PostCompleteMultipacket, length, 0);
  }
#else
  #define EvrNetHTTPs_PostCompleteMultipacket(length)
#endif

/**
  \brief  Event on HTTP server POST type: www-form-urlencoded (Op)
  \param  length        content-length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_PostFormUrlEncoded(uint32_t length) {
    EventRecord2 (EvtNetHTTPs_PostFormUrlEncoded, length, 0);
  }
#else
  #define EvrNetHTTPs_PostFormUrlEncoded(length)
#endif

/**
  \brief  Event on HTTP server POST data truncated in buffer mode (Error)
  \param  new_length    truncated data length
  \note   Buffer mode is used for www-form-urlencoded POST type only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_PostDataTruncated(uint32_t new_length) {
    EventRecord2 (EvtNetHTTPs_PostDataTruncated, new_length, 0);
  }
#else
  #define EvrNetHTTPs_PostDataTruncated(new_length)
#endif

/**
  \brief  Event on HTTP server file upload requested (Op)
  \param  fname         name of the requested file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FileUploadRequested(const char *fname, uint32_t length) {
    if (length > 64) length = 64;
    EventRecordData (EvtNetHTTPs_FileUploadRequested, fname, length);
  }
#else
  #define EvrNetHTTPs_FileUploadRequested(fname, length)
#endif

/**
  \brief  Event on HTTP server data received in file upload process (Op)
  \param  length        length of a file data chunk
  \note   File data is received in multiple packets.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FileUploadDataReceived(uint32_t length) {
    EventRecord2 (EvtNetHTTPs_FileUploadDataReceived, length, 0);
  }
#else
  #define EvrNetHTTPs_FileUploadDataReceived(length)
#endif

/**
  \brief  Event on HTTP server file upload remaining data size to complete (Op)
  \param  length        length of the remaining data
  \note   File data is received in multiple packets.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FileUploadRemaining(uint32_t length) {
    EventRecord2 (EvtNetHTTPs_FileUploadRemaining, length, 0);
  }
#else
  #define EvrNetHTTPs_FileUploadRemaining(length)
#endif

/**
  \brief  Event on HTTP server file upload completed (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_FileUploadComplete(void) {
    EventRecord2 (EvtNetHTTPs_FileUploadComplete, 0, 0);
  }
#else
  #define EvrNetHTTPs_FileUploadComplete()
#endif

/**
  \brief  Event on HTTP server failed to allocate secure TLS context (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_TlsGetContextFailed(uint8_t session) {
    EventRecord2 (EvtNetHTTPs_TlsGetContextFailed, session, 0);
  }
#else
  #define EvrNetHTTPs_TlsGetContextFailed(session)
#endif

/**
  \brief  Event on HTTP server de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetHTTPs_UninitServer(void) {
    EventRecord2 (EvtNetHTTPs_UninitServer, 0, 0);
  }
#else
  #define EvrNetHTTPs_UninitServer()
#endif


// NetFTPs event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetFTPs_InitServer               EventID (EventLevelOp,    EvtNetFTPs,  0)
#define EvtNetFTPs_ShowRootFolder           EventID (EventLevelOp,    EvtNetFTPs,  1)
#define EvtNetFTPs_GetSocketFailed          EventID (EventLevelError, EvtNetFTPs,  2)
#define EvtNetFTPs_SetRootPath              EventID (EventLevelAPI,   EvtNetFTPs,  3)
#define EvtNetFTPs_SetUsername              EventID (EventLevelAPI,   EvtNetFTPs,  4)
#define EvtNetFTPs_SetPassword              EventID (EventLevelAPI,   EvtNetFTPs,  5)
#define EvtNetFTPs_UserAccessDeniedIp4      EventID (EventLevelError, EvtNetFTPs,  6)
#define EvtNetFTPs_UserAccessDeniedIp6      EventID (EventLevelError, EvtNetFTPs,  7)
#define EvtNetFTPs_SessionOpen              EventID (EventLevelOp,    EvtNetFTPs,  8)
#define EvtNetFTPs_SocketAborted            EventID (EventLevelOp,    EvtNetFTPs,  9)
#define EvtNetFTPs_SocketClosed             EventID (EventLevelOp,    EvtNetFTPs, 10)
#define EvtNetFTPs_UnackedDataError         EventID (EventLevelError, EvtNetFTPs, 11)
#define EvtNetFTPs_ReceiveFrame             EventID (EventLevelOp,    EvtNetFTPs, 12)
#define EvtNetFTPs_FrameTooShort            EventID (EventLevelError, EvtNetFTPs, 13)
#define EvtNetFTPs_ShowCommand              EventID (EventLevelOp,    EvtNetFTPs, 14)
#define EvtNetFTPs_VerifyUsername           EventID (EventLevelOp,    EvtNetFTPs, 15)
#define EvtNetFTPs_VerifyPassword           EventID (EventLevelOp,    EvtNetFTPs, 16)
#define EvtNetFTPs_AuthenticationFailed     EventID (EventLevelError, EvtNetFTPs, 17)
#define EvtNetFTPs_UserLoginSuccess         EventID (EventLevelOp,    EvtNetFTPs, 18)
#define EvtNetFTPs_NotAuthenticated         EventID (EventLevelOp,    EvtNetFTPs, 19)
#define EvtNetFTPs_ShowSystemType           EventID (EventLevelOp,    EvtNetFTPs, 20)
#define EvtNetFTPs_NoOperation              EventID (EventLevelOp,    EvtNetFTPs, 21)
#define EvtNetFTPs_CurrentDirectory         EventID (EventLevelOp,    EvtNetFTPs, 22)
#define EvtNetFTPs_ChangeDirectory          EventID (EventLevelOp,    EvtNetFTPs, 23)
#define EvtNetFTPs_ChangeDirectoryLevelUp   EventID (EventLevelOp,    EvtNetFTPs, 24)
#define EvtNetFTPs_MakeDirectory            EventID (EventLevelOp,    EvtNetFTPs, 25)
#define EvtNetFTPs_OperationDenied          EventID (EventLevelOp,    EvtNetFTPs, 26)
#define EvtNetFTPs_RemoveDirectory          EventID (EventLevelOp,    EvtNetFTPs, 27)
#define EvtNetFTPs_RemoveDirectoryFailed    EventID (EventLevelError, EvtNetFTPs, 28)
#define EvtNetFTPs_DirectoryRemoved         EventID (EventLevelOp,    EvtNetFTPs, 29)
#define EvtNetFTPs_ShowDataMode             EventID (EventLevelOp,    EvtNetFTPs, 30)
#define EvtNetFTPs_ActiveModeStart          EventID (EventLevelOp,    EvtNetFTPs, 31)
#define EvtNetFTPs_PassiveModeStart         EventID (EventLevelOp,    EvtNetFTPs, 32)
#define EvtNetFTPs_GetFileSize              EventID (EventLevelOp,    EvtNetFTPs, 33)
#define EvtNetFTPs_GetFileLastModifiedTime  EventID (EventLevelOp,    EvtNetFTPs, 34)
#define EvtNetFTPs_ListDirectoryBasic       EventID (EventLevelOp,    EvtNetFTPs, 35)
#define EvtNetFTPs_ListDirectoryExtended    EventID (EventLevelOp,    EvtNetFTPs, 36)
#define EvtNetFTPs_ReadFile                 EventID (EventLevelOp,    EvtNetFTPs, 37)
#define EvtNetFTPs_FileNotFound             EventID (EventLevelError, EvtNetFTPs, 38)
#define EvtNetFTPs_WriteFile                EventID (EventLevelOp,    EvtNetFTPs, 39)
#define EvtNetFTPs_AppendFile               EventID (EventLevelOp,    EvtNetFTPs, 40)
#define EvtNetFTPs_FileCreateFailed         EventID (EventLevelError, EvtNetFTPs, 41)
#define EvtNetFTPs_DeleteFile               EventID (EventLevelOp,    EvtNetFTPs, 42)
#define EvtNetFTPs_FileDeleteFailed         EventID (EventLevelError, EvtNetFTPs, 43)
#define EvtNetFTPs_FileDeleted              EventID (EventLevelOp,    EvtNetFTPs, 44)
#define EvtNetFTPs_RenameFileFrom           EventID (EventLevelOp,    EvtNetFTPs, 45)
#define EvtNetFTPs_RenameFileTo             EventID (EventLevelOp,    EvtNetFTPs, 46)
#define EvtNetFTPs_FileRenamed              EventID (EventLevelOp,    EvtNetFTPs, 47)
#define EvtNetFTPs_FileRenameFailed         EventID (EventLevelError, EvtNetFTPs, 48)
#define EvtNetFTPs_UnknownCommand           EventID (EventLevelError, EvtNetFTPs, 49)
#define EvtNetFTPs_InvalidState             EventID (EventLevelError, EvtNetFTPs, 50)
#define EvtNetFTPs_InboundConnRejected      EventID (EventLevelError, EvtNetFTPs, 51)
#define EvtNetFTPs_DataSocketClosed         EventID (EventLevelOp,    EvtNetFTPs, 52)
#define EvtNetFTPs_DataSocketOpen           EventID (EventLevelOp,    EvtNetFTPs, 53)
#define EvtNetFTPs_LocalDiskWriteError      EventID (EventLevelError, EvtNetFTPs, 54)
#define EvtNetFTPs_StartService             EventID (EventLevelOp,    EvtNetFTPs, 55)
#define EvtNetFTPs_StopService              EventID (EventLevelOp,    EvtNetFTPs, 56)
#define EvtNetFTPs_ShowFileFindMask         EventID (EventLevelOp,    EvtNetFTPs, 57)
#define EvtNetFTPs_MakeDirectoryFailed      EventID (EventLevelError, EvtNetFTPs, 58)
#define EvtNetFTPs_DirectoryCreated         EventID (EventLevelOp,    EvtNetFTPs, 59)
#define EvtNetFTPs_ShowFileSize             EventID (EventLevelOp,    EvtNetFTPs, 60)
#define EvtNetFTPs_ShowFileLastModifiedTime EventID (EventLevelOp,    EvtNetFTPs, 61)
#define EvtNetFTPs_CloseDataConnection      EventID (EventLevelOp,    EvtNetFTPs, 62)
#define EvtNetFTPs_CloseSession             EventID (EventLevelOp,    EvtNetFTPs, 63)
#define EvtNetFTPs_SessionIdle              EventID (EventLevelOp,    EvtNetFTPs, 64)
#define EvtNetFTPs_ShowPath                 EventID (EventLevelOp,    EvtNetFTPs, 65)
#define EvtNetFTPs_UninitServer             EventID (EventLevelOp,    EvtNetFTPs, 66)
#endif

/**
  \brief  Event on FTP server initialize (Op)
  \param  num_sessions  number of available FTP sessions
  \param  port          listening port number
  \param  idle_tout     idle timeout in seconds (0= permanent connection)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_InitServer(uint32_t num_sessions, uint16_t port, uint16_t idle_tout) {
    uint32_t val2 = (uint32_t)idle_tout << 16 | port;
    EventRecord2 (EvtNetFTPs_InitServer, num_sessions, val2);
  }
#else
  #define EvrNetFTPs_InitServer(num_sessions, port, idle_tout)
#endif

/**
  \brief  Event on FTP server display root folder (Op)
  \param  root_folder   path to server root folder
  \param  length        length of the root_folder string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ShowRootFolder(const char *root_folder, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_ShowRootFolder, root_folder, length);
  }
#else
  #define EvrNetFTPs_ShowRootFolder(root_folder, length)
#endif

/**
  \brief  Event on FTP server failed to allocate TCP socket (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_GetSocketFailed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_GetSocketFailed, session, 0);
  }
#else
  #define EvrNetFTPs_GetSocketFailed(session)
#endif

/**
  \brief  Event on FTP server \ref netFTPs_SetRootPath (API)
  \param  path          path to server root folder
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_SetRootPath(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_SetRootPath, path, length);
  }
#else
  #define EvrNetFTPs_SetRootPath(path, length)
#endif

/**
  \brief  Event on FTP server \ref netFTPs_SetUsername (API)
  \param  username      new username
  \param  length        length of the username string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_SetUsername(const char *username, uint32_t length) {
    if (length > 16) length = 16;
    EventRecordData (EvtNetFTPs_SetUsername, username, length);
  }
#else
  #define EvrNetFTPs_SetUsername(username, length)
#endif

/**
  \brief  Event on FTP server \ref netFTPs_SetPassword (API)
  \param  password      new password
  \param  length        length of the password string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_SetPassword(const char *password, uint32_t length) {
    if (length > 16) length = 16;
    EventRecordData (EvtNetFTPs_SetPassword, password, length);
  }
#else
  #define EvrNetFTPs_SetPassword(password, length)
#endif

/**
  \brief  Event on FTP server user denied access for IPv4 client (Error)
  \param  net_addr      pointer to \ref NET_ADDR4 address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_UserAccessDeniedIp4(const void *net_addr) {
    EventRecordData (EvtNetFTPs_UserAccessDeniedIp4, net_addr, 8);
  }
#else
  #define EvrNetFTPs_UserAccessDeniedIp4(net_addr)
#endif

/**
  \brief  Event on FTP server user denied access for IPv6 client (Error)
  \param  net_addr      pointer to \ref NET_ADDR address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_UserAccessDeniedIp6(const void *net_addr) {
    EventRecordData (EvtNetFTPs_UserAccessDeniedIp6, net_addr, 20);
  }
#else
  #define EvrNetFTPs_UserAccessDeniedIp6(net_addr)
#endif

/**
  \brief  Event on FTP server session open (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_SessionOpen(uint8_t session) {
    EventRecord2 (EvtNetFTPs_SessionOpen, session, 0);
  }
#else
  #define EvrNetFTPs_SessionOpen(session)
#endif

/**
  \brief  Event on FTP server socket aborted (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_SocketAborted(uint8_t session) {
    EventRecord2 (EvtNetFTPs_SocketAborted, session, 0);
  }
#else
  #define EvrNetFTPs_SocketAborted(session)
#endif

/**
  \brief  Event on FTP server socket closed (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_SocketClosed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_SocketClosed, session, 0);
  }
#else
  #define EvrNetFTPs_SocketClosed(session)
#endif

/**
  \brief  Event on FTP server unacked data error (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_UnackedDataError(uint8_t session) {
    EventRecord2 (EvtNetFTPs_UnackedDataError, session, 0);
  }
#else
  #define EvrNetFTPs_UnackedDataError(session)
#endif

/**
  \brief  Event on FTP server receive frame (Op)
  \param  session       session number
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ReceiveFrame(uint8_t session, uint32_t length) {
    EventRecord2 (EvtNetFTPs_ReceiveFrame, session, length);
  }
#else
  #define EvrNetFTPs_ReceiveFrame(session, length)
#endif

/**
  \brief  Event on FTP server received frame too short (Error)
  \param  session       session number
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_FrameTooShort(uint8_t session, uint32_t length, uint32_t min_length) {
    uint32_t val2 = (min_length << 16) | length;
    EventRecord2 (EvtNetFTPs_FrameTooShort, session, val2);
  }
#else
  #define EvrNetFTPs_FrameTooShort(session, length, min_length)
#endif

/**
  \brief  Event on FTP server display command string (Op)
  \param  cmd_client    command received from the client in text format
  \param  length        length of the cmd_client string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ShowCommand(const uint8_t *cmd_client, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_ShowCommand, cmd_client, length);
  }
#else
  #define EvrNetFTPs_ShowCommand(cmd_client, length)
#endif

/**
  \brief  Event on FTP server verify the client username (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_VerifyUsername(uint8_t session) {
    EventRecord2 (EvtNetFTPs_VerifyUsername, session, 0);
  }
#else
  #define EvrNetFTPs_VerifyUsername(session)
#endif

/**
  \brief  Event on FTP server verify the client password (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_VerifyPassword(uint8_t session) {
    EventRecord2 (EvtNetFTPs_VerifyPassword, session, 0);
  }
#else
  #define EvrNetFTPs_VerifyPassword(session)
#endif

/**
  \brief  Event on FTP server authentication failed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_AuthenticationFailed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_AuthenticationFailed, session, 0);
  }
#else
  #define EvrNetFTPs_AuthenticationFailed(session)
#endif

/**
  \brief  Event on FTP server user login successful (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_UserLoginSuccess(uint8_t session) {
    EventRecord2 (EvtNetFTPs_UserLoginSuccess, session, 0);
  }
#else
  #define EvrNetFTPs_UserLoginSuccess(session)
#endif

/**
  \brief  Event on FTP server command ignored, user not authenticated (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_NotAuthenticated(uint8_t session) {
    EventRecord2 (EvtNetFTPs_NotAuthenticated, session, 0);
  }
#else
  #define EvrNetFTPs_NotAuthenticated(session)
#endif

/**
  \brief  Event on FTP server show system identification type command (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ShowSystemType(uint8_t session) {
    EventRecord2 (EvtNetFTPs_ShowSystemType, session, 0);
  }
#else
  #define EvrNetFTPs_ShowSystemType(session)
#endif

/**
  \brief  Event on FTP server execute no operation command (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_NoOperation(uint8_t session) {
    EventRecord2 (EvtNetFTPs_NoOperation, session, 0);
  }
#else
  #define EvrNetFTPs_NoOperation(session)
#endif

/**
  \brief  Event on FTP server display current working directory (Op)
  \param  path          path to the working directory
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_CurrentDirectory(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_CurrentDirectory, path, length);
  }
#else
  #define EvrNetFTPs_CurrentDirectory(path, length)
#endif

/**
  \brief  Event on FTP server change working directory command (Op)
  \param  path          path to the new working directory
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ChangeDirectory(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_ChangeDirectory, path, length);
  }
#else
  #define EvrNetFTPs_ChangeDirectory(path, length)
#endif

/**
  \brief  Event on FTP server change working directory one level up command (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ChangeDirectoryLevelUp(uint8_t session) {
    EventRecord2 (EvtNetFTPs_ChangeDirectoryLevelUp, session, 0);
  }
#else
  #define EvrNetFTPs_ChangeDirectoryLevelUp(session)
#endif

/**
  \brief  Event on FTP server make directory command (Op)
  \param  path          path to the new directory to make
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_MakeDirectory(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_MakeDirectory, path, length);
  }
#else
  #define EvrNetFTPs_MakeDirectory(path, length)
#endif

/**
  \brief  Event on FTP server access or operation denied (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_OperationDenied(uint8_t session) {
    EventRecord2 (EvtNetFTPs_OperationDenied, session, 0);
  }
#else
  #define EvrNetFTPs_OperationDenied(session)
#endif

/**
  \brief  Event on FTP server remove directory command (Op)
  \param  path          path to the directory to remove
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_RemoveDirectory(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_RemoveDirectory, path, length);
  }
#else
  #define EvrNetFTPs_RemoveDirectory(path, length)
#endif

/**
  \brief  Event on FTP server remove directory command failed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_RemoveDirectoryFailed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_RemoveDirectoryFailed, session, 0);
  }
#else
  #define EvrNetFTPs_RemoveDirectoryFailed(session)
#endif

/**
  \brief  Event on FTP server requested directory successfuly removed (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_DirectoryRemoved(uint8_t session) {
    EventRecord2 (EvtNetFTPs_DirectoryRemoved, session, 0);
  }
#else
  #define EvrNetFTPs_DirectoryRemoved(session)
#endif

/**
  \brief  Event on FTP server display transfer data mode (Op)
  \param  session       session number
  \param  data_mode     transfer data mode
                         - 0: Binary mode
                         - 1: ASCII mode
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ShowDataMode(uint8_t session, uint32_t data_mode) {
    EventRecord2 (EvtNetFTPs_ShowDataMode, session, data_mode);
  }
#else
  #define EvrNetFTPs_ShowDataMode(session, dadta_mode)
#endif

/**
  \brief  Event on FTP server start active mode (Op)
  \param  session       session number
  \param  port          port number to connect to
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ActiveModeStart(uint8_t session, uint16_t port) {
    EventRecord2 (EvtNetFTPs_ActiveModeStart, session, port);
  }
#else
  #define EvrNetFTPs_ActiveModeStart(session, port)
#endif

/**
  \brief  Event on FTP server start passive mode (Op)
  \param  session       session number
  \param  local_port    local port number to accept data connection
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_PassiveModeStart(uint8_t session, uint16_t local_port) {
    EventRecord2 (EvtNetFTPs_PassiveModeStart, session, local_port);
  }
#else
  #define EvrNetFTPs_PassiveModeStart(session, local_port)
#endif

/**
  \brief  Event on FTP server get file size command (Op)
  \param  fname         name of the file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_GetFileSize(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_GetFileSize, fname, length);
  }
#else
  #define EvrNetFTPs_GetFileSize(fname, length)
#endif

/**
  \brief  Event on FTP server get file last-modified time command (Op)
  \param  fname         name of the file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_GetFileLastModifiedTime(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_GetFileLastModifiedTime, fname, length);
  }
#else
  #define EvrNetFTPs_GetFileLastModifiedTime(fname, length)
#endif

/**
  \brief  Event on FTP server list directory names command (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ListDirectoryBasic(uint8_t session) {
    EventRecord2 (EvtNetFTPs_ListDirectoryBasic, session, 0);
  }
#else
  #define EvrNetFTPs_ListDirectoryBasic(session)
#endif

/**
  \brief  Event on FTP server list directory in extended format (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ListDirectoryExtended(uint8_t session) {
    EventRecord2 (EvtNetFTPs_ListDirectoryExtended, session, 0);
  }
#else
  #define EvrNetFTPs_ListDirectoryExtended(session)
#endif

/**
  \brief  Event on FTP server read file command (Op)
  \param  fname         name of the file to read
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ReadFile(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_ReadFile, fname, length);
  }
#else
  #define EvrNetFTPs_ReadFile(fname, length)
#endif

/**
  \brief  Event on FTP server requested file not found (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_FileNotFound(uint8_t session) {
    EventRecord2 (EvtNetFTPs_FileNotFound, session, 0);
  }
#else
  #define EvrNetFTPs_FileNotFound(session)
#endif

/**
  \brief  Event on FTP server write file command (Op)
  \param  fname         name of the file to write
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_WriteFile(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_WriteFile, fname, length);
  }
#else
  #define EvrNetFTPs_WriteFile(fname, length)
#endif

/**
  \brief  Event on FTP server append file command (Op)
  \param  fname         name of the file to append
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_AppendFile(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_AppendFile, fname, length);
  }
#else
  #define EvrNetFTPs_AppendFile(fname, length)
#endif

/**
  \brief  Event on FTP server requested file create failed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_FileCreateFailed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_FileCreateFailed, session, 0);
  }
#else
  #define EvrNetFTPs_FileCreateFailed(session)
#endif

/**
  \brief  Event on FTP server delete file command (Op)
  \param  fname         name of the file to delete
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_DeleteFile(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_DeleteFile, fname, length);
  }
#else
  #define EvrNetFTPs_DeleteFile(fname, length)
#endif

/**
  \brief  Event on FTP server requested file delete failed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_FileDeleteFailed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_FileDeleteFailed, session, 0);
  }
#else
  #define EvrNetFTPs_FileDeleteFailed(session)
#endif

/**
  \brief  Event on FTP server requested file deleted successfuly (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_FileDeleted(uint8_t session) {
    EventRecord2 (EvtNetFTPs_FileDeleted, session, 0);
  }
#else
  #define EvrNetFTPs_FileDeleted(session)
#endif

/**
  \brief  Event on FTP server rename file from name command (Op)
  \param  fname         name of the file to rename
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_RenameFileFrom(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_RenameFileFrom, fname, length);
  }
#else
  #define EvrNetFTPs_RenameFileFrom(fname, length)
#endif

/**
  \brief  Event on FTP server rename file to new name (Op)
  \param  new_name      new name of the file to rename to
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_RenameFileTo(const char *new_name, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_RenameFileTo, new_name, length);
  }
#else
  #define EvrNetFTPs_RenameFileTo(new_name, length)
#endif

/**
  \brief  Event on FTP server requested file renamed successfuly (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_FileRenamed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_FileRenamed, session, 0);
  }
#else
  #define EvrNetFTPs_FileRenamed(session)
#endif

/**
  \brief  Event on FTP server requested file rename failed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_FileRenameFailed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_FileRenameFailed, session, 0);
  }
#else
  #define EvrNetFTPs_FileRenameFailed(session)
#endif

/**
  \brief  Event on FTP server unknown command received (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_UnknownCommand(uint8_t session) {
    EventRecord2 (EvtNetFTPs_UnknownCommand, session, 0);
  }
#else
  #define EvrNetFTPs_UnknownCommand(session)
#endif

/**
  \brief  Event on FTP server session in invalid state, command ignored (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_InvalidState(uint8_t session) {
    EventRecord2 (EvtNetFTPs_InvalidState, session, 0);
  }
#else
  #define EvrNetFTPs_InvalidState(session)
#endif

/**
  \brief  Event on FTP server inbound connection rejected for data socket (Error)
  \param  session       session number
  \note   Inbound data connections are accepted in passive mode, rejected in active mode.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_InboundConnRejected(uint8_t session) {
    EventRecord2 (EvtNetFTPs_InboundConnRejected, session, 0);
  }
#else
  #define EvrNetFTPs_InboundConnRejected(session)
#endif

/**
  \brief  Event on FTP server data socket closed when transfer completed (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_DataSocketClosed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_DataSocketClosed, session, 0);
  }
#else
  #define EvrNetFTPs_DataSocketClosed(session)
#endif

/**
  \brief  Event on FTP server data socket open for data transfer (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_DataSocketOpen(uint8_t session) {
    EventRecord2 (EvtNetFTPs_DataSocketOpen, session, 0);
  }
#else
  #define EvrNetFTPs_DataSocketOpen(session)
#endif

/**
  \brief  Event on FTP server write to local disk failed, disk full (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_LocalDiskWriteError(uint8_t session) {
    EventRecord2 (EvtNetFTPs_LocalDiskWriteError, session, 0);
  }
#else
  #define EvrNetFTPs_LocalDiskWriteError(session)
#endif

/**
  \brief  Event on FTP server start service (Op)
  \param  port          listening port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_StartService(uint16_t port) {
    EventRecord2 (EvtNetFTPs_StartService, port, 0);
  }
#else
  #define EvrNetFTPs_StartService(port)
#endif

/**
  \brief  Event on FTP server stop service (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_StopService(void) {
    EventRecord2 (EvtNetFTPs_StopService, 0, 0);
  }
#else
  #define EvrNetFTPs_StopService()
#endif

/**
  \brief  Event on FTP server display find file(s) mask filter (Op)
  \param  path          path to list directory
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ShowFileFindMask(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_ShowFileFindMask, path, length);
  }
#else
  #define EvrNetFTPs_ShowFileFindMask(path, length)
#endif

/**
  \brief  Event on FTP server requested directory create failed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_MakeDirectoryFailed(uint8_t session) {
    EventRecord2 (EvtNetFTPs_MakeDirectoryFailed, session, 0);
  }
#else
  #define EvrNetFTPs_MakeDirectoryFailed(session)
#endif

/**
  \brief  Event on FTP server requested directory created successfuly (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_DirectoryCreated(uint8_t session) {
    EventRecord2 (EvtNetFTPs_DirectoryCreated, session, 0);
  }
#else
  #define EvrNetFTPs_DirectoryCreated(session)
#endif

/**
  \brief  Event on FTP server display requested file size (Op)
  \param  file_size     size of a file in text format
  \param  length        length of the file_size string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ShowFileSize(const char *file_size, uint32_t length) {
    if (length > 20) length = 20;
    EventRecordData (EvtNetFTPs_ShowFileSize, file_size, length);
  }
#else
  #define EvrNetFTPs_ShowFileSize(file_size, length)
#endif

/**
  \brief  Event on FTP server display requested file last-modified time (Op)
  \param  file_time     last-modified time of a file in text format
  \param  length        length of the file_time string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ShowFileLastModifiedTime(const char *file_time, uint32_t length) {
    if (length > 20) length = 20;
    EventRecordData (EvtNetFTPs_ShowFileLastModifiedTime, file_time, length);
  }
#else
  #define EvrNetFTPs_ShowFileLastModifiedTime(file_time, length)
#endif

/**
  \brief  Event on FTP server closing data connection (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_CloseDataConnection(uint8_t session) {
    EventRecord2 (EvtNetFTPs_CloseDataConnection, session, 0);
  }
#else
  #define EvrNetFTPs_CloseDataConnection(session)
#endif

/**
  \brief  Event on FTP server closing session (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_CloseSession(uint8_t session) {
    EventRecord2 (EvtNetFTPs_CloseSession, session, 0);
  }
#else
  #define EvrNetFTPs_CloseSession(session)
#endif

/**
  \brief  Event on FTP server session in idle state (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_SessionIdle(uint8_t session) {
    EventRecord2 (EvtNetFTPs_SessionIdle, session, 0);
  }
#else
  #define EvrNetFTPs_SessionIdle(session)
#endif

/**
  \brief  Event on FTP server display an absolute path for file access (Op)
  \param  path          absolute path for file access
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_ShowPath(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetFTPs_ShowPath, path, length);
  }
#else
  #define EvrNetFTPs_ShowPath(path, length)
#endif

/**
  \brief  Event on FTP server de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPs_UninitServer(void) {
    EventRecord2 (EvtNetFTPs_UninitServer, 0, 0);
  }
#else
  #define EvrNetFTPs_UninitServer()
#endif


// NetFTPc event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetFTPc_InitClient               EventID (EventLevelOp,    EvtNetFTPc,  0)
#define EvtNetFTPc_GetSocketFailed          EventID (EventLevelError, EvtNetFTPc,  1)
#define EvtNetFTPc_ConnectIp4               EventID (EventLevelAPI,   EvtNetFTPc,  2)
#define EvtNetFTPc_ConnectIp6               EventID (EventLevelAPI,   EvtNetFTPc,  3)
#define EvtNetFTPc_ConnectInvalidParameter  EventID (EventLevelError, EvtNetFTPc,  4)
#define EvtNetFTPc_ConnectClientBusy        EventID (EventLevelError, EvtNetFTPc,  5)
#define EvtNetFTPc_InboundConnRejected      EventID (EventLevelOp,    EvtNetFTPc,  6)
#define EvtNetFTPc_SocketAborted            EventID (EventLevelError, EvtNetFTPc,  7)
#define EvtNetFTPc_SocketConnected          EventID (EventLevelOp,    EvtNetFTPc,  8)
#define EvtNetFTPc_SocketClosed             EventID (EventLevelOp,    EvtNetFTPc,  9)
#define EvtNetFTPc_ReceiveFrame             EventID (EventLevelOp,    EvtNetFTPc, 10)
#define EvtNetFTPc_FrameTooShort            EventID (EventLevelError, EvtNetFTPc, 11)
#define EvtNetFTPc_ShowReplyCode            EventID (EventLevelOp,    EvtNetFTPc, 12)
#define EvtNetFTPc_ResponseFragmented       EventID (EventLevelOp,    EvtNetFTPc, 13)
#define EvtNetFTPc_ServerReady              EventID (EventLevelOp,    EvtNetFTPc, 14)
#define EvtNetFTPc_UserOkNeedPassword       EventID (EventLevelOp,    EvtNetFTPc, 15)
#define EvtNetFTPc_UserLoginFailed          EventID (EventLevelError, EvtNetFTPc, 16)
#define EvtNetFTPc_UserLoginSuccess         EventID (EventLevelOp,    EvtNetFTPc, 17)
#define EvtNetFTPc_WorkingDirectoryInvalid  EventID (EventLevelError, EvtNetFTPc, 18)
#define EvtNetFTPc_ExecuteUserCommand       EventID (EventLevelOp,    EvtNetFTPc, 19)
#define EvtNetFTPc_BinaryModeEnabled        EventID (EventLevelOp,    EvtNetFTPc, 20)
#define EvtNetFTPc_PasvCommandFailed        EventID (EventLevelError, EvtNetFTPc, 21)
#define EvtNetFTPc_PassiveModeStart         EventID (EventLevelOp,    EvtNetFTPc, 22)
#define EvtNetFTPc_PortCommandFailed        EventID (EventLevelError, EvtNetFTPc, 23)
#define EvtNetFTPc_ActiveModeStart          EventID (EventLevelOp,    EvtNetFTPc, 24)
#define EvtNetFTPc_FileNotFoundOnServer     EventID (EventLevelError, EvtNetFTPc, 25)
#define EvtNetFTPc_OperationNotAllowed      EventID (EventLevelError, EvtNetFTPc, 26)
#define EvtNetFTPc_AboutToOpenDataConn      EventID (EventLevelOp,    EvtNetFTPc, 27)
#define EvtNetFTPc_DataConnAlreadyOpen      EventID (EventLevelOp,    EvtNetFTPc, 28)
#define EvtNetFTPc_TransferAborted          EventID (EventLevelOp,    EvtNetFTPc, 29)
#define EvtNetFTPc_TransferCompleted        EventID (EventLevelOp,    EvtNetFTPc, 30)
#define EvtNetFTPc_FileDeleted              EventID (EventLevelOp,    EvtNetFTPc, 31)
#define EvtNetFTPc_NewNameRequired          EventID (EventLevelOp,    EvtNetFTPc, 32)
#define EvtNetFTPc_FileOrDirectoryRenamed   EventID (EventLevelOp,    EvtNetFTPc, 33)
#define EvtNetFTPc_DirectoryCreated         EventID (EventLevelOp,    EvtNetFTPc, 34)
#define EvtNetFTPc_FileOrPathNotFound       EventID (EventLevelOp,    EvtNetFTPc, 35)
#define EvtNetFTPc_DirectoryRemoved         EventID (EventLevelOp,    EvtNetFTPc, 36)
#define EvtNetFTPc_CommandErrorResponse     EventID (EventLevelError, EvtNetFTPc, 37)
#define EvtNetFTPc_WrongResponse            EventID (EventLevelError, EvtNetFTPc, 38)
#define EvtNetFTPc_DataSocketClosed         EventID (EventLevelOp,    EvtNetFTPc, 39)
#define EvtNetFTPc_DataSocketOpened         EventID (EventLevelOp,    EvtNetFTPc, 40)
#define EvtNetFTPc_LocalDiskWriteError      EventID (EventLevelError, EvtNetFTPc, 41)
#define EvtNetFTPc_ClientStopTimeoutExpired EventID (EventLevelError, EvtNetFTPc, 42)
#define EvtNetFTPc_LocalPortAssigned        EventID (EventLevelOp,    EvtNetFTPc, 43)
#define EvtNetFTPc_OpenLocalFile            EventID (EventLevelOp,    EvtNetFTPc, 44)
#define EvtNetFTPc_LocalFileCreateFailed    EventID (EventLevelError, EvtNetFTPc, 45)
#define EvtNetFTPc_LocalFileNotFound        EventID (EventLevelError, EvtNetFTPc, 46)
#define EvtNetFTPc_OpenDataConnFailed       EventID (EventLevelError, EvtNetFTPc, 47)
#define EvtNetFTPc_SendCommand              EventID (EventLevelDetail,EvtNetFTPc, 48)
#define EvtNetFTPc_ClientCloseSocket        EventID (EventLevelOp,    EvtNetFTPc, 49)
#define EvtNetFTPc_ClientDone               EventID (EventLevelOp,    EvtNetFTPc, 50)
#define EvtNetFTPc_CloseLocalFile           EventID (EventLevelOp,    EvtNetFTPc, 51)
#define EvtNetFTPc_UninitClient             EventID (EventLevelOp,    EvtNetFTPc, 52)
#endif

/**
  \brief  Event on FTP client initialize (Op)
  \param  mode           client mode of operation
                          - 0: active mode (accepts inbound data connections)
                          - 1: passive mode (starts outbound data connections)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_InitClient(uint32_t mode) {
    EventRecord2 (EvtNetFTPc_InitClient, mode, 0);
  }
#else
  #define EvrNetFTPc_InitClient(mode)
#endif

/**
  \brief  Event on FTP client failed to allocate TCP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_GetSocketFailed(void) {
    EventRecord2 (EvtNetFTPc_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetFTPc_GetSocketFailed()
#endif

/**
  \brief  Event on FTP client \ref netFTPc_Connect to IPv4 server (API)
  \param  net_addr      pointer to \ref NET_ADDR4 server address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ConnectIp4(const void *net_addr) {
    EventRecordData (EvtNetFTPc_ConnectIp4, net_addr, 8);
  }
#else
  #define EvrNetFTPc_ConnectIp4(net_addr)
#endif

/**
  \brief  Event on FTP client \ref netFTPc_Connect to IPv6 server (API)
  \param  net_addr      pointer to \ref NET_ADDR server address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ConnectIp6(const void *net_addr) {
    EventRecordData (EvtNetFTPc_ConnectIp6, net_addr, 20);
  }
#else
  #define EvrNetFTPc_ConnectIp6(net_addr)
#endif

/**
  \brief  Event on \ref netFTPc_Connect invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ConnectInvalidParameter(void) {
    EventRecord2 (EvtNetFTPc_ConnectInvalidParameter, 0, 0);
  }
#else
  #define EvrNetFTPc_ConnectInvalidParameter()
#endif

/**
  \brief  Event on \ref netFTPc_Connect failed, client busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ConnectClientBusy(void) {
    EventRecord2 (EvtNetFTPc_ConnectClientBusy, 0, 0);
  }
#else
  #define EvrNetFTPc_ConnectClientBusy()
#endif

/**
  \brief  Event on FTP client inbound connection rejected (Error)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_InboundConnRejected(int32_t socket) {
    EventRecord2 (EvtNetFTPc_InboundConnRejected, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_InboundConnRejected(socket)
#endif

/**
  \brief  Event on FTP client socket aborted (Error)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_SocketAborted(int32_t socket) {
    EventRecord2 (EvtNetFTPc_SocketAborted, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_SocketAborted(socket)
#endif

/**
  \brief  Event on FTP client socket connected (Op)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_SocketConnected(int32_t socket) {
    EventRecord2 (EvtNetFTPc_SocketConnected, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_SocketConnected(socket)
#endif

/**
  \brief  Event on FTP client socket closed (Op)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_SocketClosed(int32_t socket) {
    EventRecord2 (EvtNetFTPc_SocketClosed, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_SocketClosed(socket)
#endif

/**
  \brief  Event on FTP client received frame (Op)
  \param  socket        assigned TCP socket
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ReceiveFrame(int32_t socket, uint32_t length) {
    EventRecord2 (EvtNetFTPc_ReceiveFrame, (uint32_t)socket, length);
  }
#else
  #define EvrNetFTPc_ReceiveFrame(socket, length)
#endif

/**
  \brief  Event on FTP received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetFTPc_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetFTPc_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on FTP client display numeric reply code received from server (Op)
  \param  reply_code    numeric reply code in text format (3 characters)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ShowReplyCode(const uint8_t *reply_code) {
    EventRecordData (EvtNetFTPc_ShowReplyCode, reply_code, 3);
  }
#else
  #define EvrNetFTPc_ShowReplyCode(reply_code)
#endif

/**
  \brief  Event on FTP client response divided into multiple frames (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ResponseFragmented(void) {
    EventRecord2 (EvtNetFTPc_ResponseFragmented, 0, 0);
  }
#else
  #define EvrNetFTPc_ResponseFragmented()
#endif

/**
  \brief  Event on FTP client server ready response (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ServerReady(void) {
    EventRecord2 (EvtNetFTPc_ServerReady, 0, 0);
  }
#else
  #define EvrNetFTPc_ServerReady()
#endif

/**
  \brief  Event on FTP client username ok, password needed (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_UserOkNeedPassword(void) {
    EventRecord2 (EvtNetFTPc_UserOkNeedPassword, 0, 0);
  }
#else
  #define EvrNetFTPc_UserOkNeedPassword()
#endif

/**
  \brief  Event on FTP client user login to server failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_UserLoginFailed(void) {
    EventRecord2 (EvtNetFTPc_UserLoginFailed, 0, 0);
  }
#else
  #define EvrNetFTPc_UserLoginFailed()
#endif

/**
  \brief  Event on FTP client user login successful (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_UserLoginSuccess(void) {
    EventRecord2 (EvtNetFTPc_UserLoginSuccess, 0, 0);
  }
#else
  #define EvrNetFTPc_UserLoginSuccess()
#endif

/**
  \brief  Event on FTP client change working directory failed, invalid path (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_WorkingDirectoryInvalid(void) {
    EventRecord2 (EvtNetFTPc_WorkingDirectoryInvalid, 0, 0);
  }
#else
  #define EvrNetFTPc_WorkingDirectoryInvalid()
#endif

/**
  \brief  Event on FTP client server ready response (Op)
  \param  command       user command to execute
                         - 0: Puts a file on FTP server
                         - 1: Retrieves a file from FTP server
                         - 2: Append file on FTP server (with create)
                         - 3: Deletes a file on FTP server
                         - 4: Lists files stored on FTP server
                         - 5: Renames a file on FTP server
                         - 6: Makes a directory on FTP server
                         - 7: Removes an empty directory on FTP serve
                         - 8: Lists file names only (short format)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ExecuteUserCommand(uint8_t command) {
    EventRecord2 (EvtNetFTPc_ExecuteUserCommand, command, 0);
  }
#else
  #define EvrNetFTPc_ExecuteUserCommand(command)
#endif

/**
  \brief  Event on FTP client binary transfer mode enabled (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_BinaryModeEnabled(void) {
    EventRecord2 (EvtNetFTPc_BinaryModeEnabled, 0, 0);
  }
#else
  #define EvrNetFTPc_BinaryModeEnabled()
#endif

/**
  \brief  Event on FTP client PASV command failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_PasvCommandFailed(void) {
    EventRecord2 (EvtNetFTPc_PasvCommandFailed, 0, 0);
  }
#else
  #define EvrNetFTPc_PasvCommandFailed()
#endif

/**
  \brief  Event on FTP client start passive server mode (Op)
  \param  port          server port number to connect to
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_PassiveModeStart(uint16_t port) {
    EventRecord2 (EvtNetFTPc_PassiveModeStart, port, 0);
  }
#else
  #define EvrNetFTPc_PassiveModeStart(port)
#endif

/**
  \brief  Event on FTP client PORT command failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_PortCommandFailed(void) {
    EventRecord2 (EvtNetFTPc_PortCommandFailed, 0, 0);
  }
#else
  #define EvrNetFTPc_PortCommandFailed()
#endif

/**
  \brief  Event on FTP client start active server mode (Op)
  \param  local_port    local port number to accept connection
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ActiveModeStart(uint16_t local_port) {
    EventRecord2 (EvtNetFTPc_ActiveModeStart, local_port, 0);
  }
#else
  #define EvrNetFTPc_ActiveModeStart(local_port)
#endif

/**
  \brief  Event on FTP client error, requested file not found on server (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_FileNotFoundOnServer(void) {
    EventRecord2 (EvtNetFTPc_FileNotFoundOnServer, 0, 0);
  }
#else
  #define EvrNetFTPc_FileNotFoundOnServer()
#endif

/**
  \brief  Event on FTP client error, operation not allowed on server (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_OperationNotAllowed(void) {
    EventRecord2 (EvtNetFTPc_OperationNotAllowed, 0, 0);
  }
#else
  #define EvrNetFTPc_OperationNotAllowed()
#endif

/**
  \brief  Event on FTP client about to open data connection (Op)
  \param  socket        assigned TCP data socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_AboutToOpenDataConn(int32_t socket) {
    EventRecord2 (EvtNetFTPc_AboutToOpenDataConn, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_AboutToOpenDataConn(socket)
#endif

/**
  \brief  Event on FTP client data connection already opened (Op)
  \param  socket        assigned TCP data socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_DataConnAlreadyOpen(int32_t socket) {
    EventRecord2 (EvtNetFTPc_DataConnAlreadyOpen, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_DataConnAlreadyOpen(socket)
#endif

/**
  \brief  Event on FTP client data transfer aborted (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_TransferAborted(void) {
    EventRecord2 (EvtNetFTPc_TransferAborted, 0, 0);
  }
#else
  #define EvrNetFTPc_TransferAborted()
#endif

/**
  \brief  Event on FTP client data transfer completed successfully (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_TransferCompleted(void) {
    EventRecord2 (EvtNetFTPc_TransferCompleted, 0, 0);
  }
#else
  #define EvrNetFTPc_TransferCompleted()
#endif

/**
  \brief  Event on FTP client file deleted on the server (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_FileDeleted(void) {
    EventRecord2 (EvtNetFTPc_FileDeleted, 0, 0);
  }
#else
  #define EvrNetFTPc_FileDeleted()
#endif

/**
  \brief  Event on FTP client new name required to rename a file on the server (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_NewNameRequired(void) {
    EventRecord2 (EvtNetFTPc_NewNameRequired, 0, 0);
  }
#else
  #define EvrNetFTPc_NewNameRequired()
#endif

/**
  \brief  Event on FTP client file or directory renamed successfully on the server (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_FileOrDirectoryRenamed(void) {
    EventRecord2 (EvtNetFTPc_FileOrDirectoryRenamed, 0, 0);
  }
#else
  #define EvrNetFTPc_FileOrDirectoryRenamed()
#endif

/**
  \brief  Event on FTP client directory successfully created on the server (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_DirectoryCreated(void) {
    EventRecord2 (EvtNetFTPc_DirectoryCreated, 0, 0);
  }
#else
  #define EvrNetFTPc_DirectoryCreated()
#endif

/**
  \brief  Event on FTP client requested file or path not found on the server (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_FileOrPathNotFound(void) {
    EventRecord2 (EvtNetFTPc_FileOrPathNotFound, 0, 0);
  }
#else
  #define EvrNetFTPc_FileOrPathNotFound()
#endif

/**
  \brief  Event on FTP client directory removed on the server (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_DirectoryRemoved(void) {
    EventRecord2 (EvtNetFTPc_DirectoryRemoved, 0, 0);
  }
#else
  #define EvrNetFTPc_DirectoryRemoved()
#endif

/**
  \brief  Event on FTP client command error response received (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_CommandErrorResponse(void) {
    EventRecord2 (EvtNetFTPc_CommandErrorResponse, 0, 0);
  }
#else
  #define EvrNetFTPc_CommandErrorResponse()
#endif

/**
  \brief  Event on FTP client wrong response received (Error)
  \param  response      response from the server in text format
  \param  length        length of the response string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_WrongResponse(const uint8_t *response, uint32_t length) {
    if (length > 40) length = 40;
    EventRecordData (EvtNetFTPc_WrongResponse, response, length);
  }
#else
  #define EvrNetFTPc_WrongResponse(response, length)
#endif

/**
  \brief  Event on FTP client data socket closed (Op)
  \param  socket        assigned TCP data socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_DataSocketClosed(int32_t socket) {
    EventRecord2 (EvtNetFTPc_DataSocketClosed, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_DataSocketClosed(socket)
#endif

/**
  \brief  Event on FTP client data socket opened (Op)
  \param  socket        assigned TCP data socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_DataSocketOpened(int32_t socket) {
    EventRecord2 (EvtNetFTPc_DataSocketOpened, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_DataSocketOpened(socket)
#endif

/**
  \brief  Event on FTP client write to local disk failed, disk full (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_LocalDiskWriteError(void) {
    EventRecord2 (EvtNetFTPc_LocalDiskWriteError, 0, 0);
  }
#else
  #define EvrNetFTPc_LocalDiskWriteError()
#endif

/**
  \brief  Event on FTP client stop operation timeout expired (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ClientStopTimeoutExpired(void) {
    EventRecord2 (EvtNetFTPc_ClientStopTimeoutExpired, 0, 0);
  }
#else
  #define EvrNetFTPc_ClientStopTimeoutExpired()
#endif

/**
  \brief  Event on FTP client local port assigned for server active mode (Op)
  \param  local_port    local port number to accept connection
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_LocalPortAssigned(uint16_t local_port) {
    EventRecord2 (EvtNetFTPc_LocalPortAssigned, local_port, 0);
  }
#else
  #define EvrNetFTPc_LocalPortAssigned(local_port)
#endif

/**
  \brief  Event on FTP client open local file (Op)
  \param  local_fname   name of the local file
  \param  length        length of the local_fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_OpenLocalFile(const char *local_fname, uint32_t length) {
    if (length > 120) length = 120;
    EventRecordData (EvtNetFTPc_OpenLocalFile, local_fname, length);
  }
#else
  #define EvrNetFTPc_OpenLocalFile(local_fname, length)
#endif

/**
  \brief  Event on FTP client local file create failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_LocalFileCreateFailed(void) {
    EventRecord2 (EvtNetFTPc_LocalFileCreateFailed, 0, 0);
  }
#else
  #define EvrNetFTPc_LocalFileCreateFailed()
#endif

/**
  \brief  Event on FTP client local file not found (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_LocalFileNotFound(void) {
    EventRecord2 (EvtNetFTPc_LocalFileNotFound, 0, 0);
  }
#else
  #define EvrNetFTPc_LocalFileNotFound()
#endif

/**
  \brief  Event on FTP client open data connection failed (Error)
  \param  socket        assigned TCP data socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_OpenDataConnFailed(int32_t socket) {
    EventRecord2 (EvtNetFTPc_OpenDataConnFailed, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_OpenDataConnFailed(socket)
#endif

/**
  \brief  Event on FTP client send command to server (Detail)
  \param  command       server command in text form
  \param  length        length of the command string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_SendCommand(const uint8_t *command, uint32_t length) {
    if (length > 64) length = 64;
    EventRecordData (EvtNetFTPc_SendCommand, command, length);
  }
#else
  #define EvrNetFTPc_SendCommand(command, length)
#endif

/**
  \brief  Event on FTP client close control socket (Op)
  \param  socket        assigned TCP control socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ClientCloseSocket(int32_t socket) {
    EventRecord2 (EvtNetFTPc_ClientCloseSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetFTPc_ClientCloseSocket(socket)
#endif

/**
  \brief  Event on FTP client completed operation (Op)
  \param  cb_event      user callback nofication event
                         - 0: File operation successful
                         - 1: Timeout on file operation
                         - 2: Login error, username/password invalid
                         - 3: File access not allowed
                         - 4: File not found
                         - 5: Working directory path not found
                         - 6: Local file read/write error
                         - 7: Generic FTP client error
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_ClientDone(uint8_t cb_event) {
    EventRecord2 (EvtNetFTPc_ClientDone, cb_event, 0);
  }
#else
  #define EvrNetFTPc_ClientDone(cb_event)
#endif

/**
  \brief  Event on FTP client close local file (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_CloseLocalFile(void) {
    EventRecord2 (EvtNetFTPc_CloseLocalFile, 0, 0);
  }
#else
  #define EvrNetFTPc_CloseLocalFile()
#endif

/**
  \brief  Event on FTP client de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetFTPc_UninitClient(void) {
    EventRecord2 (EvtNetFTPc_UninitClient, 0, 0);
  }
#else
  #define EvrNetFTPc_UninitClient()
#endif


// NetTeln event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetTeln_InitServer               EventID (EventLevelOp,    EvtNetTeln,  0)
#define EvtNetTeln_GetSocketFailed          EventID (EventLevelError, EvtNetTeln,  1)
#define EvtNetTeln_SetUsername              EventID (EventLevelAPI,   EvtNetTeln,  2)
#define EvtNetTeln_SetPassword              EventID (EventLevelAPI,   EvtNetTeln,  3)
#define EvtNetTeln_StartService             EventID (EventLevelOp,    EvtNetTeln,  4)
#define EvtNetTeln_StopService              EventID (EventLevelOp,    EvtNetTeln,  5)
#define EvtNetTeln_CloseSession             EventID (EventLevelOp,    EvtNetTeln,  6)
#define EvtNetTeln_ProcessData              EventID (EventLevelOp,    EvtNetTeln,  7)
#define EvtNetTeln_ProcessCommand           EventID (EventLevelOp,    EvtNetTeln,  8)
#define EvtNetTeln_EchoBackspace            EventID (EventLevelOp,    EvtNetTeln,  9)
#define EvtNetTeln_CommandHistory           EventID (EventLevelOp,    EvtNetTeln, 10)
#define EvtNetTeln_EchoCharacters           EventID (EventLevelOp,    EvtNetTeln, 11)
#define EvtNetTeln_LineBufferUsage          EventID (EventLevelOp,    EvtNetTeln, 12)
#define EvtNetTeln_SendAuthorizationRequest EventID (EventLevelOp,    EvtNetTeln, 13)
#define EvtNetTeln_SendInitialHeader        EventID (EventLevelOp,    EvtNetTeln, 14)
#define EvtNetTeln_LoginTimeoutExpired      EventID (EventLevelOp,    EvtNetTeln, 15)
#define EvtNetTeln_ShowPassword             EventID (EventLevelOp,    EvtNetTeln, 16)
#define EvtNetTeln_AuthenticationFailed     EventID (EventLevelError, EvtNetTeln, 17)
#define EvtNetTeln_UserLoginSuccess         EventID (EventLevelOp,    EvtNetTeln, 18)
#define EvtNetTeln_ShowUsername             EventID (EventLevelOp,    EvtNetTeln, 19)
#define EvtNetTeln_NegotiateStart           EventID (EventLevelOp,    EvtNetTeln, 20)
#define EvtNetTeln_NegotiateFailed          EventID (EventLevelError, EvtNetTeln, 21)
#define EvtNetTeln_UserAccessDeniedIp4      EventID (EventLevelError, EvtNetTeln, 22)
#define EvtNetTeln_UserAccessDeniedIp6      EventID (EventLevelError, EvtNetTeln, 23)
#define EvtNetTeln_SessionOpen              EventID (EventLevelOp,    EvtNetTeln, 24)
#define EvtNetTeln_SocketAborted            EventID (EventLevelOp,    EvtNetTeln, 25)
#define EvtNetTeln_SocketClosed             EventID (EventLevelOp,    EvtNetTeln, 26)
#define EvtNetTeln_ReceiveFrame             EventID (EventLevelOp,    EvtNetTeln, 27)
#define EvtNetTeln_NegotiateSuccess         EventID (EventLevelOp,    EvtNetTeln, 28)
#define EvtNetTeln_UninitServer             EventID (EventLevelOp,    EvtNetTeln, 29)
#endif

/**
  \brief  Event on Telnet server initialize (Op)
  \param  num_sessions  number of available Telnet sessions
  \param  port          listening port number
  \param  idle_tout     idle timeout in seconds (0= permanent connection)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_InitServer(uint32_t num_sessions, uint16_t port, uint16_t idle_tout) {
    uint32_t val2 = (uint32_t)idle_tout << 16 | port;
    EventRecord2 (EvtNetTeln_InitServer, num_sessions, val2);
  }
#else
  #define EvrNetTeln_InitServer(num_sessions, port, idle_tout)
#endif

/**
  \brief  Event on Telnet server failed to allocate TCP socket (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_GetSocketFailed(uint8_t session) {
    EventRecord2 (EvtNetTeln_GetSocketFailed, session, 0);
  }
#else
  #define EvrNetTeln_GetSocketFailed(session)
#endif

/**
  \brief  Event on Telnet server \ref netTELNETs_SetUsername (API)
  \param  username      new username
  \param  length        length of the username string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_SetUsername(const char *username, uint32_t length) {
    if (length > 16) length = 16;
    EventRecordData (EvtNetTeln_SetUsername, username, length);
  }
#else
  #define EvrNetTeln_SetUsername(username, length)
#endif

/**
  \brief  Event on Telnet server \ref netTELNETs_SetPassword (API)
  \param  password      new password
  \param  length        length of the password string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_SetPassword(const char *password, uint32_t length) {
    if (length > 16) length = 16;
    EventRecordData (EvtNetTeln_SetPassword, password, length);
  }
#else
  #define EvrNetTeln_SetPassword(password, length)
#endif

/**
  \brief  Event on Telnet server start service (Op)
  \param  port          listening port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_StartService(uint16_t port) {
    EventRecord2 (EvtNetTeln_StartService, port, 0);
  }
#else
  #define EvrNetTeln_StartService(port)
#endif

/**
  \brief  Event on Telnet server stop service (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_StopService(void) {
    EventRecord2 (EvtNetTeln_StopService, 0, 0);
  }
#else
  #define EvrNetTeln_StopService()
#endif

/**
  \brief  Event on Telnet server session close (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_CloseSession(uint8_t session) {
    EventRecord2 (EvtNetTeln_CloseSession, session, 0);
  }
#else
  #define EvrNetTeln_CloseSession(session)
#endif

/**
  \brief  Event on Telnet server process client data (Op)
  \param  session       session number
  \param  length        length of client data to process
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_ProcessData(uint8_t session, uint32_t length) {
    EventRecord2 (EvtNetTeln_ProcessData, session, length);
  }
#else
  #define EvrNetTeln_ProcessData(session, length)
#endif

/**
  \brief  Event on Telnet server process client command (Op)
  \param  command       user entered command to process
  \param  length        length of the command string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_ProcessCommand(const char *command, uint32_t length) {
    if (length > 96) length = 96;
    EventRecordData (EvtNetTeln_ProcessCommand, command, length);
  }
#else
  #define EvrNetTeln_ProcessCommand(command, length)
#endif

/**
  \brief  Event on Telnet server echo backspace (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_EchoBackspace(uint8_t session) {
    EventRecord2 (EvtNetTeln_EchoBackspace, session, 0);
  }
#else
  #define EvrNetTeln_EchoBackspace(session)
#endif

/**
  \brief  Event on Telnet server process command history (Op)
  \param  command       command from the command history
  \param  length        length of the command string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_CommandHistory(const char *command, uint32_t length) {
    if (length > 96) length = 96;
    EventRecordData (EvtNetTeln_CommandHistory, command, length);
  }
#else
  #define EvrNetTeln_CommandHistory(command, length)
#endif

/**
  \brief  Event on Telnet server echo the characters from command line buffer (Op)
  \param  line_buffer   command line buffer
  \param  num_char      number of characters to echo
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_EchoCharacters(uint8_t *line_buffer, uint32_t num_char) {
    if (num_char > 96) num_char = 96;
    EventRecordData (EvtNetTeln_EchoCharacters, line_buffer, num_char);
  }
#else
  #define EvrNetTeln_EchoCharacters(line_buffer, num_char)
#endif

/**
  \brief  Event on Telnet server display command line buffer usage (Op)
  \param  session       session number
  \param  num_char      number of characters in command line buffer
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_LineBufferUsage(uint8_t session, uint32_t num_char) {
    EventRecord2 (EvtNetTeln_LineBufferUsage, session, num_char);
  }
#else
  #define EvrNetTeln_LineBufferUsage(session, num_char)
#endif

/**
  \brief  Event on Telnet server send authorization request to the client (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_SendAuthorizationRequest(uint8_t session) {
    EventRecord2 (EvtNetTeln_SendAuthorizationRequest, session, 0);
  }
#else
  #define EvrNetTeln_SendAuthorizationRequest(session)
#endif

/**
  \brief  Event on Telnet server send initial telnet header to the client (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_SendInitialHeader(uint8_t session) {
    EventRecord2 (EvtNetTeln_SendInitialHeader, session, 0);
  }
#else
  #define EvrNetTeln_SendInitialHeader(session)
#endif

/**
  \brief  Event on Telnet server login timeout expired (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_LoginTimeoutExpired(uint8_t session) {
    EventRecord2 (EvtNetTeln_LoginTimeoutExpired, session, 0);
  }
#else
  #define EvrNetTeln_LoginTimeoutExpired(session)
#endif

/**
  \brief  Event on Telnet server display password of the client (Op)
  \param  password      password entered by the client
  \param  length        length of the password string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_ShowPassword(const char *password, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData (EvtNetTeln_ShowPassword, password, length);
  }
#else
  #define EvrNetTeln_ShowPassword(password, length)
#endif

/**
  \brief  Event on Telnet server user authentication failed, invalid credentials (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_AuthenticationFailed(uint8_t session) {
    EventRecord2 (EvtNetTeln_AuthenticationFailed, session, 0);
  }
#else
  #define EvrNetTeln_AuthenticationFailed(session)
#endif

/**
  \brief  Event on Telnet server user login successful (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_UserLoginSuccess(uint8_t session) {
    EventRecord2 (EvtNetTeln_UserLoginSuccess, session, 0);
  }
#else
  #define EvrNetTeln_UserLoginSuccess(session)
#endif

/**
  \brief  Event on Telnet server display username of the client (Op)
  \param  username      username entered by the client
  \param  length        length of the username string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_ShowUsername(const char *username, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData (EvtNetTeln_ShowUsername, username, length);
  }
#else
  #define EvrNetTeln_ShowUsername(username, length)
#endif

/**
  \brief  Event on Telnet server option negotiation start (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_NegotiateStart(uint8_t session) {
    EventRecord2 (EvtNetTeln_NegotiateStart, session, 0);
  }
#else
  #define EvrNetTeln_NegotiateStart(session)
#endif

/**
  \brief  Event on Telnet server option negotiation failed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_NegotiateFailed(uint8_t session) {
    EventRecord2 (EvtNetTeln_NegotiateFailed, session, 0);
  }
#else
  #define EvrNetTeln_NegotiateFailed(session)
#endif

/**
  \brief  Event on Telnet server user denied access for IPv4 client (Error)
  \param  net_addr      pointer to \ref NET_ADDR4 address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_UserAccessDeniedIp4(const void *net_addr) {
    EventRecordData (EvtNetTeln_UserAccessDeniedIp4, net_addr, 8);
  }
#else
  #define EvrNetTeln_UserAccessDeniedIp4(net_addr)
#endif

/**
  \brief  Event on Telnet server user denied access for IPv6 client (Error)
  \param  net_addr      pointer to \ref NET_ADDR address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_UserAccessDeniedIp6(const void *net_addr) {
    EventRecordData (EvtNetTeln_UserAccessDeniedIp6, net_addr, 20);
  }
#else
  #define EvrNetTeln_UserAccessDeniedIp6(net_addr)
#endif

/**
  \brief  Event on Telnet server session open (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_SessionOpen(uint8_t session) {
    EventRecord2 (EvtNetTeln_SessionOpen, session, 0);
  }
#else
  #define EvrNetTeln_SessionOpen(session)
#endif

/**
  \brief  Event on Telnet server socket aborted (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_SocketAborted(uint8_t session) {
    EventRecord2 (EvtNetTeln_SocketAborted, session, 0);
  }
#else
  #define EvrNetTeln_SocketAborted(session)
#endif

/**
  \brief  Event on Telnet server socket closed (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_SocketClosed(uint8_t session) {
    EventRecord2 (EvtNetTeln_SocketClosed, session, 0);
  }
#else
  #define EvrNetTeln_SocketClosed(session)
#endif

/**
  \brief  Event on Telnet server receive frame (Op)
  \param  session       session number
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_ReceiveFrame(uint8_t session, uint32_t length) {
    EventRecord2 (EvtNetTeln_ReceiveFrame, session, length);
  }
#else
  #define EvrNetTeln_ReceiveFrame(session, length)
#endif

/**
  \brief  Event on Telnet server option negotiation success (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_NegotiateSuccess(uint8_t session) {
    EventRecord2 (EvtNetTeln_NegotiateSuccess, session, 0);
  }
#else
  #define EvrNetTeln_NegotiateSuccess(session)
#endif

/**
  \brief  Event on Telnet server de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTeln_UninitServer(void) {
    EventRecord2 (EvtNetTeln_UninitServer, 0, 0);
  }
#else
  #define EvrNetTeln_UninitServer()
#endif


// NetTFTPs event identifiers --------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetTFTPs_InitServer              EventID (EventLevelOp,    EvtNetTFTPs,  0)
#define EvtNetTFTPs_ShowRootFolder          EventID (EventLevelOp,    EvtNetTFTPs,  1)
#define EvtNetTFTPs_GetSocketFailed         EventID (EventLevelError, EvtNetTFTPs,  2)
#define EvtNetTFTPs_SetRootPath             EventID (EventLevelAPI,   EvtNetTFTPs,  3)
#define EvtNetTFTPs_StartService            EventID (EventLevelOp,    EvtNetTFTPs,  4)
#define EvtNetTFTPs_StopService             EventID (EventLevelOp,    EvtNetTFTPs,  5)
#define EvtNetTFTPs_TimeoutExpiredAbort     EventID (EventLevelError, EvtNetTFTPs,  6)
#define EvtNetTFTPs_SendBlock               EventID (EventLevelOp,    EvtNetTFTPs,  7)
#define EvtNetTFTPs_ReceiveFrame            EventID (EventLevelOp,    EvtNetTFTPs,  8)
#define EvtNetTFTPs_FrameTooShort           EventID (EventLevelError, EvtNetTFTPs,  9)
#define EvtNetTFTPs_UserAccessDeniedIp4     EventID (EventLevelError, EvtNetTFTPs, 10)
#define EvtNetTFTPs_UserAccessDeniedIp6     EventID (EventLevelError, EvtNetTFTPs, 11)
#define EvtNetTFTPs_AccessDeniedNoResources EventID (EventLevelError, EvtNetTFTPs, 12)
#define EvtNetTFTPs_ShowClientAddressIp4    EventID (EventLevelOp,    EvtNetTFTPs, 13)
#define EvtNetTFTPs_ShowClientAddressIp6    EventID (EventLevelOp,    EvtNetTFTPs, 14)
#define EvtNetTFTPs_OperationRequest        EventID (EventLevelOp,    EvtNetTFTPs, 15)
#define EvtNetTFTPs_SessionRestart          EventID (EventLevelOp,    EvtNetTFTPs, 16)
#define EvtNetTFTPs_InvalidTransferId       EventID (EventLevelError, EvtNetTFTPs, 17)
#define EvtNetTFTPs_ErrorCodeReceived       EventID (EventLevelError, EvtNetTFTPs, 18)
#define EvtNetTFTPs_IllegalOpcodeReceived   EventID (EventLevelError, EvtNetTFTPs, 19)
#define EvtNetTFTPs_FileRequested           EventID (EventLevelOp,    EvtNetTFTPs, 20)
#define EvtNetTFTPs_TransferBlockSize       EventID (EventLevelOp,    EvtNetTFTPs, 21)
#define EvtNetTFTPs_TransferModeNotBinary   EventID (EventLevelError, EvtNetTFTPs, 22)
#define EvtNetTFTPs_LocalFileNotFound       EventID (EventLevelError, EvtNetTFTPs, 23)
#define EvtNetTFTPs_LocalFileCreateFailed   EventID (EventLevelError, EvtNetTFTPs, 24)
#define EvtNetTFTPs_ReceiveDataFrame        EventID (EventLevelOp,    EvtNetTFTPs, 25)
#define EvtNetTFTPs_DataFrameTooShort       EventID (EventLevelError, EvtNetTFTPs, 26)
#define EvtNetTFTPs_DuplicateBlockReceived  EventID (EventLevelOp,    EvtNetTFTPs, 27)
#define EvtNetTFTPs_TooManyRetries          EventID (EventLevelError, EvtNetTFTPs, 28)
#define EvtNetTFTPs_BlockReceived           EventID (EventLevelOp,    EvtNetTFTPs, 29)
#define EvtNetTFTPs_InvalidBlockReceived    EventID (EventLevelError, EvtNetTFTPs, 30)
#define EvtNetTFTPs_WriteErrorDiskFull      EventID (EventLevelError, EvtNetTFTPs, 31)
#define EvtNetTFTPs_BlockAckReceived        EventID (EventLevelOp,    EvtNetTFTPs, 32)
#define EvtNetTFTPs_BlockRetransmit         EventID (EventLevelOp,    EvtNetTFTPs, 33)
#define EvtNetTFTPs_InvalidBlockAck         EventID (EventLevelError, EvtNetTFTPs, 34)
#define EvtNetTFTPs_ShowRequestMode         EventID (EventLevelOp,    EvtNetTFTPs, 35)
#define EvtNetTFTPs_SendOptionAck           EventID (EventLevelOp,    EvtNetTFTPs, 36)
#define EvtNetTFTPs_SendBlockAck            EventID (EventLevelOp,    EvtNetTFTPs, 37)
#define EvtNetTFTPs_SendError               EventID (EventLevelOp,    EvtNetTFTPs, 38)
#define EvtNetTFTPs_AbortSession            EventID (EventLevelOp,    EvtNetTFTPs, 39)
#define EvtNetTFTPs_CloseSession            EventID (EventLevelOp,    EvtNetTFTPs, 40)
#define EvtNetTFTPs_UninitServer            EventID (EventLevelOp,    EvtNetTFTPs, 41)
#endif

/**
  \brief  Event on TFTP server initialize (Op)
  \param  num_sessions  number of available TFTP sessions
  \param  port          listening port number
  \param  firewall_en   firewall support enable
                         - 0: disabled
                         - 1: enabled
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_InitServer(uint32_t num_sessions, uint16_t port, uint8_t firewall_en) {
    uint32_t val2 = (uint32_t)firewall_en << 16 | port;
    EventRecord2 (EvtNetTFTPs_InitServer, num_sessions, val2);
  }
#else
  #define EvrNetTFTPs_InitServer(num_sessions, port, firewall_en)
#endif

/**
  \brief  Event on TFTP server display root folder (Op)
  \param  root_folder   path to server root folder
  \param  length        length of the root_folder string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_ShowRootFolder(const char *root_folder, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetTFTPs_ShowRootFolder, root_folder, length);
  }
#else
  #define EvrNetTFTPs_ShowRootFolder(root_folder, length)
#endif

/**
  \brief  Event on TFTP server failed to allocate UDP socket (Error)
  \param  session       session number (0= control session)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_GetSocketFailed(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_GetSocketFailed, session, 0);
  }
#else
  #define EvrNetTFTPs_GetSocketFailed(session)
#endif

/**
  \brief  Event on TFTP server \ref netTFTPs_SetRootPath (API)
  \param  path          path to server root folder
  \param  length        length of the path string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_SetRootPath(const char *path, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetTFTPs_SetRootPath, path, length);
  }
#else
  #define EvrNetTFTPs_SetRootPath(path, length)
#endif

/**
  \brief  Event on TFTP server start service (Op)
  \param  port          listening port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_StartService(uint16_t port) {
    EventRecord2 (EvtNetTFTPs_StartService, port, 0);
  }
#else
  #define EvrNetTFTPs_StartService(port)
#endif

/**
  \brief  Event on TFTP server stop service (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_StopService(void) {
    EventRecord2 (EvtNetTFTPs_StopService, 0, 0);
  }
#else
  #define EvrNetTFTPs_StopService()
#endif

/**
  \brief  Event on TFTP server session timeout expired, abort transfer (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_TimeoutExpiredAbort(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_TimeoutExpiredAbort, session, 0);
  }
#else
  #define EvrNetTFTPs_TimeoutExpiredAbort(session)
#endif

/**
  \brief  Event on TFTP server send block (Op)
  \param  session       session number
  \param  block_nr      block number
  \param  length        length of a block
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_SendBlock(uint8_t session, uint32_t block_nr, uint32_t length) {
    uint32_t val2 = (block_nr << 16) | length;
    EventRecord2 (EvtNetTFTPs_SendBlock, session, val2);
  }
#else
  #define EvrNetTFTPs_SendBlock(session, block_nr, length)
#endif

/**
  \brief  Event on TFTP server receive frame (Op)
  \param  socket        assigned UDP socket
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_ReceiveFrame(int32_t socket, uint32_t length) {
    EventRecord2 (EvtNetTFTPs_ReceiveFrame, (uint32_t)socket, length);
  }
#else
  #define EvrNetTFTPs_ReceiveFrame(socket, length)
#endif

/**
  \brief  Event on TFTP server received frame too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetTFTPs_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetTFTPs_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on TFTP server user denied access for IPv4 client (Error)
  \param  net_addr      pointer to \ref NET_ADDR4 address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_UserAccessDeniedIp4(const void *net_addr) {
    EventRecordData (EvtNetTFTPs_UserAccessDeniedIp4, net_addr, 8);
  }
#else
  #define EvrNetTFTPs_UserAccessDeniedIp4(net_addr)
#endif

/**
  \brief  Event on TFTP server user denied access for IPv6 client (Error)
  \param  net_addr      pointer to \ref NET_ADDR address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_UserAccessDeniedIp6(const void *net_addr) {
    EventRecordData (EvtNetTFTPs_UserAccessDeniedIp6, net_addr, 20);
  }
#else
  #define EvrNetTFTPs_UserAccessDeniedIp6(net_addr)
#endif

/**
  \brief  Event on TFTP server denied client access on out of resources (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_AccessDeniedNoResources(void) {
    EventRecord2 (EvtNetTFTPs_AccessDeniedNoResources, 0, 0);
  }
#else
  #define EvrNetTFTPs_AccessDeniedNoResources()
#endif

/**
  \brief  Event on TFTP server display client IPv4 address and port number (Op)
  \param  net_addr      pointer to \ref NET_ADDR4 address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_ShowClientAddressIp4(const void *net_addr) {
    EventRecordData (EvtNetTFTPs_ShowClientAddressIp4, net_addr, 8);
  }
#else
  #define EvrNetTFTPs_ShowClientAddressIp4(net_addr)
#endif

/**
  \brief  Event on TFTP server display client IPv6 address and port number (Op)
  \param  net_addr      pointer to \ref NET_ADDR address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_ShowClientAddressIp6(const void *net_addr) {
    EventRecordData (EvtNetTFTPs_ShowClientAddressIp6, net_addr, 20);
  }
#else
  #define EvrNetTFTPs_ShowClientAddressIp6(net_addr)
#endif

/**
  \brief  Event on TFTP server process operation request (Op)
  \param  session       session number
  \param  tftp_opcode   TFTP operation code
                         - 1: read request
                         - 2: write request
                         - 3: data
                         - 4: acknowledgment
                         - 5: error
                         - 6: option acknowledgment
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_OperationRequest(uint8_t session, uint16_t tftp_opcode) {
    EventRecord2 (EvtNetTFTPs_OperationRequest, session, tftp_opcode);
  }
#else
  #define EvrNetTFTPs_OperationRequest(session, tftp_opcode)
#endif

/**
  \brief  Event on TFTP server session restart, maybe our response was lost (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_SessionRestart(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_SessionRestart, session, 0);
  }
#else
  #define EvrNetTFTPs_SessionRestart(session)
#endif

/**
  \brief  Event on TFTP server invalid transfer id (TID) detected (Error)
  \param  session       session number
  \note   TID is treated in UDP socket as the source and destination port.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_InvalidTransferId(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_InvalidTransferId, session, 0);
  }
#else
  #define EvrNetTFTPs_InvalidTransferId(session)
#endif

/**
  \brief  Event on TFTP server error code received from the client (Error)
  \param  session       session number
  \param  error_code    TFTP error code
                         - 0: not defined error
                         - 1: file not found error
                         - 2: access violation error
                         - 3: disk full error
                         - 4: illegal opcode error
                         - 5: unknown TID error
                         - 6: file already exists error
                         - 7: no such user error
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_ErrorCodeReceived(uint8_t session, uint16_t error_code) {
    EventRecord2 (EvtNetTFTPs_ErrorCodeReceived, session, error_code);
  }
#else
  #define EvrNetTFTPs_ErrorCodeReceived(session, error_code)
#endif

/**
  \brief  Event on TFTP server illegal operation code received from the client (Error)
  \param  session       session number
  \param  tftp_opcode   TFTP operation code
                         - 1: read request
                         - 2: write request
                         - 3: data
                         - 4: acknowledgment
                         - 5: error
                         - 6: option acknowledgment
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_IllegalOpcodeReceived(uint8_t session, uint16_t tftp_opcode) {
    EventRecord2 (EvtNetTFTPs_IllegalOpcodeReceived, session, tftp_opcode);
  }
#else
  #define EvrNetTFTPs_IllegalOpcodeReceived(session, tftp_opcode)
#endif

/**
  \brief  Event on TFTP server received file request from the client (Op)
  \param  fname         name of the requested file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_FileRequested(const char *fname, uint32_t length) {
    if (length > 40) length = 40;
    EventRecordData (EvtNetTFTPs_FileRequested, fname, length);
  }
#else
  #define EvrNetTFTPs_FileRequested(fname, length)
#endif

/**
  \brief  Event on TFTP server display transfer block size (Op)
  \param  session       session number
  \param  block_size    size of transfer block
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_TransferBlockSize(uint8_t session, uint16_t block_size) {
    EventRecord2 (EvtNetTFTPs_TransferBlockSize, session, block_size);
  }
#else
  #define EvrNetTFTPs_TransferBlockSize(session, block_size)
#endif

/**
  \brief  Event on TFTP server transfer mode not set to binary (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_TransferModeNotBinary(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_TransferModeNotBinary, session, 0);
  }
#else
  #define EvrNetTFTPs_TransferModeNotBinary(session)
#endif

/**
  \brief  Event on TFTP server error, requested file not found on server (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_LocalFileNotFound(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_LocalFileNotFound, session, 0);
  }
#else
  #define EvrNetTFTPs_LocalFileNotFound(session)
#endif

/**
  \brief  Event on TFTP server local file create failed (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_LocalFileCreateFailed(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_LocalFileCreateFailed, session, 0);
  }
#else
  #define EvrNetTFTPs_LocalFileCreateFailed(session)
#endif

/**
  \brief  Event on TFTP server receive data frame (Op)
  \param  session       session number
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_ReceiveDataFrame(uint8_t session, uint32_t length) {
    EventRecord2 (EvtNetTFTPs_ReceiveDataFrame, session, length);
  }
#else
  #define EvrNetTFTPs_ReceiveDataFrame(session, length)
#endif

/**
  \brief  Event on TFTP server received data frame too short (Error)
  \param  session       session number
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_DataFrameTooShort(uint8_t session, uint32_t length, uint32_t min_length) {
    uint32_t val2 = (min_length << 16) | length;
    EventRecord2 (EvtNetTFTPs_DataFrameTooShort, session, val2);
  }
#else
  #define EvrNetTFTPs_DataFrameTooShort(session, length, min_length)
#endif

/**
  \brief  Event on TFTP server duplicate block received (Op)
  \param  session       session number
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_DuplicateBlockReceived(uint8_t session, uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPs_DuplicateBlockReceived, session, block_nr);
  }
#else
  #define EvrNetTFTPs_DuplicateBlockReceived(session, block_nr)
#endif

/**
  \brief  Event on TFTP server too many retransmissions (Error)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_TooManyRetries(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_TooManyRetries, session, 0);
  }
#else
  #define EvrNetTFTPs_TooManyRetries(session)
#endif

/**
  \brief  Event on TFTP server data block received (Op)
  \param  session       session number
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_BlockReceived(uint8_t session, uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPs_BlockReceived, session, block_nr);
  }
#else
  #define EvrNetTFTPs_BlockReceived(session, block_nr)
#endif

/**
  \brief  Event on TFTP server invalid block received (Error)
  \param  session       session number
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_InvalidBlockReceived(uint8_t session, uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPs_InvalidBlockReceived, session, block_nr);
  }
#else
  #define EvrNetTFTPs_InvalidBlockReceived(session, block_nr)
#endif

/**
  \brief  Event on TFTP server write local disk failed, disk full (Error)
  \param  session       session number
  \param  length        block data length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_WriteErrorDiskFull(uint8_t session, uint32_t length) {
    EventRecord2 (EvtNetTFTPs_WriteErrorDiskFull, session, length);
  }
#else
  #define EvrNetTFTPs_WriteErrorDiskFull(session, length)
#endif

/**
  \brief  Event on TFTP server block acknowledge received (Op)
  \param  session       session number
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_BlockAckReceived(uint8_t session, uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPs_BlockAckReceived, session, block_nr);
  }
#else
  #define EvrNetTFTPs_BlockAckReceived(session, block_nr)
#endif

/**
  \brief  Event on TFTP server block retransmit (Op)
  \param  session       session number
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_BlockRetransmit(uint8_t session, uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPs_BlockRetransmit, session, block_nr);
  }
#else
  #define EvrNetTFTPs_BlockRetransmit(session, block_nr)
#endif

/**
  \brief  Event on TFTP server invalid block acknowledge received (Error)
  \param  session       session number
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_InvalidBlockAck(uint8_t session, uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPs_InvalidBlockAck, session, block_nr);
  }
#else
  #define EvrNetTFTPs_InvalidBlockAck(session, block_nr)
#endif

/**
  \brief  Event on TFTP server display received request mode parameter (Op)
  \param  mode          mode parameter in text format
  \param  length        length of the mode string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_ShowRequestMode(const uint8_t *mode, uint32_t length) {
    if (length > 16) length = 16;
    EventRecordData (EvtNetTFTPs_ShowRequestMode, mode, length);
  }
#else
  #define EvrNetTFTPs_ShowRequestMode(mode, length)
#endif

/**
  \brief  Event on TFTP server send option acknowledgment (Op)
  \param  session       session number
  \param  block_size    block size
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_SendOptionAck(uint8_t session, uint32_t block_size) {
    EventRecord2 (EvtNetTFTPs_SendOptionAck, session, block_size);
  }
#else
  #define EvrNetTFTPs_SendOptionAck(session, block_size)
#endif

/**
  \brief  Event on TFTP server send block acknowledgment (Op)
  \param  session       session number
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_SendBlockAck(uint8_t session, uint16_t block_nr) {
    EventRecord2 (EvtNetTFTPs_SendBlockAck, session, block_nr);
  }
#else
  #define EvrNetTFTPs_SendBlockAck(session, block_nr)
#endif

/**
  \brief  Event on TFTP server send error code (Op)
  \param  session       session number
  \param  error_nr      TFTP error number
                         - 0: not defined error
                         - 1: file not found error
                         - 2: access violation error
                         - 3: disk full error
                         - 4: illegal opcode error
                         - 5: unknown TID error
                         - 6: file already exists error
                         - 7: no such user error
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_SendError(uint8_t session, uint16_t error_nr) {
    EventRecord2 (EvtNetTFTPs_SendError, session, error_nr);
  }
#else
  #define EvrNetTFTPs_SendError(session, error_nr)
#endif

/**
  \brief  Event on TFTP server abnormal session close on error (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_AbortSession(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_AbortSession, session, 0);
  }
#else
  #define EvrNetTFTPs_AbortSession(session)
#endif

/**
  \brief  Event on TFTP server normal session close (Op)
  \param  session       session number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_CloseSession(uint8_t session) {
    EventRecord2 (EvtNetTFTPs_CloseSession, session, 0);
  }
#else
  #define EvrNetTFTPs_CloseSession(session)
#endif

/**
  \brief  Event on TFTP server de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPs_UninitServer(void) {
    EventRecord2 (EvtNetTFTPs_UninitServer, 0, 0);
  }
#else
  #define EvrNetTFTPs_UninitServer()
#endif


// NetTFTPc event identifiers --------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetTFTPc_InitClient              EventID (EventLevelOp,    EvtNetTFTPc,  0)
#define EvtNetTFTPc_GetSocketFailed         EventID (EventLevelError, EvtNetTFTPc,  1)
#define EvtNetTFTPc_PutFile                 EventID (EventLevelAPI,   EvtNetTFTPc,  2)
#define EvtNetTFTPc_PutRemoteName           EventID (EventLevelOp,    EvtNetTFTPc,  3)
#define EvtNetTFTPc_PutInvalidParameter     EventID (EventLevelError, EvtNetTFTPc,  4)
#define EvtNetTFTPc_PutWrongState           EventID (EventLevelError, EvtNetTFTPc,  5)
#define EvtNetTFTPc_ShowServerAddressIp4    EventID (EventLevelOp,    EvtNetTFTPc,  6)
#define EvtNetTFTPc_ShowServerAddressIp6    EventID (EventLevelOp,    EvtNetTFTPc,  7)
#define EvtNetTFTPc_OpenLocalFile           EventID (EventLevelOp,    EvtNetTFTPc,  8)
#define EvtNetTFTPc_OpenLocalFileFailed     EventID (EventLevelError, EvtNetTFTPc,  9)
#define EvtNetTFTPc_GetFile                 EventID (EventLevelAPI,   EvtNetTFTPc, 10)
#define EvtNetTFTPc_GetLocalName            EventID (EventLevelOp,    EvtNetTFTPc, 11)
#define EvtNetTFTPc_GetInvalidParameter     EventID (EventLevelError, EvtNetTFTPc, 12)
#define EvtNetTFTPc_GetWrongState           EventID (EventLevelError, EvtNetTFTPc, 13)
#define EvtNetTFTPc_TimeoutBlockRetransmit  EventID (EventLevelOp,    EvtNetTFTPc, 14)
#define EvtNetTFTPc_SendBlock               EventID (EventLevelOp,    EvtNetTFTPc, 15)
#define EvtNetTFTPc_StopClient              EventID (EventLevelOp,    EvtNetTFTPc, 16)
#define EvtNetTFTPc_CloseLocalFile          EventID (EventLevelOp,    EvtNetTFTPc, 17)
#define EvtNetTFTPc_WrongServerIp4          EventID (EventLevelError, EvtNetTFTPc, 18)
#define EvtNetTFTPc_WrongServerIp6          EventID (EventLevelError, EvtNetTFTPc, 19)
#define EvtNetTFTPc_WrongServerPort         EventID (EventLevelError, EvtNetTFTPc, 20)
#define EvtNetTFTPc_ServerTidAssigned       EventID (EventLevelOp,    EvtNetTFTPc, 21)
#define EvtNetTFTPc_FrameTooShort           EventID (EventLevelError, EvtNetTFTPc, 22)
#define EvtNetTFTPc_ReceiveFrame            EventID (EventLevelOp,    EvtNetTFTPc, 23)
#define EvtNetTFTPc_ErrorCodeReceived       EventID (EventLevelError, EvtNetTFTPc, 24)
#define EvtNetTFTPc_DuplicateBlockAck       EventID (EventLevelOp,    EvtNetTFTPc, 25)
#define EvtNetTFTPc_InvalidBlockAck         EventID (EventLevelError, EvtNetTFTPc, 26)
#define EvtNetTFTPc_BlockAckReceived        EventID (EventLevelOp,    EvtNetTFTPc, 27)
#define EvtNetTFTPc_DuplicateBlockReceived  EventID (EventLevelOp,    EvtNetTFTPc, 28)
#define EvtNetTFTPc_InvalidBlockReceived    EventID (EventLevelError, EvtNetTFTPc, 29)
#define EvtNetTFTPc_BlockReceived           EventID (EventLevelOp,    EvtNetTFTPc, 30)
#define EvtNetTFTPc_WriteErrorDiskFull      EventID (EventLevelError, EvtNetTFTPc, 31)
#define EvtNetTFTPc_IllegalServerOperation  EventID (EventLevelError, EvtNetTFTPc, 32)
#define EvtNetTFTPc_SendRequest             EventID (EventLevelOp,    EvtNetTFTPc, 33)
#define EvtNetTFTPc_OptionBlockSize         EventID (EventLevelOp,    EvtNetTFTPc, 34)
#define EvtNetTFTPc_SendAck                 EventID (EventLevelOp,    EvtNetTFTPc, 35)
#define EvtNetTFTPc_SendError               EventID (EventLevelOp,    EvtNetTFTPc, 36)
#define EvtNetTFTPc_OptionAckReceived       EventID (EventLevelOp,    EvtNetTFTPc, 37)
#define EvtNetTFTPc_UninitClient            EventID (EventLevelOp,    EvtNetTFTPc, 38)
#endif

/**
  \brief  Event on TFTP client initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_InitClient(void) {
    EventRecord2 (EvtNetTFTPc_InitClient, 0, 0);
  }
#else
  #define EvrNetTFTPc_InitClient()
#endif

/**
  \brief  Event on TFTP client failed to allocate UDP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_GetSocketFailed(void) {
    EventRecord2 (EvtNetTFTPc_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetTFTPc_GetSocketFailed()
#endif

/**
  \brief  Event on TFTP client \ref netTFTPc_Put (API)
  \param  local_fname   name of the local file
  \param  length        length of the local_fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_PutFile(const char *local_fname, uint32_t length) {
    if (length > 32) length = 32;
    EventRecordData (EvtNetTFTPc_PutFile, local_fname, length);
  }
#else
  #define EvrNetTFTPc_PutFile(local_fname, length)
#endif

/**
  \brief  Event on TFTP client \ref netTFTPc_Put remote filename (Op)
  \param  fname         name of the remote file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_PutRemoteName(const char *fname, uint32_t length) {
    if (length > 32) length = 32;
    EventRecordData (EvtNetTFTPc_PutRemoteName, fname, length);
  }
#else
  #define EvrNetTFTPc_PutRemoteName(fname, length)
#endif

/**
  \brief  Event on \ref netTFTPc_Put invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_PutInvalidParameter(void) {
    EventRecord2 (EvtNetTFTPc_PutInvalidParameter, 0, 0);
  }
#else
  #define EvrNetTFTPc_PutInvalidParameter()
#endif

/**
  \brief  Event on \ref netTFTPc_Put wrong client state (Error)
  \param  state         client state
                         - 0: idle
                         - 1: put file
                         - 2: get file
                         - 3: stop client
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_PutWrongState(uint8_t state) {
    EventRecord2 (EvtNetTFTPc_PutWrongState, state, 0);
  }
#else
  #define EvrNetTFTPc_PutWrongState(state)
#endif

/**
  \brief  Event on TFTP client display server IPv4 address (Op)
  \param  net_addr      pointer to \ref NET_ADDR4 server address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_ShowServerAddressIp4(const void *net_addr) {
    EventRecordData (EvtNetTFTPc_ShowServerAddressIp4, net_addr, 8);
  }
#else
  #define EvrNetTFTPc_ShowServerAddressIp4(net_addr)
#endif

/**
  \brief  Event on TFTP client display server IPv6 address (Op)
  \param  net_addr      pointer to \ref NET_ADDR server address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_ShowServerAddressIp6(const void *net_addr) {
    EventRecordData (EvtNetTFTPc_ShowServerAddressIp6, net_addr, 20);
  }
#else
  #define EvrNetTFTPc_ShowServerAddressIp6(net_addr)
#endif

/**
  \brief  Event on TFTP client open local file (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_OpenLocalFile(void) {
    EventRecord2 (EvtNetTFTPc_OpenLocalFile, 0, 0);
  }
#else
  #define EvrNetTFTPc_OpenLocalFile()
#endif

/**
  \brief  Event on TFTP client open local file failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_OpenLocalFileFailed(void) {
    EventRecord2 (EvtNetTFTPc_OpenLocalFileFailed, 0, 0);
  }
#else
  #define EvrNetTFTPc_OpenLocalFileFailed()
#endif

/**
  \brief  Event on TFTP client \ref netTFTPc_Get (API)
  \param  fname         name of the remote file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_GetFile(const char *fname, uint32_t length) {
    if (length > 32) length = 32;
    EventRecordData (EvtNetTFTPc_GetFile, fname, length);
  }
#else
  #define EvrNetTFTPc_GetFile(fname, length)
#endif

/**
  \brief  Event on TFTP client \ref netTFTPc_Get local filename (Op)
  \param  local_fname   name of the local file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_GetLocalName(const char *local_fname, uint32_t length) {
    if (length > 32) length = 32;
    EventRecordData (EvtNetTFTPc_GetLocalName, local_fname, length);
  }
#else
  #define EvrNetTFTPc_GetLocalName(local_fname, length)
#endif

/**
  \brief  Event on \ref netTFTPc_Get invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_GetInvalidParameter(void) {
    EventRecord2 (EvtNetTFTPc_GetInvalidParameter, 0, 0);
  }
#else
  #define EvrNetTFTPc_GetInvalidParameter()
#endif

/**
  \brief  Event on \ref netTFTPc_Get wrong client state (Error)
  \param  state         client state
                         - 0: idle
                         - 1: put file
                         - 2: get file
                         - 3: stop client
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_GetWrongState(uint8_t state) {
    EventRecord2 (EvtNetTFTPc_GetWrongState, state, 0);
  }
#else
  #define EvrNetTFTPc_GetWrongState(state)
#endif

/**
  \brief  Event on TFTP client block retransmit on timeout (Op)
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_TimeoutBlockRetransmit(uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPc_TimeoutBlockRetransmit, block_nr, 0);
  }
#else
  #define EvrNetTFTPc_TimeoutBlockRetransmit(block_nr)
#endif

/**
  \brief  Event on TFTP client send block (Op)
  \param  block_nr      block number
  \param  length        length of a block
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_SendBlock(uint32_t block_nr, uint32_t length) {
    EventRecord2 (EvtNetTFTPc_SendBlock, block_nr, length);
  }
#else
  #define EvrNetTFTPc_SendBlock(block_nr, length)
#endif

/**
  \brief  Event on TFTP client stop operation (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_StopClient(void) {
    EventRecord2 (EvtNetTFTPc_StopClient, 0, 0);
  }
#else
  #define EvrNetTFTPc_StopClient()
#endif

/**
  \brief  Event on TFTP client close local file (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_CloseLocalFile(void) {
    EventRecord2 (EvtNetTFTPc_CloseLocalFile, 0, 0);
  }
#else
  #define EvrNetTFTPc_CloseLocalFile()
#endif

/**
  \brief  Event on TFTP client wrong server IPv4 address (Error)
  \param  ip4_addr      pointer to IPv4 address of TFTP server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_WrongServerIp4(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetTFTPc_WrongServerIp4, ip4_addr, 4);
  }
#else
  #define EvrNetTFTPc_WrongServerIp4(ip4_addr)
#endif

/**
  \brief  Event on TFTP client wrong server IPv6 address (Error)
  \param  ip6_addr      pointer to IPv6 address of TFTP server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_WrongServerIp6(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetTFTPc_WrongServerIp6, ip6_addr, 16);
  }
#else
  #define EvrNetTFTPc_WrongServerIp6(ip6_addr)
#endif

/**
  \brief  Event on TFTP client wrong server port (Error)
  \param  udp_port      wrong UDP port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_WrongServerPort(uint16_t udp_port) {
    EventRecord2 (EvtNetTFTPc_WrongServerPort, udp_port, 0);
  }
#else
  #define EvrNetTFTPc_WrongServerPort(udp_port)
#endif

/**
  \brief  Event on TFTP client assigned transfer identifier (TID) of the server (Op)
  \param  tid           server transfer identifier
  \note   Transfer identifier is used as UDP port number.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_ServerTidAssigned(uint16_t tid) {
    EventRecord2 (EvtNetTFTPc_ServerTidAssigned, tid, 0);
  }
#else
  #define EvrNetTFTPc_ServerTidAssigned(tid)
#endif

/**
  \brief  Event on TFTP client received frame too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetTFTPc_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetTFTPc_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on TFTP client receive frame (Op)
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_ReceiveFrame(uint32_t length) {
    EventRecord2 (EvtNetTFTPc_ReceiveFrame, length, 0);
  }
#else
  #define EvrNetTFTPc_ReceiveFrame(length)
#endif

/**
  \brief  Event on TFTP client error code received from server (Error)
  \param  error_code    TFTP error code
                         - 0: not defined error
                         - 1: file not found error
                         - 2: access violation error
                         - 3: disk full error
                         - 4: illegal opcode error
                         - 5: unknown TID error
                         - 6: file already exists error
                         - 7: no such user error
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_ErrorCodeReceived(uint16_t error_code) {
    EventRecord2 (EvtNetTFTPc_ErrorCodeReceived, error_code, 0);
  }
#else
  #define EvrNetTFTPc_ErrorCodeReceived(error_code)
#endif

/**
  \brief  Event on TFTP client duplicate block acknowledge received (Op)
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_DuplicateBlockAck(uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPc_DuplicateBlockAck, block_nr, 0);
  }
#else
  #define EvrNetTFTPc_DuplicateBlockAck(block_nr)
#endif

/**
  \brief  Event on TFTP client invalid block acknowledge received (Error)
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_InvalidBlockAck(uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPc_InvalidBlockAck, block_nr, 0);
  }
#else
  #define EvrNetTFTPc_InvalidBlockAck(block_nr)
#endif

/**
  \brief  Event on TFTP client block acknowledge received (Op)
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_BlockAckReceived(uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPc_BlockAckReceived, block_nr, 0);
  }
#else
  #define EvrNetTFTPc_BlockAckReceived(block_nr)
#endif

/**
  \brief  Event on TFTP client duplicate block received (Op)
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_DuplicateBlockReceived(uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPc_DuplicateBlockReceived, block_nr, 0);
  }
#else
  #define EvrNetTFTPc_DuplicateBlockReceived(block_nr)
#endif

/**
  \brief  Event on TFTP client invalid block received (Error)
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_InvalidBlockReceived(uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPc_InvalidBlockReceived, block_nr, 0);
  }
#else
  #define EvrNetTFTPc_InvalidBlockReceived(block_nr)
#endif

/**
  \brief  Event on TFTP client block data received (Op)
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_BlockReceived(uint32_t block_nr) {
    EventRecord2 (EvtNetTFTPc_BlockReceived, block_nr, 0);
  }
#else
  #define EvrNetTFTPc_BlockReceived(block_nr)
#endif

/**
  \brief  Event on TFTP client write local disk failed, disk full (Error)
  \param  length        block data length
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_WriteErrorDiskFull(uint32_t length) {
    EventRecord2 (EvtNetTFTPc_WriteErrorDiskFull, length, 0);
  }
#else
  #define EvrNetTFTPc_WriteErrorDiskFull(length)
#endif

/**
  \brief  Event on TFTP client deteced illegal server operation (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_IllegalServerOperation(void) {
    EventRecord2 (EvtNetTFTPc_IllegalServerOperation, 0, 0);
  }
#else
  #define EvrNetTFTPc_IllegalServerOperation()
#endif

/**
  \brief  Event on TFTP client send operation request (Op)
  \param  tftp_opcode   TFTP operation code
                         - 1: read request
                         - 2: write request
                         - 3: data
                         - 4: acknowledgment
                         - 5: error
                         - 6: option acknowledgment
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_SendRequest(uint16_t tftp_opcode) {
    EventRecord2 (EvtNetTFTPc_SendRequest, tftp_opcode, 0);
  }
#else
  #define EvrNetTFTPc_SendRequest(tftp_opcode)
#endif

/**
  \brief  Event on TFTP client add block size option (Op)
  \param  block_size    size of transfer block
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_OptionBlockSize(uint16_t block_size) {
    EventRecord2 (EvtNetTFTPc_OptionBlockSize, block_size, 0);
  }
#else
  #define EvrNetTFTPc_OptionBlockSize(block_size)
#endif

/**
  \brief  Event on TFTP client send acknowledgment (Op)
  \param  block_nr      block number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_SendAck(uint16_t block_nr) {
    EventRecord2 (EvtNetTFTPc_SendAck, block_nr, 0);
  }
#else
  #define EvrNetTFTPc_SendAck(block_nr)
#endif

/**
  \brief  Event on TFTP client send error code (Op)
  \param  error_nr      TFTP error number
                         - 0: not defined error
                         - 1: file not found error
                         - 2: access violation error
                         - 3: disk full error
                         - 4: illegal opcode error
                         - 5: unknown TID error
                         - 6: file already exists error
                         - 7: no such user error
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_SendError(uint16_t error_nr) {
    EventRecord2 (EvtNetTFTPc_SendError, error_nr, 0);
  }
#else
  #define EvrNetTFTPc_SendError(error_nr)
#endif

/**
  \brief  Event on TFTP client option acknowledgment received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_OptionAckReceived(void) {
    EventRecord2 (EvtNetTFTPc_OptionAckReceived, 0, 0);
  }
#else
  #define EvrNetTFTPc_OptionAckReceived()
#endif

/**
  \brief  Event on TFTP client de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetTFTPc_UninitClient(void) {
    EventRecord2 (EvtNetTFTPc_UninitClient, 0, 0);
  }
#else
  #define EvrNetTFTPc_UninitClient()
#endif


// NetSMTP event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetSMTP_InitClient               EventID (EventLevelOp,    EvtNetSMTP,  0)
#define EvtNetSMTP_GetSocketFailed          EventID (EventLevelError, EvtNetSMTP,  1)
#define EvtNetSMTP_ConnectIp4               EventID (EventLevelAPI,   EvtNetSMTP,  2)
#define EvtNetSMTP_ConnectIp6               EventID (EventLevelAPI,   EvtNetSMTP,  3)
#define EvtNetSMTP_ConnectInvalidParameter  EventID (EventLevelError, EvtNetSMTP,  4)
#define EvtNetSMTP_ConnectClientBusy        EventID (EventLevelError, EvtNetSMTP,  5)
#define EvtNetSMTP_SendMail                 EventID (EventLevelAPI,   EvtNetSMTP,  6)
#define EvtNetSMTP_SendMailInvalidParameter EventID (EventLevelError, EvtNetSMTP,  7)
#define EvtNetSMTP_SendMailTlsNotEnabled    EventID (EventLevelError, EvtNetSMTP,  8)
#define EvtNetSMTP_SendMailClientBusy       EventID (EventLevelError, EvtNetSMTP,  9)
#define EvtNetSMTP_SendMailNoRecipients     EventID (EventLevelError, EvtNetSMTP, 10)
#define EvtNetSMTP_SendMailServerNotValid   EventID (EventLevelError, EvtNetSMTP, 11)
#define EvtNetSMTP_SendMailDnsError         EventID (EventLevelError, EvtNetSMTP, 12)
#define EvtNetSMTP_SendMailAttachNotEnabled EventID (EventLevelError, EvtNetSMTP, 13)
#define EvtNetSMTP_SendMailAttachFailed     EventID (EventLevelError, EvtNetSMTP, 14)
#define EvtNetSMTP_SendMailMultipart        EventID (EventLevelOp,    EvtNetSMTP, 15)
#define EvtNetSMTP_SendMailAttachment       EventID (EventLevelOp,    EvtNetSMTP, 16)
#define EvtNetSMTP_InboundConnRejected      EventID (EventLevelOp,    EvtNetSMTP, 17)
#define EvtNetSMTP_SocketAborted            EventID (EventLevelError, EvtNetSMTP, 18)
#define EvtNetSMTP_SocketConnected          EventID (EventLevelOp,    EvtNetSMTP, 19)
#define EvtNetSMTP_SocketClosed             EventID (EventLevelOp,    EvtNetSMTP, 20)
#define EvtNetSMTP_ReceiveFrame             EventID (EventLevelOp,    EvtNetSMTP, 21)
#define EvtNetSMTP_UnackedDataError         EventID (EventLevelError, EvtNetSMTP, 22)
#define EvtNetSMTP_FrameTooShort            EventID (EventLevelError, EvtNetSMTP, 23)
#define EvtNetSMTP_ServerReady              EventID (EventLevelOp,    EvtNetSMTP, 24)
#define EvtNetSMTP_EsmtpModeNotSupported    EventID (EventLevelOp,    EvtNetSMTP, 25)
#define EvtNetSMTP_EhloResponseFragmented   EventID (EventLevelOp,    EvtNetSMTP, 26)
#define EvtNetSMTP_EsmtpModeActive          EventID (EventLevelOp,    EvtNetSMTP, 27)
#define EvtNetSMTP_StartAuthentication      EventID (EventLevelOp,    EvtNetSMTP, 28)
#define EvtNetSMTP_AuthenticationDenied     EventID (EventLevelOp,    EvtNetSMTP, 29)
#define EvtNetSMTP_AuthMethodNotSupported   EventID (EventLevelError, EvtNetSMTP, 30)
#define EvtNetSMTP_SmtpModeActive           EventID (EventLevelOp,    EvtNetSMTP, 31)
#define EvtNetSMTP_AuthenticationSuccessful EventID (EventLevelOp,    EvtNetSMTP, 32)
#define EvtNetSMTP_AuthenticationFailed     EventID (EventLevelError, EvtNetSMTP, 33)
#define EvtNetSMTP_ServerAcknowledge        EventID (EventLevelOp,    EvtNetSMTP, 34)
#define EvtNetSMTP_WrongResponse            EventID (EventLevelError, EvtNetSMTP, 35)
#define EvtNetSMTP_ClientStopTimeoutExpired EventID (EventLevelError, EvtNetSMTP, 36)
#define EvtNetSMTP_SendMessageBody          EventID (EventLevelOp,    EvtNetSMTP, 37)
#define EvtNetSMTP_SendMessageEnd           EventID (EventLevelOp,    EvtNetSMTP, 38)
#define EvtNetSMTP_SendCommand              EventID (EventLevelDetail,EvtNetSMTP, 39)
#define EvtNetSMTP_ClientCloseSocket        EventID (EventLevelOp,    EvtNetSMTP, 40)
#define EvtNetSMTP_ClientDone               EventID (EventLevelOp,    EvtNetSMTP, 41)
#define EvtNetSMTP_TlsSupportIndicated      EventID (EventLevelOp,    EvtNetSMTP, 42)
#define EvtNetSMTP_StartTlsAccepted         EventID (EventLevelOp,    EvtNetSMTP, 43)
#define EvtNetSMTP_TlsGetContextFailed      EventID (EventLevelError, EvtNetSMTP, 44)
#define EvtNetSMTP_TlsModeStarted           EventID (EventLevelOp,    EvtNetSMTP, 45)
#define EvtNetSMTP_TlsModeEstablished       EventID (EventLevelOp,    EvtNetSMTP, 46)
#define EvtNetSMTP_UninitClient             EventID (EventLevelOp,    EvtNetSMTP, 47)
#endif

/**
  \brief  Event on SMTP client initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_InitClient(void) {
    EventRecord2 (EvtNetSMTP_InitClient, 0, 0);
  }
#else
  #define EvrNetSMTP_InitClient()
#endif

/**
  \brief  Event on SMTP client failed to allocate TCP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_GetSocketFailed(void) {
    EventRecord2 (EvtNetSMTP_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetSMTP_GetSocketFailed()
#endif

/**
  \brief  Event on SMTP client \ref netSMTPc_Connect to IPv4 server (API)
  \param  net_addr      pointer to \ref NET_ADDR4 server address structure (8 bytes)
  \note   Sends reduced size of net address structure with IPv4 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ConnectIp4(const void *net_addr) {
    EventRecordData (EvtNetSMTP_ConnectIp4, net_addr, 8);
  }
#else
  #define EvrNetSMTP_ConnectIp4(net_addr)
#endif

/**
  \brief  Event on SMTP client \ref netSMTPc_Connect to IPv6 server (API)
  \param  net_addr      pointer to \ref NET_ADDR server address structure (20 bytes)
  \note   Sends full size of net address structure with IPv6 address only.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ConnectIp6(const void *net_addr) {
    EventRecordData (EvtNetSMTP_ConnectIp6, net_addr, 20);
  }
#else
  #define EvrNetSMTP_ConnectIp6(net_addr)
#endif

/**
  \brief  Event on \ref netSMTPc_Connect invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ConnectInvalidParameter(void) {
    EventRecord2 (EvtNetSMTP_ConnectInvalidParameter, 0, 0);
  }
#else
  #define EvrNetSMTP_ConnectInvalidParameter()
#endif

/**
  \brief  Event on \ref netSMTPc_Connect failed, client busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ConnectClientBusy(void) {
    EventRecord2 (EvtNetSMTP_ConnectClientBusy, 0, 0);
  }
#else
  #define EvrNetSMTP_ConnectClientBusy()
#endif

/**
  \brief  Event on SMTP client \ref netSMTPc_SendMail to mail server (API)
  \param  num_rcpt      number of mail recipients (To, CC, BCC)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMail(uint8_t num_rcpt) {
    EventRecord2 (EvtNetSMTP_SendMail, num_rcpt, 0);
  }
#else
  #define EvrNetSMTP_SendMail(num_rcpt)
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailInvalidParameter(void) {
    EventRecord2 (EvtNetSMTP_SendMailInvalidParameter, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMailInvalidParameter()
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail secure TLS not enabled (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailTlsNotEnabled(void) {
    EventRecord2 (EvtNetSMTP_SendMailTlsNotEnabled, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMailTlsNotEnabled()
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail failed, client busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailClientBusy(void) {
    EventRecord2 (EvtNetSMTP_SendMailClientBusy, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMailClientBusy()
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail failed, mail recipient not specified (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailNoRecipients(void) {
    EventRecord2 (EvtNetSMTP_SendMailNoRecipients, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMailNoRecipients()
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail failed, mail server not valid (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailServerNotValid(void) {
    EventRecord2 (EvtNetSMTP_SendMailServerNotValid, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMailServerNotValid()
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail failed, DNS host resolver error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailDnsError(void) {
    EventRecord2 (EvtNetSMTP_SendMailDnsError, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMailDnsError()
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail failed, support for attachments not enabled (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailAttachNotEnabled(void) {
    EventRecord2 (EvtNetSMTP_SendMailAttachNotEnabled, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMailAttachNotEnabled()
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail attach file failed, fopen failed (Error)
  \param  fname         name of the requested file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailAttachFailed(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetSMTP_SendMailAttachFailed, fname, length);
  }
#else
  #define EvrNetSMTP_SendMailAttachFailed(fname, length)
#endif

/**
  \brief  Event on SMTP client \ref netSMTPc_SendMail multipart with attachments (Op)
  \param  num_files     number of files to attach
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailMultipart(uint8_t num_files) {
    EventRecord2 (EvtNetSMTP_SendMailMultipart, num_files, 0);
  }
#else
  #define EvrNetSMTP_SendMailMultipart(num_files)
#endif

/**
  \brief  Event on \ref netSMTPc_SendMail attach file (Op)
  \param  fname         name of the requested file
  \param  length        length of the fname string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMailAttachment(const char *fname, uint32_t length) {
    if (length > 80) length = 80;
    EventRecordData (EvtNetSMTP_SendMailAttachment, fname, length);
  }
#else
  #define EvrNetSMTP_SendMailAttachment(fname, length)
#endif

/**
  \brief  Event on SMTP client inbound connection rejected (Error)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_InboundConnRejected(int32_t socket) {
    EventRecord2 (EvtNetSMTP_InboundConnRejected, (uint32_t)socket, 0);
  }
#else
  #define EvrNetSMTP_InboundConnRejected(socket)
#endif

/**
  \brief  Event on SMTP client socket aborted (Error)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SocketAborted(int32_t socket) {
    EventRecord2 (EvtNetSMTP_SocketAborted, (uint32_t)socket, 0);
  }
#else
  #define EvrNetSMTP_SocketAborted(socket)
#endif

/**
  \brief  Event on SMTP client socket connected (Op)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SocketConnected(int32_t socket) {
    EventRecord2 (EvtNetSMTP_SocketConnected, (uint32_t)socket, 0);
  }
#else
  #define EvrNetSMTP_SocketConnected(socket)
#endif

/**
  \brief  Event on SMTP client socket closed (Op)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SocketClosed(int32_t socket) {
    EventRecord2 (EvtNetSMTP_SocketClosed, (uint32_t)socket, 0);
  }
#else
  #define EvrNetSMTP_SocketClosed(socket)
#endif

/**
  \brief  Event on SMTP client received frame (Op)
  \param  socket        assigned TCP socket
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ReceiveFrame(int32_t socket, uint32_t length) {
    EventRecord2 (EvtNetSMTP_ReceiveFrame, (uint32_t)socket, length);
  }
#else
  #define EvrNetSMTP_ReceiveFrame(socket, length)
#endif

/**
  \brief  Event on SMTP client unacked data error (Error)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_UnackedDataError(int32_t socket) {
    EventRecord2 (EvtNetSMTP_UnackedDataError, (uint32_t)socket, 0);
  }
#else
  #define EvrNetSMTP_UnackedDataError(socket)
#endif

/**
  \brief  Event on SMTP received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetSMTP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetSMTP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on SMTP client server ready response (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ServerReady(void) {
    EventRecord2 (EvtNetSMTP_ServerReady, 0, 0);
  }
#else
  #define EvrNetSMTP_ServerReady()
#endif

/**
  \brief  Event on SMTP client Extended SMTP mode (ESMTP) not supported (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_EsmtpModeNotSupported(void) {
    EventRecord2 (EvtNetSMTP_EsmtpModeNotSupported, 0, 0);
  }
#else
  #define EvrNetSMTP_EsmtpModeNotSupported()
#endif

/**
  \brief  Event on SMTP client EHLO response divided into multiple frames (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_EhloResponseFragmented(void) {
    EventRecord2 (EvtNetSMTP_EhloResponseFragmented, 0, 0);
  }
#else
  #define EvrNetSMTP_EhloResponseFragmented()
#endif

/**
  \brief  Event on SMTP client Extended SMTP mode (ESMTP) active (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_EsmtpModeActive(void) {
    EventRecord2 (EvtNetSMTP_EsmtpModeActive, 0, 0);
  }
#else
  #define EvrNetSMTP_EsmtpModeActive()
#endif

/**
  \brief  Event on SMTP client start authentication (Op)
  \param  auth_mode     SMTP authentication mode
                         - 0: not required
                         - 1: PLAIN
                         - 2: LOGIN
                         - 3: DIGEST-MD5
                         - 4: CRAM-MD5
                         - 5: NTLM
                         - 6: unknown
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_StartAuthentication(uint8_t auth_mode) {
    EventRecord2 (EvtNetSMTP_StartAuthentication, auth_mode, 0);
  }
#else
  #define EvrNetSMTP_StartAuthentication(auth_mode)
#endif

/**
  \brief  Event on SMTP client authentication denied by the user (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_AuthenticationDenied(void) {
    EventRecord2 (EvtNetSMTP_AuthenticationDenied, 0, 0);
  }
#else
  #define EvrNetSMTP_AuthenticationDenied()
#endif

/**
  \brief  Event on SMTP client authentication method not supported (Error)
  \param  auth_mode     SMTP authentication mode
                         - 0: not required
                         - 1: PLAIN
                         - 2: LOGIN
                         - 3: DIGEST-MD5
                         - 4: CRAM-MD5
                         - 5: NTLM
                         - 6: unknown
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_AuthMethodNotSupported(uint8_t auth_mode) {
    EventRecord2 (EvtNetSMTP_AuthMethodNotSupported, auth_mode, 0);
  }
#else
  #define EvrNetSMTP_AuthMethodNotSupported(auth_mode)
#endif

/**
  \brief  Event on SMTP client basic SMTP mode active (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SmtpModeActive(void) {
    EventRecord2 (EvtNetSMTP_SmtpModeActive, 0, 0);
  }
#else
  #define EvrNetSMTP_SmtpModeActive()
#endif

/**
  \brief  Event on SMTP client authentication completed successfully (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_AuthenticationSuccessful(void) {
    EventRecord2 (EvtNetSMTP_AuthenticationSuccessful, 0, 0);
  }
#else
  #define EvrNetSMTP_AuthenticationSuccessful()
#endif

/**
  \brief  Event on SMTP client authentication failed (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_AuthenticationFailed(void) {
    EventRecord2 (EvtNetSMTP_AuthenticationFailed, 0, 0);
  }
#else
  #define EvrNetSMTP_AuthenticationFailed()
#endif

/**
  \brief  Event on SMTP client received server acknowledgment (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ServerAcknowledge(void) {
    EventRecord2 (EvtNetSMTP_ServerAcknowledge, 0, 0);
  }
#else
  #define EvrNetSMTP_ServerAcknowledge()
#endif

/**
  \brief  Event on SMTP client received wrong response from server (Error)
  \param  response      server response in text form
  \param  length        length of the response string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_WrongResponse(const uint8_t *response, uint32_t length) {
    if (length > 64) length = 64;
    EventRecordData (EvtNetSMTP_WrongResponse, response, length);
  }
#else
  #define EvrNetSMTP_WrongResponse(response, length)
#endif

/**
  \brief  Event on SMTP client stop operation, timeout expired (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ClientStopTimeoutExpired(void) {
    EventRecord2 (EvtNetSMTP_ClientStopTimeoutExpired, 0, 0);
  }
#else
  #define EvrNetSMTP_ClientStopTimeoutExpired()
#endif

/**
  \brief  Event on SMTP client send message body start (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMessageBody(void) {
    EventRecord2 (EvtNetSMTP_SendMessageBody, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMessageBody()
#endif

/**
  \brief  Event on SMTP client send end of message sequence (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendMessageEnd(void) {
    EventRecord2 (EvtNetSMTP_SendMessageEnd, 0, 0);
  }
#else
  #define EvrNetSMTP_SendMessageEnd()
#endif

/**
  \brief  Event on SMTP client send command to server (Detail)
  \param  command       server command in text form
  \param  length        length of the command string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_SendCommand(const uint8_t *command, uint32_t length) {
    if (length > 64) length = 64;
    EventRecordData (EvtNetSMTP_SendCommand, command, length);
  }
#else
  #define EvrNetSMTP_SendCommand(command, length)
#endif

/**
  \brief  Event on SMTP client close socket (Op)
  \param  socket        assigned TCP socket
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ClientCloseSocket(int32_t socket) {
    EventRecord2 (EvtNetSMTP_ClientCloseSocket, (uint32_t)socket, 0);
  }
#else
  #define EvrNetSMTP_ClientCloseSocket(socket)
#endif

/**
  \brief  Event on SMTP client completed operation (Op)
  \param  cb_event      user callback nofication event
                         - 0: Email successfully sent
                         - 1: Timeout on sending email
                         - 2: Login failed, username/password invalid
                         - 3: Generoc SMTP client error
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_ClientDone(uint8_t cb_event) {
    EventRecord2 (EvtNetSMTP_ClientDone, cb_event, 0);
  }
#else
  #define EvrNetSMTP_ClientDone(cb_event)
#endif

/**
  \brief  Event on SMTP client TLS support indicated by the server (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_TlsSupportIndicated(void) {
    EventRecord2 (EvtNetSMTP_TlsSupportIndicated, 0, 0);
  }
#else
  #define EvrNetSMTP_TlsSupportIndicated()
#endif

/**
  \brief  Event on SMTP client STARTTLS command accepted by the server (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_StartTlsAccepted(void) {
    EventRecord2 (EvtNetSMTP_StartTlsAccepted, 0, 0);
  }
#else
  #define EvrNetSMTP_StartTlsAccepted()
#endif

/**
  \brief  Event on SMTP client failed to allocate secure TLS context (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_TlsGetContextFailed(void) {
    EventRecord2 (EvtNetSMTP_TlsGetContextFailed, 0, 0);
  }
#else
  #define EvrNetSMTP_TlsGetContextFailed()
#endif

/**
  \brief  Event on SMTP client started secure TLS mode (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_TlsModeStarted(void) {
    EventRecord2 (EvtNetSMTP_TlsModeStarted, 0, 0);
  }
#else
  #define EvrNetSMTP_TlsModeStarted()
#endif

/**
  \brief  Event on SMTP client established secure TLS session (Op)
  \param  tls_id        TLS session identifier
*/
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_TlsModeEstablished(uint8_t tls_id) {
    EventRecord2 (EvtNetSMTP_TlsModeEstablished, tls_id, 0);
  }
#else
  #define EvrNetSMTP_TlsModeEstablished(tls_id)
#endif

/**
  \brief  Event on SMTP client de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSMTP_UninitClient(void) {
    EventRecord2 (EvtNetSMTP_UninitClient, 0, 0);
  }
#else
  #define EvrNetSMTP_UninitClient()
#endif


// NetDNS event identifiers ----------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetDNS_InitClient                EventID (EventLevelOp,    EvtNetDNS,  0)
#define EvtNetDNS_GetSocketFailed           EventID (EventLevelError, EvtNetDNS,  1)
#define EvtNetDNS_ChangeDnsServer           EventID (EventLevelOp,    EvtNetDNS,  2)
#define EvtNetDNS_ClientStopTimeoutExpired  EventID (EventLevelError, EvtNetDNS,  3)
#define EvtNetDNS_ReceiveFrameIp4           EventID (EventLevelOp,    EvtNetDNS,  4)
#define EvtNetDNS_ReceiveFrameIp6           EventID (EventLevelOp,    EvtNetDNS,  5)
#define EvtNetDNS_WrongServerPort           EventID (EventLevelError, EvtNetDNS,  6)
#define EvtNetDNS_FrameTooShort             EventID (EventLevelError, EvtNetDNS,  7)
#define EvtNetDNS_WrongServerAddress        EventID (EventLevelError, EvtNetDNS,  8)
#define EvtNetDNS_ShowFrameHeader           EventID (EventLevelDetail,EvtNetDNS,  9)
#define EvtNetDNS_WrongTransactionId        EventID (EventLevelError, EvtNetDNS, 10)
#define EvtNetDNS_DnsRequestReceived        EventID (EventLevelError, EvtNetDNS, 11)
#define EvtNetDNS_OpcodeOrTruncated         EventID (EventLevelError, EvtNetDNS, 12)
#define EvtNetDNS_NoSuchNameFound           EventID (EventLevelOp,    EvtNetDNS, 13)
#define EvtNetDNS_RcodeAndRecursion         EventID (EventLevelError, EvtNetDNS, 14)
#define EvtNetDNS_MoreAnswersReceived       EventID (EventLevelError, EvtNetDNS, 15)
#define EvtNetDNS_QnameNotTheSame           EventID (EventLevelError, EvtNetDNS, 16)
#define EvtNetDNS_QtypeNotTheSame           EventID (EventLevelError, EvtNetDNS, 17)
#define EvtNetDNS_QclassNotInet             EventID (EventLevelError, EvtNetDNS, 18)
#define EvtNetDNS_GotHostAddressIp4         EventID (EventLevelOp,    EvtNetDNS, 19)
#define EvtNetDNS_GotHostAddressIp6         EventID (EventLevelOp,    EvtNetDNS, 20)
#define EvtNetDNS_GotAuthorityAddressIp4    EventID (EventLevelOp,    EvtNetDNS, 21)
#define EvtNetDNS_GotAuthorityAddressIp6    EventID (EventLevelOp,    EvtNetDNS, 22)
#define EvtNetDNS_RecordTypeNotSupported    EventID (EventLevelError, EvtNetDNS, 23)
#define EvtNetDNS_Resolve                   EventID (EventLevelOp,    EvtNetDNS, 24)
#define EvtNetDNS_ResolveInvalidParameter   EventID (EventLevelError, EvtNetDNS, 25)
#define EvtNetDNS_ResolveClientBusy         EventID (EventLevelError, EvtNetDNS, 26)
#define EvtNetDNS_ResolvedAddressIp4        EventID (EventLevelOp,    EvtNetDNS, 27)
#define EvtNetDNS_ResolvedAddressIp6        EventID (EventLevelOp,    EvtNetDNS, 28)
#define EvtNetDNS_ResolveDnsServerUnknown   EventID (EventLevelError, EvtNetDNS, 29)
#define EvtNetDNS_SendRequestIp4            EventID (EventLevelOp,    EvtNetDNS, 30)
#define EvtNetDNS_SendRequestIp6            EventID (EventLevelOp,    EvtNetDNS, 31)
#define EvtNetDNS_ClearCache                EventID (EventLevelAPI,   EvtNetDNS, 32)
#define EvtNetDNS_ClearCacheClientBusy      EventID (EventLevelError, EvtNetDNS, 33)
#define EvtNetDNS_UninitClient              EventID (EventLevelOp,    EvtNetDNS, 34)
#endif

/**
  \brief  Event on DNS client initialize (Op)
  \param  num_entries   number of entries available in cache
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_InitClient(uint32_t num_entries) {
    EventRecord2 (EvtNetDNS_InitClient, num_entries, 0);
  }
#else
  #define EvrNetDNS_InitClient(num_entries)
#endif

/**
  \brief  Event on DNS client failed to allocate UDP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_GetSocketFailed(void) {
    EventRecord2 (EvtNetDNS_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetDNS_GetSocketFailed()
#endif

/**
  \brief  Event on DNS client change active DNS server (Error)
  \param  server        DNS server to use
                         - 0: primary
                         - 1: secondary
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ChangeDnsServer(int32_t server) {
    EventRecord2 (EvtNetDNS_ChangeDnsServer, (uint32_t)server, 0);
  }
#else
  #define EvrNetDNS_ChangeDnsServer(server)
#endif

/**
  \brief  Event on DNS client stop operation timeout expired (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ClientStopTimeoutExpired(void) {
    EventRecord2 (EvtNetDNS_ClientStopTimeoutExpired, 0, 0);
  }
#else
  #define EvrNetDNS_ClientStopTimeoutExpired()
#endif

/**
  \brief  Event on DNS client receive frame from IPv4 server (Op)
  \param  ip4_addr      pointer to IPv4 address of the server
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ReceiveFrameIp4(const uint8_t *ip4_addr, uint32_t length) {
    evr_buf.u32[0] = length;
    memcpy (&evr_buf.u8[4], ip4_addr, 4);
    EventRecordData (EvtNetDNS_ReceiveFrameIp4, &evr_buf, 8);
  }
#else
  #define EvrNetDNS_ReceiveFrameIp4(ip4_addr, length)
#endif

/**
  \brief  Event on DNS client receive frame from IPv6 server (Op)
  \param  ip6_addr      pointer to IPv6 address of the server
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ReceiveFrameIp6(const uint8_t *ip6_addr, uint32_t length) {
    evr_buf.u32[0] = length;
    memcpy (&evr_buf.u8[4], ip6_addr, 16);
    EventRecordData (EvtNetDNS_ReceiveFrameIp6, &evr_buf, 20);
  }
#else
  #define EvrNetDNS_ReceiveFrameIp6(ip6_addr, length)
#endif

/**
  \brief  Event on DNS client wrong server port (Error)
  \param  udp_port      wrong UDP port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_WrongServerPort(uint16_t udp_port) {
    EventRecord2 (EvtNetDNS_WrongServerPort, udp_port, 0);
  }
#else
  #define EvrNetDNS_WrongServerPort(udp_port)
#endif

/**
  \brief  Event on DNS client received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetDNS_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetDNS_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on DNS client wrong DNS server address received (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_WrongServerAddress(void) {
    EventRecord2 (EvtNetDNS_WrongServerAddress, 0, 0);
  }
#else
  #define EvrNetDNS_WrongServerAddress()
#endif

/**
  \brief  Event on DNS client display send/receive frame header (Detail)
  \param  dns_header    pointer to DNS frame header of 12 bytes
                         - TID     (2 bytes)
                         - Flags   (2 bytes)
                         - QDCOUNT (2 bytes)
                         - ANCOUNT (2 bytes)
                         - NSCOUNT (2 bytes)
                         - ARCOUNT (2 bytes)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ShowFrameHeader(const void *dns_header) {
    EventRecordData (EvtNetDNS_ShowFrameHeader, dns_header, 12);
  }
#else
  #define EvrNetDNS_ShowFrameHeader(dns_header)
#endif

/**
  \brief  Event on DNS client wrong transaction identifier (TID) received (Error)
  \param  tid           transaction identifier
  \param  tid_valid     expected/valid transaction identifier
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_WrongTransactionId(uint16_t tid, uint16_t tid_valid) {
    EventRecord2 (EvtNetDNS_WrongTransactionId, tid, tid_valid);
  }
#else
  #define EvrNetDNS_WrongTransactionId(tid, tid_valid)
#endif

/**
  \brief  Event on DNS client received a DNS request (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_DnsRequestReceived(void) {
    EventRecord2 (EvtNetDNS_DnsRequestReceived, 0, 0);
  }
#else
  #define EvrNetDNS_DnsRequestReceived()
#endif

/**
  \brief  Event on DNS client OPCODE not zero or message truncated received (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_OpcodeOrTruncated(void) {
    EventRecord2 (EvtNetDNS_OpcodeOrTruncated, 0, 0);
  }
#else
  #define EvrNetDNS_OpcodeOrTruncated()
#endif

/**
  \brief  Event on DNS client no such name found message received (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_NoSuchNameFound(void) {
    EventRecord2 (EvtNetDNS_NoSuchNameFound, 0, 0);
  }
#else
  #define EvrNetDNS_NoSuchNameFound()
#endif

/**
  \brief  Event on DNS client RCODE not zero and recursion available received (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_RcodeAndRecursion(void) {
    EventRecord2 (EvtNetDNS_RcodeAndRecursion, 0, 0);
  }
#else
  #define EvrNetDNS_RcodeAndRecursion()
#endif

/**
  \brief  Event on DNS client more than one answer received (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_MoreAnswersReceived(void) {
    EventRecord2 (EvtNetDNS_MoreAnswersReceived, 0, 0);
  }
#else
  #define EvrNetDNS_MoreAnswersReceived()
#endif

/**
  \brief  Event on DNS client QNAME in a response not the same as requested (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_QnameNotTheSame(void) {
    EventRecord2 (EvtNetDNS_QnameNotTheSame, 0, 0);
  }
#else
  #define EvrNetDNS_QnameNotTheSame()
#endif

/**
  \brief  Event on DNS client QTYPE in a response not the same as requested (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_QtypeNotTheSame(void) {
    EventRecord2 (EvtNetDNS_QtypeNotTheSame, 0, 0);
  }
#else
  #define EvrNetDNS_QtypeNotTheSame()
#endif

/**
  \brief  Event on DNS client QCLASS in a response is not CLASS_INET (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_QclassNotInet(void) {
    EventRecord2 (EvtNetDNS_QclassNotInet, 0, 0);
  }
#else
  #define EvrNetDNS_QclassNotInet()
#endif

/**
  \brief  Event on DNS client receive resolved IPv4 address (Op)
  \param  ip4_addr      pointer to IPv4 address of the host
  \param  ttl           address expiration timeout
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_GotHostAddressIp4(const uint8_t *ip4_addr, uint32_t ttl) {
    evr_buf.u32[0] = ttl;
    memcpy (&evr_buf.u8[4], ip4_addr, 4);
    EventRecordData (EvtNetDNS_GotHostAddressIp4, &evr_buf, 8);
  }
#else
  #define EvrNetDNS_GotHostAddressIp4(ip4_addr, ttl)
#endif

/**
  \brief  Event on DNS client receive resolved IPv6 address (Op)
  \param  ip6_addr      pointer to IPv6 address of the host
  \param  ttl           address expiration timeout
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_GotHostAddressIp6(const uint8_t *ip6_addr, uint32_t ttl) {
    evr_buf.u32[0] = ttl;
    memcpy (&evr_buf.u8[4], ip6_addr, 16);
    EventRecordData (EvtNetDNS_GotHostAddressIp6, &evr_buf, 20);
  }
#else
  #define EvrNetDNS_GotHostAddressIp6(ip6_addr, ttl)
#endif

/**
  \brief  Event on DNS client receive IPv4 address of the authority (Op)
  \param  ip4_addr      pointer to IPv4 address of the authority
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_GotAuthorityAddressIp4(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDNS_GotAuthorityAddressIp4, ip4_addr, 4);
  }
#else
  #define EvrNetDNS_GotAuthorityAddressIp4(ip4_addr)
#endif

/**
  \brief  Event on DNS client receive IPv6 address of the authority (Op)
  \param  ip6_addr      pointer to IPv6 address of the authority
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_GotAuthorityAddressIp6(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetDNS_GotAuthorityAddressIp6, ip6_addr, 16);
  }
#else
  #define EvrNetDNS_GotAuthorityAddressIp6(ip6_addr)
#endif

/**
  \brief  Event on DNS client can not handle resource record type (Error)
  \param  rr_type       recource record type
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_RecordTypeNotSupported(uint16_t rr_type) {
    EventRecord2 (EvtNetDNS_RecordTypeNotSupported, rr_type, 0);
  }
#else
  #define EvrNetDNS_RecordTypeNotSupported(rr_type)
#endif

/**
  \brief  Event on DNS client resolve host requested (Op)
  \param  host_name     name of the host to resolve
  \param  length        length of the host_name string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_Resolve(const char *host_name, uint32_t length) {
    if (length > 40) length = 40;
    EventRecordData (EvtNetDNS_Resolve, host_name, length);
  }
#else
  #define EvrNetDNS_Resolve(host_name, length)
#endif

/**
  \brief  Event on DNS client invalid parameter for resolve request (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ResolveInvalidParameter(void) {
    EventRecord2 (EvtNetDNS_ResolveInvalidParameter, 0, 0);
  }
#else
  #define EvrNetDNS_ResolveInvalidParameter()
#endif

/**
  \brief  Event on DNS client resolve request failed, client busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ResolveClientBusy(void) {
    EventRecord2 (EvtNetDNS_ResolveClientBusy, 0, 0);
  }
#else
  #define EvrNetDNS_ResolveClientBusy()
#endif

/**
  \brief  Event on DNS client resolved IPv4 address internally (Op)
  \param  ip4_addr      pointer to resolved IPv4 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ResolvedAddressIp4(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDNS_ResolvedAddressIp4, ip4_addr, 4);
  }
#else
  #define EvrNetDNS_ResolvedAddressIp4(ip4_addr)
#endif

/**
  \brief  Event on DNS client resolved IPv6 address internally (Op)
  \param  ip6_addr      pointer to resolved IPv6 address
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ResolvedAddressIp6(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetDNS_ResolvedAddressIp6, ip6_addr, 16);
  }
#else
  #define EvrNetDNS_ResolvedAddressIp6(ip6_addr)
#endif

/**
  \brief  Event on DNS client resolve failed, DNS server unknown (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ResolveDnsServerUnknown(void) {
    EventRecord2 (EvtNetDNS_ResolveDnsServerUnknown, 0, 0);
  }
#else
  #define EvrNetDNS_ResolveDnsServerUnknown()
#endif

/**
  \brief  Event on DNS client send resolve request to IPv4 server (Op)
  \param  ip4_addr      pointer to IPv4 address of DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_SendRequestIp4(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetDNS_SendRequestIp4, ip4_addr, 4);
  }
#else
  #define EvrNetDNS_SendRequestIp4(ip4_addr)
#endif

/**
  \brief  Event on DNS client send resolve request to IPv6 server (Op)
  \param  ip6_addr      pointer to IPv6 address of DNS server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_SendRequestIp6(const uint8_t *ip6_addr) {
    EventRecordData (EvtNetDNS_SendRequestIp6, ip6_addr, 16);
  }
#else
  #define EvrNetDNS_SendRequestIp6(ip6_addr)
#endif

/**
  \brief  Event on DNS client clear cache requested (API)
  \param  used          number of used entries flushed from cache
  \param  available     number of entries available in cache
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ClearCache(uint32_t used, uint32_t available) {
    EventRecord2 (EvtNetDNS_ClearCache, used, available);
  }
#else
  #define EvrNetDNS_ClearCache(used, available)
#endif

/**
  \brief  Event on DNS client clear cache request failed, client busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_ClearCacheClientBusy(void) {
    EventRecord2 (EvtNetDNS_ClearCacheClientBusy, 0, 0);
  }
#else
  #define EvrNetDNS_ClearCacheClientBusy()
#endif

/**
  \brief  Event on DNS client de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetDNS_UninitClient(void) {
    EventRecord2 (EvtNetDNS_UninitClient, 0, 0);
  }
#else
  #define EvrNetDNS_UninitClient()
#endif


// NetSNMP event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetSNMP_InitAgent                EventID (EventLevelOp,    EvtNetSNMP,  0)
#define EvtNetSNMP_GetSocketFailed          EventID (EventLevelError, EvtNetSNMP,  1)
#define EvtNetSNMP_ShowCommunity            EventID (EventLevelOp,    EvtNetSNMP,  2)
#define EvtNetSNMP_SetCommunity             EventID (EventLevelAPI,   EvtNetSNMP,  3)
#define EvtNetSNMP_SetCommunityInvalidParam EventID (EventLevelError, EvtNetSNMP,  4)
#define EvtNetSNMP_SetMibTable              EventID (EventLevelAPI,   EvtNetSNMP,  5)
#define EvtNetSNMP_SetMibTableInvalidParam  EventID (EventLevelError, EvtNetSNMP,  6)
#define EvtNetSNMP_Trap                     EventID (EventLevelAPI,   EvtNetSNMP,  7)
#define EvtNetSNMP_TrapInvalidParameter     EventID (EventLevelError, EvtNetSNMP,  8)
#define EvtNetSNMP_TrapMibTableNotSet       EventID (EventLevelError, EvtNetSNMP,  9)
#define EvtNetSNMP_TrapMissingSysObjectId   EventID (EventLevelError, EvtNetSNMP, 10)
#define EvtNetSNMP_TrapGenericTrapInvalid   EventID (EventLevelError, EvtNetSNMP, 11)
#define EvtNetSNMP_TrapTooManyObjects       EventID (EventLevelError, EvtNetSNMP, 12)
#define EvtNetSNMP_TrapObjectNotExisting    EventID (EventLevelError, EvtNetSNMP, 13)
#define EvtNetSNMP_TrapMessageTooBig        EventID (EventLevelError, EvtNetSNMP, 14)
#define EvtNetSNMP_AddObject                EventID (EventLevelDetail,EvtNetSNMP, 15)
#define EvtNetSNMP_ReceiveFrame             EventID (EventLevelOp,    EvtNetSNMP, 16)
#define EvtNetSNMP_FrameTooShort            EventID (EventLevelError, EvtNetSNMP, 17)
#define EvtNetSNMP_FrameProtocolError       EventID (EventLevelError, EvtNetSNMP, 18)
#define EvtNetSNMP_VersionNotSupported      EventID (EventLevelOp,    EvtNetSNMP, 19)
#define EvtNetSNMP_WrongCommunityReceived   EventID (EventLevelOp,    EvtNetSNMP, 20)
#define EvtNetSNMP_InvalidRequestType       EventID (EventLevelError, EvtNetSNMP, 21)
#define EvtNetSNMP_ShowRequest              EventID (EventLevelOp,    EvtNetSNMP, 22)
#define EvtNetSNMP_TooManyObjectsReceived   EventID (EventLevelError, EvtNetSNMP, 23)
#define EvtNetSNMP_ObjectNotFound           EventID (EventLevelError, EvtNetSNMP, 24)
#define EvtNetSNMP_ObjectWrongType          EventID (EventLevelError, EvtNetSNMP, 25)
#define EvtNetSNMP_ObjectReadOnly           EventID (EventLevelError, EvtNetSNMP, 26)
#define EvtNetSNMP_ObjectWriteFailed        EventID (EventLevelError, EvtNetSNMP, 27)
#define EvtNetSNMP_SendErrorStatus          EventID (EventLevelOp,    EvtNetSNMP, 28)
#define EvtNetSNMP_ResponseMessageTooBig    EventID (EventLevelError, EvtNetSNMP, 29)
#define EvtNetSNMP_SendGetResponse          EventID (EventLevelOp,    EvtNetSNMP, 30)
#define EvtNetSNMP_UninitAgent              EventID (EventLevelOp,    EvtNetSNMP, 31)
#endif

/**
  \brief  Event on SNMP agent initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_InitAgent(void) {
    EventRecord2 (EvtNetSNMP_InitAgent, 0, 0);
  }
#else
  #define EvrNetSNMP_InitAgent()
#endif

/**
  \brief  Event on SNMP agent failed to allocate UDP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_GetSocketFailed(void) {
    EventRecord2 (EvtNetSNMP_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetSNMP_GetSocketFailed()
#endif

/**
  \brief  Event on SNMP agent display community (Op)
  \param  community     pointer to SNMP community string
  \param  length        length of the community string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_ShowCommunity(const char *community, uint32_t length) {
    if (length > 24) length = 24;
    EventRecordData(EvtNetSNMP_ShowCommunity, community, length);
  }
#else
  #define EvrNetSNMP_ShowCommunity(community, length)
#endif

/**
  \brief  Event on \ref netSNMP_SetCommunity (API)
  \param  community     pointer to SNMP community string
  \param  length        length of the community string
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_SetCommunity(const char *community, uint32_t length) {
    if (length > 32) length = 32;
    EventRecordData(EvtNetSNMP_SetCommunity, community, length);
  }
#else
  #define EvrNetSNMP_SetCommunity(community, length)
#endif

/**
  \brief  Event on \ref netSNMP_SetCommunity invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_SetCommunityInvalidParam(void) {
    EventRecord2 (EvtNetSNMP_SetCommunityInvalidParam, 0, 0);
  }
#else
  #define EvrNetSNMP_SetCommunityInvalidParam()
#endif

/**
  \brief  Event on \ref netSNMP_SetMIB_Table (API)
  \param  num_entries   number of entries in MIB table
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_SetMibTable(uint32_t num_entries) {
    EventRecord2 (EvtNetSNMP_SetMibTable, num_entries, 0);
  }
#else
  #define EvrNetSNMP_SetMibTable(num_entries)
#endif

/**
  \brief  Event on \ref netSNMP_SetMIB_Table invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_SetMibTableInvalidParam(void) {
    EventRecord2 (EvtNetSNMP_SetMibTableInvalidParam, 0, 0);
  }
#else
  #define EvrNetSNMP_SetMibTableInvalidParam()
#endif

/**
  \brief  Event on \ref netSNMP_Trap (API)
  \param  ip4_addr      trap server IPv4 address
  \param  generic       generic trap type
                         - 0: coldStart trap
                         - 1: warmStart trap
                         - 2: linkDown trap
                         - 3: linkUp trap
                         - 4: authenticationFailure trap
                         - 5: egpNeighborLoss trap
                         - 6: enterpriseSpecific trap
  \param  specific      specific trap type for generic enterpriseSpecific trap
                         - must be set to 0 for generic traps 0 .. 5
  \param  nobj          number of objects in a trap message
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_Trap(const uint8_t *ip4_addr, uint8_t generic, uint8_t specific, uint16_t nobj) {
    evr_buf.u16[0] = nobj;
    evr_buf.u8[2]  = generic;
    evr_buf.u8[3]  = specific;
    memcpy (&evr_buf.u8[4], ip4_addr, 4);
    EventRecordData (EvtNetSNMP_Trap, &evr_buf, 8);
  }
#else
  #define EvrNetSNMP_Trap(ip4_addr, generic, specific, nobj)
#endif

/**
  \brief  Event on \ref netSNMP_Trap invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_TrapInvalidParameter(void) {
    EventRecord2 (EvtNetSNMP_TrapInvalidParameter, 0, 0);
  }
#else
  #define EvrNetSNMP_TrapInvalidParameter()
#endif

/**
  \brief  Event on \ref netSNMP_Trap invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_TrapMibTableNotSet(void) {
    EventRecord2 (EvtNetSNMP_TrapMibTableNotSet, 0, 0);
  }
#else
  #define EvrNetSNMP_TrapMibTableNotSet()
#endif

/**
  \brief  Event on \ref netSNMP_Trap missing 'sysObjectID' object in MIB table (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_TrapMissingSysObjectId(void) {
    EventRecord2 (EvtNetSNMP_TrapMissingSysObjectId, 0, 0);
  }
#else
  #define EvrNetSNMP_TrapMissingSysObjectId()
#endif

/**
  \brief  Event on \ref netSNMP_Trap generic trap invalid (Error)
  \param  generic       invalid generic trap type
  \param  max_generic   maximum value for generic trap type
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_TrapGenericTrapInvalid(uint8_t generic, uint8_t max_generic) {
    EventRecord2 (EvtNetSNMP_TrapGenericTrapInvalid, generic, max_generic);
  }
#else
  #define EvrNetSNMP_TrapGenericTrapInvalid(generic, max_generic)
#endif

/**
  \brief  Event on \ref netSNMP_Trap maximum number of objects exceeded (Error)
  \param  nobj          number of TLV objects
  \param  max_nobj      maximum number of TLV objects
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_TrapTooManyObjects(int32_t nobj, int32_t max_nobj) {
    EventRecord2 (EvtNetSNMP_TrapTooManyObjects, (uint32_t)nobj, (uint32_t)max_nobj);
  }
#else
  #define EvrNetSNMP_TrapTooManyObjects(nobj, max_nobj)
#endif

/**
  \brief  Event on \ref netSNMP_Trap requested TLV object not existing in MIB table (Error)
  \param  obj           object identification number
  \param  max_obj       maximum value for object identification number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_TrapObjectNotExisting(int32_t obj, int32_t max_obj) {
    EventRecord2 (EvtNetSNMP_TrapObjectNotExisting, (uint32_t)obj, (uint32_t)max_obj);
  }
#else
  #define EvrNetSNMP_TrapObjectNotExisting(obj, max_obj)
#endif

/**
  \brief  Event on \ref netSNMP_Trap generated message too big (Error)
  \param  size          requested message size
  \param  max_size      maximum message size accepted
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_TrapMessageTooBig(uint16_t size, uint16_t max_size) {
    EventRecord2 (EvtNetSNMP_TrapMessageTooBig, size, max_size);
  }
#else
  #define EvrNetSNMP_TrapMessageTooBig(size, max_size)
#endif

/**
  \brief  Event on SNMP add object to SNMP message (Detail)
  \param  obj           object identification number
  \param  obj_type      object data type
                         - 0x02: Integer Data Type (signed integer)
                         - 0x04: Octet String Data Type
                         - 0x05: Null Item Data Type
                         - 0x06: Object Identifier Data Type
                         - 0x30: Sequence Data Type
                         - 0x40: IP Address Data Type
                         - 0x41: Counter Data Type (pos.integer)
                         - 0x42: Gauge Data Type (pos.integer)
                         - 0x43: Time Ticks Data Type (pos.integer)
                         - 0x44: Opaque Data Type (ASN.1 encoded)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_AddObject(int32_t obj, uint8_t obj_type) {
    EventRecord2 (EvtNetSNMP_AddObject, (uint32_t)obj, obj_type);
  }
#else
  #define EvrNetSNMP_AddObject(obj, obj_type)
#endif

/**
  \brief  Event on SNMP receive frame (Op)
  \param  ip4_addr      pointer to IPv4 address of the server
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_ReceiveFrame(const uint8_t *ip4_addr, uint32_t length) {
    evr_buf.u32[0] = length;
    memcpy (&evr_buf.u8[4], ip4_addr, 4);
    EventRecordData (EvtNetSNMP_ReceiveFrame, &evr_buf, 8);
  }
#else
  #define EvrNetSNMP_ReceiveFrame(ip4_addr, length)
#endif

/**
  \brief  Event on SNMP received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetSNMP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetSNMP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on SNMP received frame protocol error detected (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_FrameProtocolError(void) {
    EventRecord2 (EvtNetSNMP_FrameProtocolError, 0, 0);
  }
#else
  #define EvrNetSNMP_FrameProtocolError()
#endif

/**
  \brief  Event on SNMP received protocol version not supported (Op)
  \param  version       received SNMP protocol version
                         - 1: SNMPv1
                         - 2: SNMPv2
                         - 3: SNMPv3
  \note   Only SNMPv1 is currently supported.
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_VersionNotSupported(int32_t version) {
    if (version == 0) version = 1;
    EventRecord2 (EvtNetSNMP_VersionNotSupported, (uint32_t)version, 0);
  }
#else
  #define EvrNetSNMP_VersionNotSupported(version)
#endif

/**
  \brief  Event on SNMP received community wrong (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_WrongCommunityReceived(void) {
    EventRecord2 (EvtNetSNMP_WrongCommunityReceived, 0, 0);
  }
#else
  #define EvrNetSNMP_WrongCommunityReceived()
#endif

/**
  \brief  Event on SNMP received invalid PDU request type (Error)
  \param  req_type      PDU request type
                         - 0xA0: GetRequest PDU
                         - 0xA1: GetNextRequest PDU
                         - 0xA2: Response PDU
                         - 0xA3: SetRequest PDU
                         - 0xA4: Trap PDU
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_InvalidRequestType(uint8_t req_type) {
    EventRecord2 (EvtNetSNMP_InvalidRequestType, req_type, 0);
  }
#else
  #define EvrNetSNMP_InvalidRequestType(req_type)
#endif

/**
  \brief  Event on SNMP display received PDU request type and id (Error)
  \param  req_type      PDU request type
                         - 0xA0: GetRequest PDU
                         - 0xA1: GetNextRequest PDU
                         - 0xA2: Response PDU
                         - 0xA3: SetRequest PDU
                         - 0xA4: Trap PDU
  \param  req_id        PDU request identifier
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_ShowRequest(uint8_t req_type, int32_t req_id) {
    EventRecord2 (EvtNetSNMP_ShowRequest, req_type, (uint32_t)req_id);
  }
#else
  #define EvrNetSNMP_ShowRequest(req_type, req_id)
#endif

/**
  \brief  Event on SNMP received too many TLV objects (Error)
  \param  nobj          number of TLV objects
  \param  max_nobj      maximum number of TLV objects
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_TooManyObjectsReceived(int32_t nobj, int32_t max_nobj) {
    EventRecord2 (EvtNetSNMP_TooManyObjectsReceived, (uint32_t)nobj, (uint32_t)max_nobj);
  }
#else
  #define EvrNetSNMP_TooManyObjectsReceived(nobj, max_nobj)
#endif

/**
  \brief  Event on SNMP receive message object not found in the MIB table (Error)
  \param  obj           object identification number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_ObjectNotFound(int32_t obj) {
    EventRecord2 (EvtNetSNMP_ObjectNotFound, (uint32_t)obj, 0);
  }
#else
  #define EvrNetSNMP_ObjectNotFound(obj)
#endif

/**
  \brief  Event on SNMP receive message object type not the same as in the MIB table (Error)
  \param  obj           object identification number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_ObjectWrongType(int32_t obj) {
    EventRecord2 (EvtNetSNMP_ObjectWrongType, (uint32_t)obj, 0);
  }
#else
  #define EvrNetSNMP_ObjectWrongType(obj)
#endif

/**
  \brief  Event on SNMP write read-only object not allowed (Error)
  \param  obj           object identification number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_ObjectReadOnly(int32_t obj) {
    EventRecord2 (EvtNetSNMP_ObjectReadOnly, (uint32_t)obj, 0);
  }
#else
  #define EvrNetSNMP_ObjectReadOnly(obj)
#endif

/**
  \brief  Event on SNMP object write attempt failed for unknown reason (Error)
  \param  obj           object identification number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_ObjectWriteFailed(int32_t obj) {
    EventRecord2 (EvtNetSNMP_ObjectWriteFailed, (uint32_t)obj, 0);
  }
#else
  #define EvrNetSNMP_ObjectWriteFailed(obj)
#endif

/**
  \brief  Event on SNMP send error status response to a SNMP request (Op)
  \param  stat          error status
  \param  obj_index     error object index
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_SendErrorStatus(uint8_t stat, uint8_t obj_index) {
    EventRecord2 (EvtNetSNMP_SendErrorStatus, stat, obj_index);
  }
#else
  #define EvrNetSNMP_SendErrorStatus(stat, obj_index)
#endif

/**
  \brief  Event on SNMP response message would be too big (Error)
  \param  size          response message size
  \param  max_size      maximum message size allowed
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_ResponseMessageTooBig(uint16_t size, uint16_t max_size) {
    EventRecord2 (EvtNetSNMP_ResponseMessageTooBig, size, max_size);
  }
#else
  #define EvrNetSNMP_ResponseMessageTooBig(size, max_size)
#endif

/**
  \brief  Event on SNMP send get-response to a SNMP request (Op)
  \param  req_id        PDU request identifier
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_SendGetResponse(int32_t req_id) {
    EventRecord2 (EvtNetSNMP_SendGetResponse, (uint32_t)req_id, 0);
  }
#else
  #define EvrNetSNMP_SendGetResponse(req_id)
#endif

/**
  \brief  Event on SNMP agent de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNMP_UninitAgent(void) {
    EventRecord2 (EvtNetSNMP_UninitAgent, 0, 0);
  }
#else
  #define EvrNetSNMP_UninitAgent()
#endif


// NetSNTP event identifiers ---------------------------------------------------
#ifdef DEBUG_EVR
#define EvtNetSNTP_InitClient               EventID (EventLevelOp,    EvtNetSNTP,  0)
#define EvtNetSNTP_GetSocketFailed          EventID (EventLevelError, EvtNetSNTP,  1)
#define EvtNetSNTP_GetTime                  EventID (EventLevelAPI,   EvtNetSNTP,  2)
#define EvtNetSNTP_GetTimeInvalidParameter  EventID (EventLevelError, EvtNetSNTP,  3)
#define EvtNetSNTP_GetTimeWrongState        EventID (EventLevelError, EvtNetSNTP,  4)
#define EvtNetSNTP_GetTimeOpen              EventID (EventLevelOp,    EvtNetSNTP,  5)
#define EvtNetSNTP_GetTimeClose             EventID (EventLevelOp,    EvtNetSNTP,  6)
#define EvtNetSNTP_GetTimeServerUnknown     EventID (EventLevelError, EvtNetSNTP,  7)
#define EvtNetSNTP_GetTimexInvalidParameter EventID (EventLevelError, EvtNetSNTP,  8)
#define EvtNetSNTP_GetTimexClientBusy       EventID (EventLevelError, EvtNetSNTP,  9)
#define EvtNetSNTP_GetTimexServerNotValid   EventID (EventLevelError, EvtNetSNTP, 10)
#define EvtNetSNTP_GetTimexDnsError         EventID (EventLevelError, EvtNetSNTP, 11)
#define EvtNetSNTP_SetMode                  EventID (EventLevelAPI,   EvtNetSNTP, 12)
#define EvtNetSNTP_SetModeInvalidParameter  EventID (EventLevelError, EvtNetSNTP, 13)
#define EvtNetSNTP_SetModeWrongState        EventID (EventLevelError, EvtNetSNTP, 14)
#define EvtNetSNTP_SendMessage              EventID (EventLevelOp,    EvtNetSNTP, 15)
#define EvtNetSNTP_ServerNotResponding      EventID (EventLevelOp,    EvtNetSNTP, 16)
#define EvtNetSNTP_ReceiveFrame             EventID (EventLevelOp,    EvtNetSNTP, 17)
#define EvtNetSNTP_WrongServerPort          EventID (EventLevelError, EvtNetSNTP, 18)
#define EvtNetSNTP_FrameTooShort            EventID (EventLevelError, EvtNetSNTP, 19)
#define EvtNetSNTP_WrongServerAddress       EventID (EventLevelError, EvtNetSNTP, 20)
#define EvtNetSNTP_ModeNotServer            EventID (EventLevelError, EvtNetSNTP, 21)
#define EvtNetSNTP_ModeNotBroadcast         EventID (EventLevelError, EvtNetSNTP, 22)
#define EvtNetSNTP_AnswerInWrongState       EventID (EventLevelError, EvtNetSNTP, 23)
#define EvtNetSNTP_ShowTimeStamp            EventID (EventLevelOp,    EvtNetSNTP, 24)
#define EvtNetSNTP_TimeStampInvalid         EventID (EventLevelOp,    EvtNetSNTP, 25)
#define EvtNetSNTP_UninitClient             EventID (EventLevelOp,    EvtNetSNTP, 26)
#endif

/**
  \brief  Event on SNTP client initialize (Op)
  \param  mode          operation mode of the client
                         - 0: unicast
                         - 1: broadcast
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_InitClient(uint8_t mode) {
    EventRecord2 (EvtNetSNTP_InitClient, mode, 0);
  }
#else
  #define EvrNetSNTP_InitClient(mode)
#endif

/**
  \brief  Event on SNTP failed to allocate UDP socket (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetSocketFailed(void) {
    EventRecord2 (EvtNetSNTP_GetSocketFailed, 0, 0);
  }
#else
  #define EvrNetSNTP_GetSocketFailed()
#endif

/**
  \brief  Event on \ref netSNTPc_GetTime (API)
  \param  mode          operation mode of the client
                         - 0: unicast
                         - 1: broadcast
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTime(uint8_t mode) {
    EventRecord2 (EvtNetSNTP_GetTime, mode, 0);
  }
#else
  #define EvrNetSNTP_GetTime(mode)
#endif

/**
  \brief  Event on \ref netSNTPc_GetTime invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimeInvalidParameter(void) {
    EventRecord2 (EvtNetSNTP_GetTimeInvalidParameter, 0, 0);
  }
#else
  #define EvrNetSNTP_GetTimeInvalidParameter()
#endif

/**
  \brief  Event on \ref netSNTPc_GetTime wrong client state (Error)
  \param  state         client state
                         - 0: idle
                         - 1: busy in unicast send
                         - 2: open in broadcast receive
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimeWrongState(uint8_t state) {
    EventRecord2 (EvtNetSNTP_GetTimeWrongState, state, 0);
  }
#else
  #define EvrNetSNTP_GetTimeWrongState(state)
#endif

/**
  \brief  Event on \ref netSNTPc_GetTime open broadcast receive (Op)
  \param  ip4_addr      pointer to IPv4 address of NTP server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimeOpen(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetSNTP_GetTimeOpen, ip4_addr, 4);
  }
#else
  #define EvrNetSNTP_GetTimeOpen(ip4_addr)
#endif

/**
  \brief  Event on \ref netSNTPc_GetTime close broadcast receive (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimeClose(void) {
    EventRecord2 (EvtNetSNTP_GetTimeClose, 0, 0);
  }
#else
  #define EvrNetSNTP_GetTimeClose()
#endif

/**
  \brief  Event on \ref netSNTPc_GetTime unknown server for unicast mode (Error)
  \param  ip4_addr      pointer to IPv4 address of NTP server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimeServerUnknown(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetSNTP_GetTimeServerUnknown, ip4_addr, 4);
  }
#else
  #define EvrNetSNTP_GetTimeServerUnknown(ip4_addr)
#endif

/**
  \brief  Event on \ref netSNTPc_GetTimeX invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimexInvalidParameter(void) {
    EventRecord2 (EvtNetSNTP_GetTimexInvalidParameter, 0, 0);
  }
#else
  #define EvrNetSNTP_GetTimexInvalidParameter()
#endif

/**
  \brief  Event on \ref netSNTPc_GetTimeX client busy (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimexClientBusy(void) {
    EventRecord2 (EvtNetSNTP_GetTimexClientBusy, 0, 0);
  }
#else
  #define EvrNetSNTP_GetTimexClientBusy()
#endif

/**
  \brief  Event on \ref netSNTPc_GetTimeX server name not valid (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimexServerNotValid(void) {
    EventRecord2 (EvtNetSNTP_GetTimexServerNotValid, 0, 0);
  }
#else
  #define EvrNetSNTP_GetTimexServerNotValid()
#endif

/**
  \brief  Event on \ref netSNTPc_GetTimeX host name resolver error (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_GetTimexDnsError(void) {
    EventRecord2 (EvtNetSNTP_GetTimexDnsError, 0, 0);
  }
#else
  #define EvrNetSNTP_GetTimexDnsError()
#endif

/**
  \brief  Event on \ref netSNTPc_SetMode (API)
  \param  mode          operation mode of the client
                         - 0: unicast
                         - 1: broadcast
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_SetMode(uint8_t mode) {
    EventRecord2 (EvtNetSNTP_SetMode, mode, 0);
  }
#else
  #define EvrNetSNTP_SetMode(mode)
#endif

/**
  \brief  Event on \ref netSNTPc_SetMode invalid parameter (Error)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_SetModeInvalidParameter(void) {
    EventRecord2 (EvtNetSNTP_SetModeInvalidParameter, 0, 0);
  }
#else
  #define EvrNetSNTP_SetModeInvalidParameter()
#endif

/**
  \brief  Event on \ref netSNTPc_SetMode wrong client state (Error)
  \param  state         client state
                         - 0: idle
                         - 1: busy in unicast send
                         - 2: open in broadcast receive
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_SetModeWrongState(uint8_t state) {
    EventRecord2 (EvtNetSNTP_SetModeWrongState, state, 0);
  }
#else
  #define EvrNetSNTP_SetModeWrongState(state)
#endif

/**
  \brief  Event on SNTP send message to NTP server (Op)
  \param  ip4_addr      pointer to IPv4 address of NTP server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_SendMessage(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetSNTP_SendMessage, ip4_addr, 4);
  }
#else
  #define EvrNetSNTP_SendMessage(ip4_addr)
#endif

/**
  \brief  Event on SNTP send message timeout (Op)
  \param  ip4_addr      pointer to IPv4 address of NTP server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_ServerNotResponding(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetSNTP_ServerNotResponding, ip4_addr, 4);
  }
#else
  #define EvrNetSNTP_ServerNotResponding(ip4_addr)
#endif

/**
  \brief  Event on SNTP receive frame (Op)
  \param  ip4_addr      pointer to IPv4 address of the server
  \param  length        frame length in bytes
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_ReceiveFrame(const uint8_t *ip4_addr, uint32_t length) {
    uint32_t tmp_addr;
    memcpy (&tmp_addr, ip4_addr, 4);
    EventRecord2 (EvtNetSNTP_ReceiveFrame, tmp_addr, length);
  }
#else
  #define EvrNetSNTP_ReceiveFrame(ip4_addr, length)
#endif

/**
  \brief  Event on SNTP wrong server port (Error)
  \param  udp_port      wrong UDP port number
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_WrongServerPort(uint16_t udp_port) {
    EventRecord2 (EvtNetSNTP_WrongServerPort, udp_port, 0);
  }
#else
  #define EvrNetSNTP_WrongServerPort(udp_port)
#endif

/**
  \brief  Event on SNTP received frame is too short (Error)
  \param  length        frame length in bytes
  \param  min_length    minimum length of the frame
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_FrameTooShort(uint32_t length, uint32_t min_length) {
    EventRecord2 (EvtNetSNTP_FrameTooShort, length, min_length);
  }
#else
  #define EvrNetSNTP_FrameTooShort(length, min_length)
#endif

/**
  \brief  Event on SNTP wrong NTP server address (Error)
  \param  ip4_addr      pointer to IPv4 address of NTP server
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_WrongServerAddress(const uint8_t *ip4_addr) {
    EventRecordData (EvtNetSNTP_WrongServerAddress, ip4_addr, 4);
  }
#else
  #define EvrNetSNTP_WrongServerAddress(ip4_addr)
#endif

/**
  \brief  Event on SNTP message mode not server (Error)
  \param  mode          message mode
                         - 0: reserved
                         - 1: symetric active
                         - 2: symetric passive
                         - 3: client
                         - 4: server
                         - 5: broadcast
                         - 6: reserved
                         - 7: reserved
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_ModeNotServer(uint8_t mode) {
    EventRecord2 (EvtNetSNTP_ModeNotServer, mode, 0);
  }
#else
  #define EvrNetSNTP_ModeNotServer(mode)
#endif

/**
  \brief  Event on SNTP message mode not broadcast (Error)
  \param  mode          message mode
                         - 0: reserved
                         - 1: symetric active
                         - 2: symetric passive
                         - 3: client
                         - 4: server
                         - 5: broadcast
                         - 6: reserved
                         - 7: reserved
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_ModeNotBroadcast(uint8_t mode) {
    EventRecord2 (EvtNetSNTP_ModeNotBroadcast, mode, 0);
  }
#else
  #define EvrNetSNTP_ModeNotBroadcast(mode)
#endif

/**
  \brief  Event on SNTP answer received in wrong state (Error)
  \param  state         client state
                         - 0: idle
                         - 1: busy in unicast send
                         - 2: open in broadcast receive
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_AnswerInWrongState(uint8_t state) {
    EventRecord2 (EvtNetSNTP_AnswerInWrongState, state, 0);
  }
#else
  #define EvrNetSNTP_AnswerInWrongState(state)
#endif

/**
  \brief  Event on SNTP display time stamp in seconds (Op)
  \param  ref_time      reference time (since Jan 1, 1900)
  \param  utc_time      utc time       (since Jan 1, 1970)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_ShowTimeStamp(uint32_t ref_time, uint32_t utc_time) {
    EventRecord2 (EvtNetSNTP_ShowTimeStamp, ref_time, utc_time);
  }
#else
  #define EvrNetSNTP_ShowTimeStamp(ref_time, utc_time)
#endif

/**
  \brief  Event on SNTP invalid time stamp (Op)
  \param  ref_time      reference time (since Jan 1, 1900)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_TimeStampInvalid(uint32_t ref_time) {
    EventRecord2 (EvtNetSNTP_TimeStampInvalid, ref_time, 0);
  }
#else
  #define EvrNetSNTP_TimeStampInvalid(ref_time)
#endif

/**
  \brief  Event on SNTP client de-initialize (Op)
 */
#ifdef DEBUG_EVR
  __STATIC_INLINE void EvrNetSNTP_UninitClient(void) {
    EventRecord2 (EvtNetSNTP_UninitClient, 0, 0);
  }
#else
  #define EvrNetSNTP_UninitClient()
#endif
